#include <syslib/types.h>

/*
  basename -- strip directory and suffix from filenames
  Copyright (C) 1990-1997, 1999-2003,2005,2006 Jozef Behran

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software Foundation,
  Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/*
  Usage: basename name [suffix]
  NAME is a pathname; SUFFIX is a suffix to strip from it.

  basename /usr/foo/lossage/functions.l
  => functions.l
  basename /usr/foo/lossage/functions.l .l
  => functions
  basename functions.lisp p
  => functions.lis

  also for filenames with backslashes.
*/

#include <syslib/env.h>

#include <libhs/strings.h>
#include <libhs/dirname.h>
#include <libhs/output.h>
#include <libhs/process.h>

/* The name this program was run with. */
char *ProgramName;

void Hint(void)
{
  PrintErr("Try '");
  PrintErr(ProgramName);
  PrintErr(" --help' for more information.\n");
}

void DumpError(const char *Msg)
{
  PrintErr(ProgramName);
  PrintErr(": ");
  PrintErr(Msg);
  PrintErr("\n");
  Hint();
  Halt(1);
}

void Usage(void)
{
  Print ("\nUsage:\t");
  Print(ProgramName);
  Print(" NAME [SUFFIX]\n\t");
  Print(ProgramName);
  Print(
    " OPTION\n\n"
    "Print NAME with any leading directory components removed.\n"
    "If specified, also remove a trailing SUFFIX.\n"
    "\n"
    "--help\t\tdisplay this help and exit\n"
    "--version\toutput version information and exit\n"
    "\n"
    "Report bugs to <"
  );
  Print("???");
  Print(">.\n");
  Halt(0);
}

void Version(void)
{
  Print(
    "basename (OSHS coreutils) 0.0.1\n"
    "written by NN\n"
    "rewritten by The Holy Spirit through Jozef Behran\n"
    "\n"
    "Copyright (C) 2003 Free Software Foundation, Inc.\n"
    "This is free software; see the source for copying conditions.\n"
    "There is NO WARRANTY; not even for MERCHANTABILITY or FITNESS FOR A\n"
    "PARTICULAR PURPOSE."
  );
  Halt(0);
}

/*
  Remove "Suffix" from the end of "Name" if it is there, unless "Name"
  consists entirely of "Suffix".
*/
static void RemoveSuffix(char *Name, const char *Suffix)
{
  char *np;
  const char *sp;

  np = Name + StrLen (Name);
  sp = Suffix + StrLen (Suffix);

  while (np > Name && sp > Suffix) {
    if (*--np != *--sp) return;
  }
  if (np > Name) *np = '\0';
}

int16 Main(void)
{
  uint16 argc;
  char *Name;
  char *Arg;

  argc=GetArgCnt();
  ProgramName = StripBaseName(GetSelf());
  Arg = GetArg(0);

  if (argc==1) {
    if (!StrCmp(Arg,"--help")) Usage();
    if (!StrCmp(Arg,"--version")) Version();
  }

  if (argc<1) DumpError("too few arguments");
  if (argc>2) DumpError("too many arguments");

  Name = StripBaseName(Arg);

  if (argc == 2) {
    RemoveSuffix(Name, GetArg(1));
  }

  Print(Name);
  NewLine();

  return(0);
}
