#include <syslib/types.h>
#define __INTERFACE__
#include "errno.h"

#include <syslib/kernel.h>
#include <syslib/pdgfxcon.h>
#include <syslib/pdbase.h>
#include <syslib/pdfsys.h>
#include <syslib/pdinet.h>
#include <syslib/pdexec.h>

static int16 ErrNo;

int16 GetErrNo(void)
{
  return(ErrNo);
}

void SetErrNo(int16 Value)
{
  ErrNo=Value;
}

bool IsError(void)
{
  return(GetErrNo()<0);
}

/*
  Procedure to get a human-readable error message for the code specified.
*/
const char *ComplainError(int16 Error)
{
  switch (Error) {
    default             :return("Unknown error");
    /*
      Errors reported by the kernel
    */
    case keBADHND   :return("Invalid handle");
    case keNOMEM    :return("Not enough physical memory for the operation");
    case keNOHND    :return("No room for new handle");
    case keNOPBK    :return("Handle is not a valid pageblock handle");
    case keNOPROC   :return("Handle is not a valid process handle");
    case keNOTSUP   :return("Feture not supported");
    case keCLOSED   :return("Blocking call aborted due to handle close");
    case keMSGSIZE  :return("Message has invalid size");
    case keRANGE    :return("Value out of range");
    case keNOCHN    :return("Handle is not a valid channel handle");
    case keFAULT    :return("User supplied buffer not in valid memory");
    case keMTHREAD  :return("Too many threads");
    case keNOTHR    :return("Handle is not a valid thread handle");
    /*
      Standard errors reported by any server
    */
    case beREQLEN        :return("Request too short");
    case beINVCMD        :return("Invalid command in request");
    case beNOTSUP        :return("Requested action not supported");
    case beINVAL         :return("Invalid value in the request");
    case beUNKERR        :return("Unknown server error");
    case beNOMEM         :return("Server is out of memory");
    case beIO            :return("Low level I/O error");
    case beFAULT         :return("Attempt to I/O to/from invalid addr");
    case beINVDAT        :return("Invalid data from device");
    case beRANGE         :return("Argument out of range");
    case beTIMEOUT       :return("Device timeout error");
    case beCMDSEQ        :return("Command sequence error");
    /*
      POSIX compatibility errors (not used by native OSHS implementation)
    */
    case be2BIG          :return("Argument list too big");
    case beACCESS        :return("Permission denied");
    case beAGAIN         :return("Resource temporarily unavailable");
    case beBADF          :return("Bad file descriptor");
    case beBADMSG        :return("Bad message");
    case beBUSY          :return("Resource busy");
    case beCANCELLED     :return("Operation cancelled");
    case beCHILD         :return("No child process");
    case beDEADLK        :return("Resource deadlock avoided");
    case beDOM           :return("Domain error");
    case beEXIST         :return("File already exists");
    case beFBIG          :return("File too large");
    case beINPROGRESS    :return("Operation in progress");
    case beINTR          :return("Can't restart function call after intr");
    case beISDIR         :return("Object is a directory");
    case beMFILE         :return("Too many open files in process");
    case beMLINK         :return("Too many links");
    case beMSGSIZE       :return("Message buffer size not appropiate");
    case beNAMETOOLONG   :return("Filename too long");
    case beNFILE         :return("Too many open files in system");
    case beNODEV         :return("No such device");
    case beNOENT         :return("No such file or directory");
    case beNOEXEC        :return("Exec format error");
    case beNOLOCK        :return("No locks available");
    case beNOSPC         :return("No more free space on device");
    case beNOSYS         :return("Function not implemented");
    case beNOTDIR        :return("Object is not a directory");
    case beNOTEMPTY      :return("Directory not empty");
    case beNOTTY         :return("Inapropiate I/O control operation");
    case beNXIO          :return("No such device or adress");
    case bePERM          :return("Operation not permitted");
    case bePIPE          :return("Broken pipe");
    case beROFS          :return("Filesystem is read only");
    case beSPIPE         :return("Invalid seek operation");
    case beSRCH          :return("No such process");
    case beTIMEDOUT      :return("Operation timed out");
    case beXDEV          :return("Improper link");
    /*
      Filesystem errors
    */
    case fseINVDRV     :return("Drive not present");
    case fseEXIST      :return("Filesystem object already exists");
    case fseISDIR      :return("Filesystem object is a directory");
    case fseISLNK      :return("Filesystem object is a symlink");
    case fseISFILE     :return("Filesystem object is a file");
    case fseISDEV      :return("Filesystem object is a device access point");
    case fseACCESS     :return("Filesystem access denied");
    case fseNAMELEN    :return("File specification too long");
    case fseNOENT      :return("Filesystem object does not exist");
    case fsePLNDIR     :return("Path leads through non-directory");
    case fseNOIO       :return("Device or pipe not ready");
    case fseROFS       :return("Attempt to write on a read-only filesystem");
    case fsePRGUSE     :return("Program file is in use");
    case fseLOOP       :return("Too many symbolic link indirections");
    case fseNOSPC      :return("No room on storage device");
    case fseFILELEN    :return("File too large");
    case fseNOTSUP     :return("Feature not supported by the filesystem");
    case fseNOSEEK     :return("Descriptor is not seekable");
    case fseBADNAME    :return("Invalid filename for the filesystem");
    case fseINVDLET    :return("Invalid drive specification");
    case fseRENADRV    :return("Cannot rename across drive");
    case fseNODEV      :return("No such device");
    case fseCDATA      :return("Filesystem control data corrupted");
    case fseLONGIO     :return("File does not support long I/O");
    /*
      GFXCON errors
    */
    case screBADSIZE    :return("Bad request struct length");
    case screBADFLAGS   :return("Reserved flags set in mode info");
    case screBADPXSIZE  :return("Invalid pixel size");
    case screSCRINIT    :return("Screen hardware init failure");
    case screMODESUP    :return("Mode not supported");
    case screNOTSUP	:return("Command not supported by the driver");
    case screKBDINIT    :return("Keyboard hardware init failure");
    case screNOMODESET  :return("No video mode set");
    case screSCRHWFAIL	:return("Screen hardware failure");
    case screREGWRT	:return("A region is already being written");
    case screBPXID	:return("Bad pixmap ID");
    case screINCINS	:return("Incomplete instruction in code");
    case screINVINS	:return("Invalid instruction in code");
    case screMRECTOUT	:return("Mount rectangle outside window");
    case screSRECTOUT	:return("Source rectangle outside boundaries");
    case screDRECTOUT	:return("Destination rectangle outside boundaries");
    case screMOVERLAP	:return("Mount rectangle overlaps another");
    case screNOTMNT	:return("Attempt to unmount non-mounted pixmap");
    case screPARTPIX	:return("Partial pixel in pixmap input data");
    /*
      INET errors
    */
    case ineteNOHOST      :return("The host is unknown");
    case ineteNOADDR      :return("The host has no address");
    case ineteUNSERR      :return("Unrecoverable nameserver error");
    case ineteTMPERROR    :return("Temporary nameserver error");
    case ineteNRERR       :return("Unknown name resolution error");
    case inetePROTOSUP    :return("Protocol not supported");
    case ineteAFSUP       :return("Adress family not supported");
    case ineteTOOMANY     :return("Too many open sockets");
    case ineteSACCESS     :return("Access denied to the socket");
    case ineteADDRACC     :return("Access to adress denied");
    case ineteADDRUSE     :return("Address is in use");
    case ineteSSETUP      :return("Error performing socket setup");
    case ineteCMDSUP      :return("Command not supported");
    case ineteMISNULL     :return("Name is not null-terminated");
    case ineteMADDRS      :return("Too many adresses returned");
    case ineteADDRFMT     :return("Invalid host adresses format");
    case ineteREFUSED     :return("Connection refused");
    case ineteTIMEOUT     :return("Connection timed out");
    case ineteUNREACH     :return("Network unreachable");
    case ineteCACCESS     :return("Connection access denied");
    case ineteCERROR      :return("Unknown connection error");
    case ineteNORESOLV    :return("No address resolution in progress");
    case ineteSDOWN       :return("Socket is down");
    /*
      Execution server errors
    */
    case execeMANYARGS     :return("Too many program arguments");
    case execeFORMAT       :return("Executable file format error");
    case execeSUIDACC      :return("Suid program execution denied");
    case execeTXTBSY       :return("Cannot execute file open for writing");
    case execeNONPROG      :return("No program code to execute");
    case execeNONLIB       :return("No library code to load");
    /*
      LIBHS specific errors
    */
    case lheINVREP         :return("Invalid reply from server");
    case lheREOF           :return("Unexpected end of file");
    case lheIO             :return("File I/O error");
    case lheACCESS         :return("Buffered file access denied");
    case lheINVAL          :return("Invalid value passed to system library");
    case lheBUSY           :return("File position is busy");
    case lheNOMEM          :return("Library call went out of memory");
    case lheNOFILE         :return("No file at I/O position");
    case lheINVNUM         :return("Invalid numeric format");
  }
}

const char *ComplainLastError(void)
{
  return(ComplainError(GetErrNo()));
}
