#include <syslib/types.h>
#define __INTERFACE__
#include "exec.h"

#include <syslib/kernel.h>
#include <syslib/pdexec.h>
#include <syslib/pdbase.h>
#include <syslib/pdfsys.h>

#include "strings.h"
#include "servers.h"
#include "errno.h"
#include "libman.h"
#include "diskio.h"

bool Load(const char *LibSpec)
{
  TLoadCmd C;
  TServerInfo I;
  uint8 Dummy;
  char *Ptr;

  C.Command=cmdLOAD;
  C.LibSpec[MAX_MESSAGE_SIZE-2]='\0';
  Ptr=StrNCpy(C.LibSpec,LibSpec,MAX_MESSAGE_SIZE-1);
  if (C.LibSpec[MAX_MESSAGE_SIZE-2]!='\0') {
    SetErrNo(fseNAMELEN);
    return(false);
  }
  I.Server=hndEXECDRV;
  I.Cmd=&C;
  I.CmdSize=Ptr-C.LibSpec+2;
  I.Reply=&Dummy;
  I.RepSize=sizeof(Dummy);
  if (!SendCmd(&I)) return(false);
  return(I.RepSize==0);
}

static bool CheckFileExists(const char *FileSpec)
{
  int16 Handle;

  Handle=Open(FileSpec,oeOPEN+oRDONLY);
  if (Handle>=0) Close(Handle);
  return(Handle>=0);
}

static bool LoadMissingLibraries(const char *LibList,uint16 Size)
{
  char Buffer[MAX_FILESPEC_SIZE+4];
  const char *LibName;
  char *Ptr;
  char Ch;

  Buffer[MAX_FILESPEC_SIZE]='\0';
  LibName=LibList;
  while (Size>0) {
    Ch=*(LibList++);
    Size--;
    if (Ch=='\0') {
      Ptr=GetBinaryDirectory(Buffer,sizeof(Buffer));
      Ptr=StrNCpy(Ptr,LibName,sizeof(Buffer)-(Ptr-Buffer));
      PutBinaryExtension(Ptr,true);
      if (Buffer[MAX_FILESPEC_SIZE]!='\0') {
        SetErrNo(fseNAMELEN);
        return(false);
      }
      if (CheckFileExists(Buffer)) {
        if (!Load(Buffer)) return(false);
      } else {
        return(false);
      }
      LibName=LibList;
    }
  }
  if (Ch!='\0') SetErrNo(beUNKERR);
  return(Ch=='\0');
}

int16 Call(const char *const* Args,const char *const* Env)
{
  TExecCmd C;
  TExecReply R;
  TServerInfo I;

  C.Command=cmdCALL;
  C.ArgList=(void *)Args;
  C.EnvList=(void *)Env;
  I.Server=hndEXECDRV;
  I.Cmd=&C;
  I.CmdSize=sizeof(C);
  I.Reply=&R;
  I.RepSize=sizeof(R);
  while (true) {
    if (!SendCmd(&I)) return(-1);
    if (I.RepSize==2) break;
    if (I.RepSize>2) {
      if (R.ExitCode==execeNOLIB) {
        if (!LoadMissingLibraries((void *)R.Info,I.RepSize-2)) return(-1);
        continue;
      }
    }
    SetErrNo(beUNKERR);
    return(-1);
  }
  return(R.ExitCode);
}
