#include <syslib/types.h>
#define __INTERFACE__
#include "exparr.h"

/*
  Expandable array support.
*/

#include <syslib/traps.h>

#include "memory.h"

#define ARRAY_INCREMENT 64

void InitArray(TArray *Array)
{
  Array->Cnt=0;
  Array->Arr=NULL;
}

TArrayHandle *OpenArray(TArray *Array)
{
  TArrayHandle *Handle;

  Handle=GetMem(sizeof(*Handle));
  if (Handle==NULL) return(NULL);
  OpenArrayEx(Array,Handle);
  Handle->FreeIt=true;
  return(Handle);
}

TArrayHandle *CreateArray(TArray *Array)
{
  InitArray(Array);
  return(OpenArray(Array));
}

void OpenArrayEx(TArray *Array, TArrayHandle *Handle)
{
  Handle->FreeIt=false;
  Handle->Array=Array;
  Handle->Space=Array->Cnt;
}

void ClearOpenedArray(TArrayHandle *Handle)
{
  Handle->Array->Cnt=0;
}

void CreateArrayEx(TArray *Array, TArrayHandle *Handle)
{
  InitArray(Array);
  OpenArrayEx(Array,Handle);
}

bool ExpandArray(TArrayHandle *Handle)
{
  int16 NewSize;
  void *New;

  if (Handle->Space==Handle->Array->Cnt) {
    Handle->Space+=ARRAY_INCREMENT;
    NewSize=Handle->Space*sizeof(*Handle->Array->Arr);
    New=ReallocMem(Handle->Array->Arr,NewSize);
    if (New==NULL) {
      Handle->Space-=ARRAY_INCREMENT;
      return(false);
    }
    Handle->Array->Arr=New;
  }
  return(true);
}

bool PutItem(TArrayHandle *Handle, void *Item)
{
  bool Result;

  Result=ExpandArray(Handle);
  if (Result) Handle->Array->Arr[Handle->Array->Cnt++]=Item;
  return(Result);
}

void DeleteItems(TArrayHandle *Handle, uint16 Count)
{
  Handle->Array->Cnt-=Count;
}

void CloseArray(TArrayHandle *Handle)
{
  int16 Size;

  Size=Handle->Array->Cnt*sizeof(*Handle->Array->Arr);
  Handle->Array->Arr=ReallocMem(Handle->Array->Arr,Size);
  if (Size>0) {
    if (Handle->Array->Arr==NULL) ReportBug("Memory shrinking failed");
  }
  if (Handle->FreeIt) FreeMem(Handle);
}

void ClearArray(TArray *Array)
{
  DestroyArray(Array);
  InitArray(Array);
}

void DestroyArray(TArray *Array)
{
  if (Array->Arr!=NULL) FreeMem(Array->Arr);
}
