#include <syslib/hhead.h>
#ifndef __h__LIBHS_EXPARR__
#define __h__LIBHS_EXPARR__

/*** Informations about expandable array ***/
/*
  This structure holds informations about an expandable array.
*/
typedef struct tagTArray {
  /*
    Count of items in the array.
  */
  uint16 Cnt;
  /*
    Pointer to the array of the items.
  */
  void **Arr;
} TArray;
/*
  This structure is used to hold basic informations about the array. The
  array itself is allocated on the heap.
  
  The sparse names of the fields of this structure are intentional; normally
  the identifier which references the array itself says what is the
  particular array for.
*/

/*** The array handle internals ***/
/*
  This structure holds informations about an open expandable array.
*/
typedef struct tagTArrayHandle {
  /*
    Pointer to the asociated array.
  */
  TArray *Array;
  /*
    How many allocated item slots are in the array.
  */
  uint16 Space;
  /*
    If True, ~CloseArray() will free the handle after the array is closed.
  */
  bool FreeIt;
} TArrayHandle;
/*
  This 'opaque' structure acts as an identifier for an open expandable array.
  Although exposed to public, the content of this structure is internal to
  the module and the caller should not care about it nor inspect it or worse
  modify it.
*/

/*** Calls: Calls about the expandable arrays ***/

void InitArray(TArray *Array);
TArrayHandle *CreateArray(TArray *Array);
TArrayHandle *OpenArray(TArray *Array);
void CreateArrayEx(TArray *Array, TArrayHandle *Handle);
void OpenArrayEx(TArray *Array, TArrayHandle *Handle);
void ClearOpenedArray(TArrayHandle *Handle);
bool ExpandArray(TArrayHandle *Handle);
bool PutItem(TArrayHandle *Handle, void *Item);
void DeleteItems(TArrayHandle *Handle, uint16 Count);
void CloseArray(TArrayHandle *Handle);
void ClearArray(TArray *Array);
void DestroyArray(TArray *Array);

#endif
#include <syslib/hfoot.h>
