#include <syslib/hhead.h>
#ifndef __h__LIBHS_FILEBUF__
#define __h__LIBHS_FILEBUF__

#define fm_LITTLEENDIAN 1

/*
  Structure for the file buffer. Each file can act as input or output file.
  For each of the files the input and output positions are stored and they
  are switched to each other as needed. The input and output formats are
  also specified.
*/
typedef struct {
  int16 F;
  int16 IOError;
  int16 Flags;
  uint16 IOBytes;
  uint32 Position;
  uint32 StoredPosition;
  uint16 BufferSize;
  uint16 DataSize;
  uint16 DataPos;
  upint8 *Buffer;
} TBufferedFile, *PBufferedFile;

#define F_READ          1
#define F_WRITE         2
#define F_CLEAR         4
#define F_EXCL          8

bool OpenFile(PBufferedFile F,const char *Spec,int16 Mode);
bool FlushFile(PBufferedFile F);
bool CloseFile(PBufferedFile F);
int16 FileIOResult(PBufferedFile F);
bool SeekRead(PBufferedFile F,uint32 NewPos);
uint32 TellRead(PBufferedFile F);
bool SeekWrite(PBufferedFile F,uint32 NewPos);
uint32 TellWrite(PBufferedFile F);
uint32 TellFileSize(PBufferedFile F);
bool FileReadBlock(PBufferedFile F,void *Buf,uint32 Count);
bool FileWriteBlock(PBufferedFile F,const void *Buffer,uint32 Count);

int16 IOResult(void);
const char *ComplainLastIOError(void);

bool SetupInputFile(PBufferedFile F);
bool AllocateAndOpenInputFile(const char *FileSpec);
bool SetupAndOpenInputFile(PBufferedFile F,const char *FileSpec);
bool SeekInputFile(uint32 NewPosition);
bool CheckInputFileReady(void);
uint32 GetInputFileSize(void);
uint32 GetInputFilePos(void);
uint8 ReadByte(void);
uint16 Read16BE(void);
uint16 Read16LE(void);
uint32 Read32BE(void);
uint32 Read32LE(void);
bool ReadBlock(void *Buffer,uint32 Size);
bool SkipRestOfLine(void);
PBufferedFile ReleaseInputFile(void);
bool CloseAndFreeInputFile(void);
bool CloseAndReleaseInputFile(void);

bool SetupOutputFile(PBufferedFile F);
bool AllocateAndCreateOutputFile(const char *FileSpec);
bool SetupAndCreateOutputFile(PBufferedFile F,const char *FileSpec);
bool AllocateAndOpenOutputFile(const char *FileSpec);
bool SetupAndOpenOutputFile(PBufferedFile F,const char *FileSpec);
bool AllocateAndCreateOutputFileIfNonex(const char *FileSpec);
bool SetupAndCreateOutputFileIfNonex(PBufferedFile F,const char *FileSpec);
bool SeekOutputFile(uint32 NewPosition);
bool CheckOutputFileReady(void);
uint32 GetOutputFileSize(void);
uint32 GetOutputFilePos(void);
void WriteByte(uint8 What);
void Write16LE(uint16 What);
void Write16BE(uint16 What);
void Write32LE(uint32 What);
void Write32BE(uint32 What);
bool WriteBlock(const void *Buffer,uint32 Size);
bool WriteText(const char *Str);
bool WriteString(const char *Str);
void FlushOutputFile(void);
PBufferedFile ReleaseOutputFile(void);
bool CloseAndFreeOutputFile(void);
bool CloseAndReleaseOutputFile(void);

bool SetupInOutFile(PBufferedFile F);
bool AllocateAndCreateInOutFile(char *FileSpec);
bool SetupAndCreateInOutFile(PBufferedFile F,char *FileSpec);
bool AllocateAndOpenInOutFile(char *FileSpec);
bool SetupAndOpenInOutFile(PBufferedFile F,char *FileSpec);
void MapInOutFileRead(void);
void UnmapInOutFileRead(void);
void MapInOutFileWrite(void);
void UnmapInOutFileWrite(void);
PBufferedFile ReleaseInOutFile(void);
bool CloseAndFreeInOutFile(void);
bool CloseAndReleaseInOutFile(void);

#endif
#include <syslib/hfoot.h>
