#include <syslib/types.h>
#define __INTERFACE__
#include "inet.h"

#include "strings.h"
#include "errclass.h"
#include "misc.h"
#include "errno.h"
#include "devices.h"

static bool INETOpen;
static int16 INETHandle;
static bool ResolveInProgress;

static bool CommandDevice(void *Cmd,uint16 CmdSize,void *Rep,uint16 RepSize)
{
  int16 ErrNo;

  /*
    Ensure the device is open
  */
  if (!INETOpen) {
    INETHandle=OpenFirstDeviceOfType(dtInternet);
    if (INETHandle==-1) return(false);
    INETOpen=true;
  }
  /*
    Send the command packet into the device
  */
  ErrNo=KCALL__SEND(INETHandle,Cmd,CmdSize);
  if (ErrNo!=0) {
    SetErrNo(ErrNo);
    return(false);
  }
  /*
    Read the reply from the screen device
  */
  ErrNo=KCALL__RECV(INETHandle,Rep,RepSize,NULL);
  SetErrNo(ErrNo);
  /*
    Return success flag
  */
  return(ErrNo>=0);
}

int16 OpenServerSocket(uint16 Port,uint16 QueueSize)
{
  TOpenTCPServerCommand C;
  TOpenTCPServerReply R;

  C.Command=inetcmdOpenTCPServer;
  C.Port=Port;
  C.QueueSize=QueueSize;
  if (!CommandDevice(&C,sizeof(C),&R,sizeof(R))) return(-1);
  return(R.Handle);
}

bool ListenServer(int16 Handle,TConnectionRecord *R)
{
  int16 ErrNo;

  ErrNo=KCALL__SEND(Handle,NULL,0);
  if (ErrNo==0) ErrNo=KCALL__RECV(Handle,R,sizeof(*R),NULL);
  SetErrNo(ErrNo);
  return(ErrNo>=0);
}

bool OpenClientSocket(THostAddr *Host,uint16 Port,TOpenTCPClientReply *R)
{
  TOpenTCPClientCommand C;

  C.Command=inetcmdOpenTCPClient;
  MemCpy(C.Host,Host,sizeof(C.Host));
  C.Port=Port;
  return(CommandDevice(&C,sizeof(C),R,sizeof(*R)));
}

static bool CheckIP(const char *Name,TOpenTCPClientCommand *R)
{
  uint8 IPNums[3];
  int8 Current;
  int16 Value;
  char Ch;
  bool IsNumber;
  int16 i;

  Current=0;
  Value=0;
  IsNumber=false;
  do {
    Ch=*(Name++);
    if (Ch>='0' && Ch<='9') {
      Value=Value*10+Ch-48;
      if (Value>255) return(false);
      IsNumber=true;
    } else if (Ch=='.') {
      if (!IsNumber || Current==3) return(false);
      IPNums[Current++]=Value;
      Value=0;
      IsNumber=false;
    } else {
      if (Ch) return(false);
    }
  } while (Ch);
  for (i=0;i<5;i++) R->Host[i]=0;
  R->Host[5]=0xFFFF;
  R->Host[6]=IPNums[0]*256+IPNums[1];
  R->Host[7]=IPNums[2]*256+Value;
  return(true);
}

bool FindFirstAddress(const char *Name,TOpenTCPClientCommand *R)
{
  TResolveNameCommand C;

  if (CheckIP(Name,R)) return(true);
  ResolveInProgress=true;
  C.Command=inetcmdResolveName;
  StrNCpy(C.Name,Name,MAX_MESSAGE_SIZE-2);
  C.Name[MAX_MESSAGE_SIZE-2]='\0';
  return(CommandDevice(&C,sizeof(C),R,sizeof(*R)));
}

bool FindNextAddress(TOpenTCPClientCommand *R)
{
  upint8 C;

  if (!ResolveInProgress) return(false);
  C=inetcmdNextAddress;
  ResolveInProgress=CommandDevice(&C,1,R,sizeof(*R));
  return(ResolveInProgress);
}

bool ResolveAndConnect(char *Host,uint16 Port,TOpenTCPClientReply *R)
{
  TOpenTCPClientCommand C;

  if (!FindFirstAddress(Host,&C)) return(-1);
  C.Command=inetcmdOpenTCPClient;
  C.Port=Port;
  return(CommandDevice(&C,sizeof(C),R,sizeof(*R)));
}
