#include <syslib/types.h>
#define __INTERFACE__
#include "keyword.h"

#include <syslib/traps.h>

#include "strings.h"

#define RESERVED_WORD_LEN_LIMIT 32

int16 RecognizeWord(int16 Sensitivity,const char *W,const char *L)
{
  char BufferWord[RESERVED_WORD_LEN_LIMIT+1];
  char BufferIn[RESERVED_WORD_LEN_LIMIT+1];
  char *Ptr;
  int16 WordID;

  /*
    Copy the reserved word into the reserved word input buffer. If case
    insensitivity is requested, convert the command to uppercase since the
    reserved word values are.
  */
  switch (Sensitivity) {
    case sensINSENSITIVE: {
      Ptr=StrNCpy(BufferIn,W,RESERVED_WORD_LEN_LIMIT+1);
      break;
    }
    case sensUPPERCASE: {
      Ptr=StrNToUpr(BufferIn,W,RESERVED_WORD_LEN_LIMIT+1);
      break;
    }
    case sensLOWERCASE: {
      Ptr=StrNToLwr(BufferIn,W,RESERVED_WORD_LEN_LIMIT+1);
      break;
    }
    default: {
      TrapUndefined("Invalid value passed to RecognizeWord()");
      return(0);
    }
  }

  /*
    If the potential reserved word name is longer than the longest possible
    reserved word, return "unrecognized" value, which is represented as zero.
    This check is here to avoid "buffer overrun" security hole bugs.
  */
  if (Ptr-BufferIn>RESERVED_WORD_LEN_LIMIT) return(0);

  /*
    Go through the reserved word list and search for the reserved word.
  */

  WordID=1;
  do {

    /*
      Read a command into the command compare buffer and convert to uppercase.
    */

    Ptr=BufferWord;
    do {
      if (Sensitivity==sensLOWERCASE) {
        *Ptr=DownCase(*L);
      } else {
        *Ptr=UpCase(*L);
      }
      L++;
      Ptr++;
    } while (*L>'Z');
    *Ptr='\0';

    /*
      Compare the command already read to the one given by the caller. If they
      match, return apropiate number, which is stored in "WordID".
    */

    if (!StrCmp(BufferIn,BufferWord)) return(WordID);

    /*
      Update "WordID" and loop again.
    */

    WordID++;

  } while (*L);

  return(0);
}

void GetWord(const char *W,uint16 ID,char *Dest,bool UpperCase)
{
  const char *Ptr;
  char Ch;

  Ptr=W;
  /*
    Find the word in the list
  */
  while (ID>0) {
    ID--;
    do {
      Ptr++;
      assert(*Ptr!='\0');
    } while (*Ptr>'Z');
  }
  /*
    Construct the result
  */
  do {
    Ch=DownCase(*(Ptr++));
    if (UpperCase) Ch=UpCase(Ch);
    *(Dest++)=Ch;
  } while (*Ptr>'Z');
  *Dest='\0';
}
