#include <syslib/types.h>
#define __INTERFACE__
#include "output.h"

#include <syslib/kernel.h>
#include <syslib/pdbase.h>
#include <syslib/pdfsys.h>
#include <syslib/pdfileio.h>

#include "strings.h"
#include "diskio.h"
#include "filebuf.h"
#include "errno.h"
#include "numconvi.h"

static int16 Output;
static uint16 Counter;

void Print(const char *Str)
{
  uint16 Len;
  int16 ErrorTmp;

  if (Output==0) Output=1;
  Len=StrLen(Str);
  if (Len>0) {
    ErrorTmp=GetErrNo();
    if (Output>0) {
      Write(Output,Str,Len);
    } else {
      WriteBlock(Str,Len);
    }
    SetErrNo(ErrorTmp);
  }
}

void PrintChar(char Ch)
{
  char Chars[2];

  Chars[0]=Ch;
  Chars[1]='\0';
  Print(Chars);
}

void ActivateErr(void)
{
  if (Output==0) Output=1;
  assert(Output==1 || (Output==2 && Counter>0));
  Output=2;
  Counter++;
}

void DeactivateErr(void)
{
  if (Output==0) Output=1;
  assert(Output==2 && Counter>0);
  Counter--;
  if (Counter==0) Output=1;
}

void ActivateCurrentOutputFile(void)
{
  if (Output==0) Output=1;
  assert(Output>0);
  Output=-Output;
}

void DeactivateCurrentOutputFile(void)
{
  if (Output==0) Output=1;
  assert(Output<0);
  Output=-Output;
}

void PrintErr(const char *Str)
{
  int16 Tmp;
  
  Tmp=Output;
  Output=2;
  Print(Str);
  Output=Tmp;
}

void Repeat(char Ch,uint32 Count)
{
  char Buf[2];
  
  Buf[0]=Ch;
  Buf[1]='\0';
  for (;Count>0;Count--) Print(Buf);
}

void Spaces(uint32 Count)
{
  Repeat(' ',Count);
}

void PrintDWord(uint32 Num,uint16 Places)
{
  char Str[11];
  char *Ptr;
  int16 Size;

  Ptr=FormatDWordCore(Num,Str+11);
  Size=Str-Ptr+10;
  Spaces(Places>Size?Places-Size:0);
  Print(Ptr);
}

void PrintNum(int16 Num,uint16 Places)
{
  char Str[11];
  char *Ptr;
  int16 Size;

  if (Num<0) {
    Ptr=FormatDWordCore(-(uint32)Num,Str+11);
    Size=Str-Ptr+11;
    Spaces(Places>Size?Places-Size:0);
    Print("-");
    Print(Ptr);
  } else {
    PrintDWord(Num,Places);
  }
}

static void DispHex(uint32 Num, uint16 Places, bool DownCase)
{
  char Str[9];
  char *Ptr;
  int16 Size;

  Ptr=FormatHexCore(Num,Str+9,DownCase);
  Size=Str-Ptr+8;
  Repeat('0',Places>Size?Places-Size:0);
  Print(Ptr);
}

void PrintHex(uint32 DWord,uint16 Places)
{
  DispHex(DWord,Places,true);
}

void PrintHEX(uint32 DWord,uint16 Places)
{
  DispHex(DWord,Places,false);
}

void NewLine(void)
{
  Print("\n");
}

void PrintLn(const char *Str)
{
  Print(Str);
  NewLine();
}

void PrintFloat(float Number,uint16 Places,uint16 Decimal)
{
  Print("n/a");
}
