#include <syslib/types.h>
/*
  Test: The RB tree dictionaries torture
*/

#include "../strings.h"
#include "../memory.h"

#define BUILD_COUNT 10000
#define ADD_DELETE_COUNT 10000

uint32 UsedMemory;

void *GetCountMem(uint16 MemSize)
{
  upint16 *Ptr;

  Ptr=GetMem(MemSize+sizeof(upint16));
  if (Ptr==NULL) return(NULL);
  UsedMemory+=MemSize;
  *Ptr=MemSize;
  return(Ptr+1);
}

void FreeCountMem(const void *Ptr)
{
  const upint16 *P;

  P=Ptr;
  P--;
  UsedMemory-=(*P);
  FreeMem(P);
}

char *StrDupCount(const char *Source)
{
  uint16 Len;
  char *Mem;

  Len=StrLen(Source);
  Mem=GetCountMem(Len+1);
  if (Mem) MemCpy(Mem,Source,Len+1);
  return(Mem);
}

#define GetMem GetCountMem
#define FreeMem FreeCountMem
#define StrDup StrDupCount

#include "../misc.c"
#include "../dictrb.c"

#include "../output.h"

#define IDENT_SIZE 3

uint32 RandSeed;
uint32 TotalKeys;

int16 Random()
{
  RandSeed=(0x8088405L*RandSeed)%0x100000000L+1;
  return(RandSeed % 32768);
}

void Error(char *Msg)
{
  Print(Msg);
  NewLine();
}

void GenerateName(char *Buf)
{
  int16 i;
  int16 r;
  int16 Num;

  Buf[IDENT_SIZE]='\0';
  r=0;
  for (i=0;i<IDENT_SIZE;i++) {
    if (r==0) {
      Num=Random();
      r=3;
    }
    *(Buf++)='A'+Num%26;
    Num/=26;
    r--;
  }
}

TRBDict *BuildDict(void)
{
  char Buf[IDENT_SIZE+1];
  TRBDict *D;
  int16 i;

  D=CreateDict();
  if (D==NULL) {
    Error("Error creating dictionary");
    return(NULL);
  }
  for (i=0;i<BUILD_COUNT;i++) {
    GenerateName(Buf);
    if (AddItem(D,Buf,Buf)==rbSUCCESS) TotalKeys++;
  }
  return(D);
}

bool CheckDictProperties(TRBDict *D)
{
  TRBDict *Node;
  TRBDict *New;
  int16 Status;
  const char *Key;
  int16 Height;
  bool Counting;
  int16 CurrentCnt;
  uint32 KeysFound;

  /*
    Check if the structure is a RB-tree.
  */
  NIL.Flags=0;
  Node=D->Left;
  if (Node==&NIL) return(true);
  if (Node->Parent!=D) {
    Error("Root integrity error");
    return(false);
  }
  if (!(D->Flags & ndTREE)) {
    Error("The tree has not it's tree flag set");
    return(false);
  }
  Key="";
  Height=0;
  CurrentCnt=0;
  KeysFound=0;
  Counting=true;
  do {
    Status=Node->Flags & ndVISIT_FLAGS;
    Node->Flags |= ndVISIT_FLAGS;
    NIL.Parent=Node;
    switch (Status) {
      case stFRESH: {
        Status=stGOING_LEFT;
        New=Node->Left;
        if (New==NULL) {
          Error("Unexpected NULL");
	  return(false);
        }
        if (New->Flags & ndVISIT_FLAGS) {
          Error("Structure is not a tree");
          return(false);
        }
        if (New->Parent!=Node) {
          Error("Parent pointer consistency error");
          return(false);
        }
	if (New==&NIL) {
          if (Counting) {
            Counting=false;
            Height=CurrentCnt;
          } else {
            if (Height!=CurrentCnt) {
              Error("Black height incorrect");
              return(false);
            }
          }
	} else {
          if (New->Flags & rbRED) {
            if (Node->Flags & rbRED) {
              Error("Red node has red parent");
              return(false);
            }
          } else {
            CurrentCnt++;
          }
	}
	break;
      }
      case stGOING_LEFT: {
        if (Node->Key==NULL) {
          Error("Node without key");
          return(false);
        }
        if (Node->Data==NULL) {
          Error("Node with NULL data");
          return(false);
        }
        if (StrCmp(Key,Node->Key)>=0) {
          Error("Structure is not a ballanced tree");
          return(false);
        }
        Key=Node->Key;
        KeysFound++;
        Status=stGOING_RIGHT;
        New=Node->Right;
        if (New==NULL) {
          Error("Unexpected NULL");
	  return(false);
        }
        if (New->Flags & ndVISIT_FLAGS) {
          Error("Structure is not a tree");
          return(false);
        }
        if (New->Parent!=Node) {
          Error("Parent pointer consistency error");
          return(false);
        }
	if (New==&NIL) {
          if (Height!=CurrentCnt) {
            Error("Black height incorrect");
            return(false);
          }
	} else {
          if (New->Flags & rbRED) {
            if (Node->Flags & rbRED) {
              Error("Red node has red parent");
              return(false);
            }
          } else {
            CurrentCnt++;
          }
	}
	break;
      }
      case stGOING_RIGHT: {
        Status=stPROCESSED;
        New=Node->Parent;
        if (!(Node->Flags & rbRED)) CurrentCnt--;
	break;
      }
      default: {
        Error("Node with an invalid status found");
        return(false);
      }
    }
    Node->Flags &= ~ndVISIT_FLAGS;
    Node->Flags |= Status;
    if (New!=&NIL) Node=New;
  } while (Node!=D);
  if (KeysFound!=TotalKeys) {
    Error("Incorrect number of keys found in the tree");
    return(false);
  }
  /*
    Check the work of FindFirstKey/FindNextKey(). These calls also must clear
    the visiting flags.
  */
  KeysFound=0;
  if (FindFirstKey(D)) {
    KeysFound++;
    Key=D->Key;
    while (FindNextKey(D)) {
      KeysFound++;
      if (StrCmp(Key,D->Key)>=0) {
        Error("Wrong order of keys in FindFirst/FindNext");
        return(false);
      }
      Key=D->Key;
    }
  }
  if (KeysFound!=TotalKeys) {
    Error("Incorrect number of keys returned by the iteration");
    return(false);
  }
  /*
    Return success.
  */
  return(true);
}

void AddDeleteTest(TRBDict *D)
{
  char Buf[IDENT_SIZE+1];
  int16 i;
  int16 Res;

  for (i=0;i<ADD_DELETE_COUNT;i++) {
    GenerateName(Buf);
    Res=AddItem(D,Buf,Buf);
    if (Res==rbNOMEM) {
      Error("Out of memory");
      return;
    }
    if (Res) {
      DeleteItem(D,Buf);
      TotalKeys--;
    } else {
      TotalKeys++;
    }
  }
}

int16 Main(void)
{
  TRBDict *D;

  UsedMemory=0;
  D=BuildDict();
  CheckDictProperties(D);
  AddDeleteTest(D);
  CheckDictProperties(D);
  DestroyDict(D);
  if (UsedMemory!=0) Error("Memory leak");

  return(0);
}
