#include <syslib/types.h>
#define __INTERFACE__
#include "thread.h"

#include <syslib/traps.h>

int16 CreateNewThread(TMainFunction *MainFnc,void *Data)
{
  return(KCALL__THRNEW(MainFnc,Data,NULL));
}

int16 WaitThread(int16 ThreadID)
{
  return(KCALL__THRWAIT(ThreadID));
}

int16 KillThread(int16 ThreadID)
{
  return(KCALL__THRKILL(ThreadID));
}

int32 CreateLock(void)
{
  return(KCALL__LOCKNEW());
}

static void InvalidLock(void)
{
  ReportBug("Attempt to use an invalid lock handle");
}

static void UnexpectedFailure(void)
{
  ReportBug("Unexpected failure");
}

void Lock(int32 LockID)
{
  int16 Error;

  Error=KCALL__LOCK(LockID);
  if (Error==keBADHND) {
    InvalidLock();
  } else if (Error!=0) {
    UnexpectedFailure();
  }
}

void Unlock(int32 LockID)
{
  int16 Error;

  Error=KCALL__UNLOCK(LockID);
  if (Error==keBADHND) {
    InvalidLock();
  } else if (Error!=0) {
    UnexpectedFailure();
  }
}

void DestroyLock(int32 LockID)
{
  int16 Error;

  Error=KCALL__LOCKDEL(LockID);
  if (Error==keBADHND) {
    InvalidLock();
  } else if (Error!=0) {
    UnexpectedFailure();
  }
}

static void InvalidCond(void)
{
  ReportBug("Attempt to use an invalid conditional variable handle");
}

int32 CreateCond(void)
{
  return(KCALL__CONDNEW());
}

void SetCond(int32 CondID)
{
  int16 Error;

  Error=KCALL__SET(CondID);
  if (Error==keBADHND) {
    InvalidCond();
  } else if (Error!=0) {
    UnexpectedFailure();
  }
}

void ResetCond(int32 CondID)
{
  int16 Error;

  Error=KCALL__RESET(CondID);
  if (Error==keBADHND) {
    InvalidCond();
  } else if (Error!=0) {
    UnexpectedFailure();
  }
}

void WaitCond(int32 CondID)
{
  int16 Error;

  Error=KCALL__WAIT(CondID);
  if (Error==keBADHND) {
    InvalidCond();
  } else if (Error!=0) {
    UnexpectedFailure();
  }
}

void DestroyCond(int32 CondID)
{
  int16 Error;

  Error=KCALL__CONDDEL(CondID);
  if (Error==keBADHND) {
    InvalidCond();
  } else if (Error!=0) {
    UnexpectedFailure();
  }
}
