#include <syslib/types.h>
#define __INTERFACE__
#include "args.h"

/*
  Command line arguments parser
*/

#include <syslib/env.h>
#include <syslib/kernel.h>

#include <libhs/strings.h>

#include "errors.h"

static bool CheckAndCopyIntoBuf(char *Buf,const char *Arg,bool ModuleName)
{
  bool Result;

  Buf[MAX_FILENAME_LENGTH]='\0';
  StrNCpy(Buf,Arg,MAX_FILENAME_LENGTH+1);
  Result=(Buf[MAX_FILENAME_LENGTH]=='\0');
  if (!Result) {
    if (ModuleName) {
      PlainError("Module name too long");
    } else {
      PlainError("Package item name too long");
    }
    Buf[MAX_FILENAME_LENGTH]='\0';
  }
  return(Result);
}

bool AnalyzeArgs(void)
{
  char *Arg;
  bool Result;
  uint16 i;
  int16 MakeModule;

  /*
    Initialize the environment to default values.
  */
  i=0;
  Result=true;
  DebugDump=false;
  RootDir=NULL;
  DescFile=NULL;
  AddOnLibs=NULL;
  Platform=NULL;
  BuildAll=false;
  LinkStatically=false;
  ClearDirectory=false;
  InstallThings=false;
  MakeModule=0;
  PackageItemName[0]='\0';
  ModuleName[0]='\0';
  DisplayRescans=false;
  LinkSystemLibsDifferently=false;
  TreatWarningsAsErrors=true;
  /*
    Analyze the arguments and edit the environment according to them.
  */
  do {
    Arg=GetArg(i++);
    if (Arg==NULL) break;
    if (Arg[0]=='-') {
      switch (Arg[1]) {
        case 'R': {
	  RootDir=Arg+2;
	  break;
	}
	case 'L': {
	  AddOnLibs=Arg+2;
	  break;
	}
	case 's': {
          LinkStatically=true;
	  break;
	}
	case 'S': {
          LinkSystemLibsDifferently=true;
	  break;
	}
        case 'p': {
          Platform=Arg+2;
          break;
        }
        case 'd': {
          DebugDump=true;
          break;
        }
        case 'b': {
          BuildAll=true;
          break;
        }
        case 'r': {
          DisplayRescans=true;
          break;
        }
        case 'w': {
          TreatWarningsAsErrors=false;
          break;
        }
        default: {
          PlainError("Unrecognized option\1");
          PutArg(Arg);
          Result=false;
          break;
        }
      }
    } else {
      if (MakeModule>0) {
        if (MakeModule==1) {
          Result=CheckAndCopyIntoBuf(ModuleName,Arg,true);
        } else if (MakeModule==2) {
          Result=CheckAndCopyIntoBuf(PackageItemName,Arg,false);
        } else {
          PlainError("Too many arguments to 'mkmod' command");
          Result=false;
        }
        if (!Result) break;
        MakeModule++;
      } else {
        if (StrCmp(Arg,"clean")==0) {
          ClearDirectory=true;
        } else if (StrCmp(Arg,"install")==0) {
          InstallThings=true;
        } else if (StrCmp(Arg,"mkmod")==0) {
          MakeModule++;
        } else {
          if (DescFile!=NULL) {
            PlainError("Too many description files stated on command line");
            Result=false;
          }
          DescFile=Arg;
        }
      }
    }
  } while (true);
  /*
    Check whether the 'mkmod' command has at least one argument (the name of
    the module to make).
  */
  if (MakeModule && Result) {
    if (ModuleName[0]=='\0') {
      PlainError("You must specify the name for the module to make");
      Result=false;
    }
  }
  /*
    If the platform is configured to not support shared libraries, set static
    linking of the libraries.
  */
  if (!KCFG__SHARED_LIBRARIES_SUPPORTED()) LinkStatically=true;
  /*
    Done.
  */
  return(Result);
}
