#include <syslib/hhead.h>
#ifndef __h__OBS_FILEEXT__
#define __h__OBS_FILEEXT__

/*** The types of the file ***/
typedef enum {
  /*
    A source that is translated to another source (or binary) using a
    translator. Translations of these files cannot trigger more modules
    being added to the program.
  */
  ftSimpleSource,
  /*
    Header files declare objects in the module that are available to
    other modules. The objects are only declared here; the definitions of
    the objects are in the source code file. The import statements in the
    modules always refer to header files. Header file can refer to other
    header files (via import statements). Header file of a particular
    extension always belongs to the same language in a single package.
  */
  ftHeader,
  /*
    Source code file. It contains all definitions of objects mentioned in
    its associated header plus some definitions of private objects used by
    the public ones.
  */
  ftSource,
  /*
    A compiled module produced by the compiler. Each file contains compiled
    code for one single module. After all required modules were compiled,
    these files are passed to the linker which assembles a binary from them.
  */
  ftCompiledModule,
  /*
    Binaries are files that contain code for single program and/or library.
    They are assembled by the linker from the object module files produced
    by the compilers. Note that other systems use different extensions for
    the programs and their binaries; in OSHS a program is only a special case
    of a library with no header files and an entry point.
  */
  ftBinary,
  /*
    File which was not yet categorized.
  */
  ftUnknown
} TFileType;

/*** The informations about a particular file extension ***/
typedef struct tagTFileExtension {
  /*
    The extension about which this record is.
  */
  char *Ext;
  /*
    The type of the file with this extension.
  */
  TFileType Type;
  /*
    This field is used for these bindings: ftHeader->ftSource,
    ftSource->ftCompiledModule and ftCompiledModule->ftBinary. If none of
    these is applicable to the extension, this field must contain NULL.
  */
  struct tagTFileExtension *Assoc1;
  /*
    This field is used for the ftSource->ftHeader binding. If this is not
    applicable, this field must contain NULL. This binding is used to
    check if the source file has header in C compatible P language file
    parsing where the file with a header has some more required things at its
    beginning than that with no header.
  */
  struct tagTFileExtension *Assoc2;
  /*
    Pointer to the translator used to generate the file. If NULL, the file is
    not generated and must be present on input.
  */
  void *Translator;
} TFileExtension;

pubconst TFileExtension * pubconst MachineCodeExt;

void InitExtensions(void);
void DestroyExtensions(void);
TFileExtension *RegisterExtension(const char *Ext);
TFileExtension *SearchExtension(const char *Ext);

#endif
#include <syslib/hfoot.h>
