#include <syslib/hhead.h>
#ifndef __h__OBS_FILESTK__
#define __h__OBS_FILESTK__

typedef enum {
  stkFileProcessing,
  /*
    Stack of files to be linked. This stack is used during compilation to
    gather files that have been touched by the compilation process.
  */
  stkFilesToLink,
  /*
    Stack of files to be linked into a library. These are all files that are
    referenced (either directly or indirectly) by the headers introduced by
    the HEADER directive.
  */
  stkFilesToLinkIntoLibrary,
  /*
    Stack of libraries. Any referenced libraries discovered during the file
    dependencies analysis are passed through this stack to avoid duplicates.
  */
  stkLibraries,
  /*
    This stack is used to tag files that have been seen in the course of
    header inclusion tracking. If a header is referenced multiple times or
    headers are referencing themselves m
  */
  stkIncludedHeaders,
  /*
    This stack is used to detect headers that are referenced multiple times
    in a source code file.
  */
  stkAlreadyImportedHeaders,
  /*
    Tag for an invalid file stack. This must be the last value of the
    TStackID type.
  */
  stkInvalid
} TStackID;

/*
  Since dependency tree is saved only when package processing code is not
  active, we can share stacks.
*/
#define stkIdentifiedFiles stkFilesToLink
#define stkUsedLibraries stkIncludedHeaders

#include "files.h"

void InitStackOfFiles(TStackID Number);
bool PushFileOntoStack(TStackID Number,TFileInModule *File);
TFileInModule *PopFileOffStack(TStackID Number);
void CleanupFileStack(TStackID Number);

#endif
#include <syslib/hfoot.h>
