#include <syslib/types.h>
#define __INTERFACE__
#include "mkidir.h"

#include <syslib/kernel.h>
#include <syslib/pdbase.h>
#include <syslib/pdfsys.h>

#include <libhs/output.h>
#include <libhs/diskio.h>
#include <libhs/errno.h>

#define MAX_SPEC_SIZE 4096

static int16 CheckIsDirectory(const char *Spec)
{
  /*
    Attempt to open the specification as a file. If it is a directory, the
    call will fail with eISDIR, so "True" will be returned. Otherwise we
    return "False". Don't forget to close the file if it really was opened.
  */
  int16 fd;

  fd=Open(Spec,oeOPEN+oRDONLY);
  if (fd!=-1) Close(fd);
  return((fd==-1) && (GetErrNo()==fseISDIR));
}

static int16 MakeDir(const char *Name)
{
  if (!CheckIsDirectory(Name)) {
    if (!MkDir(Name)) {
      Print("Unable to make directory ");
      Print(Name);
      Print("\nSystem said: ");
      Print(ComplainLastError());
      NewLine();
      return(1);
    }
  }
  return(0);
}

int16 CreateDirectoryForInstall(const char *DirSpec)
{
  char Buf[MAX_SPEC_SIZE];
  char Ch;
  char OCh;
  const char *Ptr;
  char *Out;
  char *Pieces[MAX_SPEC_SIZE+1];
  uint16 PieceCnt;
  int16 i;
  char *Piece;
  char *PieceEnd;

  assert(DirSpec!=NULL);
  /*
    The directory must always be absolute.
  */
  Ptr=DirSpec;
  if (
    (Ptr[1]!=':') ||
    ((Ptr[2]!='\\') && (Ptr[2]!='/'))
  ) {
    Print(
      "The installation directory specification must always be absolute\n"
      "Bogus param: "
    );
    Print(Ptr);
    NewLine();
    return(1);
  }
  /*
    Divide the directory specification into the pieces. The [Pieces] array
    will contain pointers. The "Ptr" pointer was already initialized by
    the check above.
  */
  Out=Buf;
  PieceCnt=0;
  Pieces[0]=Buf;
  Piece=Buf;
  for (i=0;i<2;i++) *(Out++)=*(Ptr++);
  do {
    Ch=*(Ptr++);
    OCh=Ch;
    if ((Ch=='\\') || (Ch=='/')) OCh='\0';
    *(Out++)=OCh;
    if (OCh=='\0') {
      Pieces[PieceCnt]=Out;
      if (Ch!='\0') PieceCnt++;
    }
  } while (Ch!='\0');
  /*
    Create the directories
  */
  PieceEnd=Pieces[0];
  for (i=0;i<PieceCnt;i++) {
    Piece=PieceEnd;
    PieceEnd=Pieces[i];
    *(PieceEnd-1)='\\';
    if (MakeDir(Buf)) return(1);
  }
  /*
    Return success
  */
  return(0);
}
