#include <syslib/types.h>
#define __INTERFACE__
#include "mkmod.h"

#include <syslib/kernel.h>
#include <syslib/pdfsys.h>

#include <libhs/strings.h>
#include <libhs/filebuf.h>
#include <libhs/output.h>

#include "args.h"
#include "errors.h"
#include "files.h"
#include "filespec.h"

static const char *DetectPackageItemName(TPackage *P)
{
  TPackageItem *I;
  int16 i;
  bool Valid;

  if (PackageItemName[0]=='\0') {
    if (P->Items.Cnt!=1) {
      PlainError("You need to specify a package item name");
      return(NULL);
    }
    I=P->Items.Arr[0];
    return(I->Output->Module->Name);
  } else {
    for (i=0;i<P->Items.Cnt;i++) {
      I=P->Items.Arr[i];
      Valid=(StrCmp(PackageItemName,I->Output->Module->Name)==0);
      if (Valid) break;
    }
    if (!Valid) {
      PlainError("There is no package item with the name specified");
      return(NULL);
    }
    return(PackageItemName);
  }
}

static bool CloseCurrentFile(bool IsHeader)
{
  bool Result;

  Result=CloseAndFreeOutputFile();
  if (!Result) {
    if (IsHeader) {
      PlainError("Error writing the header file");
    } else {
      PlainError("Error writing the source code file");
    }
  }
  return(Result);
}

static void WriteNames(const char *ItemName)
{
  char Buf[MAX_FILENAME_SIZE+1];

  StrToUpr(Buf,ItemName);
  Print(Buf);
  Print("_");
  StrToUpr(Buf,ModuleName);
  Print(Buf);
}

static void OutputHeaderFileContent(const char *ItemName)
{
  ActivateCurrentOutputFile();
  Print(
    "#include <syslib/hhead.h>\n"
    "#ifndef __h__"
  );
  WriteNames(ItemName);
  Print(
    "__\n"
    "#define __h__"
  );
  WriteNames(ItemName);
  Print(
    "__\n"
    "\n"
    "/* Write the content of the module header file here */\n"
    "\n"
    "#endif\n"
    "#include <syslib/hfoot.h>\n"
  );
  DeactivateCurrentOutputFile();
}

static TFileInModule *CreateHeaderFile(const char *ItemName)
{
  TFileInModule *H;

  H=RegisterFileSplitted(ModuleName,"h");
  if (H!=NULL) {
    if (!OpenFileInModule(H,OF_OUTPUT)) {
      if (IOResult()==fseEXIST) {
        PlainError("Header file already exists");
      } else {
        PlainError("Could not create header file");
      }
    }
  }
  OutputHeaderFileContent(ItemName);
  if (!CloseCurrentFile(true)) {
    DeleteFileInModule(H);
    H=NULL;
  }
  return(H);
}

static void OutputSourceCodeFileContent(void)
{
  ActivateCurrentOutputFile();
  Print(
    "#include <syslib/types.h>\n"
    "#define __INTERFACE__\n"
    "#include \""
  );
  Print(ModuleName);
  Print(
    ".h\"\n"
    "\n"
    "/* Write the content of the module source code file here */\n"
  );
  DeactivateCurrentOutputFile();
}

static TFileInModule *CreateSourceCodeFile(void)
{
  TFileInModule *S;

  S=RegisterFileSplitted(ModuleName,"c");
  if (S!=NULL) {
    if (!OpenFileInModule(S,OF_OUTPUT)) {
      if (IOResult()==fseEXIST) {
        PlainError("Source code file already exists");
      } else {
        PlainError("Could not create source code file");
      }
    }
  }
  OutputSourceCodeFileContent();
  if (!CloseCurrentFile(false)) {
    DeleteFileInModule(S);
    S=NULL;
  }
  return(S);
}

void MkMod(TPackage *P)
{
  const char *ItemName;
  TFileInModule *H;

  if (!CheckNameValid(ModuleName)) return;
  ItemName=DetectPackageItemName(P);
  if (ItemName==NULL) return;
  H=CreateHeaderFile(ItemName);
  if (H==NULL) return;
  if (!CreateSourceCodeFile()) {
    DeleteFileInModule(H);
    return;
  }
}
