#include <syslib/types.h>
#define __INTERFACE__
#include "pkgbuild.h"

#include <libhs/strings.h>
#include <libhs/output.h>
#include <libhs/exec.h>

#include "args.h"
#include "fileext.h"
#include "files.h"
#include "pkgdata.h"
#include "errors.h"

static bool PerformRebuild(TFileInModule *F)
{
  const char *Args[10];
  const char **APtr;
  TFileSpec Buffer;
  int16 Result;

  assert(F->Ext->Type==ftSource);
  Print("Compiling ");
  PrintFileName(F);
  NewLine();
  ExtractNameFromFile(F,Buffer);
  APtr=Args;
  *(APtr++)="@:/compile";
  *(APtr++)=Buffer;
  if (TreatWarningsAsErrors) *(APtr++)="-w";
  *(APtr++)=NULL;
  Result=Call(Args,NULL);
  return(Result==0);
}

/*
  This procedure is used to build source code only modules. These modules
  are never generated as generators are assigned to headers of the modules.
  Returns 0=no rebuild, 1=successfull rebuild, 2=errors while rebuild.
*/
TRebuildStatus CheckAndBuildSource(TFileInModule *F)
{
  TFileInModule *FObj;
  bool RebuildIt;
  TRebuildStatus Result;

  assert(F->Ext->Type==ftSource);
  FObj=RegisterFileSplitted(F->Module->Name,F->Ext->Assoc1->Ext);
  RebuildIt=FObj->TimeStamp<F->MaxTimeStamp;
  Result=rsSuccessfullRebuild;
  if (RebuildIt) {
    Result=rsErrorsWhileRebuild;
    if (PerformRebuild(F)) {
      Result=rsSuccessfullRebuild;
      UpdateTimeStamp(FObj);
    }
  }
  return Result;
}

TRebuildStatus CheckAndRegenerate(TFileInModule *F,bool Clean)
{
  TPackageItem *T;
  TRebuildStatus Result;
  TFileInModule *FTrans;
  TFileInModule *ISrc;
  TFileInModule *OHdr;
  TFileInModule *OSrc;
  bool Rebuild;
  char ArgList[4*(MAX_FILESPEC_LENGTH+1)];
  char *Ptr;
  char *Args[10];
  char **CurArg;

  /*
    If the translator is not yet built, return a "pending" status to indicate
    the current package item cannot be rebuilt.
  */
  T=F->Ext->Translator;
  if (T->Status!=stDone) {
    F->Status=statPending;
    return(rsPendingRebuild);
  }
  /*
    Determine whether to regenerate the generated file. The file needs
    regenerating when the generated output is older than either the
    translator binary or the source file.
  */
  FTrans=T->Output;
  OHdr=RegisterFileSplitted(F->Module->Name,T->OHdr->Ext);
  if (OHdr==NULL) return(rsErrorsWhileRebuild);
  Rebuild=FTrans->TimeStamp>OHdr->TimeStamp;
  ISrc=RegisterFileSplitted(F->Module->Name,T->ISrc->Ext);
  if (ISrc==NULL) return(rsErrorsWhileRebuild);
  if (ISrc->TimeStamp>OHdr->TimeStamp) Rebuild=true;
  OSrc=NULL;
  if (T->OSrc!=NULL) {
    OSrc=RegisterFileSplitted(F->Module->Name,T->OSrc->Ext);
    if (OSrc==NULL) return(rsErrorsWhileRebuild);
    if (FTrans->TimeStamp>OSrc->TimeStamp) Rebuild=true;
    if (ISrc->TimeStamp>OSrc->TimeStamp) Rebuild=true;
  }
  /*
    If cleanup was requested, do so.
  */
  if (Clean) { 
    DeleteFileInModule(OHdr);
    if (OSrc!=NULL) {
      DeleteFileInModule(OSrc);
      PushFileToBeProcessed(OSrc);
    }
    return(rsSuccessfullRebuild);
  }
  /*
    Regenerate the generated file if it needs that.
  */
  Result=rsSuccessfullRebuild;
  if (Rebuild) {
    Print("Translating ");
    PrintFileName(ISrc);
    NewLine();
    CurArg=Args;
    *(CurArg++)=ArgList;
    Ptr=StrCpy(ArgList,"./");
    Ptr=ExtractNameFromFile(FTrans,Ptr);
    Ptr++;
    *(CurArg++)=Ptr;
    Ptr=ExtractNameFromFile(ISrc,Ptr);
    Ptr++;
    *(CurArg++)=Ptr;
    Ptr=ExtractNameFromFile(OHdr,Ptr);
    Ptr++;
    if (OSrc!=NULL) {
      *(CurArg++)=Ptr;
      Ptr=ExtractNameFromFile(OSrc,Ptr);
      Ptr++;
    }
    *CurArg=NULL;
    Result=rsErrorsWhileRebuild;
    if (Call((void *)Args,NULL)==0) {
      Result=rsSuccessfullRebuild;
      UpdateTimeStamp(OHdr);
      if (OSrc!=NULL) UpdateTimeStamp(OSrc);
    } else {
      SysError("Unable to execute\1");
      PutFileArg(FTrans);
    }
    OHdr->Status=statOutdated;
    if (OSrc!=NULL) OSrc->Status=statOutdated;
  } else {
    OHdr->Status=statUpToDate;
    if (OSrc!=NULL) OSrc->Status=statUpToDate;
  }
  if (OSrc!=NULL) {
    OSrc->MaxTimeStamp=OSrc->TimeStamp;
    CheckAndBuildSource(OSrc);
    PushFileOntoStack(stkFilesToLink,OSrc);
  }
  return(Result);
}
