#include <syslib/types.h>
#define __INTERFACE__
#include "pkgclean.h"

#include "files.h"
#include "pkgdeps.h"
#include "pkgbuild.h"

static void DeleteObjectFile(TFileInModule *F)
{
  TFileInModule *FObj;

  assert(F->Ext->Type==ftSource);
  FObj=RegisterFileSplitted(F->Module->Name,F->Ext->Assoc1->Ext);
  if (FObj!=NULL) DeleteFileInModule(FObj);
}

static void CleanupGeneratedModules(TFileInModule *F)
{
  CheckAndRegenerate(F,true);
}

static void ClearGeneratedFilesInItem(TPackageItem *I)
{
  TFileInModule *F;

  InitStackOfFilesToBeProcessed();
  PushAllFilesInList(&I->Headers);
  PushAllFilesInList(&I->Sources);
  while (true) {
    F=PopFileToBeProcessed();
    if (F==NULL) break;
    PushAllFilesInList(&F->References);
    if (F->Ext->Type==ftSource) {
      DeleteObjectFile(F);
    } else {
      assert(F->Ext->Type==ftHeader);
      if (F->Ext->Translator!=NULL) {
        CleanupGeneratedModules(F);
      } else {
        PushSourceFileOfHeaderIfExists(F);
      }
    }
  }
  CleanupStackOfFilesToBeProcessedAfterUse();
}

static void ClearOutputFiles(TPackage *P)
{
  TPackageItem *I;
  uint16 i;

  for (i=0;i<P->Items.Cnt;i++) {
    I=P->Items.Arr[i];
    DeleteFileInModule(I->Output);
    ClearGeneratedFilesInItem(I);
  }
}

void CleanPackage(TPackage *P)
{
  EraseDepTreeFile(P,false);
  ClearOutputFiles(P);
}
