#include <syslib/types.h>
#define __INTERFACE__
#include "pkgdata.h"

/*
  Package metadata handling module
*/

#include <libhs/strings.h>
#include <libhs/memory.h>

#include "errors.h"

TPackageItem *CreatePackageItem(const char *Name)
{
  TPackageItem *I;

  I=GetMem(sizeof(*I));
  if (I==NULL) {
    PlainError("No memory for package item information structure");
    return(NULL);
  }
  I->Output=NULL;
  I->ISrc=NULL;
  I->IHdr=NULL;
  I->OSrc=NULL;
  I->OHdr=NULL;
  InitArray(&I->Headers);
  InitArray(&I->Sources);
  I->UpToDate=true;
  I->Status=stNew;
  return(I);
}

TPackage *CreatePackage(const char *Name)
{
  TPackage *P;

  P=GetMem(sizeof(*P));
  if (P==NULL) {
    PlainError("No memory for package information structure");
    return(NULL);
  }
  P->Name=StrDup(Name);
  if (P->Name==NULL) {
    PlainError("No memory for the name of the package");
    return(NULL);
  }
  InitArray(&P->Items);
  return(P);
}

void DestroyPackageItem(TPackageItem *I)
{
  DestroyArray(&I->Headers);
  DestroyArray(&I->Sources);
  FreeMem(I);
}

void DestroyPackage(TPackage *P)
{
  uint16 i;

  for (i=0;i<P->Items.Cnt;i++) DestroyPackageItem(P->Items.Arr[i]);
  DestroyArray(&P->Items);
  FreeMem(P->Name);
  FreeMem(P);
}
