#include <syslib/hhead.h>
#ifndef __h__OBS_PKGDATA__
#define __h__OBS_PKGDATA__

#include <libhs/exparr.h>

#include "files.h"

#define M_IS_PRIVATE 0x20
#define M_IS_GENERATOR 0x40
#define M_IS_LIBRARY 0x80
#define M_ALL_FLAGS (M_IS_GENERATOR+M_IS_LIBRARY+M_IS_PRIVATE)

typedef enum {
  ptProgram=0,
  ptLibrary=0+M_IS_LIBRARY,
  ptTranslator=0+M_IS_GENERATOR+M_IS_PRIVATE,
  ptModuleGenerator=1+M_IS_GENERATOR+M_IS_PRIVATE
} TPackageItemType;

typedef enum {
  stNew,
  stBuilding,
  stPending,
  stDone
} TItemBuildStatus;

typedef struct tagTPackageItem {
  /*
    The type of the package item
  */
  TPackageItemType Type;
  /*
    The output file for the package item. The name of this file (without
    the extension) is also the name of the whole package.
  */
  TFileInModule *Output;
  /*
    List of module source files to be linked into the package item. The
    compilation process starts from these. Only the files that are mentioned
    in the package description are listed here.
  */
  TArray Sources;
  /*
    List of header files connected to the package item.
  */
  TArray Headers;
  /*
    Input extension 1. This is filled for the translators only, all other
    package items should have this set to NULL.
  */
  TFileExtension *ISrc;
  /*
    Input extension 2. This is filled for translators which take more than
    one input file.
  */
  TFileExtension *IHdr;
  /*
    Output extension. Filled for all translators.
  */
  TFileExtension *OHdr;
  /*
    Output extension. Filled for module generators, which create both, the
    header and the source code.
  */
  TFileExtension *OSrc;
  /*
    The package item build status
  */
  TItemBuildStatus Status;
  /*
    Set to true if the package item is up-to-date.
  */
  bool UpToDate;
} TPackageItem;

typedef struct tagTPackage {
  /*
    The name of the package.
  */
  const char *Name;
  /*
    The list of items the package is composed of.
  */
  TArray Items;
} TPackage;

TPackageItem *CreatePackageItem(const char *Name);
TPackage *CreatePackage(const char *Name);
void DestroyPackageItem(TPackageItem *I);
void DestroyPackage(TPackage *P);

#endif
#include <syslib/hfoot.h>
