#include <syslib/types.h>
#define __INTERFACE__
#include "fbvideo.h"

#include <libhs/memory.h>
#include <libhs/gfxcon.h>

#include "menu.h"
#include "bench.h"
#include "settings.h"
#include "screen.h"
#include "random.h"

#define DATA_LINES (600*4)

static upint8 *CreateRandomContent(void)
{
  uint32 Size;
  upint8 *Ptr;
  upint8 *Pos;
  uint32 i,j;
  uint32 Num;

  Size=DATA_LINES*ScrWidth*ScrPixelSize;
  Ptr=GetHugeMem(Size);
  if (Ptr!=NULL) {
    Pos=Ptr;
    Size/=4;
    for (i=0;i<Size;i++) {
      Num=Rand();
      for (j=0;j<4;j++) {
        *(Pos++)=Num%256;
        Num>>=8;
      }
    }
  }
  return(Ptr);
}

static void RunBenchmark(uint16 BlockLines)
{
  upint8 *Data;
  upint8 *Ptr;
  uint32 ScrSize;
  uint32 Remaining;
  uint16 VidMemPos;
  uint16 ScreenRange;
  uint16 ScreenPos;
  bool GoBack;
  uint16 TransferLines;
  uint32 TransferSize;
  uint16 Height;

  Data=CreateRandomContent();
  if (Data==NULL) {
    return;
  }
  Height=((ScrHeight/16)-2)*16;
  ScrSize=ScrWidth*Height;
  ScreenRange=DATA_LINES-ScrHeight;
  ScreenPos=0;
  GoBack=false;
  Remaining=0;
  StartBenchmark();
  while (InProgress) {
    if (GoBack) {
      ScreenPos--;
      GoBack=(ScreenPos>0);
    } else {
      ScreenPos++;
      GoBack=(ScreenPos==ScreenRange);
    }
    Ptr=Data+ScreenPos*ScrWidth*ScrPixelSize;
    Remaining=Height;
    VidMemPos=0;
    while (Remaining>0) {
      TransferLines=BlockLines;
      if (TransferLines>Remaining) TransferLines=Remaining;
      TransferSize=TransferLines*ScrWidth;
      OpenScreenRegion(0,VidMemPos,ScrWidth,TransferLines);
      ScreenWrite(Ptr,TransferSize);
      Ptr+=TransferSize*ScrPixelSize;
      VidMemPos+=TransferLines;
      Remaining-=TransferLines;
      AddToCounter(TransferSize);
    }
    UpdateScreen();
  }
  StopBenchmark();
  FreeMem(Data);
}

static void FullScreenBenchmark(void)
{
  RunBenchmark((ScrHeight-32));
}

static void HalfScreenBenchmark(void)
{
  RunBenchmark((ScrHeight-32)/2);
}

static void QuarterScreenBenchmark(void)
{
  RunBenchmark((ScrHeight-32)/4);
}

static void HalfQuarterScreenBenchmark(void)
{
  RunBenchmark((ScrHeight-32)/8);
}

static void OneLineScreenBenchmark(void)
{
  RunBenchmark(1);
}

staticdatablock(VideoMemoryTransferMenu)
  streami8u mitItem endstream
  streamstr "Full screen size" endstream
  streamptr FullScreenBenchmark endstream

  streami8u mitItem endstream
  streamstr "Half screen size" endstream
  streamptr HalfScreenBenchmark endstream

  streami8u mitItem endstream
  streamstr "Quarter screen size" endstream
  streamptr QuarterScreenBenchmark endstream

  streami8u mitItem endstream
  streamstr "1/8 of the screen size" endstream
  streamptr HalfQuarterScreenBenchmark endstream

  streami8u mitItem endstream
  streamstr "One line at a time" endstream
  streamptr OneLineScreenBenchmark endstream

  streami8u mitEndMarker endstream
enddatablock

datablock(FramebufferMenu)
  streami8u mitSubMenu endstream
  streamstr "Video memory transfer" endstream
  streamptr VideoMemoryTransferMenu() endstream

  streami8u mitEndMarker endstream
enddatablock

