#include <syslib/types.h>
#define __INTERFACE__
#include "keytest.h"

#include <syslib/kernel.h>
#include <syslib/pdkeys.h>
#include <syslib/pdgfxcon.h>

#include <libhs/strings.h>
#include <libhs/gfxcon.h>
#include <libhs/numconv.h>

#include "colors.h"
#include "settings.h"
#include "screen.h"

static char *GetKeyName(uint16 Key)
{
  switch (Key & ~MK_RELEASED) {
    case RK_ESC              :return("ESC");
    case RK_F1               :return("F1");
    case RK_F2               :return("F2");
    case RK_F3               :return("F3");
    case RK_F4               :return("F4");
    case RK_F5               :return("F5");
    case RK_F6               :return("F6");
    case RK_F7               :return("F7");
    case RK_F8               :return("F8");
    case RK_F9               :return("F9");
    case RK_F10              :return("F10");
    case RK_F11              :return("F11");
    case RK_F12              :return("F12");
    case RK_BACKSPACE        :return("BACKSPACE");
    case RK_TAB              :return("TAB");
    case RK_ENTER            :return("ENTER");
    case RK_SPACE            :return("SPACE");
    case RK_INSERT           :return("INSERT");
    case RK_DELETE           :return("DELETE");
    case RK_HOME             :return("HOME");
    case RK_END              :return("END");
    case RK_PGUP             :return("PGUP");
    case RK_PGDN             :return("PGDN");
    case RK_LEFT             :return("LEFT");
    case RK_RIGHT            :return("RIGHT");
    case RK_UP               :return("UP");
    case RK_DOWN             :return("DOWN");
    case RK_KP_SLASH         :return("KP_SLASH");
    case RK_KP_STAR          :return("KP_STAR");
    case RK_KP_MINUS         :return("KP_MINUS");
    case RK_KP7              :return("KP7");
    case RK_KP8              :return("KP8");
    case RK_KP9              :return("KP9");
    case RK_KP_PLUS          :return("KP_PLUS");
    case RK_KP4              :return("KP4");
    case RK_KP5              :return("KP5");
    case RK_KP6              :return("KP6");
    case RK_KP1              :return("KP1");
    case RK_KP2              :return("KP2");
    case RK_KP3              :return("KP3");
    case RK_KP_ENTER         :return("KP_ENTER");
    case RK_KP0              :return("KP0");
    case RK_KP_PERIOD        :return("KP_PERIOD");
    case RK_BACKAPOSTROPHE   :return("BACKAPOSTROPHE");
    case RK_1                :return("1");
    case RK_2                :return("2");
    case RK_3                :return("3");
    case RK_4                :return("4");
    case RK_5                :return("5");
    case RK_6                :return("6");
    case RK_7                :return("7");
    case RK_8                :return("8");
    case RK_9                :return("9");
    case RK_0                :return("0");
    case RK_MINUS            :return("MINUS");
    case RK_EQUAL            :return("EQUAL");
    case RK_BACKSLASH        :return("BACKSLASH");
    case RK_Q                :return("Q");
    case RK_W                :return("W");
    case RK_E                :return("E");
    case RK_R                :return("R");
    case RK_T                :return("T");
    case RK_Y                :return("Y");
    case RK_U                :return("U");
    case RK_I                :return("I");
    case RK_O                :return("O");
    case RK_P                :return("P");
    case RK_LEFTBRACKET      :return("LEFTBRACKET");
    case RK_RIGHTBRACKET     :return("RIGHTBRACKET");
    case RK_A                :return("A");
    case RK_S                :return("S");
    case RK_D                :return("D");
    case RK_F                :return("F");
    case RK_G                :return("G");
    case RK_H                :return("H");
    case RK_J                :return("J");
    case RK_K                :return("K");
    case RK_L                :return("L");
    case RK_SEMICOLON        :return("SEMICOLON");
    case RK_APOSTROPHE       :return("APOSTROPHE");
    case RK_Z                :return("Z");
    case RK_X                :return("X");
    case RK_C                :return("C");
    case RK_V                :return("V");
    case RK_B                :return("B");
    case RK_N                :return("N");
    case RK_M                :return("M");
    case RK_COMMA            :return("COMMA");
    case RK_PERIOD           :return("PERIOD");
    case RK_SLASH            :return("SLASH");
    case RK_LSHIFT           :return("LSHIFT");
    case RK_RSHIFT           :return("RSHIFT");
    case RK_LCTRL            :return("LCTRL");
    case RK_RCTRL            :return("RCTRL");
    case RK_LALT             :return("LALT");
    case RK_RALT             :return("RALT");
    case RK_MENU             :return("MENU");
    case RK_LWIN             :return("LWIN");
    case RK_RWIN             :return("RWIN");
    case RK_NUMLOCK          :return("NUMLOCK");
    case RK_CAPSLOCK         :return("CAPSLOCK");
    case RK_SCROLLOCK        :return("SCROLLOCK");
    case RK_PRINTSCREEN      :return("PRINTSCREEN");
    case RK_PAUSE            :return("PAUSE");
    default                  :return(NULL);
  }
}

void TestKeyboard(void)
{
  uint8 Line;
  bool WasESC;
  bool Running;
  uint16 Key;
  char Buffer[31];
  char *Status;
  char *Ptr;
  char *Name;

  Buffer[30]='\0';
  Line=0;
  WasESC=false;
  Running=true;
  do {
    MemSet(Buffer,30,' ');
    PrintXY(0,Line,colKeyTestCursor,Buffer);
    Key=RawReadKey();
    FormatHex(Key,4,false,'0',Buffer);
    Buffer[4]=' ';
    if ((Key & MK_RELEASED)>0) {
      Status="Rele";
    } else {
      Status="Pres";
      if (Key==RK_ESC) {
        if (WasESC) Running=false;
        WasESC=true;
      } else {
        WasESC=false;
      }
    }
    StrCpy(Buffer+5,Status);
    Buffer[9]=' ';
    Name=GetKeyName(Key);
    Ptr=Buffer+10;
    if (Name!=NULL) Ptr=StrCpy(Ptr,Name);
    do {
      *(Ptr++)=' ';
    } while (*Ptr!='\0');
    PrintXY(0,Line,colKeyTestNormal,Buffer);
    Line++;
    if (Line==ScrHeight/16) Line=0;
  } while (Running);
}
