#include <syslib/types.h>
#define __INTERFACE__
#include "menu.h"

#include <syslib/traps.h>
#include <syslib/kernel.h>
#include <syslib/pdkeys.h>
#include <syslib/pdgfxcon.h>

#include <libhs/strings.h>
#include <libhs/gfxcon.h>
#include <libhs/output.h>
#include <libhs/errno.h>

#include "settings.h"
#include "colors.h"
#include "screen.h"

#define MAX_MENU_ITEMS 20
#define MENU_WIDTH 38
#define MAX_MENU_DEPTH 16

typedef struct {
  TMenuItemType Type;
  char *Title;
  void *Link;
} TMenuItem;

typedef struct tagTMenuStackItem {
  void *BackLink;
  uint8 Cursor;
} TMenuStackItem;

static void *MenuLink;
static uint8 MenuCount;
static uint8 MenuCursor;
static TMenuItem CurrentMenu[MAX_MENU_ITEMS];
static uint8 MenuY;
static TMenuStackItem MenuStack[MAX_MENU_DEPTH];
static uint8 MenuDepth;

static void FetchMenu(upint8 *Data)
{
  TMenuItemType Type;
  TMenuItem *I;
  uint32 Len;

  /*
    Initialize the menu information
  */
  MenuLink=Data;
  MenuCount=0;
  MenuCursor=0;
  /*
    Parse the provided datablock
  */
  do {
    Type=*(Data++);
    if (Type==mitEndMarker) break;
    I=CurrentMenu+MenuCount;
    MenuCount++;
    I->Type=Type;
    I->Title=(char *)Data;
    Len=StrLen(I->Title);
    assert(Len<=MENU_WIDTH);
    Data+=Len+1;
    I->Link=*((void **)Data);
    Data+=sizeof(void *);
  } while (true);
}

/*** isplay a menu item on the screen ***/
static void DisplayMenuItem(uint8 Number,uint8 Color)
{
  TMenuItem *I;
  uint8 Len;
  char Text[MENU_WIDTH+3];
  int16 i;

  assert(Number<MenuCount);
  for (i=0;i<MENU_WIDTH+2;i++) Text[i]=' ';
  Text[MENU_WIDTH+2]='\0';
  I=CurrentMenu+Number;
  Len=StrLen(I->Title);
  assert(Len<=MENU_WIDTH);
  MemCpy(Text+(MENU_WIDTH-Len)/2+1,I->Title,Len);
  PrintXY(((ScrWidth/8)-(MENU_WIDTH+2))/2,MenuY+Number,Color,Text);
}

/*** Display the menu on the screen ***/
static void ShowMenuItem(uint8 Number)
{
  uint8 Color;

  Color=colMenuTextNormal;
  if (Number==MenuCursor) Color=colMenuTextCursor;
  DisplayMenuItem(Number,Color);
}

/*** Display the menu on the screen ***/
static void DisplayMenu(void)
{
  int16 i;

  MenuY=((ScrHeight/16)-MenuCount)/2;
  for (i=0;i<MenuCount;i++) {
    ShowMenuItem(i);
  }
}

/*** Hide the menu ***/
static void HideMenu(void)
{
  int16 i;

  for (i=0;i<MenuCount;i++) {
    DisplayMenuItem(i,0);
  }
}

static void CursorUp(void)
{
  if (MenuCursor==0) {
    MenuCursor=MenuCount-1;
  } else {
    MenuCursor-=1;
  }
}

static void CursorDown(void)
{
  if (MenuCursor==MenuCount-1) {
    MenuCursor=0;
  } else {
    MenuCursor+=1;
  }
}

typedef void TVoidProc(void);

static void SelectItem(void)
{
  TMenuItem *I;
  TVoidProc *Proc;
  TMenuStackItem *S;

  HideMenu();
  I=CurrentMenu+MenuCursor;
  switch (I->Type) {
    case mitSubMenu: {
      assert(MenuDepth<MAX_MENU_DEPTH);
      S=MenuStack+MenuDepth;
      S->BackLink=MenuLink;
      S->Cursor=MenuCursor;
      MenuDepth++;
      FetchMenu(I->Link);
      break;
    }
    case mitItem: {
      Proc=I->Link;
      Proc();
      ClearScreen();
      break;
    }
    default: {
      ReportBug("Undefined menu item occured");
      break;
    }
  }
  DisplayMenu();
}

static bool GoBackInMenu(void)
{
  TMenuStackItem *S;

  HideMenu();
  if (MenuDepth==0) return(false);
  MenuDepth--;
  S=MenuStack+MenuDepth;
  FetchMenu(S->BackLink);
  MenuCursor=S->Cursor;
  DisplayMenu();
  return(true);
}

void RunMenu(void *Data)
{
  uint16 Key;
  uint8 OldMenuCursor;

  FetchMenu(Data);
  DisplayMenu();
  do {
    OldMenuCursor=MenuCursor;
    Key=ReadKey();
    switch (Key) {
      case RK_UP:
      case RK_KP8: CursorUp(); break;
      case RK_DOWN:
      case RK_KP2: CursorDown(); break;
      case RK_ENTER:
      case RK_KP_ENTER: SelectItem(); continue;
      case RK_ESC: {
        if (GoBackInMenu()) {
          Key=RK_F1;
          continue;
        }
      }
      default: {
        break;
      }
    }
    if (OldMenuCursor!=MenuCursor) {
      ShowMenuItem(MenuCursor);
      ShowMenuItem(OldMenuCursor);
    }
  } while (Key!=RK_ESC);
}
