#include <syslib/types.h>
#define __INTERFACE__
#include "scrmode.h"

#include <syslib/kernel.h>
#include <syslib/pdkeys.h>
#include <syslib/pdgfxcon.h>

#include <libhs/strings.h>
#include <libhs/numconv.h>
#include <libhs/gfxcon.h>

#include "settings.h"
#include "colors.h"
#include "errors.h"
#include "screen.h"

staticdatablock(ScreenModes)
  streami16u
    (num320<<lWidth)+(num200<<lHeight),
    (num640<<lWidth)+(num400<<lHeight),
    (num800<<lWidth)+(num600<<lHeight),
    (num1024<<lWidth)+(num768<<lHeight),
    (num1280<<lWidth)+(num1024<<lHeight)
  endstream
enddatablock

staticdatablock(PixelSizeList)
  streami8u px8bit,px16bit,px24bit,px32bit endstream
enddatablock

#define MODES_TABLE_POS 2

static int16 GetModesXPos(void)
{
  return((ScrWidth/8-(15+4*4))/2);
}

static int16 GetModesYPos(void)
{
  int16 Count;

  Count=datasize(ScreenModes)/2;
  return((ScrHeight/16-Count)/2);
}

static int16 DetermineYPos(uint16 ModeNum)
{
  int16 y;
  uint16 ModeWH;
  upint16 *ModePtr;

  y=GetModesYPos()+MODES_TABLE_POS;
  ModeWH=ModeNum&(mWidth+mHeight);
  ModePtr=ScreenModes();
  while (*ModePtr!=ModeWH) {
    y++;
    ModePtr++;
    assert(y-GetModesYPos()-MODES_TABLE_POS < datasize(ScreenModes)/2);
  }
  return(y);
}

static void DisplayModeNumber(uint8 Attr,uint16 ModeNum)
{
  TSetModeCommand C;
  int16 x,y;
  char Num[3];

  /*
    Determine the line on which to display
  */
  y=DetermineYPos(ModeNum);
  /*
    Determine the column
  */
  UnpackModeNumber(ModeNum,&C);
  x=GetModesXPos()+15+C.PixelSize*4;
  /*
    Display the pixel size
  */
  FormatDWord((C.PixelSize+1)*8,2,'0',Num);
  Attr+=(ModeNum==ScrMode?colModeMenuActive:colModeMenuInactive);
  PrintXY(x,y,Attr,Num);
}

char *PlaceNumber(char *Buffer, uint16 Num)
{
  char NumBuffer[6];
  char *NumPtr;

  FormatDWord(Num,5,' ',NumBuffer);
  NumPtr=NumBuffer;
  while (*NumPtr==' ') NumPtr++;
  return(StrCpy(Buffer,NumPtr));
}

void BuildModeChooserDisplay(void)
{
  char Buffer[16];
  char *Ptr;
  int16 Count;
  int16 x,y;
  int16 i;
  upint16 *ModePtr;
  uint16 ModeNum;
  TSetModeCommand C;
  uint8 Attr;

  Count=datasize(ScreenModes)/2;
  x=GetModesXPos();
  y=GetModesYPos();
  PrintXY(x,y,colModeMenuSizes,"U: Screen update");
  y+=MODES_TABLE_POS;
  ModePtr=ScreenModes();
  for (i=0;i<Count;i++) {
    ModeNum=*(ModePtr++);
    UnpackModeNumber(ModeNum,&C);
    Ptr=PlaceNumber(Buffer,C.Width);
    *(Ptr++)=' ';
    *(Ptr++)='x';
    *(Ptr++)=' ';
    Ptr=PlaceNumber(Ptr,C.Height);
    PrintXY(x,y,colModeMenuSizes,Buffer);
    Attr=colModeMenuNormal;
    DisplayModeNumber(Attr,ModeNum+(px8bit<<lPixelSize));
    DisplayModeNumber(Attr,ModeNum+(px16bit<<lPixelSize));
    DisplayModeNumber(Attr,ModeNum+(px24bit<<lPixelSize));
    DisplayModeNumber(Attr,ModeNum+(px32bit<<lPixelSize));
    y++;
  }
}

void DisplayModeInfo(uint16 Mode)
{
  int16 x,y;
  const char *Text;

  x=GetModesXPos();
  y=GetModesYPos();
  Text=((Mode & flgSoftScreen)>0)?"Demand ":"Instant";
  PrintXY(x+20,y,colModeMenuActive,Text);
}

uint16 SelectMode(uint16 StartMode)
{
  bool Running;
  uint16 Count;
  TSetModeCommand C;
  int16 OldX,OldY;
  int16 X,Y;
  upint16 *ModeSizes;
  upint8 *PixelSizes;
  uint16 Key;
  uint16 OldMode,Mode;
  uint16 ModeFlags;

  ModeFlags=StartMode & flgAllFlags;
  ModeSizes=ScreenModes();
  PixelSizes=PixelSizeList();
  Count=datasize(ScreenModes)/2;
  UnpackModeNumber(StartMode,&C);
  X=C.PixelSize;
  Y=DetermineYPos(StartMode)-GetModesYPos()-MODES_TABLE_POS;
  OldX=X^1;
  OldY=Y;
  Running=true;
  do {
    Mode=ModeSizes[Y]+(PixelSizes[X]<<lPixelSize);
    if (X!=OldX || Y!=OldY) {
      OldMode=ModeSizes[OldY]+(PixelSizes[OldX]<<lPixelSize);
      DisplayModeNumber(colModeMenuNormal,OldMode);
      DisplayModeNumber(colModeMenuCursor,Mode);
      DisplayModeInfo(ModeFlags);
      OldX=X;
      OldY=Y;
    }
    Key=ReadKey();
    switch (Key) {
      case RK_U: {
        ModeFlags^=flgSoftScreen;
	OldX=99;
	break;
      }
      case RK_LEFT: {
        X=(X==0)?3:X-1;
        break;
      }
      case RK_RIGHT: {
        X=(X==3)?0:X+1;
        break;
      }
      case RK_UP: {
        Y=(Y==0)?Count-1:Y-1;
        break;
      }
      case RK_DOWN: {
        Y=(Y==Count-1)?0:Y+1;
        break;
      }
      case RK_ESC: {
        Mode=0xFFFF;
        /* FALL THROUGH */
      }
      case RK_ENTER: {
        Running=false;
      }
    }
  } while (Running);
  return(Mode | ModeFlags);
}

void ChooseMode(void)
{
  uint16 NewMode;       

  DisplayCorners(16+14);
  for (;;) {
    BuildModeChooserDisplay();
    NewMode=SelectMode(ScrMode);
    if (NewMode==0xFFFF) break;
    if (ScrMode!=NewMode) {
      ScrMode=NewMode;
      if (!SetTheScreenMode()) Fatal("Unable to set video mode");
      ClearScreen();
      DisplayCorners(16+14);
    }
  }
  DisplayCorners(0);
}
