#! /bin/sh

# if user requested cleanup, do so.
if test "x$1" = "xclean"; then
  cd gps
  rm -f *.exe
  rm -f filelist.ne1 filelist.new filelist.gen arglist.gen echo_n.gen
  sh config.sh clean
  cd ../makegen
  rm -f filein.[ch] traps.h 1ststage.gen
  rm -f makefile 1ststage makegen *.o *.exe
  cd ../syslib
  rm -f makefile *.o syslib.so syslib.a syslib.soo defs.gh
  cd ../tools
  rm -f *.o gatherm4 toolbox *.exe
  exit 0
fi

# Determine whether we are verbose
Verbose=false
if test "x$1" = "x-v"; then
  Verbose=:
fi

# Analyze the arguments of the script. The "--usedefaults", "--help" and
# "--version" arguments are handled by this script. Everything other is
# passed to the generated configurator. This passing is performed via a file
# called "arglist.gen"
ArgError=false
for Arg
do
  case $Arg in
    --version | --help | --defaults)
      if test "$#" != "1"; then
        echo "$Arg may be used only alone"
        ArgError=:
      else
        case $Arg in
          --version)
            exit 0
          ;;
          --help)
            exit 0
          ;;
        esac
      fi
    ;;
  esac
done
if $ArgError; then
  exit 1
fi

# Go to the directory with GPS
if $Verbose; then
  echo "cd gps"
fi
cd gps

# Build a file listing the pieces of the platform scanner scripts and their
# modification times. Then compare this to a prepared list of files to the
# one left there by a previous call to the build system. If they differ,
# arrange for rebuild. On systems lacking the `stat' utility also arrange
# for rebuild regardless to up-to-date-ness (which cannot be determined
# in that case). This rebuild affects only the configuration of the build
# process.
PerformScan=false
BuildFileList=false
FinalizeScan=false
if test -f ../tools/gatherm4; then
  cd src
  ../../tools/gatherm4
  cd ..
  sort <filelist.ne1 >filelist.new
  rm -f filelist.ne1
  rebuild=""
  if cmp -s filelist.new filelist.gen; then
    rebuild="leavescanner"
    rm -f filelist.new
  else
    rm -f filelist.gen
    PerformScan=:
    FinalizeScan=:
  fi
else
  rebuild=""
  PerformScan=:
  BuildFileList=:
fi

# If rebuild was requested, do so now.
if $PerformScan; then
  if $Verbose; then
    echo "sh config.sh $rebuild"
  fi
  sh config.sh $rebuild || exit
fi
if $FinalizeScan; then
  if $Verbose; then
    echo "mv filelist.new filelist.gen"
  fi
  mv filelist.new filelist.gen
fi

# Fetch the results of the scan and prepare them for use by the rest of the
# build system.
. ./results.gen
export GPS_RANLIB

# Go to the directory with the makefile generator program
if $Verbose; then
  echo "cd ../makegen"
fi
cd ../makegen

# Configure the compiler and linker
export CC="$GPS_CC -g -O0 -Wall -Werror"
export LINK="$CC"
export GPS_LIB

# Check and possibly rebuild the first stage of the makefile generator
# program. This first stage builds the makefile for the more complex second
# stage generator. It fits into single and fairly small file so its makefile
# is also very simple. The check is made first to avoid messages produced
# by the makefile when there is nothing to do on the target(s) indicated.
# The makefile for the first stage must be built here because its content
# depends on the requested verbosity of the script.
CompileCmd='$(CC) -c 1ststage.c'
LinkCmd='$(LINK) -o 1ststage 1ststage.o'
CompileEcho='compiling makegen/1ststage.c'
LinkEcho='linking makegen/1ststage'
if $Verbose; then
  CompileEcho="$CompileCmd"
  LinkEcho="$LinkCmd"
fi
cat >1ststage.gen <<EOF
1ststage: 1ststage.o
	@echo "$LinkEcho"
	@$LinkCmd

1ststage.o: 1ststage.c
	@echo "$CompileEcho"
	@$CompileCmd
EOF
if make -f 1ststage.gen -q; then :; else
  make -f 1ststage.gen || exit
fi

# Prepare the second stage of the makefile generator program for rebuild.
# The makefile generator uses some files from the SYSLIB library.
SRCDIR="../syslib"
if $Verbose; then :; else
  if test -f filein.c; then :; else
    echo "creating makegen/filein.c"
  fi
fi
if $Verbose; then
  echo "rm -f filein.c"
fi
rm -f filein.c
if $Verbose; then
  echo "$SH_FASTCP $SRCDIR/filein.c filein.c"
fi
$SH_FASTCP $SRCDIR/filein.c filein.c

if $Verbose; then :; else
  if test -f filein.h; then :; else
    echo "creating makegen/filein.h"
  fi
fi
if $Verbose; then
  echo "rm -f filein.h"
fi
rm -f filein.h
if $Verbose; then
  echo "$SH_FASTCP $SRCDIR/filein.h filein.h"
fi
$SH_FASTCP $SRCDIR/filein.h filein.h
if $Verbose; then :; else
  if test -f traps.h; then :; else
    echo "creating makegen/traps.h"
  fi
fi
if $Verbose; then
  echo "rm -f traps.h"
fi
rm -f traps.h
if $Verbose; then
  echo "$SH_FASTCP $SRCDIR/traps.h traps.h"
fi
$SH_FASTCP $SRCDIR/traps.h traps.h

# Rebuild the makefile for the second stage of the makefile builder program
# using the compiled first stage.
if $Verbose; then :; else
  if test -f makefile; then :; else
    echo "creating makegen/makefile"
  fi
fi
if $Verbose; then
  echo "./1ststage $1"
fi
./1ststage $1 || exit

# Check whether the second stage of the makefile generator program is
# up-to-date and possibly rebuild it.
if make -q; then :; else
  make || exit
fi

# We are now ready for the source code of the real thing
if $Verbose; then
  echo "cd ../syslib"
fi
cd ../syslib

# Generate the makefile for the source code
if $Verbose; then :; else
  if test -f makefile; then :; else
    echo "creating syslib/makefile"
  fi
fi
if $Verbose; then
  echo "../makegen/makegen $1"
fi
../makegen/makegen $1 || exit

# Update the `defs.gh' file if needed.
MAKEDEFS=:
if test -f defs.gh; then
  if cmp defs.gh defs.ghn >/dev/null 2>&1; then
    MAKEDEFS="false"
  fi
else
  if $Verbose; then :; else
    echo "creating syslib/defs.gh"
  fi
fi
if $MAKEDEFS; then
  rm -f defs.gh
  mv defs.ghn defs.gh
else
  rm -f defs.ghn
fi

# Fetch the list of libraries to be used.
PROG='s/^#define .*ADDON_LIBS.* "\(.*\)"$/\1/'
export GPS_LIB="`grep '^#define .*ADDON_LIBS.* ".*"$' defs.gh | sed "$PROG"`"

# Finally we can build the interesting sources
if make -q; then :; else
  make || exit
fi

# Now try to build the m4 gathering tool.
if $Verbose; then
  echo "cd ../tools"
fi
cd ../tools
if make -q; then :; else
  rm -f
  make || exit
fi
if $Verbose; then
  echo "cd ../syslib"
fi
cd ../syslib

# If the m4 gathering tool changed or
if $BuildFileList; then
  cd ../gps/src
  ../../tools/gatherm4
  cd ..
  sort <filelist.ne1 >filelist.new
  rm -f filelist.ne1 filelist.gen
  mv filelist.new filelist.gen
  cd ../syslib
fi

# If installation was requested, do so. First the OSHS codebase directory
# is created, then the directory structure in it is created and finally the
# library is installed into it.
InstallIt=false
if test "x$1" = "xinstall"; then
  InstallIt=:
fi
if test -n "$ALWAYS_INSTALL_SYSLIB"; then
  InstallIt=:
fi
if $InstallIt; then
  echo "Installing SYSLIB"
  DirList=`../tools/toolbox CB`
  TotalDir=""
  for Dir in $DirList; do
    TotalDir="$TotalDir$Dir"
    if test -d $TotalDir; then :; else
      if mkdir $TotalDir; then :; else
        echo "Unable to create OSHS codebase directory"
        exit 1
      fi
    fi
  done
  Base=$TotalDir
  IntBase=`../tools/toolbox I`
  if test -d $Base/bin; then :; else
    mkdir $Base/bin
  fi
  if test -d $Base/lib; then :; else
    mkdir $Base/lib
  fi
  if test -d $IntBase/lib; then :; else
    mkdir $IntBase/lib
  fi
  InstStatic=false
  if test -f syslib.a; then
    InstStatic=:
    if test -f syslib.soo; then
      InstStatic=false
    fi
  fi
  if $InstStatic; then
    cp -f syslib.a $IntBase/lib
  else
    rm -f $IntBase/lib/syslib.a
  fi
  if test -f syslib.so; then
    cp -f syslib.so $IntBase/lib
  else
    rm -f $IntBase/lib/syslib.so
  fi
  ../tools/toolbox L >$IntBase/lib/syslib.lib
  if test -d $Base/include; then :; else
    mkdir $Base/include
  fi
  INC=$Base/include/syslib
  rm -rf $INC
  mkdir $INC
  for Header in `grep ^HEADER syslib.dop`; do
    if test "$Header" != "HEADER"; then
      cp $Header $INC
    fi
  done
fi

# Return to the root of the source code tree
cd ..
