# Check if correct m4 is available. The only m4 processors that are
# acceptable are the GNU m4 and the OSHS m4 ones. Prefer $HOME/local/bin/m4
# to system-wide m4 to allow users to substitute their own m4 program without
# having to get root privileges first.
$ECHO_N "checking for working m4 macro processor... "
export gps_m4="m4"
if test -f $HOME/local/bin/m4; then
  gps_m4="$HOME/local/bin/m4"
fi
ERROR=""
ECHO_NOTE="no"
if $gps_m4 --version </dev/null >report.tmp 2>/dev/null; then
  :
else
  if $gps_m4 </dev/null >/dev/null 2>&1; then
    touch report.tmp
  else
    rm -f report.tmp
  fi
fi
if test -f report.tmp; then
  RESULT=`grep GNU report.tmp`
  if test "x$RESULT" = "x"; then
    RESULT=`grep OSHS report.tmp`
    if test "x$RESULT" = "x"; then
      RESULT="m4 of third party provider"
      if test "x$1" != "x-f"; then
        ERROR="Incompatible"
        ECHO_NOTE="yes"
      fi
    else
      RESULT="OSHS m4"
    fi
  else
    RESULT="GNU m4"
  fi
else
  RESULT="none"
  ERROR="No"
fi
echo "$RESULT"
rm -f report.tmp
if test "x$ERROR" != "x"; then
  echo "****** Error: $ERROR \`m4' macro processor found ******"
  echo "This program works only with either the GNU m4 or the OSHS m4"
  echo "macro processor programs. Other m4 implementations are either"
  echo "buggy or have hard-coded limitations on the size and number of"
  echo "macros, which this program exceeds. They also lack several"
  echo "builtin macros that it would be difficult to get along without"
  echo "in a sophisticated application like this build system."
  if test "$ECHO_NOTE" = "yes"; then
    echo
    echo "(Note: If you want to proceed anyway, use the -f commandline"
    echo "option)"
  fi
  exit 1
fi

