import colors
import socket
import exceptions
import sys

statAttr=0
statShape=1

class ConnectionLost(exceptions.Exception):
  def __init__(s,args=None):
    s.args=args

class TRawTerminal:
  """
  Internal terminal class. Instances of this class know how to operate the
  terminal but they don't know how to close the terminal and are not capable
  for asynchronous updates of other threads so they aren't much useful. This
  class server as an ancestor of all terminals.
  """

  def SetupSocket(s,Socket):
    s.Socket=Socket

  def InitStatus(s):
    s.Stack=[]
    s.Status=[7,shpUNDERSCORE]
    s.Colors=colors.List[:]

  def __init__(s,Socket):
    s.SetupSocket(Socket)
    s.InitStatus()

  def Read(s,Size):
    return(s.Socket.recv(Size,socket.MSG_WAITALL))

  def Write(s,string):
    s.Socket.send(string)

  def WriteRep(s,Char,Count):
    if len(Char)!=1:
      raise ValueError,"Character expected"
    if Count<0 or Count>258:
      raise ValueError,"Count out of range"
    if Count==0: return
    if Count==258 and Char==" ":
      s.Write(" ");
      Count-=1
    if (Count<3 and Char==" ") or (Count<4 and Char!=" "):
      s.Write(Char*Count);
      return
    if Char==' ':
      s.Write("\x02"+chr(Count-2))
    else:
      s.Write("\x03"+chr(Count-3)+Char)

  def WriteLn(s,What):
    s.Write(What+'\n')

  def ClrScr(s,Count=40,Start=1):
    s.DrawRect(1,Start,120,Count)

  def ClrWorkArea(s):
    s.ClrScr(36)

  def GotoXY(s,x,y):
    s.Write("\x0B"+chr(y)+chr(x))

  def SetAttr(s,Attr):
    Attr=colors.Combine(s.Status[statAttr],Attr)
    CurAttr=s.Status[statAttr]
    Str=""
    Color=Attr&0x0F
    if Color!=CurAttr&0x0F:
      Str=chr(Color+0xE0)
    Color=Attr&0xF0
    if Color!=CurAttr&0xF0:
      Str+=chr((Color>>4)+0xF0)
    s.Status[statAttr]=Attr
    s.Write(Str)

  def TranslateSimpleColor(s,Color):
    return s.Colors[Color]

  def TranslateColor(s,Color):
    if type(Color) is type([]):
      Attr=colors.EmptyAttr
      for SimpleColor in Color:
        Attr=colors.Combine(Attr,s.TranslateSimpleColor(SimpleColor))
      return Attr
    else:
      return s.TranslateSimpleColor(Color)

  def SetColor(s,Color):
    s.SetAttr(s.TranslateColor(Color))

  def SetShape(s,Shape):
    if Shape==s.Status[statShape]: return
    s.Status[statShape]=Shape
    s.Write("\x0B"+chr(Shape))

  def PushCursorStatus(s):
    s.Write('\x0B\xC5')
    s.Stack.append(s.Status[:])

  def PopCursorStatus(s):
    s.Write('\x0B\xC6')
    s.Status[:]=[]
    s.Status.extend(s.Stack.pop())
  
  def DrawLine(s,X,Y,Len,Type,Vertical,Attr):
    Seq='\x0C'+chr(Type*64+Y)+chr(Vertical+X)
    if Attr:
      Seq+=chr(Len+128)+chr(Attr)
    else:
      Seq+=chr(Len)
    s.Write(Seq)

  def DrawHorizontalLine(s,X,Y,Len,Type=0,Attr=None):
    s.DrawLine(X,Y,Len,Type,0,Attr)

  def DrawVerticalLine(s,X,Y,Len,Type=0,Attr=None):
    s.DrawLine(X,Y,Len,Type,128,Attr)

  def DrawRect(s,X,Y,Width,Height,Attr=None):
    s.PushCursorStatus()
    if Attr:
      s.SetAttr(Attr)
    for Y in xrange(Y,Y+Height):
      s.GotoXY(X,Y)
      s.WriteRep(' ',Width)
    s.PopCursorStatus()

  def DrawBox(s,X,Y,Width,Height,Type=0,Attr=None):
    s.DrawHorizontalLine(X,Y,Width,Type,Attr)
    s.DrawHorizontalLine(X,Y+Height,Width,Type,Attr)
    s.DrawVerticalLine(X,Y,Height,Type,Attr)
    s.DrawVerticalLine(X+Width,Y,Height,Type,Attr)

  def __del__(s):
    s.Close()

  def Close(s):
    if not s.Socket is None:
      s.Socket.close()

  def ReadKey(s):
    Key=s.Read(2)
    if len(Key)<2:
      raise ConnectionLost,"Connection to the terminal lost"
    return ord(Key[0])+256*ord(Key[1])

# List of valid base keystrokes
ESC=0xE080
F1=0xE081
F2=0xE082
F3=0xE083
F4=0xE084
F5=0xE085
F6=0xE086
F7=0xE087
F8=0xE088
F9=0xE089
F10=0xE08A
F11=0xE08B
F12=0xE08C
INSERT=0xE091
DELETE=0xE092
HOME=0xE093
END=0xE094
PGUP=0xE095
PGDN=0xE096
LEFT=0xE097
RIGHT=0xE098
DOWN=0xE09A
UP=0xE099
BACKSPACE=0xE08D
TAB=0xE08E
ENTER=0xE08F
SPACE=0xE090
BACKAPOSTROPHE=0xE040
T_1=0xE041
T_2=0xE042
T_3=0xE043
T_4=0xE044
T_5=0xE045
T_6=0xE046
T_7=0xE047
T_8=0xE048
T_9=0xE049
T_0=0xE04A
MINUS=0xE04B
EQUAL=0xE04C
BACKSLASH=0xE04D
T_Q=0xE04E
T_W=0xE04F
T_E=0xE050
T_R=0xE051
T_T=0xE052
T_Y=0xE053
T_U=0xE054
T_I=0xE055
T_O=0xE056
T_P=0xE057
LEFTBRACKET=0xE058
RIGHTBRACKET=0xE059
T_A=0xE05A
T_S=0xE05B
T_D=0xE05C
T_F=0xE05D
T_G=0xE05E
T_H=0xE05F
T_J=0xE060
T_K=0xE061
T_L=0xE062
SEMICOLON=0xE063
APOSTROPHE=0xE064
T_Z=0xE065
T_X=0xE066
T_C=0xE067
T_V=0xE068
T_B=0xE069
T_N=0xE06A
T_M=0xE06B
COMMA=0xE06C
PERIOD=0xE06D
SLASH=0xE06E
KP_SLASH=0xE09B
KP_STAR=0xE09C
KP_MINUS=0xE09D
KP_PLUS=0xE0A1

# List of key modifiers
SHIFT=0x0400
CTRL=0x0200
ALT=0x0100
NUM=0x1000
CAPS=0x2000
SHIFTS=0x0700
BASEKEY=0xE0FF
ISFNKEY=0xF800
YES=0xE000

#List of cursor shapes
shpNONE=0x80
shpUNDERSCORE=0x81
shpHALFBLOCK=0x82
shpBLOCK=0x83

def ConnectSocket(Host,Port):
  Socket=socket.socket(socket.AF_INET,socket.SOCK_STREAM)
  try:
    Socket.connect((Host,Port))
  except socket.error:
    return None
  return Socket

def Connect(HOST,PORT):
  Socket=ConnectSocket(HOST,PORT)
  if Socket is None:
    sys.exit("Unable to open connection to the OSHS Terminal")
  T=TRawTerminal(Socket)
  T.ReadKey()
  return T
