# Attempt to guess canonical platform name. This code was copied from
# 'config.guess' from autoconf. The result will be 'unknown' if the
# platform is not recognized.
#
# Timestamp: 2006-03-13

# Detect the release of MIPS. Some MIPS systems declare additional release
# info in their C headers.
GPS_DEFCHECK_C_NO_STDIO([GPSL_MIPS_RELEASE],[[
  #ifdef __cplusplus
  #include <stdio.h>  /* for printf() prototype */
  int main (int argc, char *argv[])
  #else
  int main (argc, argv)
  int argc;
  char *argv[];
  #endif
  {
  #if defined (host_mips) && defined (MIPSEB)
  #if defined (SYSTYPE_SYSV)
    printf ("mips-mips-riscos%ssysv\n", argv[1]);
  #endif
  #if defined (SYSTYPE_SVR4)
    printf ("mips-mips-riscos%ssvr4\n", argv[1]);
  #endif
  #if defined (SYSTYPE_BSD43) || defined(SYSTYPE_BSD)
    printf ("mips-mips-riscos%sbsd\n", argv[1]);
  #endif
  #endif
    exit(0);
  }
]])

# Distinguish between aix3.2.5 for powerpc and for rs6000
GPS_DEFCHECK_C_NO_STDIO([GPSL_AIX_3_2_5],[[
  #include <sys/systemcfg.h>
  main()
  {
    if (__power_pc()) {
      puts("powerpc-ibm-aix3.2.5");
    } else {
      puts("rs6000-ibm-aix3.2.5");
    }
    exit(0);
  }
]])

# Linux on x86 has some different flavours depending on the C library used.
# These are required to be recognized as some portability issues could be
# then unresolveable.
GPS_DEFCHECK_C_NO_STDIO([GPSL_Ix86_LINUX_BIN_FMT],[[
  #undef LIBC
  #include <features.h>
  #ifdef __ELF__
  # ifdef __GLIBC__
  #  if __GLIBC__ >= 2
  #   define LIBC "gnu"
  #  else
  #   define LIBC "gnulibc1"
  #  endif
  # else
  #  define LIBC "gnulibc1"
  # endif
  #else
  # if defined(__INTEL_COMPILER) || defined(__PGI) || defined(__sun)
  #  define LIBC "gnu"
  # else
  #  ifdef __dietlibc__
  #   define LIBC "dietlibc"
  #  else
  #   define LIBC "gnuaout"
  #  endif
  # endif
  #endif
  #include <stdio.h>
  int main() {printf(LIBC);return(0);}
]])

# Dig HP UX architecture from the compiler.
GPS_DEFCHECK_C_NO_STDIO([GPSL_HPUX_GET_ARCH],[[
  #define _HPUX_SOURCE
  #include <stdlib.h>
  #include <unistd.h>
  int main ()
  {
  #if defined(_SC_KERNEL_BITS)
    long bits = sysconf(_SC_KERNEL_BITS);
  #endif
    long cpu  = sysconf (_SC_CPU_VERSION);
    switch (cpu) {
      case CPU_PA_RISC1_0: {
        puts ("hppa1.0");
        break;
      }
      case CPU_PA_RISC1_1: {
        puts ("hppa1.1");
        break;
      }
      case CPU_PA_RISC2_0: {
  #if defined(_SC_KERNEL_BITS)
        switch (bits) {
          case 64: {
            puts ("hppa2.0w");
            break;
          }
          case 32: {
            puts ("hppa2.0n");
            break;
          }
          default: {
            puts ("hppa2.0");
            break;
          }
        }
  #else  /* !defined(_SC_KERNEL_BITS) */
        puts ("hppa2.0");
  #endif
        break;
      }
      default: {
        puts ("hppa1.0");
        break;
      }
    }
    exit (0);
  }
]])

# HITACHI systems have no useful output in uname so digging in the compiler
# is necessary.
GPS_DEFCHECK_C_NO_STDIO([GPSL_HITACHI_IDENTIFY],[[
  #include <unistd.h>
  int main ()
  {
    long cpu = sysconf (_SC_CPU_VERSION);
    /*
      The order matters, because CPU_IS_HP_MC68K erroneously returns
      true for CPU_PA_RISC1_0.  CPU_IS_PA_RISC returns correct
      results, however.  
    */
    if (CPU_IS_PA_RISC (cpu)) {
      switch (cpu) {
        case CPU_PA_RISC1_0: puts ("hppa1.0-hitachi-hiuxwe2"); break;
        case CPU_PA_RISC1_1: puts ("hppa1.1-hitachi-hiuxwe2"); break;
        case CPU_PA_RISC2_0: puts ("hppa2.0-hitachi-hiuxwe2"); break;
        default: puts ("hppa-hitachi-hiuxwe2"); break;
      }
    } else if (CPU_IS_HP_MC68K (cpu)) {
      puts ("m68k-hitachi-hiuxwe2");
    } else puts ("unknown-hitachi-hiuxwe2");
    exit (0);
  }
]])

# On MIPS 'uname' does not report the CPU correctly
GPS_DEFCHECK_C_NO_STDIO([GPSL_GET_MIPS_LINUX_CPU],[[
  #undef CPU
  #undef mips
  #undef mipsel
  #if defined(__MIPSEL__) || defined(__MIPSEL) || defined(_MIPSEL) || defined(MIPSEL)
  CPU="mipsel"
  #else
  #if defined(__MIPSEB__) || defined(__MIPSEB) || defined(_MIPSEB) || defined(MIPSEB)
  CPU="mips"
  #else
  CPU=""
  #endif
  #endif
  #include <stdio.h>
  int main() {printf(CPU);return(0);}
]])

# The same with 'uname' on MIPS64
GPS_DEFCHECK_C_NO_STDIO([GPSL_GET_MIPS64_LINUX_CPU],[[
  #undef CPU
  #undef mips64
  #undef mips64el
  #if defined(__MIPSEL__) || defined(__MIPSEL) || defined(_MIPSEL) || defined(MIPSEL)
  CPU="mips64el"
  #else
  #if defined(__MIPSEB__) || defined(__MIPSEB) || defined(_MIPSEB) || defined(MIPSEB)
  CPU=mips64
  #else
  CPU=""
  #endif
  #endif
  #include <stdio.h>
  int main() {printf(CPU);return(0);}
]])

# 'uname' failed to report us something recognizeable, perhaps it was
# completely missing. One of the last hopes: attempt to dig into the compiler
# if we can get something useful there.
GPS_DEFCHECK_C_NO_STDIO([GPSL_DIG_ID_FROM_COMPILER],[[
  #ifdef _SEQUENT_
  # include <sys/types.h>
  # include <sys/utsname.h>
  #endif
  main ()
  {
  #if defined (sony)
  #if defined (MIPSEB)
    /*
      BFD wants "bsd" instead of "newsos".  Perhaps BFD should be changed,
      I don't know....
    */
    printf ("mips-sony-bsd");
  #else
  #include <sys/param.h>
    printf ("m68k-sony-newsos%s",
  #ifdef NEWSOS4
      "4"
  #else
      ""
  #endif
    );
  #endif
  #endif
  #if defined (__arm) && defined (__acorn) && defined (__unix)
    printf ("arm-acorn-riscix");
  #endif
  #if defined (hp300) && !defined (hpux)
    printf ("m68k-hp-bsd");
  #endif
  #if defined (NeXT)
  #if !defined (__ARCHITECTURE__)
  #define __ARCHITECTURE__ "m68k"
  #endif
    int version;
    version=`(hostinfo | sed -n 's/.*NeXT Mach \([0-9]*\).*/\1/p') 2>/dev/null`;
    if (version < 4) {
      printf ("%s-next-nextstep%d", __ARCHITECTURE__, version);
    } else {
      printf ("%s-next-openstep%d", __ARCHITECTURE__, version);
    }
  #endif
  #if defined (MULTIMAX) || defined (n16)
  #if defined (UMAXV)
    printf ("ns32k-encore-sysv");
  #else
  #if defined (CMU)
    printf ("ns32k-encore-mach");
  #else
    printf ("ns32k-encore-bsd");
  #endif
  #endif
  #endif
  #if defined (__386BSD__)
    printf ("i386-pc-bsd");
  #endif
  #if defined (sequent)
  #if defined (i386)
    printf ("i386-sequent-dynix");
  #endif
  #if defined (ns32000)
    printf ("ns32k-sequent-dynix");
  #endif
  #endif
  #if defined (_SEQUENT_)
    struct utsname un;
    uname(&un);
    if (strncmp(un.version, "V2", 2) == 0) {
      printf ("i386-sequent-ptx2");
    } else if (strncmp(un.version, "V1", 2) == 0) {
      /* XXX is V1 correct? */
      printf ("i386-sequent-ptx1");
    } else printf ("i386-sequent-ptx");
  #endif
  #if defined (vax)
  # if !defined (ultrix)
  #  include <sys/param.h>
  #  if defined (BSD)
  #   if BSD == 43
    printf ("vax-dec-bsd4.3");
  #   else
  #    if BSD == 199006
    printf ("vax-dec-bsd4.3reno");
  #    else
    printf ("vax-dec-bsd");
  #    endif
  #   endif
  #  else
    printf ("vax-dec-bsd");
  #  endif
  # else
    printf ("vax-dec-ultrix");
  # endif
  #endif
  #if defined (alliant) && defined (i860)
    printf ("i860-alliant-bsd");
  #endif
  exit (0);
}
]])

GPS_DEFCHECK_C([GPSL_CHECK_BORLAND_COMPILER],[[
  int main(void)
  {
  #ifdef __BORLANDC__
    printf("yes");
  #endif
    return(0);
  }
]])

GPS_DEFPIECE([GPSL_PROCESS_BORLAND_COMPILER_CHECK_RESULTS],[[
  if test "x]CODE_OUTPUT[" = "xyes"; then
    ]TST_RESULT[="${UNAME_MACHINE}-pc-msdosbc"
  else
    ]TST_RESULT[="${UNAME_MACHINE}-pc-msdosdjgpp"
  fi
]])

# This is the main platform detection script itself.
GPS_DEFCHECKING([GPS_PLATFORM],[for canonical platform name],[[
  ]TST_RESULT[="unknown"
  # This is needed to find uname on a Pyramid OSx when run in the BSD
  # universe. (ghazi@noc.rutgers.edu 1994-08-24)
  if (test -f /.attbin/uname) >/dev/null 2>&1 ; then
    PATH=$PATH:/.attbin ; export PATH
  fi
  UNAME_MACHINE=`(uname -m) 2>/dev/null` || UNAME_MACHINE=unknown
  UNAME_RELEASE=`(uname -r) 2>/dev/null` || UNAME_RELEASE=unknown
  UNAME_SYSTEM=`(uname -s) 2>/dev/null`  || UNAME_SYSTEM=unknown
  UNAME_VERSION=`(uname -v) 2>/dev/null` || UNAME_VERSION=unknown
  # Note: order is significant - the case branches are not exclusive.
  case "${UNAME_MACHINE}:${UNAME_SYSTEM}:${UNAME_RELEASE}:${UNAME_VERSION}" in
    *:NetBSD:*:*)
      # NetBSD (nbsd) targets should (where applicable) match one or
      # more of the tupples: *-*-netbsdelf*, *-*-netbsdaout*,
      # *-*-netbsdecoff* and *-*-netbsd*.  For targets that recently
      # switched to ELF, *-*-netbsd* would select the old
      # object file format.  This provides both forward
      # compatibility and a consistent mechanism for selecting the
      # object file format.
      #
      # Note: NetBSD doesn't particularly care about the vendor
      # portion of the name.  We always set it to "unknown".
      sysctl="sysctl -n hw.machine_arch"
      UNAME_MACHINE_ARCH=`(/sbin/$sysctl 2>/dev/null || \
        /usr/sbin/$sysctl 2>/dev/null || echo unknown)`
      case "${UNAME_MACHINE_ARCH}" in
        armeb) machine=armeb-unknown ;;
        arm*) machine=arm-unknown ;;
        sh3el) machine=shl-unknown ;;
        sh3eb) machine=sh-unknown ;;
        *) machine=${UNAME_MACHINE_ARCH}-unknown ;;
      esac
      # The Operating System including object format, if it has switched
      # to ELF recently, or will in the future.
      case "${UNAME_MACHINE_ARCH}" in
        arm*|i386|m68k|ns32k|sh3*|sparc|vax)
          eval $set_cc_for_build
          if echo __ELF__ | $CC_FOR_BUILD -E - 2>/dev/null \
            | grep __ELF__ >/dev/null
          then
            # Once all utilities can be ECOFF (netbsdecoff) or a.out
            # (netbsdaout). Return netbsd for either.  FIX?
            os=netbsd
          else
            os=netbsdelf
          fi
        ;;
        *)
          os=netbsd
        ;;
      esac
      # The OS release
      # Debian GNU/NetBSD machines have a different userland, and
      # thus, need a distinct triplet. However, they do not need
      # kernel version information, so it can be replaced with a
      # suitable tag, in the style of linux-gnu.
      case "${UNAME_VERSION}" in
        Debian*)
          release='-gnu'
        ;;
        *)
          release=`echo ${UNAME_RELEASE}|sed -e 's/[-_].*/\./'`
        ;;
      esac
      # Since CPU_TYPE-MANUFACTURER-KERNEL-OPERATING_SYSTEM:
      # contains redundant information, the shorter form:
      # CPU_TYPE-MANUFACTURER-OPERATING_SYSTEM is used.
      ]TST_RESULT[="${machine}-${os}${release}"
    ;;
    *:OpenBSD:*:*)
      ]TST_RESULT[="${UNAME_MACHINE}-unknown-openbsd${UNAME_RELEASE}"
      if arch >/dev/null 2>&1; then
        UNAME_MACHINE_ARCH=`arch | sed 's/OpenBSD.//'`
        ]TST_RESULT[="${UNAME_MACHINE_ARCH}-unknown-openbsd${UNAME_RELEASE}"
      fi
    ;;
    *:ekkoBSD:*:*)
      ]TST_RESULT[="${UNAME_MACHINE}-unknown-ekkobsd${UNAME_RELEASE}"
    ;;
    *:SolidBSD:*:*)
      ]TST_RESULT[="${UNAME_MACHINE}-unknown-solidbsd${UNAME_RELEASE}"
    ;;
    macppc:MirBSD:*:*)
      ]TST_RESULT[="powerppc-unknown-mirbsd${UNAME_RELEASE}"
    ;;
    *:MirBSD:*:*)
      ]TST_RESULT[="${UNAME_MACHINE}-unknown-mirbsd${UNAME_RELEASE}"
    ;;
    alpha:OSF1:*:*)
      case $UNAME_RELEASE in
        *4.0)
          UNAME_RELEASE=`/usr/sbin/sizer -v | awk '{print $3}'`
        ;;
        *5.*)
          UNAME_RELEASE=`/usr/sbin/sizer -v | awk '{print $4}'`
        ;;
      esac
      # According to Compaq, /usr/sbin/psrinfo has been available on
      # OSF/1 and Tru64 systems produced since 1995.  I hope that
      # covers most systems running today.  This code pipes the CPU
      # types through head -n 1, so we only detect the type of CPU 0.
      ALPHA_CPU_TYPE=`/usr/sbin/psrinfo -v | sed -n -e 's/^  The alpha \(.*\) processor.*$/\1/p' | head -n 1`
      case "$ALPHA_CPU_TYPE" in
        "EV4 (21064)")
          UNAME_MACHINE="alpha"
        ;;
        "EV4.5 (21064)")
          UNAME_MACHINE="alpha"
        ;;
        "LCA4 (21066/21068)")
          UNAME_MACHINE="alpha"
        ;;
        "EV5 (21164)")
          UNAME_MACHINE="alphaev5"
        ;;
        "EV5.6 (21164A)")
          UNAME_MACHINE="alphaev56"
        ;;
        "EV5.6 (21164PC)")
          UNAME_MACHINE="alphapca56"
        ;;
        "EV5.7 (21164PC)")
          UNAME_MACHINE="alphapca57"
        ;;
        "EV6 (21264)")
          UNAME_MACHINE="alphaev6"
        ;;
        "EV6.7 (21264A)")
          UNAME_MACHINE="alphaev67"
        ;;
        "EV6.8CB (21264C)")
          UNAME_MACHINE="alphaev68"
        ;;
        "EV6.8AL (21264B)")
          UNAME_MACHINE="alphaev68"
        ;;
        "EV6.8CX (21264D)")
          UNAME_MACHINE="alphaev68"
        ;;
        "EV6.9A (21264/EV69A)")
          UNAME_MACHINE="alphaev69"
        ;;
        "EV7 (21364)")
          UNAME_MACHINE="alphaev7"
        ;;
        "EV7.9 (21364A)")
          UNAME_MACHINE="alphaev79"
        ;;
      esac
      # A Pn.n version is a patched version.
      # A Vn.n version is a released version.
      # A Tn.n version is a released field test version.
      # A Xn.n version is an unreleased experimental baselevel.
      # 1.2 uses "1.2" for uname -r.
      ]TST_RESULT[="${UNAME_MACHINE}-dec-osf`echo ${UNAME_RELEASE} | sed -e 's/^[PVTX]//' | tr ']AS_CR_LETTERS[' ']AS_CR_letters['`"
    ;;
    Alpha\ *:Windows_NT*:*)
      # How do we know it's Interix rather than the generic POSIX subsystem?
      # Should we change UNAME_MACHINE based on the output of uname instead
      # of the specific Alpha model?
      ]TST_RESULT[="alpha-pc-interix"
    ;;
    21064:Windows_NT:50:3)
      ]TST_RESULT[="alpha-dec-winnt3.5"
    ;;
    Amiga*:UNIX_System_V:4.0:*)
      ]TST_RESULT[="m68k-unknown-sysv4"
    ;;
    *:[Aa]miga[Oo][Ss]:*:*)
      ]TST_RESULT[="${UNAME_MACHINE}-unknown-amigaos"
    ;;
    *:[Mm]orph[Oo][Ss]:*:*)
      ]TST_RESULT[="${UNAME_MACHINE}-unknown-morphos"
    ;;
    *:OS/390:*:*)
      ]TST_RESULT[="i370-ibm-openedition"
    ;;
    *:z/VM:*:*)
      ]TST_RESULT[="s390-ibm-zvmoe"
    ;;
    *:OS400:*:*)
      ]TST_RESULT[="powerpc-ibm-os400"
    ;;
    arm:RISC*:1.[012]*:*|arm:riscix:1.[012]*:*)
      ]TST_RESULT[="arm-acorn-riscix${UNAME_RELEASE}"
    ;;
    arm:riscos:*:*|arm:RISCOS:*:*)
      ]TST_RESULT[="arm-unknown-riscos"
    ;;
    SR2?01:HI-UX/MPP:*:* | SR8000:HI-UX/MPP:*:*)
      ]TST_RESULT[="hppa1.1-hitachi-hiuxmpp"
    ;;
    Pyramid*:OSx*:*:* | MIS*:OSx*:*:* | MIS*:SMP_DC-OSx*:*:*)
      # akee@wpdis03.wpafb.af.mil (Earle F. Ake) contributed MIS and NILE.
      if test "`(/bin/universe) 2>/dev/null`" = att ; then
        ]TST_RESULT[="pyramid-pyramid-sysv3"
      else
        ]TST_RESULT[="pyramid-pyramid-bsd"
      fi
    ;;
    NILE*:*:*:dcosx)
      ]TST_RESULT[="pyramid-pyramid-svr4"
    ;;
    DRS?6000:unix:4.0:6*)
      ]TST_RESULT[="sparc-icl-nx6"
    ;;
    DRS?6000:UNIX_SV:4.2*:7* | DRS?6000:isis:4.2*:7*)
      case `/usr/bin/uname -p` in
        sparc)
          ]TST_RESULT[="sparc-icl-nx7"
        ;;
      esac
    ;;
    sun4H:SunOS:5.*:*)
      UNAME_RELEASE=`echo ${UNAME_RELEASE}|sed -e 's/[^.]*//'`
      ]TST_RESULT[="sparc-hal-solaris2$UNAME_RELEASE"
    ;;
    sun4*:SunOS:5.*:* | tadpole*:SunOS:5.*:*)
      UNAME_RELEASE=`echo ${UNAME_RELEASE}|sed -e 's/[^.]*//'`
      ]TST_RESULT[="sparc-sun-solaris2$UNAME_RELEASE"
    ;;
    i86pc:SunOS:5.*:*)
      UNAME_RELEASE=`echo ${UNAME_RELEASE}|sed -e 's/[^.]*//'`
      ]TST_RESULT[="i386-pc-solaris2$UNAME_RELEASE"
    ;;
    sun4*:SunOS:6*:*)
      # According to config.sub, this is the proper way to canonicalize
      # SunOS6.  Hard to guess exactly what SunOS6 will be like, but
      # it's likely to be more like Solaris than SunOS4.
      UNAME_RELEASE=`echo ${UNAME_RELEASE}|sed -e 's/[^.]*//'`
      ]TST_RESULT[="sparc-sun-solaris3$UNAME_RELEASE"
    ;;
    sun4*:SunOS:*:*)
      case "`/usr/bin/arch -k`" in
        Series*|S4*)
          UNAME_RELEASE=`uname -v`
        ;;
      esac
      # Japanese Language versions have a version number like '4.1.3-JL'.
      UNAME_RELEASE=`echo ${UNAME_RELEASE}|sed -e 's/-/_/'`
      ]TST_RESULT[="sparc-sun-sunos$UNAME_RELEASE"
    ;;
    sun3*:SunOS:*:*)
      ]TST_RESULT[="m68k-sun-sunos${UNAME_RELEASE}"
    ;;
    sun*:*:4.2BSD:*)
      UNAME_RELEASE=`(sed 1q /etc/motd | awk '{print substr($5,1,3)}') 2>/dev/null`
      test "x${UNAME_RELEASE}" = "x" && UNAME_RELEASE=3
      case "`/bin/arch`" in
        sun3)
          ]TST_RESULT[="m68k-sun-sunos${UNAME_RELEASE}"
        ;;
        sun4)
          ]TST_RESULT[="sparc-sun-sunos${UNAME_RELEASE}"
        ;;
      esac
    ;;
    aushp:SunOS:*:*)
      echo sparc-auspex-sunos${UNAME_RELEASE}
    ;;
    # The situation for MiNT is a little confusing.  The machine name
    # can be virtually everything (everything which is not
    # "atarist" or "atariste" at least should have a processor
    # > m68000).  The system name ranges from "MiNT" over "FreeMiNT"
    # to the lowercase version "mint" (or "freemint").  Finally
    # the system name "TOS" denotes a system which is actually not
    # MiNT.  But MiNT is downward compatible to TOS, so this should
    # be no problem.
    atarist[e]:*MiNT:*:* | atarist[e]:*mint:*:* | atarist[e]:*TOS:*:*)
      ]TST_RESULT[="m68k-atari-mint${UNAME_RELEASE}"
    ;;
    atari*:*MiNT:*:* | atari*:*mint:*:* | atarist[e]:*TOS:*:*)
      ]TST_RESULT[="m68k-atari-mint${UNAME_RELEASE}"
    ;;
    *falcon*:*MiNT:*:* | *falcon*:*mint:*:* | *falcon*:*TOS:*:*)
      ]TST_RESULT[="m68k-atari-mint${UNAME_RELEASE}"
    ;;
    milan*:*MiNT:*:* | milan*:*mint:*:* | *milan*:*TOS:*:*)
      ]TST_RESULT[="m68k-milan-mint${UNAME_RELEASE}"
    ;;
    hades*:*MiNT:*:* | hades*:*mint:*:* | *hades*:*TOS:*:*)
      ]TST_RESULT[="m68k-hades-mint${UNAME_RELEASE}"
    ;;
    *:*MiNT:*:* | *:*mint:*:* | *:*TOS:*:*)
      ]TST_RESULT[="m68k-unknown-mint${UNAME_RELEASE}"
    ;;
    m68k:machten:*:*)
      ]TST_RESULT[="m68k-apple-machten${UNAME_RELEASE}"
    ;;
    powerpc:machten:*:*)
      ]TST_RESULT[="powerpc-apple-machten${UNAME_RELEASE}"
    ;;
    RISC*:Mach:*:*)
      ]TST_RESULT[="mips-dec-mach_bsd4.3"
    ;;
    RISC*:ULTRIX:*:*)
      ]TST_RESULT[="mips-dec-ultrix${UNAME_RELEASE}"
    ;;
    VAX*:ULTRIX*:*:*)
      ]TST_RESULT[="vax-dec-ultrix${UNAME_RELEASE}"
    ;;
    2020:CLIX:*:* | 2430:CLIX:*:*)
      ]TST_RESULT[="clipper-intergraph-clix${UNAME_RELEASE}"
    ;;
    mips:*:*:UMIPS | mips:*:*:RISCos)
      ]GPSL_MIPS_RELEASE([${UNAME_RELEASE}])[
      if test ! "x]CODE_OUTPUT[" = "x"; then
        UNAME_RELEASE=`echo ]CODE_OUTPUT[ | sed -n 's/\([0-9]*\).*/\1/p'`
      fi
      ]TST_RESULT[="mips-mips-riscos${UNAME_RELEASE}"
    ;;
    Motorola:PowerMAX_OS:*:*)
      ]TST_RESULT[="powerpc-motorola-powermax"
    ;;
    Motorola:*:4.3:PL8-*)
      ]TST_RESULT[="powerpc-harris-powermax"
    ;;
    Night_Hawk:*:*:PowerMAX_OS | Synergy:PowerMAX_OS:*:*)
      ]TST_RESULT[="powerpc-harris-powermax"
    ;;
    Night_Hawk:Power_UNIX:*:*)
      ]TST_RESULT[="powerpc-harris-powerunix"
    ;;
    m88k:CX/UX:7*:*)
      ]TST_RESULT[="m88k-harris-cxux7"
    ;;
    m88k:*:4*:R4*)
      ]TST_RESULT[="m88k-motorola-sysv4"
    ;;
    m88k:*:3*:R3*)
      ]TST_RESULT[="m88k-motorola-sysv3"
    ;;
    AViiON:dgux:*:*)
      # DG/UX returns AViiON for all architectures
      PROCESSOR=`/usr/bin/uname -p`
      if [ $PROCESSOR = mc88100 ] || [ $PROCESSOR = mc88110 ]; then
        if [ ${TARGET_BINARY_INTERFACE}x = m88kdguxelfx ] || \
          [ ${TARGET_BINARY_INTERFACE}x = x ]
        then
          ]TST_RESULT[="m88k-dg-dgux${UNAME_RELEASE}"
        else
          ]TST_RESULT[="m88k-dg-dguxbcs${UNAME_RELEASE}"
        fi
      else
        ]TST_RESULT[="i586-dg-dgux${UNAME_RELEASE}"
      fi
    ;;
    M88*:DolphinOS:*:*)	# DolphinOS (SVR3)
      ]TST_RESULT[="m88k-dolphin-sysv3"
    ;;
    M88*:*:R3*:*)
      # Delta 88k system running SVR3
      ]TST_RESULT[="m88k-motorola-sysv3"
    ;;
    XD88*:*:*:*) # Tektronix XD88 system running UTekV (SVR3)
      ]TST_RESULT[="m88k-tektronix-sysv3"
    ;;
    Tek43[0-9][0-9]:UTek:*:*) # Tektronix 4300 system running UTek (BSD)
      ]TST_RESULT[="m68k-tektronix-bsd"
    ;;
    *:IRIX*:*:*)
      ]TST_RESULT[="mips-sgi-irix`echo ${UNAME_RELEASE}|sed -e 's/-/_/g'`"
    ;;
    ????????:AIX?:[12].1:2)   # AIX 2.2.1 or AIX 2.1.1 is RT/PC AIX.
      ]TST_RESULT[="romp-ibm-aix"  # uname -m gives an 8 hex-code CPU id
    ;;                        # Note that: echo "'`uname -s`'" gives 'AIX '
    i*86:AIX:*:*)
      ]TST_RESULT[="i386-ibm-aix"
    ;;
    ia64:AIX:*:*)
      if [ -x /usr/bin/oslevel ] ; then
        IBM_REV=`/usr/bin/oslevel`
      else
        IBM_REV=${UNAME_VERSION}.${UNAME_RELEASE}
      fi
      ]TST_RESULT[="${UNAME_MACHINE}-ibm-aix${IBM_REV}"
    ;;
    *:AIX:2:3)
      if grep bos325 /usr/include/stdio.h >/dev/null 2>&1; then
        ]GPSL_AIX_3_2_5[
        ]TST_RESULT[="]CODE_OUTPUT["
        if "x$]TST_RESULT[" = "x"; then
          ]TST_RESULT[="rs6000-ibm-aix3.2.5"
        fi
      elif grep bos324 /usr/include/stdio.h >/dev/null 2>&1; then
        ]TST_RESULT[="rs6000-ibm-aix3.2.4"
      else
        ]TST_RESULT[="rs6000-ibm-aix3.2"
      fi
    ;;
    *:AIX:*:[45])
      IBM_CPU_ID=`/usr/sbin/lsdev -C -c processor -S available | sed 1q | awk '{ print $1 }'`
      if /usr/sbin/lsattr -El ${IBM_CPU_ID} | grep ' POWER' >/dev/null 2>&1; then
        IBM_ARCH=rs6000
      else
        IBM_ARCH=powerpc
      fi
      if [ -x /usr/bin/oslevel ] ; then
        IBM_REV=`/usr/bin/oslevel`
      else
        IBM_REV=${UNAME_VERSION}.${UNAME_RELEASE}
      fi
      ]TST_RESULT[="${IBM_ARCH}-ibm-aix${IBM_REV}"
    ;;
    *:AIX:*:*)
      ]TST_RESULT[="rs6000-ibm-aix"
    ;;
    ibmrt:4.4BSD:*|romp-ibm:BSD:*)
      ]TST_RESULT[="romp-ibm-bsd4.4"
    ;;
    ibmrt:*BSD:*|romp-ibm:BSD:*)                   # covers RT/PC BSD and
      ]TST_RESULT[="romp-ibm-bsd${UNAME_RELEASE}"  # 4.3 with uname added to
    ;;                                             # report: romp-ibm BSD 4.3
    *:BOSX:*:*)
      ]TST_RESULT[="rs6000-bull-bosx"
    ;;
    DPX/2?00:B.O.S.:*:*)
      ]TST_RESULT[="m68k-bull-sysv3"
    ;;
    9000/[34]??:4.3bsd:1.*:*)
      ]TST_RESULT[="m68k-hp-bsd"
    ;;
    hp300:4.4BSD:*:* | 9000/[34]??:4.3bsd:2.*:*)
      ]TST_RESULT[="m68k-hp-bsd4.4"
    ;;
    9000/[34678]??:HP-UX:*:*)
      HPUX_REV=`echo ${UNAME_RELEASE}|sed -e 's/[^.]*.[0B]*//'`
      case "${UNAME_MACHINE}" in
        9000/31? )
          HP_ARCH=m68000 
        ;;
        9000/[34]?? )
          HP_ARCH=m68k 
        ;;
        9000/[678][0-9][0-9])
          if [ -x /usr/bin/getconf ]; then
            sc_cpu_version=`/usr/bin/getconf SC_CPU_VERSION 2>/dev/null`
            sc_kernel_bits=`/usr/bin/getconf SC_KERNEL_BITS 2>/dev/null`
            case "${sc_cpu_version}" in
              523)
                # CPU_PA_RISC1_0
                HP_ARCH="hppa1.0"
              ;;
              528)
                # CPU_PA_RISC1_1
                HP_ARCH="hppa1.1"
              ;;
              532)
                # CPU_PA_RISC2_0
                case "${sc_kernel_bits}" in
                  32)
                    HP_ARCH="hppa2.0n"
                  ;;
                  64)
                    HP_ARCH="hppa2.0w"
                  ;;
                  '')
                    # HP-UX 10.20
                    HP_ARCH="hppa2.0"
                  ;;
                esac 
              ;;
            esac
          fi
          if [ "${HP_ARCH}" = "" ]; then
            ]GPSL_HPUX_GET_ARCH[
            HP_ARCH=]CODE_OUTPUT[
            if test "x$HP_ARCH" = "x"; then
              HP_ARCH=hppa
            fi
          fi 
        ;;
      esac
      if [ ${HP_ARCH} = "hppa2.0w" ]; then
        # hppa2.0w-hp-hpux* has a 64-bit kernel and a compiler generating
        # 32-bit code.  hppa64-hp-hpux* has the same kernel and a compiler
        # generating 64-bit code.  GNU and HP use different nomenclature
        # (these examples use config.guess instead of GPS):
        #
        # $ CC_FOR_BUILD=cc ./config.guess
        # => hppa2.0w-hp-hpux11.23
        # $ CC_FOR_BUILD="cc +DA2.0w" ./config.guess
        # => hppa64-hp-hpux11.23
        if echo __LP64__ | (CCOPTS= ]GPS_CC[ -E -) | grep __LP64__ >/dev/null; then
          HP_ARCH="hppa2.0w"
        else
          HP_ARCH="hppa64"
        fi
      fi
      ]TST_RESULT[="${HP_ARCH}-hp-hpux${HPUX_REV}"
    ;;
    ia64:HP-UX:*:*)
      HPUX_REV=`echo ${UNAME_RELEASE}|sed -e 's/[^.]*.[0B]*//'`
      ]TST_RESULT[="ia64-hp-hpux${HPUX_REV}"
    ;;
    3050*:HI-UX:*:*)
      ]GPSL_HITACHI_IDENTIFY[
      if "x]CODE_OUTPUT[" = "x"; then
        ]TST_RESULT[="]CODE_OUTPUT["
      else
        ]TST_RESULT[="unknown-hitachi-hiuxwe2"
      fi
    ;;
    9000/7??:4.3bsd:*:* | 9000/8?[79]:4.3bsd:*:* )
      ]TST_RESULT[="hppa1.1-hp-bsd"
    ;;
    9000/8??:4.3bsd:*:*)
      ]TST_RESULT[="hppa1.0-hp-bsd"
    ;;
    *9??*:MPE/iX:*:* | *3000*:MPE/iX:*:*)
      ]TST_RESULT[="hppa1.0-hp-mpeix"
    ;;
    hp7??:OSF1:*:* | hp8?[79]:OSF1:*:* )
      ]TST_RESULT[="hppa1.1-hp-osf"
    ;;
    hp8??:OSF1:*:*)
      ]TST_RESULT[="hppa1.0-hp-osf"
    ;;
    i*86:OSF1:*:*)
      if [ -x /usr/sbin/sysversion ] ; then
        ]TST_RESULT[="${UNAME_MACHINE}-unknown-osf1mk"
      else
        ]TST_RESULT[="${UNAME_MACHINE}-unknown-osf1"
      fi
    ;;
    parisc*:Lites*:*:*)
      ]TST_RESULT[="hppa1.1-hp-lites"
    ;;
    C1*:ConvexOS:*:* | convex:ConvexOS:C1*:*)
      ]TST_RESULT[="c1-convex-bsd"
    ;;
    C2*:ConvexOS:*:* | convex:ConvexOS:C2*:*)
      if getsysinfo -f scalar_acc; then
        ]TST_RESULT[="c32-convex-bsd"
      else
      ]TST_RESULT[="c2-convex-bsd"
      fi
    ;;
    C34*:ConvexOS:*:* | convex:ConvexOS:C34*:*)
      ]TST_RESULT[="c34-convex-bsd"
    ;;
    C38*:ConvexOS:*:* | convex:ConvexOS:C38*:*)
      ]TST_RESULT[="c38-convex-bsd"
    ;;
    C4*:ConvexOS:*:* | convex:ConvexOS:C4*:*)
      ]TST_RESULT[="c4-convex-bsd"
    ;;
    CRAY*Y-MP:*:*:*)
      ]TST_RESULT[=`echo ymp-cray-unicos${UNAME_RELEASE} | sed -e 's/\.[^.]*$/.X/'`
    ;;
    CRAY*[A-Z]90:*:*:*)
      ]TST_RESULT[=`echo ${UNAME_MACHINE}-cray-unicos${UNAME_RELEASE} \
        | sed -e 's/CRAY.*\([A-Z]90\)/\1/' \
              -e y/ABCDEFGHIJKLMNOPQRSTUVWXYZ/abcdefghijklmnopqrstuvwxyz/ \
              -e 's/\.[^.]*$/.X/'`
    ;;
    CRAY*TS:*:*:*)
      ]TST_RESULT[=`echo t90-cray-unicos${UNAME_RELEASE} | sed -e 's/\.[^.]*$/.X/'`
    ;;
    CRAY*T3E:*:*:*)
      ]TST_RESULT[=`echo alphaev5-cray-unicosmk${UNAME_RELEASE} | sed -e 's/\.[^.]*$/.X/'`
    ;;
    CRAY*SV1:*:*:*)
      ]TST_RESULT[=`echo sv1-cray-unicos${UNAME_RELEASE} | sed -e 's/\.[^.]*$/.X/'`
    ;;
    *:UNICOS/mp:*:*)
      ]TST_RESULT[=`echo craynv-cray-unicosmp${UNAME_RELEASE} | sed -e 's/\.[^.]*$/.X/'`
    ;;
    F30[01]:UNIX_System_V:*:* | F700:UNIX_System_V:*:*)
      FUJITSU_PROC=`uname -m | tr ']AS_CR_LETTERS[' ']AS_CR_letters['`
      FUJITSU_SYS=`uname -p | tr ']AS_CR_LETTERS[' ']AS_CR_letters[' | sed -e 's/\///'`
      FUJITSU_REL=`echo ${UNAME_RELEASE} | sed -e 's/ /_/'`
      ]TST_RESULT[="${FUJITSU_PROC}-fujitsu-${FUJITSU_SYS}${FUJITSU_REL}"
    ;;
    5000:UNIX_System_V:4.*:*)
      FUJITSU_SYS=`uname -p | tr ']AS_CR_LETTERS[' ']AS_CR_letters[' | sed -e 's/\///'`
      FUJITSU_REL=`echo ${UNAME_RELEASE} | tr ']AS_CR_LETTERS[' ']AS_CR_letters[' | sed -e 's/ /_/'`
      ]TST_RESULT[="sparc-fujitsu-${FUJITSU_SYS}${FUJITSU_REL}"
    ;;
    i*86:BSD/386:*:* | i*86:BSD/OS:*:* | *:Ascend\ Embedded/OS:*:*)
      ]TST_RESULT[="${UNAME_MACHINE}-pc-bsdi${UNAME_RELEASE}"
    ;;
    sparc*:BSD/OS:*:*)
      ]TST_RESULT[="sparc-unknown-bsdi${UNAME_RELEASE}"
    ;;
    *:BSD/OS:*:*)
      ]TST_RESULT[="${UNAME_MACHINE}-unknown-bsdi${UNAME_RELEASE}"
    ;;
    *:FreeBSD:*:*)
      UNAME_RELEASE=`echo ${UNAME_RELEASE}|sed -e 's/[-(].*//'`
      if test "x${UNAME_MACHINE}" = "xpc98"; then
        UNAME_MACHINE="i386"
      fi
      ]TST_RESULT[="${UNAME_MACHINE}-unknown-freebsd${UNAME_RELEASE}"
    ;;
    i*:CYGWIN*:*)
      ]TST_RESULT[="${UNAME_MACHINE}-pc-cygwin"
    ;;
    i*:MINGW*:*)
      ]TST_RESULT[="${UNAME_MACHINE}-pc-mingw32"
    ;;
    i*:windows32*:*)
      # uname -m includes "-pc" on this system.
      ]TST_RESULT[="${UNAME_MACHINE}-mingw32"
    ;;
    i*:PW*:*)
      ]TST_RESULT[="${UNAME_MACHINE}-pc-pw32"
    ;;
    x86:Interix*:[345]*)
      ]TST_RESULT[="i586-pc-interix${UNAME_RELEASE}"
    ;;
    EM64T:Interix*:[345]*)
      ]TST_RESULT[="x86_64-unknown-interix${UNAME_RELEASE}"
    ;;
    [345]86:Windows_95:* | [345]86:Windows_98:* | [345]86:Windows_NT:*)
      ]TST_RESULT[="i${UNAME_MACHINE}-pc-mks"
    ;;
    i*:Windows_NT*:* | Pentium*:Windows_NT*:*)
      # How do we know it's Interix rather than the generic POSIX subsystem?
      # It also conflicts with pre-2.0 versions of AT&T UWIN. Should we
      # UNAME_MACHINE based on the output of uname instead of i386?
      ]TST_RESULT[="i586-pc-interix"
    ;;
    i*:UWIN*:*)
      ]TST_RESULT[="${UNAME_MACHINE}-pc-uwin"
    ;;
    amd64:CYGWIN*:*:* | x86_64:CYGWIN*:*:*)
      ]TST_RESULT[="x86_64-unknown-cygwin"
    ;;
    p*:CYGWIN*:*)
      ]TST_RESULT[="powerpcle-unknown-cygwin"
    ;;
    prep*:SunOS:5.*:*)
      UNAME_RELEASE=`echo ${UNAME_RELEASE}|sed -e 's/[^.]*//'`
      ]TST_RESULT[="powerpcle-unknown-solaris2${UNAME_RELEASE}"
    ;;
    *:GNU:*:*)
      # the GNU system
      ]TST_RESULT[=`echo ${UNAME_MACHINE}|sed -e 's,[-/].*$,,'`-unknown-gnu`echo ${UNAME_RELEASE}|sed -e 's,/.*$,,'`
    ;;
    *:GNU/*:*:*)
      # other systems with GNU libc and userland
      ]TST_RESULT[="${UNAME_MACHINE}-unknown-"`echo ${UNAME_SYSTEM} | sed 's,^[^/]*/,,' | tr '[A-Z]' '[a-z]'``echo ${UNAME_RELEASE}|sed -e 's/[-(].*//'`"-gnu"
    ;;
    i*86:Minix:*:*)
      ]TST_RESULT[="${UNAME_MACHINE}-pc-minix"
    ;;
    arm*:Linux:*:*)
      ]TST_RESULT[="${UNAME_MACHINE}-unknown-linux-gnu"
    ;;
    cris:Linux:*:*)
      ]TST_RESULT[="cris-axis-linux-gnu"
    ;;
    crisv32:Linux:*:*)
      ]TST_RESULT[="crisv32-axis-linux-gnu"
    ;;
    frv:Linux:*:*)
      ]TST_RESULT[="frv-unknown-linux-gnu"
    ;;
    ia64:Linux:*:*)
      ]TST_RESULT[="${UNAME_MACHINE}-unknown-linux-gnu"
    ;;
    m32r*:Linux:*:*)
      ]TST_RESULT[="${UNAME_MACHINE}-unknown-linux-gnu"
    ;;
    m68*:Linux:*:*)
      ]TST_RESULT[="${UNAME_MACHINE}-unknown-linux-gnu"
    ;;
    mips:Linux:*:* | mips64:Linux:*:*)
      if test "${UNAME_MACHINE}" = "mips64"; then
        ]GPSL_GET_MIPS64_LINUX_CPU[
      else
        ]GPSL_GET_MIPS_LINUX_CPU[
      fi
      CPU="]CODE_OUTPUT["
      if test "x$CPU" = "x"; then
        CPU="unknown"
      fi
      ]TST_RESULT[="${CPU}-unknown-linux-gnu"
    ;;
    or32:Linux:*:*)
      ]TST_RESULT[="or32-unknown-linux-gnu"
    ;;
    ppc:Linux:*:*)
      ]TST_RESULT[="powerpc-unknown-linux-gnu"
    ;;
    ppc64:Linux:*:*)
      ]TST_RESULT[="powerpc64-unknown-linux-gnu"
    ;;
    alpha:Linux:*:*)
      case `sed -n '/^cpu model/s/^.*: \(.*\)/\1/p' < /proc/cpuinfo` in
        EV5)   UNAME_MACHINE=alphaev5 ;;
        EV56)  UNAME_MACHINE=alphaev56 ;;
        PCA56) UNAME_MACHINE=alphapca56 ;;
        PCA57) UNAME_MACHINE=alphapca56 ;;
        EV6)   UNAME_MACHINE=alphaev6 ;;
        EV67)  UNAME_MACHINE=alphaev67 ;;
        EV68*) UNAME_MACHINE=alphaev68 ;;
      esac
      objdump --private-headers /bin/sh | grep ld.so.1 >/dev/null
      if test "$?" = 0 ; then LIBC="libc1" ; else LIBC="" ; fi
      ]TST_RESULT[="${UNAME_MACHINE}-unknown-linux-gnu${LIBC}"
    ;;
    parisc:Linux:*:* | hppa:Linux:*:*)
      # Look for CPU level
      case `grep '^cpu[^a-z]*:' /proc/cpuinfo 2>/dev/null | cut -d' ' -f2` in
        PA7*) ]TST_RESULT[="hppa1.1-unknown-linux-gnu" ;;
        PA8*) ]TST_RESULT[="echo hppa2.0-unknown-linux-gnu" ;;
        *)    ]TST_RESULT[="echo hppa-unknown-linux-gnu" ;;
      esac
    ;;
    parisc64:Linux:*:* | hppa64:Linux:*:*)
      ]TST_RESULT[="hppa64-unknown-linux-gnu"
    ;;
    s390:Linux:*:* | s390x:Linux:*:*)
      ]TST_RESULT[="${UNAME_MACHINE}-ibm-linux"
    ;;
    sh64*:Linux:*:*)
      ]TST_RESULT[="${UNAME_MACHINE}-unknown-linux-gnu"
    ;;
    sh*:Linux:*:*)
      ]TST_RESULT[="${UNAME_MACHINE}-unknown-linux-gnu"
    ;;
    sparc:Linux:*:* | sparc64:Linux:*:*)
      ]TST_RESULT[="${UNAME_MACHINE}-unknown-linux-gnu"
    ;;
    vax:Linux:*:*)
      ]TST_RESULT[="${UNAME_MACHINE}-dec-linux-gnu"
    ;;
    x86_64:Linux:*:*)
      ]TST_RESULT[="x86_64-unknown-linux-gnu"
    ;;
    i*86:Linux:*:*)
      # The BFD linker knows what the default object file format is, so
      # first see if it will tell us. cd to the root directory to prevent
      # problems with other programs or directories called 'ld' in the path.
      # Set LC_ALL=C to ensure ld outputs messages in English.
      ld_supported_targets=`cd /; LC_ALL=C ld --help 2>&1 \
        | sed -ne '/supported targets:/!d
          s/[ 	][ 	]*/ /g
          s/.*supported targets: *//
          s/ .*//
          p'`
      case "$ld_supported_targets" in
        elf32-i386)
          TENTATIVE="${UNAME_MACHINE}-pc-linux-gnu"
        ;;
        a.out-i386-linux)
          ]TST_RESULT[="${UNAME_MACHINE}-pc-linux-gnuaout"
        ;;
        coff-i386)
          ]TST_RESULT[="${UNAME_MACHINE}-pc-linux-gnucoff"
        ;;
        "")
          # Either a pre-BFD a.out linker (linux-gnuoldld) or
          # one that does not give us useful --help.
          ]TST_RESULT[="${UNAME_MACHINE}-pc-linux-gnuoldld"
        ;;
      esac
      # ELF32 binary file format found. Now determine
      # whether the default compiler is a.out or elf
      ]GPSL_Ix86_LINUX_BIN_FMT[
      if test "x]CODE_OUTPUT[" != "x"; then
        ]TST_RESULT[="${UNAME_MACHINE}-pc-linux-]CODE_OUTPUT["
      elif test "x${TENTATIVE}" != "x"; then
        ]TST_RESULT[="$TENTATIVE"
      fi
    ;;
    i*86:DYNIX/ptx:4*:*)
      # ptx 4.0 does uname -s correctly, with DYNIX/ptx in there.
      # earlier versions are messed up and put the nodename in both
      # sysname and nodename.
      ]TST_RESULT[="i386-sequent-sysv4"
    ;;
    i*86:UNIX_SV:4.2MP:2.*)
      # Unixware is an offshoot of SVR4, but it has its own version
      # number series starting with 2...
      # I am not positive that other SVR4 systems won't match this,
      # I just have to hope.  -- rms.
      # Use sysv4.2uw... so that sysv4* matches it.
      ]TST_RESULT[="${UNAME_MACHINE}-pc-sysv4.2uw${UNAME_VERSION}"
    ;;
    i*86:OS/2:*:*)
      # If we were able to find 'uname', then EMX Unix compatibility
      # is probably installed.
      ]TST_RESULT[="${UNAME_MACHINE}-pc-os2-emx"
    ;;
    i*86:XTS-300:*:STOP)
      ]TST_RESULT[="${UNAME_MACHINE}-unknown-stop"
    ;;
    i*86:atheos:*:*)
      ]TST_RESULT[="${UNAME_MACHINE}-unknown-atheos"
    ;;
    i*86:syllable:*:*)
      ]TST_RESULT[="${UNAME_MACHINE}-pc-syllable"
    ;;
    i*86:LynxOS:2.*:* | i*86:LynxOS:3.[01]*:* | i*86:LynxOS:4.0*:*)
      ]TST_RESULT[="i386-unknown-lynxos${UNAME_RELEASE}"
    ;;
    i*86:*DOS:*:*)
      ]GPSL_CHECK_BORLAND_COMPILER[
      ]GPSL_PROCESS_BORLAND_COMPILER_CHECK_RESULTS[
    ;;
    i*86:*:4.*:* | i*86:SYSTEM_V:4.*:*)
      UNAME_REL=`echo ${UNAME_RELEASE} | sed 's/\/MP$//'`
      if grep Novell /usr/include/link.h >/dev/null 2>/dev/null; then
        ]TST_RESULT[="${UNAME_MACHINE}-univel-sysv${UNAME_REL}"
      else
        ]TST_RESULT[="${UNAME_MACHINE}-pc-sysv${UNAME_REL}"
      fi
    ;;
    i*86:*:5:[678]*)
      # UnixWare 7.x, OpenUNIX and OpenServer 6.
      case `/bin/uname -X | grep "^Machine"` in
        *486*)           UNAME_MACHINE=i486 ;;
        *Pentium)        UNAME_MACHINE=i586 ;;
        *Pent*|*Celeron) UNAME_MACHINE=i686 ;;
      esac
      ]TST_RESULT[="${UNAME_MACHINE}-unknown-sysv${UNAME_RELEASE}${UNAME_SYSTEM}${UNAME_VERSION}"
    ;;
    i*86:*:3.2:*)
      if test -f /usr/options/cb.name; then
        UNAME_REL=`sed -n 's/.*Version //p' </usr/options/cb.name`
        ]TST_RESULT[="${UNAME_MACHINE}-pc-isc$UNAME_REL"
      elif /bin/uname -X 2>/dev/null >/dev/null ; then
        UNAME_REL=`(/bin/uname -X|grep Release|sed -e 's/.*= //')`
        (/bin/uname -X|grep i80486 >/dev/null) && UNAME_MACHINE=i486
        (/bin/uname -X|grep '^Machine.*Pentium' >/dev/null) && UNAME_MACHINE=i586
        (/bin/uname -X|grep '^Machine.*Pent *II' >/dev/null) && UNAME_MACHINE=i686
        (/bin/uname -X|grep '^Machine.*Pentium Pro' >/dev/null) && UNAME_MACHINE=i686
        ]TST_RESULT[="${UNAME_MACHINE}-pc-sco$UNAME_REL"
      else
        ]TST_RESULT[="${UNAME_MACHINE}-pc-sysv32"
      fi
    ;;
    pc:*:*:*)
      # Left here for compatibility:
      # uname -m prints for DJGPP always 'pc', but it prints nothing about
      # the processor, so we play safe by assuming i386.
      UNAME_MACHINE="i386"
      ]GPSL_CHECK_BORLAND_COMPILER[
      ]GPSL_PROCESS_BORLAND_COMPILER_CHECK_RESULTS[
    ;;
    Intel:Mach:3*:*)
      ]TST_RESULT[="i386-pc-mach3"
    ;;
    paragon:*:*:*)
      ]TST_RESULT[="i860-intel-osf1"
    ;;
    i860:*:4.*:*) # i860-SVR4
      if grep Stardent /usr/include/sys/uadmin.h >/dev/null 2>&1 ; then
        # Stardent Vistra i860-SVR4
        ]TST_RESULT[="i860-stardent-sysv${UNAME_RELEASE}"
      else # Add other i860-SVR4 vendors below as they are discovered.
        # Unknown i860-SVR4
        ]TST_RESULT[="i860-unknown-sysv${UNAME_RELEASE}"
      fi
    ;;
    mini*:CTIX:SYS*5:*)
      # "miniframe"
      ]TST_RESULT[="m68010-convergent-sysv"
    ;;
    mc68k:UNIX:SYSTEM5:3.51m)
      ]TST_RESULT[="m68k-convergent-sysv"
    ;;
    M680?0:D-NIX:5.3:*)
      ]TST_RESULT[="m68k-diab-dnix"
    ;;
    M68*:*:R3V[5678]*:*)
      if test -r /sysV68; then
        ]TST_RESULT[='m68k-motorola-sysv'
      fi
    ;;
    3[345]??:*:4.0:3.0 | 3[34]??A:*:4.0:3.0 | 3[34]??,*:*:4.0:3.0 | 3[34]??/*:*:4.0:3.0 | 4400:*:4.0:3.0 | 4850:*:4.0:3.0 | SKA40:*:4.0:3.0 | SDS2:*:4.0:3.0 | SHG2:*:4.0:3.0 | S7501*:*:4.0:3.0)
      OS_REL=''
      if test -r /etc/.relid; then
	OS_REL=.`sed -n 's/[^ ]* [^ ]* \([0-9][0-9]\).*/\1/p' < /etc/.relid`
	if /bin/uname -p 2>/dev/null | grep 86 >/dev/null ; then
          ]TST_RESULT[="i486-ncr-sysv4.3${OS_REL}"
	elif /bin/uname -p 2>/dev/null | /bin/grep entium >/dev/null ; then
          ]TST_RESULT[="i586-ncr-sysv4.3${OS_REL}"
        fi
      fi
    ;;
    3[34]??:*:4.0:* | 3[34]??,*:*:4.0:*)
      if /bin/uname -p 2>/dev/null | grep 86 >/dev/null ; then
        ]TST_RESULT[="i486-ncr-sysv4"
      fi
    ;;
    m68*:LynxOS:2.*:* | m68*:LynxOS:3.0*:*)
      ]TST_RESULT[="m68k-unknown-lynxos${UNAME_RELEASE}"
    ;;
    mc68030:UNIX_System_V:4.*:*)
      ]TST_RESULT[="m68k-atari-sysv4"
    ;;
    TSUNAMI:LynxOS:2.*:*)
      ]TST_RESULT[="sparc-unknown-lynxos${UNAME_RELEASE}"
    ;;
    rs6000:LynxOS:2.*:*)
      ]TST_RESULT[="rs6000-unknown-lynxos${UNAME_RELEASE}"
    ;;
    PowerPC:LynxOS:2.*:* | PowerPC:LynxOS:3.[01]*:* | PowerPC:LynxOS:4.0*:*)
      ]TST_RESULT[="powerpc-unknown-lynxos${UNAME_RELEASE}"
    ;;
    SM[BE]S:UNIX_SV:*:*)
      ]TST_RESULT[="mips-dde-sysv${UNAME_RELEASE}"
    ;;
    RM*:ReliantUNIX-*:*:*)
      ]TST_RESULT[="mips-sni-sysv4"
    ;;
    RM*:SINIX-*:*:*)
      ]TST_RESULT[="mips-sni-sysv4"
    ;;
    *:SINIX-*:*:*)
      if uname -p 2>/dev/null >/dev/null ; then
        UNAME_MACHINE=`(uname -p) 2>/dev/null`
        ]TST_RESULT[="${UNAME_MACHINE}-sni-sysv4"
      else
        ]TST_RESULT[="ns32k-sni-sysv"
      fi
    ;;
    PENTIUM:*:4.0*:*)
      # Unisys 'ClearPath HMP IX 4000' SVR4/MP effort
      # says <Richard.M.Bartel@ccMail.Census.GOV>
      ]TST_RESULT[="i586-unisys-sysv4"
    ;;
    *:UNIX_System_V:4*:FTX*)
      # From Gerald Hewes <hewes@openmarket.com>.
      # How about differentiating between stratus architectures? -djm
      ]TST_RESULT[="hppa1.1-stratus-sysv4"
    ;;
    *:*:*:FTX*)
      # From seanf@swdc.stratus.com.
      ]TST_RESULT[="i860-stratus-sysv4"
    ;;
    i*86:VOS:*:*)
      # From Paul.Green@stratus.com.
      ]TST_RESULT[="${UNAME_MACHINE}-stratus-vos"
    ;;
    *:VOS:*:*)
      # From Paul.Green@stratus.com.
      ]TST_RESULT[="hppa1.1-stratus-vos"
    ;;
    mc68*:A/UX:*:*)
      ]TST_RESULT[="m68k-apple-aux${UNAME_RELEASE}"
    ;;
    news*:NEWS-OS:6*:*)
      ]TST_RESULT[="mips-sony-newsos6"
    ;;
    R[34]000:*System_V*:*:* | R4000:UNIX_SYSV:*:* | R*000:UNIX_SV:*:*)
      if [ -d /usr/nec ]; then
        ]TST_RESULT[="mips-nec-sysv${UNAME_RELEASE}"
      else
        ]TST_RESULT[="mips-unknown-sysv${UNAME_RELEASE}"
      fi
    ;;
    BeBox:BeOS:*:*)
      # BeOS running on hardware made by Be, PPC only.
      ]TST_RESULT[="powerpc-be-beos"
    ;;
    BeMac:BeOS:*:*)
      # BeOS running on Mac or Mac clone, PPC only.
      ]TST_RESULT[="powerpc-apple-beos"
    ;;
    BePC:BeOS:*:*)
      # BeOS running on Intel PC compatible.
      ]TST_RESULT[="i586-pc-beos"
    ;;
    SX-4:SUPER-UX:*:*)
      ]TST_RESULT[="sx4-nec-superux${UNAME_RELEASE}"
    ;;
    SX-5:SUPER-UX:*:*)
      ]TST_RESULT[="sx5-nec-superux${UNAME_RELEASE}"
    ;;
    SX-6:SUPER-UX:*:*)
      ]TST_RESULT[="sx6-nec-superux${UNAME_RELEASE}"
    ;;
    Power*:Rhapsody:*:*)
      ]TST_RESULT[="powerpc-apple-rhapsody${UNAME_RELEASE}"
    ;;
    *:Rhapsody:*:*)
      ]TST_RESULT[="${UNAME_MACHINE}-apple-rhapsody${UNAME_RELEASE}"
    ;;
    *:Darwin:*:*)
      UNAME_PROCESSOR=`uname -p` || UNAME_PROCESSOR=unknown
      case $UNAME_PROCESSOR in
        unknown) UNAME_PROCESSOR=powerpc ;;
      esac
      ]TST_RESULT[="${UNAME_PROCESSOR}-apple-darwin${UNAME_RELEASE}"
    ;;
    *:procnto*:*:* | *:QNX:[0123456789]*:*)
      UNAME_PROCESSOR=`uname -p`
      if test "$UNAME_PROCESSOR" = "x86"; then
        UNAME_PROCESSOR=i386
        UNAME_MACHINE=pc
      fi
      ]TST_RESULT[="${UNAME_PROCESSOR}-${UNAME_MACHINE}-nto-qnx${UNAME_RELEASE}"
    ;;
    *:QNX:*:4*)
      ]TST_RESULT[="i386-pc-qnx"
    ;;
    NSE-?:NONSTOP_KERNEL:*:*)
      ]TST_RESULT[="nse-tandem-nsk${UNAME_RELEASE}"
    ;;
    NSR-?:NONSTOP_KERNEL:*:*)
      ]TST_RESULT[="nsr-tandem-nsk${UNAME_RELEASE}"
    ;;
    *:NonStop-UX:*:*)
      ]TST_RESULT[="mips-compaq-nonstopux"
    ;;
    BS2000:POSIX*:*:*)
      ]TST_RESULT[="bs2000-siemens-sysv"
    ;;
    DS/*:UNIX_System_V:*:*)
      ]TST_RESULT[="${UNAME_MACHINE}-${UNAME_SYSTEM}-${UNAME_RELEASE}"
    ;;
    *:Plan9:*:*)
      # "uname -m" is not consistent, so use $cputype instead. 386
      # is converted to i386 for consistency with other x86
      # operating systems.
      if test "$cputype" = "386"; then
        UNAME_MACHINE=i386
      else
        UNAME_MACHINE="$cputype"
      fi
      ]TST_RESULT[="${UNAME_MACHINE}-unknown-plan9"
    ;;
    *:TOPS-10:*:*)
      ]TST_RESULT[="pdp10-unknown-tops10"
    ;;
    *:TENEX:*:*)
      ]TST_RESULT[="pdp10-unknown-tenex"
    ;;
    KS10:TOPS-20:*:* | KL10:TOPS-20:*:* | TYPE4:TOPS-20:*:*)
      ]TST_RESULT[="pdp10-dec-tops20"
    ;;
    XKL-1:TOPS-20:*:* | TYPE5:TOPS-20:*:*)
      ]TST_RESULT[="pdp10-xkl-tops20"
    ;;
    *:TOPS-20:*:*)
      ]TST_RESULT[="pdp10-unknown-tops20"
    ;;
    *:ITS:*:*)
      ]TST_RESULT[="pdp10-unknown-its"
    ;;
    SEI:*:*:SEIUX)
      ]TST_RESULT[="mips-sei-seiux${UNAME_RELEASE}"
    ;;
    *:DragonFly:*:*)
      ]TST_RESULT[="${UNAME_MACHINE}-unknown-dragonfly"`echo ${UNAME_RELEASE}|sed -e 's/[-(].*//'`
    ;;
    *:*VMS:*:*)
      UNAME_MACHINE=`(uname -p) 2>/dev/null`
      case "${UNAME_MACHINE}" in
        A*) ]TST_RESULT[="alpha-dec-vms" ;;
        I*) ]TST_RESULT[="ia64-dec-vms" ;;
        V*) ]TST_RESULT[="vax-dec-vms" ;;
      esac
    ;;
    *:XENIX:*:SysV)
      ]TST_RESULT[="i386-pc-xenix"
    ;;
    i*86:skyos:*:*)
      RELEASE=`echo ${UNAME_RELEASE} | sed -e 's/ .*$//'`
      ]TST_RESULT[="${UNAME_MACHINE}-pc-skyos${RELEASE}"
    ;;
    i*86:rdos:*:*)
      ]TST_RESULT[="${UNAME_MACHINE}-pc-rdos"
    ;;
  esac
  if test "x$]TST_RESULT[" = "xunknown"; then
    ]GPSL_DIG_ID_FROM_COMPILER[
    if test "x]CODE_OUTPUT[" != "x"; then
      ]TST_RESULT[=]CODE_OUTPUT[
    fi
  fi
  # Apollos put the system type in the environment.
  if test "x$]TST_RESULT[" = "xunknown"; then
    if test -d /usr/apollo; then
      ]TST_RESULT[="${ISP}-apollo-${SYSTYPE}"
    fi
  fi
  # Convex versions that predate uname can use getsysinfo(1)
  if test "x$]TST_RESULT[" = "xunknown"; then
    if [ -x /usr/convex/getsysinfo ]; then
      case `getsysinfo -f cpu_type` in
        c1*)
          ]TST_RESULT[="c1-convex-bsd"
        ;;
        c2*)
          if getsysinfo -f scalar_acc; then
            ]TST_RESULT[="c32-convex-bsd"
          else
            ]TST_RESULT[="c2-convex-bsd"
          fi
        ;;
        c34*)
          ]TST_RESULT[="c34-convex-bsd"
        ;;
        c38*)
          ]TST_RESULT[="c38-convex-bsd"
        ;;
        c4*)
          ]TST_RESULT[="c4-convex-bsd"
        ;;
      esac
    fi
  fi
]],[[
  if test "x$]TST_RESULT[" = "xunknown"; then
    ]GPS_ERROR([Unable to guess canonical platform name])[
  fi
]])
