# Generic Platform Scanner for OSHS SYSLIB.
# Execute the tests and produce a 'config.h' file for the current platform

GPS_DEFSNIPPET([GPS_LIBINIT],[[
  # Initialize the library detection subsystem
  ]GPS_LIBCHECK_CLEAR[
  ]GPSL_LIBCHECK_CLEAR_FETCHED[
  rm -f libargs.gen libdeps.gen
  touch libargs.gen libdeps.gen
  ]SH_CLEANUP[ ]SH_SELFDIR[/libargs.gen ]SH_SELFDIR[/libdeps.gen
]])

GPS_DEFSNIPPET([GPS_MAIN],[DNL[
  ]SH_SETUP_BOURNE[
  ]SH_KSH_BUGFIX[
  ]SH_KILL_NLS_NUISANCES[
  ]SH_SETUP_LINENO[
  ]DNL SH_FASTCP[
  ]GPS_LIBINIT[
  ]GPS_OPEN_LOG[
  ]GPS_C_CHECK_STANDARD_HEADERS_PRESENT[
  ]DNL GPS_RANLIB[

  # Run the tests and produce the output. The tests were executed above this
  # command, because the test generation machinery already placed them there.
  # So we just need to collect the results and write them into the 'config.h'
  # file. The 'config.h' file must be created in the source code directory,
  # which is one level below us.
  echo "creating gps/config.gen"
  exec 3>&1
  exec >]SH_SELFDIR[/config.gen
  echo "# This configuration file was automatically generated by GPS."
  echo "# It will be taken by the BUILDER tool to create defs.gh in the"
  echo "# SYSLIB source code directory (/src). It is not intended to be"
  echo "# read or updated by a human as the format is very terse and also"
  echo "# this file is deliberately overwritten each time GPS is rerun"
  echo "# (which happens at every time when GPS sources are changed). See"
  echo "# the build documentation for instructions how to introduce the"
  echo "# configuration changes you need in a more persistent way."
  echo
  ]GPS_CONFIG_H_OUTPUT[]DNL[
  echo
  echo "# This portion of the configuration states which driver settings"
  echo "# need special libraries to be explicitly linked into the program."
  echo "# This data is used by the SYSLIB build system to construct the"
  echo "# content of the library linking config item called ADDON_LIBS."
  echo "# This facility is here because if the user reconfigures the driver"
  echo "# settings, the SYSLIB build system must update the list of addon"
  echo "# libraries to be linked into the programs or the SYSLIB will fail"
  echo "# if configured for use of drivers that have their dependency"
  echo "# libraries unmentioned in ADDON_LIBS."
  echo "#"
  echo "# The lines begin with a type character which denotes what is"
  echo "# described on the line. This type character must be on the first"
  echo "# position on the line otherwise the SYSLIB build system will not"
  echo "# recognize it as a valid library dependency specifier line."
  echo "#"
  echo "# Lines with the type character set to '@' state which libraries"
  echo "# must be linked into the binary when a particular config item is"
  echo "# set to a particular value (the config item may be an exclusive or"
  echo "# a boolean config item). The line contains at least three"
  echo "# identifiers. The first identifier identifies the configuration"
  echo "# item, the second one identifies the value (it must be one of the"
  echo "# valid values for an exclusive config item or yes/no for a boolean"
  echo "# config item) and the remaining ones identify the specific"
  echo "# libraries to be linked into the binary when the specified config"
  echo "# item is set to the specified value (these identifiers are assigned"
  echo "# to their libraries in the GPS sources by the GPS_DETECT_LIBRARY"
  echo "# macro invocations, mostly in sources in the 'progs' subdirectory)."
  echo "#"
  echo "# When the type character of the line is set to '!', that line shows"
  echo "# how to link a particular library into the binary. The first"
  echo "# identifier on the line shows the identifier of the library which"
  echo "# is used in the '@' lines to specify the library and the rest of"
  echo "# the line is a blank-separated list of linker arguments. If this"
  echo "# argument portion of the line contains only "n/a", the specified"
  echo "# library is not available on the platform and settings that rely"
  echo "# on it should be rejected."
  echo
  cat libdeps.gen
  exec 1>&3

  # Clear all the temporary files generated by the act of platform analysis.
  # All these files are recorded in the clean.gen file which itself is a
  # shell script which removes the temporary files when executed.
  if [ -f clean.gen ]; then
    $SH_EXEC ./clean.gen
    rm -f clean.gen
  fi
]])

GPS_MAIN
