# The check declaration related directives for GPS

# ------------------------------------------------------------------------- #
# GPS_CURRENT
# ------------------------------------------------------------------------- #
m4_define([GPS_CURRENT],1)

# ------------------------------------------------------------------------- #
# GPSL_PUSH_STREAM
# ------------------------------------------------------------------------- #
m4_prog([GPSL_PUSH_STREAM],[
  [m4_define([GPS_CURRENT],m4_eval(GPS_CURRENT+1))],
  [m4_divert(GPS_CURRENT)],
])

# ------------------------------------------------------------------------- #
# GPSL_POP_STREAM
# ------------------------------------------------------------------------- #
m4_prog([GPSL_POP_STREAM],[
  [m4_divert(0)],
  [m4_undivert(GPS_CURRENT)],
  [m4_define([GPS_CURRENT],m4_eval(GPS_CURRENT-1))],
  [m4_divert(GPS_CURRENT)],
])

# ------------------------------------------------------------------------- #
# DNL
# ------------------------------------------------------------------------- #
# This macro is synonymous to 'dnl' except that its real exexution is done
# by 'sed' after the script is generated.
m4_define([DNL],[@DELETE_TO_NEW_LINE@])
m4_define([IBEGIN],[@START_INDENTATION_BLOCK@])
m4_define([IEND],[@END_INDENTATION_BLOCK@])

# ------------------------------------------------------------------------- #
# GPS_DEFVAR(NAME:newname,BODY:script,COMMENT:string,PRED:string,POST:string)
# $BODY defines $NAME
# ------------------------------------------------------------------------- #
# Declares a variable, which is automatically initialized on demand with a
# check of something similar. When the check is being generated, the COMMENT
# is outputted first, then all initializations which this one depends on and 
# finally the initialization of the variable just requested. The check is
# generated only once.
m4_prog([GPS_DEFVAR],[
  [m4_prog([$1],[
    [m4_divert(0)],
    [$3],
    [m4_divert(GPS_CURRENT)],
    [m4_pushdef([TST_RESULT],[res_$1])],
    [m4_define([$1],[$4]TST_RESULT[$5])],
    [GPSL_PUSH_STREAM],
    [$2],
    [echo [$1]=\"[$4]TST_RESULT[$5]\" >>results.gen]NL[],
    [GPSL_POP_STREAM],
    [$1],
    [m4_popdef([TST_RESULT])],
  ])],
])

# ------------------------------------------------------------------------- #
GPS_DEFCHECK(NAME:newname,BODY:script,COMMENT:script)
# $BODY defines $NAME
# ------------------------------------------------------------------------- #
m4_define([GPS_DEFCHECK],[GPS_DEFVAR([$1],[$2],[$3],@S|@)])

# ------------------------------------------------------------------------- #
GPS_DEFCHECKING(NAME:newname,TITLE:string,BODY1:script,BODY2:script,COMMENT:script)
# $BODY defines $NAME
# ------------------------------------------------------------------------- #
m4_define([GPS_DEFCHECKING],[GPS_DEFVAR([$1],]m4__cat(
  [NL],
  [[[# check $2]]NL],
  [[GPS_LOG([checking $2])]NL],
  [[[]SH_ECHO_N[ "checking $2... "]]],
  [[$3]],
  [[[echo @S|@]TST_RESULT]NL],
  [[GPS_LOG([result: @S|@]TST_RESULT[])]NL],
  [[m4_ifelse([$4],[],[],DNL[$4])]],
)[,[$5],@S|@)])

# ------------------------------------------------------------------------- #
# GPS_DEFSNIPPET(NAME:newname,BODY:script,COMMENT:string)
# $BODY defines $NAME
# ------------------------------------------------------------------------- #
# Declares a named snippet of code. The difference between this and a test is
# that snippets produce no output that can be assigned into a variable and
# tested for later. Another difference is that the snippet is generated each
# time its name is specified. Snippets are usually used to set various
# internal settings in the enviroment to proper values.
m4_prog([GPS_DEFSNIPPET],[
  [m4_prog([$1],[
    [m4_divert(0)],
    [$3],
    [m4_divert(GPS_CURRENT)],
    [GPSL_PUSH_STREAM],
    [$2],
    [GPSL_POP_STREAM],
    [DNL],
  ])],
])

# The difference between "piece" and "snippet" is that "snippet" acts like
# "outputless check", while "piece" acts like a procedure, which gets added
# into another snippet/check. This one is internal to GPS and is used in some
# special situations where the indentation feature of the publicly visible
# piece definition macro is not desired (mainly in blocks that already are
# being indented).
m4_prog([GPSL_DEFPIECE_NOINDENT],[
  [m4_prog([$1],[
    [DNL],
    [$2],
    [DNL],
  ])],
])

# This is the publicly available piece generator. This one ensures that the
# resulting block will be properly indented in the output.
m4_prog([GPS_DEFPIECE],[
  [m4_prog([$1],[
    []IBEGIN[],
    [$2],
    []IEND[],
  ])],
])

# ------------------------------------------------------------------------- #
# GPS_ERROR(MSG:string)
# ------------------------------------------------------------------------- #
# Writes the error message msg as a hard error and aborts the script.
GPS_DEFPIECE([GPS_ERROR],[[
  echo
  echo "Error: $1"
  exit 1
]])

# ------------------------------------------------------------------------- #
# GPS_UNUSED()
# ------------------------------------------------------------------------- #
# Use this macro at the beginning of a check that may be unused on some
# platforms. It will initialize ]TST_RESULT[ to a descriptive value that
# the result is not used on the platform. You then can leave the value
# in ]TST_RESULT[ as is when you determine that the check is not required
# for the current platform.
GPS_DEFPIECE([GPS_UNUSED],[[
  ]TST_RESULT[="(unused)"
]])

# ------------------------------------------------------------------------- #
# GPS_DEFCHECK_C(NAME:newname,BODY:string)
# $BODY defines $NAME
# ------------------------------------------------------------------------- #
# Declares a check which is written in C rather than shell script. This
# script must output its result to stdout. The output then can be retrieved
# as ]CHECK_OUTPUT[. Unlike shell script checks, these checks have no
# automatic linkage tracking and they are always generated.
m4_define([CODE_BOL],[[  echo "]])
m4_define([CODE_EOL],[[" >>"$as_file.c"]])
m4_define([CODE_OUTPUT],[$as_result])
m4_define([GPSL_PREPARE_COMPILING],[DNL
  GPSL_ALLLIB="${GPS_FETCHEDLIB} ${GPS_LIB} ${GPS_CURLIB}"
  Cmd="]GPSL_COMPILE_CMD["
  GPS_LOG([compiling: $Cmd])
  GPS_LOG([Libraries: ${GPSL_ALLLIB}])
DNL])
m4_define([GPSL_CONVERT_C_TO_SH],
  [m4_sub(
    m4_sub(
      m4_sub(
        m4_sub(
          [[[[[$1]]]]], [^  ], []),
        [\"], [\\"]),
      [^], CODE_BOL),
    [$],CODE_EOL)])
m4_define([GPSL_C_TO_SH_NO_STDIO],[[  as_file="]SH_SELFDIR[/dummy"
  ]SH_CLEANUP[ "$as_file.c" "$as_file"
  rm -f "$as_file.c"
]GPSL_CONVERT_C_TO_SH([$1])])
m4_define([GPSL_C_TO_SH],[GPSL_C_TO_SH_NO_STDIO([@%:@include <stdio.h>]NL[$1])])
m4_define([GPSL_COMPILE_CMD],m4__cat(
  [[]GPS_CC[]m4_ifelse([$1],[],[],[ $1])[ -o $as_file $as_file.c]m4_ifelse([$2],[],[],[ $2])[] ${GPSL_ALLLIB}],
))
m4_define([GPSL_COMPILE],m4__cat(
  [[]$Cmd >&5 2>&5],
))
m4_define([GPSL_BUILD_CHECK],[[IBEGIN[
]$2([$1])[
  as_result=""
  ]GPSL_PREPARE_COMPILING[
  ]GPSL_COMPILE[ && as_result=`$as_file ]]$[1][[`]]
]IEND)
m4_define([GPS_DEFCHECK_C],[m4_define([$1],GPSL_BUILD_CHECK([$2],[GPSL_C_TO_SH]))])
m4_define([GPS_DEFCHECK_C_NO_STDIO],[m4_define([$1],GPSL_BUILD_CHECK([$2],[GPSL_C_TO_SH_NO_STDIO]))])

# ------------------------------------------------------------------------- #
# GPS_CONFIG_H_OUTPUT
# ------------------------------------------------------------------------- #
# This variable stacks the results of the checks to be included in the
# 'config.h' file.
m4_define([GPS_CONFIG_H_OUTPUT],[])

# ------------------------------------------------------------------------- #
# GPS_DEFEXCL(
#   NAME:exclcfg,
#   TITLE:string,
#   BODY1:script,
#   BODY2:script,
#   COMMENT:script
# )
# ------------------------------------------------------------------------- #
m4_define([GPS_DEFEXCL_CORE],[m4_define([GPS_CONFIG_H_OUTPUT],[DNL[
  echo "$1 ]$2["
]]m4_defn([GPS_CONFIG_H_OUTPUT]))])

m4_prog([GPS_DEFEXCL],[
  [m4_define([GPSL_SYMBOL],[[GPSL_CONFIG_ITEM_EXCLUSIVE__$1]])],
  [m4_define([GPSC_$1],GPSL_SYMBOL)],
  [GPS_DEFCHECKING(GPSL_SYMBOL,[$2],[$3],[$4],[$5])],
  [GPS_DEFEXCL_CORE([$1],GPSL_SYMBOL)],
  [m4_undef([GPSL_SYMBOL])],
])

# ------------------------------------------------------------------------- #
# GPSL_DEFINCL(
#   NAME:inclcfg,
#   VALUE:value,
#   TITLE:string,
#   BODY1:script,
#   BODY2:script,
#   COMMENT:script,
#   TRUERESPONSE:string
# )
# ------------------------------------------------------------------------- #
m4_define([GPSL_DEFINCL_CORE],[m4_define([GPS_CONFIG_H_OUTPUT],[DNL[
  if test "x]$3[" = "x$4"; then
    echo "$1 $2"
  fi
]]m4_defn([GPS_CONFIG_H_OUTPUT]))])

m4_prog([GPSL_DEFINCL],[
  [m4_define([GPSL_SYMBOL],[[GPSL_CONFIG_ITEM_INCLUSIVE__$1__$2]])],
  [m4_define([GPSC_$1],GPSL_SYMBOL)],
  [GPS_DEFCHECKING(GPSL_SYMBOL,[$3],[$4],[$5],[$6])],
  [GPSL_DEFINCL_CORE([$1],[$2],GPSL_SYMBOL,[$7])],
  [m4_undef([GPSL_SYMBOL])],
])

# ------------------------------------------------------------------------- #
# GPS_DEFINCL(
#   NAME:inclcfg,
#   VALUE:value,
#   TITLE:string,
#   BODY1:script,
#   BODY2:script,
#   COMMENT:script,
# )
# ------------------------------------------------------------------------- #
m4_prog([GPS_DEFINCL],[
  [GPSL_DEFINCL([$1],[$2],[$3],[$4],[$5],[$6],[yes])],
])

# ------------------------------------------------------------------------- #
# GPS_DEFINCL_NEGATED(
#   NAME:inclcfg,
#   VALUE:value,
#   TITLE:string,
#   BODY1:script,
#   BODY2:script,
#   COMMENT:script,
# )
# ------------------------------------------------------------------------- #
m4_prog([GPS_DEFINCL_NEGATED],[
  [GPSL_DEFINCL([$1],[$2],[$3],[$4],[$5],[$6],[no])],
])

# ------------------------------------------------------------------------- #
# GPSL_DEFBOOL(NAME:boolcfg,TEST:oldname,TRUERESPONSE:string)
# ------------------------------------------------------------------------- #
m4_define([GPSL_DEFBOOL],[m4_define([GPS_CONFIG_H_OUTPUT],[DNL[
  if test "x]$2[" = "x$3"; then
    echo "$1 yes"
  fi
]]m4_defn([GPS_CONFIG_H_OUTPUT]))])

# ------------------------------------------------------------------------- #
# GPS_DEFBOOL(NAME:boolcfg,TEST:oldname)
# ------------------------------------------------------------------------- #
m4_prog([GPS_DEFBOOL_NAMED],[
  [GPSL_DEFBOOL([$1],[$2],[yes])],
])

# ------------------------------------------------------------------------- #
# GPS_DEFBOOL_NEGATED(NAME:boolcfg,TEST:oldname)
# ------------------------------------------------------------------------- #
m4_prog([GPS_DEFBOOL_NEGATED_NAMED],[
  [GPSL_DEFBOOL([$1],[$2],[no])],
])

# ------------------------------------------------------------------------- #
# GPSL_DEFBOOL_DIRECT(
#   NAME:inclcfg,
#   TITLE:string,
#   BODY1:script,
#   LIBS_IF_YES:string,
#   LIBS_IF_NO:string,
#   BODY2:script,
#   COMMENT:script,
#   TRUERESPONSE:string
# )
# ------------------------------------------------------------------------- #
m4_define(GPSL_COMBINE,[[$2]m4_ifelse([$3][$4],[],[],[GPSL_LIBCHECK_BOOL_SETUP([$1],[$3],[$4])NL])])
[[[m4_define(GPSL_COMBINE,[[$2]])]]]
m4_prog([GPSL_DEFBOOL_DIRECT],[
  [m4_define([GPSL_SYMBOL],[[GPSL_CONFIG_ITEM_BOOLEAN__$1]])],
  [m4_define([GPSC_$1],GPSL_SYMBOL)],
  [GPS_DEFCHECKING(GPSL_SYMBOL,[$2],GPSL_COMBINE([$1],[$3],[$4],[$5]),[$6],[$7])],
  [GPSL_DEFBOOL([$1],GPSL_SYMBOL,[$8])],
  [m4_undef([GPSL_SYMBOL])],
])

# ------------------------------------------------------------------------- #
# GPS_DEFBOOL(
#   NAME:inclcfg,
#   TITLE:string,
#   BODY1:script,
#   LIBS_IF_YES:string,
#   LIBS_IF_NO:string,
#   BODY2:script,
#   COMMENT:script,
#   TRUERESPONSE:string
# )
# ------------------------------------------------------------------------- #
m4_prog([GPS_DEFBOOL],[
  [GPSL_DEFBOOL_DIRECT([$1],[$2],[$3],[$4],[$5],[$6],[$7],[yes])],
])

# ------------------------------------------------------------------------- #
# GPS_DEFBOOL_NEGATED(
#   NAME:inclcfg,
#   TITLE:string,
#   BODY1:script,
#   LIBS_IF_YES:string,
#   LIBS_IF_NO:string,
#   BODY2:script,
#   COMMENT:script,
#   TRUERESPONSE:string
# )
# ------------------------------------------------------------------------- #
m4_prog([GPS_DEFBOOL_NEGATED],[
  [GPSL_DEFBOOL_DIRECT([$1],[$2],[$3],[$4],[$5],[$6],[$7],[no])],
])

# ------------------------------------------------------------------------- #
# GPS_BUILDTOOL(NAME:newname,TITLE:string,CODE:string)
# ------------------------------------------------------------------------- #
m4_define([GPSL_CURRENT_TOOL],[0])
m4_prog([GPS_BUILD_TOOL],[
  [m4_prog([$1],[
    [m4_define([$1],[SH_SELFDIR/[tool]]GPSL_CURRENT_TOOL)],
    [GPSL_PUSH_STREAM],
    [NL[# Building tool for $2]NL],
    [SH_ECHO_N[ "building tool for $2... "]NL],
    [GPS_LOG([building tool for $2])NL],
    [[]SH_CLEANUP[ "]SH_SELFDIR[/tool]GPSL_CURRENT_TOOL"NL],
    [GPSL_C_TO_SH([$3])[]NL],
    [m4_undefine([GSPL_TMP])],
    [[]GPSL_PREPARE_COMPILING[]NL],
    [[if ]GPSL_COMPILE[; then :; else]NL],
    [[    ]GPS_ERROR([Error building tool $1])[]NL],
    [[fi]NL],
    [[mv -f "$as_file" ]$1[]NL],
    [[echo "done"]NL],
    [GPS_LOG([done])NL],
    [GPSL_POP_STREAM],
    [$1],
    [m4_define([GPSL_CURRENT_TOOL],m4_incr(GPSL_CURRENT_TOOL))],
  ])],
])

# ------------------------------------------------------------------------- #
# Macros to aid with library detection.
# ------------------------------------------------------------------------- #
# When detecting the library, assign library linker arguments into GPS_CURLIB
# and attempt to link against it. Set the result to "no" when the linking
# attempt fails and to "yes" when it succeeded. At the end of the library
# detection check place ]GPS_LIBCHECK_STORE(["name"])[ macro call, where
# "name" is an identifier used for the library.
#
# When using the library, place ]GPS_LIBCHECK_ACCESS("name")[ (where "name"
# is the name used in ]GPS_LIBCHECK_STORE[ call to identify the library) into
# the branch selecting a driver that uses the library to activate the
# library. Use ]GPS_LIBCHECK_CLEAR[ to cancel a previous activation of the
# library. At the end place the ]GPS_LIBCHECK_FINISH[ macro call to
# permanently record the library activation.
m4_prog([GPS_LIBCHECK_CLEAR],[
  [GPS_CURLIB=""],
])
m4_prog([GPSL_LIBCHECK_CLEAR_FETCHED],[
  [GPS_FETCHEDLIB=""],
])
m4_define([GPSL_LIBSYMBOL],[GPS_LIBARGS_$1])
GPS_DEFPIECE([GPS_LIBCHECK_STORE],[[
  ]GPSL_LIBSYMBOL([$1])[=""
  ac_lib_link="n/a"
  if test "x${]TST_RESULT[}" = "xyes"; then
    ]GPSL_LIBSYMBOL([$1])[="${GPS_CURLIB}"
    ac_lib_link="${GPS_CURLIB}"
  fi
  echo "! $1 @S|@ac_lib_link" >> libdeps.gen
  ]GPS_LIBCHECK_CLEAR[
]])
m4_define([GPS_LIBCHECK_ACCESS],[[GPS_CURLIB="]m4_ifelse([$1],[],[],[${GPSL_LIBSYMBOL([$1])}])["]])
GPS_DEFPIECE([GPS_LIBCHECK_FINISH],[[
  if test -n "${GPS_CURLIB}"; then
    for GPS_LIBARG in ${GPS_CURLIB}; do
      if grep -qxe "${GPS_LIBARG}" libargs.gen; then :; else
        test -n "${GPS_LIB}" && GPS_LIB="${GPS_LIB} "
        GPS_LIB="${GPS_LIB}${GPS_LIBARG}"
        echo "${GPS_LIBARG}" >>libargs.gen
      fi
    done
  fi
  ]GPS_LIBCHECK_CLEAR[
]])
GPS_DEFPIECE([GPSL_LIBCHECK_BOOL_SETUP],[[
  if test "$]TST_RESULT[" = "yes"; then
    ]GPS_LIBCHECK_ACCESS($2)[
    ]m4_ifelse([$2],[],[DNL],[echo "@ $1 yes $2" >> libdeps.gen])[
  elif test "$]TST_RESULT[" = "no"; then
    ]GPS_LIBCHECK_ACCESS($3)[
    ]m4_ifelse([$3],[],[DNL],[echo "@ $1 no $3" >> libdeps.gen])[
  fi
  ]GPS_LIBCHECK_FINISH[
]])
GPS_DEFPIECE([GPS_FETCH_LIB],[[
  ac_library_references="@S|@ac_library_references $1"
  if test "x@S|@]TST_RESULT[" = "xyes"; then
    if ]GPS_TEST_LIB_$1[; then
      if test -n "${]GPSL_LIBSYMBOL([$1])[}"; then
        GPS_FETCHEDLIB="@S|@{GPS_FETCHEDLIB} ${]GPSL_LIBSYMBOL([$1])[}"
      fi
    else
      ]TST_RESULT[="no"
    fi
  fi
]])
GPS_DEFPIECE([GPSL_PREPARE_LIB_FETCHING],[[
  ]GPSL_LIBCHECK_CLEAR_FETCHED[
  ac_current_lib="$1"
  ac_library_references=""
]])
GPS_DEFPIECE([GPSL_CLEANUP_LIB_FETCHING],[[
  if test "x@S|@ac_library_references" != "x"; then
    echo ": @S|@ac_current_lib@S|@ac_library_references" >> libdeps.gen
  fi
  ]GPSL_LIBCHECK_CLEAR_FETCHED[
]])

m4_define([GPS_DETECT_LIBRARY],[

GPS_DEFCHECKING([GPSLIB_$1],
  [for $2],[[
  ]GPSL_PREPARE_LIB_FETCHING([$1])[
  ]DNL[]$3[]DNL[
  ]GPS_LIBCHECK_STORE($1)[
  ]GPSL_CLEANUP_LIB_FETCHING[
]])

m4_define([GPS_TEST_LIB_$1],[[test "x]GPSLIB_$1[" = "xyes"]])

])

# ------------------------------------------------------------------------- #
# GPS_TRY_HEADER(HEADER:string,PREREQUISITIES:string)
# ------------------------------------------------------------------------- #
# Try a header.
m4_define([GPSL_EXPAND_PREREQ_HEADERS],[dnl
m4_ifelse($#,0,[],$#,1,[],
[[@%:@include <$1>][]CODE_EOL[]NL[]CODE_BOL[]dnl
GPSL_EXPAND_PREREQ_HEADERS(m4_shift($@))])dnl
])
m4_prog([GPS_TRY_HEADER],[
  [GPSL_C_TO_SH(
  [GPSL_EXPAND_PREREQ_HEADERS($2)]@%:@include <$1>
  int main()
  {
    return(0);
  }
  )],
  [NL[as_result="no"]NL],
  [[]GPSL_PREPARE_COMPILING[]NL],
  [[if ]GPSL_COMPILE[; then]NL],
  [[    as_result="yes"]NL],
  [[fi]],
])

# ------------------------------------------------------------------------- #
# GPSL_HEADER_TO_IDENT(HEADER:string)
# ------------------------------------------------------------------------- #
m4_define([GPSL_HDR_ORIG],[[./abcdefghijklmnopqrstuvwxyz]])
m4_define([GPSL_HDR_TRAN],[[__ABCDEFGHIJKLMNOPQRSTUVWXYZ]])
m4_prog([GPSL_HEADER_TO_IDENT],[
  [[HAVE_]m4_translit([$1],GPSL_HDR_ORIG,GPSL_HDR_TRAN)],
])

# ------------------------------------------------------------------------- #
# GPS_CHECK_HEADER(HEADER:string)
# ------------------------------------------------------------------------- #
m4_prog([GPS_CHECK_HEADER],[
  [m4_pushdef([GPS_TMP1],GPSL_HEADER_TO_IDENT($1))],
  [m4_pushdef([GPS_TMP],[GPS_]m4_quote(GPS_TMP1))],
  [GPS_DEFCHECKING(GPS_TMP,[for $1],]m4_quote(m4__cat(
    NL[]GPS_TRY_HEADER([$1],[$2]),
    NL[]TST_RESULT[=]CODE_OUTPUT[]NL,
  ))[)],
  [m4_popdef([GPS_TMP1])],
  [m4_popdef([GPS_TMP])],
])

# ------------------------------------------------------------------------- #
# GPS_IS_HEADER(HEADER:string)
# ------------------------------------------------------------------------- #
m4_prog([GPS_IS_HEADER],[
  [m4_pushdef([GPS_TMP],[[GPS_]GPSL_HEADER_TO_IDENT($1)])],
  [m4_ifdef(GPS_TMP,,],
    [[GPS_CHECK_HEADER([$1])]],
  [)],
  [[test "]m4_unquote(GPS_TMP)[" $2= "yes"]],
  [m4_popdef([GPS_TMP])],
])
m4_define([GPS_IS_NOT_HEADER],[GPS_IS_HEADER([$1],[!])])

# ------------------------------------------------------------------------- #
# GPS_ARE_HEADERS(HEADERLIST:string)
# ------------------------------------------------------------------------- #
GPS_DEFPIECE([GPSL_PROCESS_ONE_HEADER],[[
  if ]GPS_IS_NOT_HEADER($1)[; then
    ]TST_RESULT[="no"
  fi
]])
m4_prog([GPSL_LIST_HEADERS],[
  [m4_ifelse(
    $#,1,[],
    [GPSL_PROCESS_ONE_HEADER([$1])NL[]GPSL_LIST_HEADERS(m4_shift($@))]DNL
  )],
])
m4_prog([GPS_ARE_HEADERS],[
  []TST_RESULT[="yes"]NL,
  [GPSL_LIST_HEADERS($1)DNL],
])

# ------------------------------------------------------------------------- #
# GPS_TRY_FUNCTION(HEADER:string,DATA:string)
# ------------------------------------------------------------------------- #
# Try a header.
m4_prog([GPS_TRY_FUNCTION],[
  [[GPSL_C_TO_SH(
  $2
  int main()
  {
    void *p;
    p=$1;
    return(p==printf);
  }
  )]],
  [NL[as_result="no"]NL],
  [[]GPSL_PREPARE_COMPILING[]NL],
  [[if ]GPSL_COMPILE[; then]NL],
  [[    as_result="yes"]NL],
  [[fi]],
])

# ------------------------------------------------------------------------- #
# GPS_CHECK_FUNCTION(FUNCTION:string)
# ------------------------------------------------------------------------- #
m4_prog([GPS_CHECK_FUNCTION],[
  [m4_pushdef([GPS_TMP1],[HAVE_]m4_translit([$1],[.abcdefghijklmnopqrstuvwxyz],[_ABCDEFGHIJKLMNOPQRSTUVWXYZ]))],
  [m4_pushdef([GPS_TMP],[GPS_]m4_quote(GPS_TMP1))],
  [GPS_DEFCHECKING(GPS_TMP,[for $1],]m4_quote(m4__cat(
    NL[]GPS_TRY_FUNCTION([$1],[$2]),
    NL[]TST_RESULT[=]CODE_OUTPUT[]NL,
  ))[)],
  [m4_popdef([GPS_TMP1])],
  [m4_popdef([GPS_TMP])],
])

# ------------------------------------------------------------------------- #
# Macros to aid with driver selection.
# ------------------------------------------------------------------------- #

m4_define([GPSL_BUILD_DRIVER_SELECTION],[[DNL[
  if ]GPS_TEST_LIB_$2[; then
    ]TST_RESULT[="$1"
    ]GPS_LIBCHECK_ACCESS([$2])[
  fi
]]])

m4_define([GPSL_BUILD_DRIVER_SELECTOR],
  [m4_for([$1],[GPSL_BUILD_DRIVER_SELECTION])]dnl
)

m4_define([GPSL_BUILD_DRIVER_TABLE_LINE],[m4_ifelse($2,[],[],[DNL[
  echo "@ ]]m4_quote(GPSL_CURRENT_CONFIG_ITEM)[[ $1 $2" >> libdeps.gen
]])])

m4_prog([GPSL_BUILD_DRIVER_TABLE],[
  [m4_pushdef([GPSL_CURRENT_CONFIG_ITEM],[$1])],
  [m4_for([$2],[GPSL_BUILD_DRIVER_TABLE_LINE])],
  [m4_popdef([GPSL_CURRENT_CONFIG_ITEM])],
])

m4_define([GPS_SELECT_DRIVER],[

GPS_DEFEXCL([$1],
  [$2],[[
  ]TST_RESULT[="]]m4_ifelse([$4],[],none,$4)[["
  ]]GPSL_BUILD_DRIVER_SELECTOR([$3])[DNL[
  ]GPS_LIBCHECK_FINISH[
  ]]GPSL_BUILD_DRIVER_TABLE([$1],[$3])[DNL[
]])

])
