divert(-1)

# Generic Platform Scanner for OSHS SYSLIB.
# The m4 initialization script
#
# The Generic Platform Scanner for OSHS SYSLIB (or 'GPS') is a M4/shell
# program that is used to scan the platform on which the SYSLIB library is
# being built.

## ------------------- ##
##   About this file   ##
## ------------------- ##

# The GPS includes a m4 debugger called M4DEBUG. If something breaks, I need
# this m4 debugger to be able to figure what went wrong. However some of the
# m4 features are not used in the rest of the GPS (only at initialization)
# so I need to avoid the need to implement them in the m4 debugger (it would
# take significant amount of effort, which I consider to be a waste of time).
# An example of such a feature is the ability to obtain the definition of a
# builtin using 'defn'.

# Therefore I decided to go through an easy way. I created this file, which
# brings the m4 enviroment to the right way. The M4DEBUG program just
# initializes its enviroment so that there is no need to feed this file into
# it. 

# The enviroment changes done are:
#
# - Output is disabled (use m4_divert(0) or m4_divert to enable it).
# - All macros get the 'm4_' prefix except these:
#     Old name          New name
#     m4exit            m4_exit
#     m4wrap            m4_wrap
#     patsubst          m4_sub
#     changequote       (deleted)
#     changecom         (deleted)
#
# Remark: These macros have different names in autoconf:
#     m4 name           Autoconf name
#     regexp            m4_bregexp
#     patsubst          m4_bpatsubst

## -------------------------------- ##
##   M4 enviroment initialization   ##
## -------------------------------- ##

# This program requires GNU M4 or OSHS M4. However there is no need to check
# this out since the invoking script does it automagically before trying to
# feed the M4 with this script.

# Set the quotes, whatever the current quoting system. The default m4 quotes
# ("`" and "'") are inapropiate for our purposes since they are also
# extensively used in the shell scripts (which we are going to generate.
# The quotes the GPS is using are "[" and "]", which not only are quite
# uncommon but more importantly they are really unlikely to be unbalanced.

changequote()
changequote([, ])

## ------------------------------------- ##
##   Making order in M4 builtin macros   ##
## ------------------------------------- ##

# The M4 builtins are moved into the 'm4' namespace. Also some builtins are
# enhanced and some are 'invented'.

# First things first. Move the macro definition managing builtins into our
# 'm4' namespace.

define([m4_define],   defn([define]))
define([m4_defn],     defn([defn]))
define([m4_undefine], defn([undefine]))

m4_undefine([define])
m4_undefine([defn])
m4_undefine([undefine])

# Some m4 internals have names colliding with tokens we might use.
# Rename them a'la 'm4 --prefix-builtins'.

# ------------------------------------------------------------------------- #
# m4_copy(SRC:oldname, DST:newname)
# $DST is $SRC
# ------------------------------------------------------------------------- #
# Define DST as the definition of SRC.
# What's the difference between:
# 1. m4_copy([from], [to])
# 2. m4_define([from], [to($@)])
# Well, obviously 1 is more expansive in space.  Maybe 2 is more expansive
# in time, but because of the space cost of 1, it's not that obvious.
# Nevertheless, one huge difference is the handling of '$0'.  If 'from'
# uses '$0', then with 1, 'to''s '$0' is 'to', while it is 'from' in 2.
# The user will certainly prefer see 'from'.

m4_define([m4_copy], [m4_define([$2], m4_defn([$1]))])

# ------------------------------------------------------------------------- #
# m4_rename(SRC:delname, DST:newname)
# $DST is $SRC
# ------------------------------------------------------------------------- #
# Rename the macro SRC as DST.

m4_define([m4_rename], [m4_copy([$1], [$2])m4_undefine([$1])])

# ------------------------------------------------------------------------- #
# m4_rename_m4(NAME:delname)
# m4_$NAME is $NAME
# ------------------------------------------------------------------------- #
# Rename macro NAME as m4_NAME.

m4_define([m4_rename_m4], [m4_rename([$1], [m4_$1])])

# Some m4 internals have names colliding with tokens we might use.
# Move them into the 'm4' name space.

m4_rename_m4([builtin])
m4_undefine([changecom])
m4_undefine([changequote])
m4_rename_m4([debugfile])
m4_rename_m4([debugmode])
m4_rename_m4([decr])
m4_rename_m4([divert])
m4_rename_m4([divnum])
m4_rename_m4([dumpdef])
m4_rename_m4([errprint])
m4_rename_m4([esyscmd])
m4_rename_m4([eval])
m4_rename_m4([format])
m4_rename_m4([ifdef])
m4_rename_m4([ifelse])
m4_rename_m4([include])
m4_rename_m4([incr])
m4_rename_m4([index])
m4_rename_m4([indir])
m4_rename_m4([len])
m4_rename([m4exit], [m4_exit])
m4_rename([m4wrap], [m4_wrap])
m4_rename_m4([maketemp])
m4_rename([patsubst], [m4_sub])
m4_rename_m4([popdef])
m4_rename_m4([pushdef])
m4_rename_m4([regexp])
m4_rename_m4([shift])
m4_rename_m4([sinclude])
m4_rename_m4([substr])
m4_rename_m4([symbols])
m4_rename_m4([syscmd])
m4_rename_m4([sysval])
m4_rename_m4([traceoff])
m4_rename_m4([traceon])
m4_rename_m4([translit])
m4_rename_m4([undivert])

# This macro is used to break execution of a macro at particular place. It
# expands to nothing.

m4_define([m4_break],[])

# Remove macros used by the initialization only

m4_undefine([m4_copy])
m4_undefine([m4_rename])
m4_undefine([m4_rename_m4])
