# Detecting the shared memory extension of X11
#
# Some X11 versions have the ability to use shared memory for transfers of
# image data between the server and the client. This improves performance of
# the screen updates as the huge image data don't need to be transferred over
# a pipe or similar mechanism. This ability of course works only when the
# client and the server are running at the same machine but the programmer
# does not need to worry about this, because the driver will take care about
# these details.

GPS_CHECK_HEADER([X11/extensions/XShm.h],[X11/Xlib.h,X11/Xutil.h,])

GPS_DEFCHECK_C([GPSL_PROBE_X11_SHM],[[
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/extensions/XShm.h>
int main(void)
{
  void *p;
  p=XShmAttach;
  if (p!=printf) {
    printf("yes");
  }
  return(0);
}
]])

GPS_DETECT_LIBRARY([X11SHM],
  [X11_SHM extension],[[
  ]GPS_ARE_HEADERS([sys/ipc.h,sys/shm.h,X11/extensions/XShm.h,])[
  ]GPS_FETCH_LIB([X11])[
  if test "x$]TST_RESULT[" = "xyes"; then
    ]GPSL_PROBE_X11_SHM[
    if test -z "]CODE_OUTPUT["; then
      GPS_CURLIB="-lXext"
      ]GPSL_PROBE_X11_SHM[
      if test -z "]CODE_OUTPUT["; then
        ]TST_RESULT[="no"
      fi
    fi
  fi
]])
