# Ensure the $LINENO variable is working

GPS_DEFCHECK([SHL_LINENO_NATIVE],[[
  # Check if LINENO variable works
  ]SH_CLEANUP[ ]SH_SELFDIR[/lineno.gen
  cat >]SH_SELFDIR[/lineno.gen <<EOF
  as_lineno_1=\$LINENO
  as_lineno_2=\$LINENO
  as_lineno_3=\`(expr \$as_lineno_1 + 1) 2>/dev/null\`
  if test \"x\$as_lineno_1\" != \"x\$as_lineno_2\"; then
    if test \"x\$as_lineno_3\"  = \"x\$as_lineno_2\"; then
      echo yes
    fi
  fi
  EOF
  ]TST_RESULT[=`. ]SH_SELFDIR[/lineno.gen`
]])

GPS_DEFSNIPPET([SH_SETUP_LINENO],[[
  # Ensure the LINENO variable is working properly even on shells that don't
  # support it natively. This is required for the logging machinery.
  # We checked if $LINENO works as expected just above.
  if test "x]SHL_LINENO_NATIVE[" != "xyes"; then
    # Oops, the current shell does not support LINENO so we have to fixup
    # things ourselves.
    # To do that we need to find where we are first. Look in the path if we
    # contain no path at all relative or not.
    case @S|@0 in
      *[\\/]* ) as_myself=@S|@0 ;;
      *) as_myself="" ;;
    esac
    if test "x$as_myself" = "x"; then
      as_save_IFS=$IFS
      IFS=]SH_PATH_SEPARATOR[
      for as_dir in $PATH; do
        IFS=$as_save_IFS
        test -z "$as_dir" && as_dir=.
        test -r "$as_dir/@S|@0" && as_myself=$as_dir/@S|@0 && break
      done
    fi
    # If we did not find ourselves, most probably we were run as 'sh COMMAND'
    # in which case we are not to be found in the path. Try $0 in this case
    # as a guess.
    if test "x$as_myself" = x; then
      as_myself=@S|@0
    fi
    # Check if we really found ourself.
    if test ! -f "$as_myself"; then
      { echo "]SH_SELF[: error: cannot find myself; rerun with an absolute path" >&2
      { (exit 1); exit 1; }; }
    fi
    # Being really lazy we try to search the system for a shell that can
    # handle $LINENO first. If we find such a shell, we simply feed ourselves
    # into it and go ahead.
    case $CONFIG_SHELL in
      '')
        as_save_IFS=$IFS; IFS=]SH_PATH_SEPARATOR[
        for as_dir in /bin$PATH_SEPARATOR/usr/bin$PATH_SEPARATOR$PATH
        do
          IFS=$as_save_IFS
          if test "x$as_dir" = "x"; then 
            as_dir=.
          fi
          for as_base in sh bash ksh sh5; do
            case $as_dir in
              /*)
                as_test=` "$as_dir/$as_base" ]SH_SELFDIR[/lineno.gen 2>/dev/null`
                if test "x$as_test" = "xyes"; then
                  ]SH_UNSET[ BASH_ENV || test "${BASH_ENV+set}" != set || { BASH_ENV=; export BASH_ENV; }
                  ]SH_UNSET[ ENV || test "${ENV+set}" != set || { ENV=; export ENV; }
                  CONFIG_SHELL=$as_dir/$as_base
                  export CONFIG_SHELL
                  exec "$CONFIG_SHELL" "@S|@0" ${1+"$@"}
                fi
              ;;
            esac
          done
        done
      ;;
    esac
    # No shell capable of $LINENO handling. The only way to get past this
    # obstacle is to create a copy of ourself, with all occurences
    # of the '$LINENO' string uniformly replaced with their respective
    # numbers. This will not work perfectly when someone tries to reference
    # LINENO indirectly but the checks are created in such a way that no
    # one will do such an ugly and nonportable thing.
    #
    # So create 'scansys.sh' as a copy of $as_myself, but with $LINENO
    # uniformly replaced by the line number. The first 'sed' inserts a
    # line-number line before each line; the second 'sed' does the real
    # work. The second script uses 'N' to pair each line-number line
    # with the numbered line, and appends trailing '-' during
    # substitution so that $LINENO is not a special case at line end.
    # This 'scansys.sh' will then be fed into the current shell.
    # Also remember to register the 'scansys.sh' file for removal.
    # (Raja R Harinath suggested sed '=', and Paul Eggert wrote the
    # second 'sed' script.  Blame Lee E. McMahon for sed's syntax. :-)
    as_start="\(.*\)"
    as_lineno="[$]LINENO"
    as_end="\(@<:@^']AS_CR_ALNUM['_@:>@\)"
    sed '=' <$as_myself |
      sed '
        N
        s,$,-,
        : loop
        s,^\(@<:@']AS_CR_DIGITS['@:>@*\)'$as_start$as_lineno$as_end',\1\2\1\3,
        t loop
        s,-$,,
        s,^@<:@']AS_CR_DIGITS['@:>@*\n,,
      ' >scansys.sh &&
    chmod +x ]SH_SELFDIR[/scansys.sh ||
    {
      echo "]SH_SELF[: error: cannot create 'scansys.lin'" >&2
      echo "]SH_SELF[: try to rerun with a POSIX shell" >&2
      { (exit 1); exit 1; };
    }
    # Don't try to exec as it changes $[0], causing all sort of problems
    # (the dirname of $[0] is not the place where we might find the
    # original and so on.  Autoconf is especially sensible to this).
    . ]SH_SELFDIR[/scansys.sh
    # Exit status is that of the last command. Keep it in a variable as we
    # still have some things to do before returning.
    ac_stored=$?
    # Delete the generated script after use.
    rm -f ]SH_SELFDIR[/scansys.sh
    # Return the exit status stored previously.
    exit $ac_stored
  fi
]])
