# NLS nuisances removal
#
# SCO Unix doesn't like LANG=C and friends, but without LC_MESSAGES=C,
# Solaris will translate variable values in 'set'! Similarly, without
# LC_CTYPE=C, compiled C code can behave unexpectedly. The trick here is to
# set the values of these variables to 'C' except for if they are not already
# set at all. And it is better to ensure the variables are unset rather than
# set to something like empty string or so.

GPS_DEFSNIPPET([SH_KILL_NLS_NUISANCES],[[
  # NLS nuisances.
  for as_var in \
    LANG LANGUAGE LC_ADDRESS LC_ALL LC_COLLATE LC_CTYPE LC_IDENTIFICATION \
    LC_MEASUREMENT LC_MESSAGES LC_MONETARY LC_NAME LC_NUMERIC LC_PAPER \
    LC_TELEPHONE LC_TIME
  do
    if (set +x; test -z "`(eval $as_var=C; export $as_var) 2>&1`"); then
      eval $as_var=C; export $as_var
    else
      ]SH_UNSET[ $as_var
    fi
  done
]])
