# Attempts to allocate huge memory blocks may hurt on some platforms. If the
# platform has malloc() with a 16 bit argument and a request for a block
# larger than 64K is made, the compiler silently truncates the high bits of
# the requested size and passes the low 16 bits to malloc(). The malloc()
# happily returns a non-NULL pointer but this pointer points to a block that
# is much shorter than the caller requested. The result is an ugly and
# undetectable buffer overflow bug.
#
# This check assumes that malloc has the memory block size of the size_t. If
# the size_t cannot represent memory sizes larger than 64K, the support for
# large memory blocks is disabled to avoid the bug just mentioned. The
# library will return NULL if a memory block above 64K is requested in that
# case.

GPS_DEFBOOL_NEGATED([HUGE_MEMORY],
  [whether >64K memory block requests cause buffer overflow],[[
  ]TST_RESULT[="yes"
  if test "]GPS_SIZE_T_SIZE[" != "no"; then
    if test "x`expr "]GPS_SIZE_T_SIZE[" ">" 3`" = "x1"; then
      ]TST_RESULT[="no"
    fi
  fi
]])
