# Check which method of using locks is used. Mutexes are faster but they may
# be unapropiate as POSIX does not allow a mutex to be released using thread
# other than the one that locked it. The other way is "rwlocks", which can be
# released by any thread but they are slower.

GPS_DEFCHECK_C([GPSL_PROBE_MUTEXES],[
  #include <pthread.h>
  #include <stdlib.h>
  #include <time.h>
  pthread_mutex_t Mutex;
  pthread_cond_t Cond;
  pthread_mutex_t CondWait;
  int Failed;
  int Done;
  void InitializeMutex(pthread_mutex_t *Mutex)
  {
    if (pthread_mutex_init(Mutex,NULL)!=0) Failed=1;
  }
  void *RunThread(void *Data)
  {
    Failed=(pthread_mutex_unlock(&Mutex)!=0);
    if (pthread_mutex_lock(&CondWait)!=0) exit(1);
    Done=1;
    pthread_cond_broadcast(&Cond);
    if (pthread_mutex_unlock(&CondWait)!=0) exit(1);
  }
  void *RunMainThread(void *Data)
  {
    pthread_t *ID;
    ID=Data;
    InitializeMutex(&Mutex);
    InitializeMutex(&CondWait);
    if (Failed==1 || pthread_cond_init(&Cond,NULL)!=0) return(1);
    if (pthread_mutex_lock(&CondWait)!=0) return(1);
    if (pthread_mutex_lock(&Mutex)!=0) return(1);
    if (pthread_create(ID,NULL,RunThread,NULL)!=0) return(1);
    if (pthread_cond_wait(&Cond,&CondWait)!=0) return(1);
    if (Failed==0) printf("yes");
    exit(0);
  }
  int main(void)
  {
    pthread_t ID;
    pthread_t OID;
    if (pthread_create(&ID,NULL,RunMainThread,&OID)!=0) return(1);
    sleep(1);
    pthread_cancel(ID);
    pthread_cancel(OID);
    return(0);
  }
])

GPS_DEFBOOL_NEGATED([USE_RWLOCK],
  [whether pthread mutexes are suitable for us],[[
  ]GPS_UNUSED[
  if ]GPS_TEST_LIB_PTHREAD[; then
    ]TST_RESULT[="no"
    ]GPS_LIBCHECK_ACCESS([PTHREAD])[
    ]GPSL_PROBE_MUTEXES[
    ]GPS_LIBCHECK_CLEAR[
    if test -n "]CODE_OUTPUT["; then
      ]TST_RESULT[="yes"
    fi
  fi
]])
