#include <string.h>

#include "filewrt.h"
#include "types.h"
#include "files.h"
#include "fileread.h"
#include "scandop.h"
#include "prepsym.h"

static int VerboseMakefile;

static void PushDependencies(TFileInfo *F)
{
  int i;

  for (i=0;i<F->DepCount;i++) {
    RegisterFileToBeProcessed(F->DepList[i]);
  }
}

static void AccessDependencies(TFileInfo *F)
{
  int i;

  for (i=0;i<F->DepCount;i++) {
    RegisterAccessedFile(F->DepList[i]);
  }
}

static void NewRule(void)
{
  NewLine();
  NewLine();
}

static void StartNewCommand(int Echo)
{
  NewLine();
  Print("\t");
  if (!Echo) Print("@");
}

static void NewCommand(void)
{
  StartNewCommand(VerboseMakefile);
}

static void MessageCommand(const char *Prefix,const char *N1,const char *N2)
{
  if (!VerboseMakefile) {
    StartNewCommand(0);
    Print("echo ");
    Print(Prefix);
    Print(" ");
    Print(N1);
    Print(N2);
  }
}

static void PrintObjectFile(TFileInfo *F)
{
  int Len;

  Len=strlen(F->FileName);
  PrintLen(F->FileName,Len-2);
  Print(".o");
}

static void BuildLibraryBuildRule(void)
{
  bool Space;
  TFileInfo *F;
  bool StaticLibs;
  bool SharedLibs;

  Space=false;
  Print("OBJECTS=");
  PrepareFileProcessing();
  while (true) {
    F=GetNextFileToBeProcessed();
    if (F==NULL) break;
    PushDependencies(F);
    F=F->Source;
    if (F==NULL) continue;
    PushDependencies(F);
    if (Space) Print(" ");
    Space=true;
    PrintObjectFile(F);
  }
  StaticLibs=CheckSymbol("__i__LIB_SUPPORT__static__",NULL);
  SharedLibs=CheckSymbol("__i__LIB_SUPPORT__shared__",NULL);
  NewRule();
  Print("all:");
  if (SharedLibs) {
    Print(" syslib.so");
  } else {
    Print(" cleanup");
  }
  Print(" syslib.a");
  if (!SharedLibs) {
    NewRule();
    Print("cleanup:");
    NewCommand();
    Print("rm -f syslib.so");
  }
  if (SharedLibs) {
    NewRule();
    Print("syslib.so: $(OBJECTS)");
    MessageCommand("linking","syslib/syslib.so","");
    NewCommand();
    Print("$(LINK) -shared -o syslib.so $(OBJECTS) $(GPS_LIB)");
  }
  NewRule();
  Print("syslib.a: $(OBJECTS)");
  MessageCommand("linking","syslib/syslib.a","");
  NewCommand();
  Print("rm -f syslib.a");
  NewCommand();
  Print("ar -qc syslib.a $(OBJECTS) >/dev/null");
  NewCommand();
  Print("$(GPS_RANLIB) syslib.a");
  NewCommand();
  if (StaticLibs) {
    Print("rm -f syslib.soo");
  } else {
    Print(
      "echo \"Existence of this file prevents syslib.a from installing. "
      "The content of this\" >syslib.soo"
    );
    NewCommand();
    Print(
      "echo \"file does not matter, only whether it exists. This file "
      "is created only when\" >>syslib.soo"
    );
    NewCommand();
    Print(
      "echo \"shared library support in SYSLIB is active and static library "
      "support is not.\" >>syslib.soo"
    );
    NewCommand();
    Print(
      "echo \"The SYSLIB build scripts require that syslib.a is always "
      "built and available,\" >>syslib.soo"
    );
    NewCommand();
    Print(
      "echo \"even when static library support is turned off.\" >>syslib.soo"
    );
  }
}

static void BuildAccessedFileList(TFileInfo *F)
{
  InitFileAccessStack();
  RegisterAccessedFile(F);
  while (true) {
    F=GetNextAccessedFile();
    if (F==NULL) break;
    Print(" ");
    Print(F->FileName);
    AccessDependencies(F);
  }
  DestroyFileAccessStack();
}

static void BuildCompilationRule(TFileInfo *F)
{
  NewRule();
  PrintObjectFile(F);
  Print(":");
  BuildAccessedFileList(F->Source);
  MessageCommand("compiling","syslib/",F->Source->FileName);
  NewCommand();
  Print("$(CC) -c ");
  Print(F->Source->FileName);
  Print(" -o ");
  PrintObjectFile(F);
}

static void BuildCompilationRules(void)
{
  TFileInfo *F;

  PrepareFileProcessing();
  while (true) {
    F=GetNextFileToBeProcessed();
    if (F==NULL) break;
    PushDependencies(F);
    if (F->Source==NULL) continue;
    PushDependencies(F->Source);
    BuildCompilationRule(F);
  }
}

void BuildMakefile(int Verbose)
{
  VerboseMakefile=Verbose;
  OpenOutputFile("makefile");
  BuildLibraryBuildRule();
  BuildCompilationRules();
  NewLine();
  CloseOutputFile();
}
