This is the platform definition header file. It is processed by the MAKEGEN
program to produce the "defs.gh" header file.

The meaningfull content of this file is contained in a block lists. This
block list is introduced by the "------------------------" line and the same
line terminates it. The blocks in the list are delimited by the
"------------" lines. The empty lines at the beginning and at the end of the
block are ignored but the empty lines in the middle of the block are
meaningful. Also any whitespace at the end of the block lines is ignored.

This list contains descriptions off all configuration items produced by GPS
and checked in the SYSLIB source code. These configuration items describe the
various platform dependent settings the SYSLIB must be aware of. The MAKEGEN
program uses this list to check for malformed or otherwise invalid platform
dependency checks in SYSLIB source code.

There are three types of the platform dependent features. "Boolean" settings
can be set to "yes" (or simply set) or "no" (or simply unset). "Inclusive"
setting can be set to any arbitrary set of allowed values and "Exclusive"
settings must be set to exactly one value.

Each block in this list contains the name of the setting and its type in
brackets. The type must be "Exclusive", "Inclusive" or "Boolean". The next
line is empty.

After the name of the setting the description of the setting follows. In the
description there must be exactly one space between words (more exactly,
there must be no multiple spaces or tabs at the first line between the first
and the seond word of the paragraph). The paragraphs are delimited by the
empty lines.

If the setting is not "Boolean", the last paragraph contains the description
of the values valid for the setting. The paragraph is identified by the
whispace between the first and second word of the paragraph, where must be
something other than a single space (for example multiple spaces or a tab).
The first word of each line of this paragraph contains the name of the value
(if it starts at the first column of the line) and the rest of the line is
the description of the value. If the description is too long and cannot fit
onto one single line, it can be split to multiple lines by prepending the
continuation with whitespace.

Each configuration item must have different name.

------------------------

TIMESHIFT	(Exclusive)

Specifies how to obtain the UTC<->LocalTime shift value.

always_zero	Always report 0
localtime	Use localtime()
thread_safe	Use localtime_r() which is thread safe

------------

ERRNO		(Inclusive)

Error code sets recognized by the convertor.

posix		Add POSIX defined error codes
unix		Add UNIX specific error codes

------------

UNISTD		(Boolean)

Set to 1 if the target has <unistd.h> header file.

------------

NEED_IO		(Boolean)

Set to 1 if the target has open() and close() functions declared in <io.h>
instead of <fcntl.h> header file.

------------

SYNC_SUPPORT	(Boolean)

Set to 1 if the target supports O_SYNC flag for open(). This flag ensures
that data are written to the device before the write() operation on the
resulting file descriptor returns.

------------

MODE		(Exclusive)

Select default file mode flags for new files. This is passed to open() as
third argument.

posix		Use POSIX compliant rationale value
msdos		Use MSDOS flags
zero		Use zero

------------

IO32		(Boolean)
  
Set if the 32 bit values may be passed to the low-level read()/write() calls
directly. Unset if these calls cannot perform I/O larger than 64K. This
should be unset for platforms without the HUGE_MEMORY capability.

------------

TRUNC		(Exclusive)
  
Determines how to truncate the file
  
ftruncate	Use ftruncate() system call
reopen		Reopen the file with O_TRUNC flag

------------

STATINFO	(Exclusive)
  
Determines how to interpret informations from stat() system call.
  
bcmsdos		MS-DOS file information, using Borland C
unix		UNIX file information

------------

CHMOD		(Exclusive)

Select CHMOD implementation. This has been made an exclusive feature, because
one might want add more methods later.

secure		Secure implementation using fchmod()
posix		Less secure implementation using chmod()

------------

CHOWN		(Exclusive)

Select CHOWN/CHGRP implementation. This has been made an exclusive feature,
because one might want add more methods later.

secure		Full and secure implementation using POSIX calls (fchown)
posix		Full implementation using POSIX calls (chown)
root_only	Allow only chown/chgrp to uid/gid 0, return eNOTSUP otherwise

------------

DIR_MANIP	(Exclusive)

Determine, how to manipulate with directories (create and remove them).

posix		Use POSIX directory manipulation methods
bcmsdos		Use MS-DOS with Borland C directory manipulation methods
tcmsdos		Use MS-DOS with Tirbo C++ 2.01 directory manipulation methods

------------

FSPEC_TYPE	(Exclusive)

Specify the type of file specifications accepted by the underlying platform.

msdos		Device ID, upper-case names, delimiter is backslash
unix		No device ID, lower-case names, delimiter is slash

------------

READDIR_BAD	(Boolean)

Set to 1 if readdir() returns NULL and error condition when end of
directory is reached.In such a case the ReadDir() will simply return
"end of directory" when it encounters NULL and will not check for any
errors. The normal operation is to return NULL with no error condition when
there are no more items in the directory being read.

------------

DIRITEM_TYPE	(Exclusive)

Specify how to handle directory items read from directories.

native		Use the items as they were returned
unix		Convert the items to uppercase

------------

CASE_SENSE	(Boolean)

Set to 1 if the platform has case sensitive filenames and to 0 (or undefine)
if not.

------------

OPEN_MODE	(Exclusive)
  
Specifies platform specific open mode flags that are constant and are
needed to force the system to use binary file reading or similar things.

zero		No platform specific mode flags
force_binary	Set "binary mode" platform specific flag

------------

SCREEN_DRV	(Exclusive)

Selects the screen video memory direct access driver.

none		No driver; pretend impossibility to work with the screen.
SDL		Use SDL.
windows		Use Windows GUI.
X11		Use X11 calls

------------

USE_SHM		(Boolean)

When the SCREEN_DRV is set to X11, this allows the system to use the SHM
extension of X. This fastens the screen updates when both, the server and
the client run on the same machine but reduces portability (not all systems
support shared memory).

------------

ALT_BUG		(Boolean)

Set if the underlying system reports the "Right Alt" key to be "Mode Shift".
This causes the "Mode Shift" keystrokes to be converted to "Right Alt"
keystrokes. On PC keyboards there is no "Mode Shift" key.

------------

NUMCAPS_BUG	(Boolean)

On some platforms Num Lock and Caps Lock are incorrectly reported. When the
offending key is pressed a keypress is reported for it and when it is
released, nothing is reported. When the same key is pressed again, key
release is reported for it and when it is released, nothing is reported for
it again. To cure this weirdness set this item. It will cause the offending
reports to be converted into two immediate reports, one for the key press and
the other for the key release.

------------

THREAD_DRV	(Exclusive)

Tell the source how to support threads.

none		No driver; pretend impossibility to create threads.
SDL		Use SDL.
windows		Use Windows API.
pthread		Use POSIX pthread library

------------

THR_USE_CANCEL	(Boolean)

If set to "yes", it tells the pthread thread driver that threads should be
killed using the pthread_cancel() call. Set to "yes" if pthread_kill() is
broken on your platform (I seen that it may either do nothing or kill the
entire process instead of just the one thread specified). Generally this
should be set on platforms that provide the PTHREAD_CANCEL_ASYNCHRONOUS macro
in the pthreads headers.

------------

USE_RWLOCK	(Boolean)

When the THREAD_DRV config item is set to POSIX pthread, this boolean
config item select one of the two implementations of locks. When set to
"no", the mutexes are used to implement them while it is set to "yes",
rwlocks are used. The mutexes are faster but they may be nonportable as
the OSHS locks are allowed to be unlocked by any thread, not only the one
that owns it.

When the THREAD_DRV config item is set to something other, this boolean
configuration item will be ignored.

------------

TIGHT_MEMORY	(Boolean)

Set this if the underlying system imposes restrictive limits on the data
segment size preventing many programs from being successfully built. This
symbol causes the resulting library to use much less size for the static
variables. However it greatly reduces the usability of the library so it
should be unset for platforms which don't put such restrictive limits to
their executables.

------------

GETCWD		(Exclusive)

Set how to obtain the current working directory

posix		Use POSIX call getcwd from <unistd.h>
bcmsdos		Use getcwd from <dir.h> (from Borland C)

------------

DEF_DRIVES	(Exclusive)

Set the pattern of default drives that is installed when no configuration
is found.

none		Empty diskset (configuration is mandatory)
bcmsdos		Mirror MSDOS drives to OSHS (using Borland C)
tcmsdos		Mirror MSDOS drives to OSHS (using Turbo C++ 2.01)
mingw		Mirror Windows drives to OSHS (using MINGV method; only
		harddisks are mirrored to avoid delays)
unix		The root of the UNIX tree is mapped to U: with UNIX
		namesystem.

------------

INET_DRV	(Exclusive)

Select how to access the Internet.

none		No Internet access
linux		Use Linux method
windows		Use Windows API

------------

STDIO_EMUL	(Exclusive)

Select how to implement the STDIN, STDOUT and STDERR handles

direct		Use direct mapping

------------

SLEEP_DRV	(Exclusive)

Select how to implement the sleep() call

posix		Use POSIX nanosleep()
bcmsdos		Use delay() from Borland C
sleep		Use sleep() call
windows		Use Windows API
select          Use select()
poll		Use poll() from <poll.h>
syspoll		Use poll() from <sys/poll.h>
usleep		Use usleep() (where it returns void and thus cannot fail)
usleep2		Use usleep() (where it can fail, for example on SUSv2)
apollo		Use apollo driver (sleep etc are broken on apollo)

------------

EXEC_DRV	(Exclusive)

Select how to execute programs (the EXEC module implementation).

none		No execution driver
bcmsdos		Use Borland specific system calls
unix		Use UNIX method
mingw		Use Windows/MINGW method

------------

EXEC_PATH	(Boolean)

Determines whether the program execution drive uses the PATH enviroment
variable (or similar thing) when a program name without path is passed to
it.

------------

HUGE_MEMORY     (Boolean)

Set to True on platforms where memory blocks above 64K can be allocated to
enable huge memory blocks support.

------------

HEAP_CHECK      (Exclusive)

Select which memory heap checking facility to use.

none		No memory heap checking facility available
gmcheck		GNU mcheck() and company

------------

CODEBASE_DIR    (Exclusive)

Defines the default for the codebase OSHS directory. This directory is mapped
as Z: in OSHS and defines the location where the OSHS installed applications
and libraries will be placed. The default can be overriden by OSHSCODEBASEDIR
envitoment variable.

none		No default (OSHSCODEBASEDIR must be set)
msdos		"C:\OSHS"
linux		"$HOME/oshs" where $HOME is replaced with the value of the
		enviroment variable called HOME (the home directory of the
		user currently logged in).

------------

INTERNAL_DIR    (Exclusive)

Defines the default for the internal OSHS directory. This internal directory
is normally not visible from the OSHS enviroment and it contains the OSHS
configuration and filesystem masquerading directories. The default can be
overriden by OSHSINTERNALDIR enviroment variable.

none		No default (OSHSINTERNALDIR must be set)
msdos		"C:\OSHSINT"
linux		"$HOME/.oshs" where $HOME is replaced with the value of the
		enviroment variable called HOME (the home directory of the
		user currently logged in).
optional        Make the internal directory optional. To enable it the user
                must set OSHSINTERNALDIR. If OSHSINTERNALDIR is not set, the
                programs will not complain but actions which require this
                directory will fail (either return a failure error code or
                output an error message saying the OSHS internal directory is
                not enabled).

------------

COMPILER_DRV    (Exclusive)

Select the compiler driver to be used.

none            Do not support compilation
gnu             Use GNU toolchain with shared libraries and position
                independed code generated for them or static libraries if
                requested. The shared libraries are optional and not included
                on platforms that don't support them.

------------

EASY_DEVEL      (Boolean)

If set (and the compiler driver supports it), turnes on the "easy development
mode". In that mode the extensions used by the binaries in the OSHS system
will be the same as the extensions used for executables by the platform. Also
The compiler driver changes the content of some of the generated files to
resemble the platform closely enough to allow the executables to be executed
without any assistance of the SYSLIB library.

------------

GNU_LINUX	(Boolean)

Set to "yes" on GNU/Linux and "no" on anything else.

------------

ADDON_LIBS      (Text)

Add various other libraries that must be linked into the program into this
symbol.

------------

MEMCMP_OK       (Boolean)

According to the SunOS man page, memcmp returns indeterminate sign
depending on whether characters are signed or not. On such a platform
set this to "false" to switch on an alternate implementation without this
bug.

------------

ACCEPT_THREAD   (Boolean)

If set to yes, the accept() call will be executed in a thread. When the
handle is closed by another thread, the thread executing the accept() will
be killed.

------------

WINSOCK_HACK	(Boolean)

Windows has bug in winsock that causes second and later packet after last
send operation in a TCP stream to be received after a short delay. By
setting this to true you can enable "Windows Blatant Hack" which tries to
circumvent this problem by sending dumb data in the stream. Note that this
makes all networked OSHS software compiled under Windows incompatible with
non-OSHS software which does not know about the "Windows Blatant Hack".

------------

WINSOCK_SUPPORT	(Boolean)

This configuration item makes the SYSLIB compatible with WINSOCK_HACK enabled
network code. It turns autodetection of the WINSOCK_HACK streams and ignored
chars removal.

------------

LIB_SUPPORT	(Inclusive)

Determines which types of libraries are supported on a platform. The list
of possible library types shows what library models are supported.

shared		Shared libraries support, extension ".so", created by "gcc
		-shared".
static		Static libraries, extension ".a", created by "ar" and
		"ranlib".

------------

OPEN_ON_DIR_BAD	(Boolean)

The Open() command must return fseISDIR because this is how various utilities
on OSHS check for directory and because it helps speed the directory tree
traversal (try to open a file, error=fseISDIR means the file is a directory,
so dive down else error=0 means normal file and already opened and error!=0
means something went badly wrong; this avoids one stat() per filename and
since open() internally performs stat(), this leads to double performance;
when block devices etc need to be excluded, one can use fstat after
successfully opened, fstat is faster since it doesn't to search for file's
inode (its inode is already cached to allow the fsys server to update it)).

The older code relies on open() returning EISDIR when applied to a directory
but on some platforms this does not work so we need to enable a patch for
this and this is what this item does.

When this item is set to true, the code performs a stat() on the destination
file and returns fseISDIR when it detects that the filespec specifies a
directory and calls open() otherwise. When this item is false, the code
relies on open() returning EISDIR when fed with a directory specification.
The reason of having the old code behavior preserved is that I don't want
to do these stat()s when they are not necessary.

------------

LITTLE_ENDIAN   (Boolean)

Set to true on machines with little endian order (or "Intel endian") and to
false to big endian ones (or "Motorola endian" or "Apple endian"). There is
GPS check that determines the endiannes automatically so setting this value
in the user config file is not necessary. If the value is misset, random
crashes or misbehavior at various places may occur.

------------------------

The MAKEGEN program reads this list and the GPS platform scan report (in the
"gps/config.gen" file) and the platform scan tweaks file (from "config.usr"
file located in the same directory as the building script called "build") and
produces two files. The first is the platform definition header file called
and the second is the makefile.

- "config.usr": The platform scan tweaks file -

This file is located at the root of the source code file (at the same place
where the building script lives). The format is simple. Empty lines and
lines beginning with "#" are ignored. The remaining lines contain two words.
The first word is one of the configuration items defined above and the second
one is a value for it.

If the config item is an inclusive one, it may have multiple values specified
on its line and it may appear more than once in the file. Also values can be
removed from the inclusive config item by prepending them with "-" as in
"-somevalue".

- The platform definition header file -

The platform definition header file contains the defines that describe the
platform dependent features of the system and choose the proper ways how to
do things the SYSLIB is here for in a form digestable by a C compiler.
Each such symbol has the name with this shape:

  __<class>__<name>__<value>__

The <class> field is one character field that describes the class of the
symbol. The <name> field holds the name of the configuration item (or the
value of an exclusive configuration item) and <value> is used for some symbol
classes to distinguish between symbols belonging to the same <name>. Valid
symbol classes are the following:

  i  Inclusive configuration item symbol. The <name> is the name of the item
     and <value> is the value for the item. These symbols are boolean.
     "Inclusive" means that more than one symbols with the same <name> may be
     defined. Typically these symbols select sets of items to be taken into
     account during compilation.

  e  Exclusive configuration item symbol. The <name> is the name of the item
     and the <value> is omitted. There exist a list of value symbols that are
     valid for that particular feature symbol (the value symbols may be
     shared by multiple exclusive feature symbols). Each exclusive feature
     symbol must be fed by exactly one of these values otherwise compilation
     will fail with "Invalid value for XXX" error report.

  v  Configuration item value symbol. The <name> is the name of the value and
     <value> is omitted. These symbols define values for the exclusive
     configuration item symbols. They all must have distinct values and they
     must be defined only once in this file. These symbols are assigned to
     the exclusive configuration item symbols.

  b  Boolean configuration item symbol. The <name> is the name of the symbol
     and the <value> is omitted. These symbols are either undefined, 0 or 1.
     If it is undefined or is defined as 0, the corresponding item is
     considered to be "no" or unset. If it is defined to 1, the considered
     item is considered to be "yes" or set.

  t  Temporary symbol. The <name> is the name of the symbol and <value> is
     not used. These symbols are available for the system when complicated
     condition evaulation is required to choose the proper way how to do a
     platform-specific thing. A value passed from the outer vorld in a
     temporary symbol will never affect the compilation as these symbols must
     be always deleted before they are used (otherwise a compilation error
     at the makefile generation stage will occur).

Each platform configuration item has one or more corresponding "i", "e" or
"b" symbol. Various platforms use various symbols to select different parts
of the source code that are suitable for them.

The format of all symbols is enforced by the makefile generator and any
malformed symbol or any symbols referencing undefined configuration item
(not mentioned in this file) will cause a makefile generation error. This
check can be turned off by placing the string "__SPECIAL__" in front of the
condition which does not follow the rules but this should be used sparingly
and only when in extreme need (it is for example better to write a GPS check
that will evaulate the condition and assign the result to a boolean
configuration item symbol so it can be referenced by the file using
nonsprcial directive conditions).
