#ifndef __h__SYSLIB_DISKDATA__
#define __h__SYSLIB_DISKDATA__

#define seekSET 0
#define seekCUR 1
#define seekEND 2

typedef int32 (*PReadProc)(void *fd,void *Buf,int32 Size);
typedef int32 (*PWriteProc)(void *fd,const void *Buf,int32 Size);
typedef int16 (*PSeekProc)(void *fd,int32 Where,int16 How);
typedef int16 (*PTellProc)(void *fd,TTellReply *R);
typedef int16 (*PStatProc)(void *fd,TFileInfo *Info);
typedef int16 (*PDirReadProc)(void *dd,TDirItem *I);
typedef int16 (*PChangeAttr)(void *fd,uint16 NewAttr);

typedef struct {
  /*
    Handling procedures. These procedures are called for each of the file
    operations performed on the handle.
  */
  PReadProc Read;
  PWriteProc Write;
  PSeekProc Seek;
  PTellProc Tell;
  PStatProc Stat;
  PChangeAttr ChMod;
  PChangeAttr ChOwn;
  PChangeAttr ChGrp;
  TCloseProc *Truncate;
  /*
    Pointer to the channel information record. This pointer is required for
    the "classic file I/O" to be able to install post-message handlers.
  */
  void *Channel;
  /*
    When reading using a "classic file I/O", this attribute holds the size
    of the remaining data to be transferred to the reading process. The data
    will be sent in next message(s).
  */
  int32 RemainingSize;
  /*
    Used to communicate the packet size to the classic write handler. There
    is no packet size in the handlers since formely I assumed all commands
    will have constant sizes, so I must this variable-sized command solve
    using this not-so-clean technique.
  */
  int16 PacketSize;
} TGenericFileHandle;

typedef struct {
  /*
    The directory handle is a special kind of a file handle.
  */
  TGenericFileHandle H;
  /*
    Here is stored address to the directory reading procedure by the low
    level directory open calls.
  */
  PDirReadProc ReadDir;
  /*
    When set to true, the item names read by ReadDir() will be capitalized.
    This is used for MS-DOS based namesystem drives. In file handles this is
    unused and usually contains garbage.
  */
  bool MsDosProcessing;
} TGenericDirHandle;

void *CreateFileHandleData(uint16 Size);

typedef struct {
  TGenericFileHandle H;
  void *DevNode;
} TGenericDeviceHandle;

typedef struct {
  void *Data;
  THandler *Handler;
  TCloseProc *Close;
  TDestroyProc *Destroy;
  void *Channel;
} TOpenReport;

typedef char TFileSpec[MAX_FILESPEC_SIZE+1];

typedef struct {
  uint8 Type;
  uint8 FSType;
  TFileSpec Dir;
  TFileSpec Cwd;
} TEmulatedDisk;

#define dtEMPTY 0
#define dtDIRECT 1

#define fsMSDOS 0
#define fsWINLFN 1
#define fsUNIX 2

typedef struct {
  /*
    Structures about emulated OSHS drives. Each drive letter has its own
    structure here and the drive letters start from "@" (a drive letter
    reserved for the device filesystem).
  */
  TEmulatedDisk Drives[26];
  /*
    Drive letter for the current drive
  */
  char CurrentDrive;
  /*
    If a file specification conversion fails, the error code denoting the
    failure reason will be stored here.
  */
  int16 Result;
} TRootStatus;

#endif
