#include "defs.gh"

/*
  File I/O module (POSIX 1003.1-1996 version)
*/

#if __b__UNISTD__
#include <unistd.h>
#endif
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#if __b__NEED_IO__
#include <io.h>
#endif
#undef __t__USE_BORLAND_MKDIR__
#if __e__DIR_MANIP__ == __v__bcmsdos__
#define __t__USE_BORLAND_MKDIR__ 1
#elif __e__DIR_MANIP__ == __v__tcmsdos__
#define __t__USE_BORLAND_MKDIR__ 1
#else
#define __t__USE_BORLAND_MKDIR__ 0
#endif
#if __t__USE_BORLAND_MKDIR__
#include <dir.h>
#endif
#include <errno.h>

#include "types.h"
#include "traps.h"
#include "errno.h"
#include "memory.h"
#include "strings.h"
#include "kernel.h"
#include "kernmsg.h"
#include "pdbase.h"
#include "pdfsys.h"
#include "pdfileio.h"
#include "disklow.h"
#include "diskdata.h"
#include "disklett.h"
#include "diskdumb.h"
#include "diskspec.h"
#include "diskmain.h"

#define __INTERFACE__
#include "diskdir.h"
#undef __INTERFACE__

#include "typesnd.h"

#define MAX_HOST_FILESPEC_SIZE 256

typedef struct {
  TRootStatus *Root;
  TFileSpec Buf;
  uint16 Size;
  bool Gathering;
  bool Bad;
} TSpecialHandleInfo;

typedef struct {
  TGenericFileHandle H;
  int16 ID;
  TSpecialHandleInfo *I;
  char FileName[MAX_HOST_FILESPEC_SIZE+1];
} TDirectHandle;

static int16 DirectLSeek(void *fd,int32 Where,int16 How)
{
  TDirectHandle *Data;

  Data=fd;
  return(LowLevelSeek(Data->ID,Where,How));
}

static int16 DirectTell(void *fd,TTellReply *R)
{
  TDirectHandle *Data;

  Data=fd;
  R->Page=0;
  R->Position=LowLevelTell(Data->ID);
  return(TranslateCommonErrors(GetError(R->Position==(uint32)-1)));
}

static int32 DirectRead(void *fd,void *Buf,int32 Size)
{
  int32 Result;
  TDirectHandle *Data;

  Data=fd;
  UnlockChannels();
  Result=LowLevelRead(Data->ID,Buf,Size);
  LockChannels();
  return(Result);
}

static int32 DirectWrite(void *fd,const void *Buf,int32 Size)
{
  int32 Result;
  TDirectHandle *Data;

  Data=fd;
  UnlockChannels();
  Result=LowLevelWrite(Data->ID,Buf,Size);
  LockChannels();
  return(Result);
}

static int32 SpecialWrite(void *fd,const void *Buf,int32 Size)
{
  TDirectHandle *Data;
  TSpecialHandleInfo *I;
  TFileSpec SpecBuf;
  bool FlushIt;
  const char *Ptr;
  const char *Block;
  char Ch;
  void *Result;
  int32 Written;

  Data=fd;
  I=Data->I;
  Ptr=Buf;
  Block=Buf;
  Written=0;
  while (Size>0) {
    Ch=*Ptr;
    Size--;
    FlushIt=(Size==0);
    if (I->Gathering) {
      if (Ch=='\377') Ch='\0';
      Written++;
      if (I->Size<sizeof(I->Buf)) {
        I->Buf[I->Size++]=Ch;
      } else {
        I->Bad=true;
      }
      if (Ch=='\0') {
        if (!I->Bad) {
          Result=ConvertFileSpecEx(I->Root,I->Buf,SpecBuf);
          I->Bad=(Result==NULL);
        }
        if (I->Bad) {
          DirectWrite(fd,"[bad]",5);
        } else {
          DirectWrite(fd,SpecBuf,StrLen(SpecBuf));
        }
        Block=Ptr;
        I->Gathering=false;
      }
    } else {
      if (Ch=='\377') {
        I->Gathering=true;
        I->Bad=false;
        I->Size=0;
        FlushIt=true;
      }
    }
    if (FlushIt && Block!=Ptr) {
      Written+=DirectWrite(fd,Block,Ptr-Block);
    }
    Ptr++;
  }
  return(Written);
}

static int16 DirectFStat(void *fd,TFileInfo *Info)
{
  int16 Result;
  TDirectHandle *Data;

  Data=fd;
  UnlockChannels();
  Result=LowLevelFStat(Data->ID,Info);
  LockChannels();
  return(Result);
}

#if __e__CHMOD__ == __v__secure__
int16 DoDirectChMod(int16 fd,uint16 NewAttr)
#else
int16 DoDirectChMod(const char *FileSpec,uint16 NewAttr)
#endif
{
  int16 NewMod;
  int16 Result;

  /*
    Check for undefined situation: setting reserved bits.
  */
  if (NewAttr&~(
    fmUSRREAD | fmUSRWRITE | fmUSREXEC |
    fmGRPREAD | fmGRPWRITE | fmGRPEXEC |
    fmOTHREAD | fmOTHWRITE | fmOTHEXEC |
    fmSETUID | fmSETGID | fmSTICKY
  )) {
    TrapUndefined("Attempt to ChMod reserved bits");
  }
  /*
    Convert the provided file mode to the file mode expected by POSIX.
  */
  NewMod=0;
#if __e__STATINFO__ == __v__unix__
  if (NewAttr&fmUSRREAD) NewMod+=S_IRUSR;
  if (NewAttr&fmUSRWRITE) NewMod+=S_IWUSR;
  if (NewAttr&fmUSREXEC) NewMod+=S_IXUSR;
  if (NewAttr&fmGRPREAD) NewMod+=S_IRGRP;
  if (NewAttr&fmGRPWRITE) NewMod+=S_IWGRP;
  if (NewAttr&fmGRPEXEC) NewMod+=S_IXGRP;
  if (NewAttr&fmOTHREAD) NewMod+=S_IROTH;
  if (NewAttr&fmOTHWRITE) NewMod+=S_IWOTH;
  if (NewAttr&fmOTHEXEC) NewMod+=S_IXOTH;
  if (NewAttr&fmSETUID) NewMod+=S_ISUID;
  if (NewAttr&fmSETGID) NewMod+=S_ISGID;
  if (NewAttr&fmSTICKY) NewMod+=S_ISVTX;
#elif __e__STATINFO__ == __v__bcmsdos__
  if (NewAttr&fmUSRREAD) NewMod+=S_IREAD;
  if (NewAttr&fmUSRWRITE) NewMod+=S_IWRITE;
  if (NewAttr&(
    fmUSREXEC|
    fmGRPREAD|fmGRPWRITE|fmGRPEXEC|
    fmOTHREAD|fmOTHWRITE|fmOTHEXEC|
    fmSETUID|fmSETGID|fmSTICKY
  )) {
    return(beNOTSUP);
  }
#endif
  /*
    Attempt to change the file permissions.
  */
  PrepareSystemCall();
#if __e__CHMOD__ == __v__secure__
  Result=fchmod(fd,NewMod);
#elif __e__CHMOD__ == __v__posix__
  Result=chmod(FileSpec,NewMod);
#endif
  if (Result==-1) Result=TranslateCommonErrors(GetError(true));
  return(Result);
}

int16 DirectChMod(void *fd,uint16 NewAttr)
{
  TDirectHandle *H;

  H=fd;
#if __e__CHMOD__ == __v__secure__
  return(DoDirectChMod(H->ID,NewAttr));
#else
  return(DoDirectChMod(H->FileName,NewAttr));
#endif
}

#if __e__CHOWN__ == __v__secure__
int16 DoDirectChOwn(int16 fd,uint16 NewOwner)
#else
int16 DoDirectChOwn(const char *FileSpec,uint16 NewOwner)
#endif
{
  int16 Result;

#if __e__CHOWN__ == __v__root_only__
  Result=(NewOwner==0)?0:fseNOTSUP;
#else
  PrepareSystemCall();
#if __e__CHOWN__ == __v__posix__
  Result=chown(FileSpec,NewOwner,-1);
#elif __e__CHOWN__ == __v__secure__
  Result=fchown(fd,NewOwner,-1);
#endif
  if (Result==-1) Result=TranslateCommonErrors(GetError(true));
#endif
  return(Result);
}

int16 DirectChOwn(void *fd,uint16 NewOwner)
{
  TDirectHandle *H;

  H=fd;
#if __e__CHOWN__ == __v__secure__
  return(DoDirectChOwn(H->ID,NewOwner));
#else
  return(DoDirectChOwn(H->FileName,NewOwner));
#endif
}

#if __e__CHOWN__ == __v__secure__
int16 DoDirectChGrp(int16 fd,uint16 NewGroup)
#else
int16 DoDirectChGrp(const char *FileSpec,uint16 NewGroup)
#endif
{
#if __e__CHOWN__ == __v__root_only__
  return(DoDirectChOwn(FileSpec,NewGroup));
#else
  int16 Result;

  PrepareSystemCall();
#if __e__CHOWN__ == __v__posix__
  Result=chown(FileSpec,-1,NewGroup);
#elif __e__CHOWN__ == __v__secure__
  Result=fchown(fd,-1,NewGroup);
#endif
  if (Result==-1) Result=TranslateCommonErrors(GetError(true));
  return(Result);
#endif
}

int16 DirectChGrp(void *fd,uint16 NewGroup)
{
  TDirectHandle *H;

  H=fd;
#if __e__CHOWN__ == __v__secure__
  return(DoDirectChGrp(H->ID,NewGroup));
#else
  return(DoDirectChGrp(H->FileName,NewGroup));
#endif
}

static int16 DirectClose(void *fd)
{
  int16 Result;
  TDirectHandle *Data;

  Data=fd;
  UnlockChannels();
  PrepareSystemCall();
  Result=LowLevelClose(Data->ID);
  LockChannels();
  return(Result);
}

static void FillPipeHandlers(TGenericFileHandle *FD)
{
  FD->Read=DirectRead;
  FD->Write=DirectWrite;
}

void PipeToChannel(int16 fd,TOpenReport *R)
{
  TDirectHandle *Data;

  Data=CreateFileHandleData(sizeof(*Data));
  if (Data!=NULL) {
    Data->ID=fd;
    FillPipeHandlers((void *)Data);
  }
  R->Close=DirectClose;
  R->Destroy=DumbDestroy;
  R->Data=Data;
}

void SetupSpecialWriteHandling(void *FD,TRootStatus *Root)
{
  TDirectHandle *Data;

  Data=FD;
  Data->I=KMEM__malloc(sizeof(*Data->I));
  if (Data->I==NULL) Fatal("Unable to create root buffer");
  Data->H.Write=SpecialWrite;
  Data->I->Root=Root;
  Data->I->Size=0;
  Data->I->Gathering=false;
}

static int16 Truncate(void *fd)
{
  TDirectHandle *Data;
  int16 Result;

  Data=fd;
  UnlockChannels();
  Result=LowLevelTruncate(Data->ID);
  LockChannels();
  return(Result);
}

int16 DirectOpen(const char *FileSpec,int16 Mode,TOpenReport *R)
{
  int16 Result;
  TDirectHandle *Data;
  int16 Len;

  /*
    Create and fill in a file descriptor structure. 
  */
  Len=StrLen(FileSpec);
  Data=CreateFileHandleData(sizeof(*Data)-(MAX_HOST_FILESPEC_SIZE-Len));
  if (Data==NULL) return(beNOMEM);
  MemCpy(Data->FileName,FileSpec,Len+1);
  FillPipeHandlers((void *)Data);
  Data->H.Seek=DirectLSeek;
  Data->H.Tell=DirectTell;
  Data->H.Stat=DirectFStat;
  Data->H.Truncate=Truncate;
  Data->H.ChMod=DirectChMod;
  Data->H.ChOwn=DirectChOwn;
  Data->H.ChGrp=DirectChGrp;
  R->Close=DirectClose;
  R->Destroy=DumbDestroy;
  /*
    Open the file and fill the private data area with informations provided
    by the underlying OS.
  */
  Data->ID=LowLevelOpen(FileSpec,Mode);
  /*
    Return the resulting file descriptor.
  */
  if (Data->ID<0) {
    Result=Data->ID;
    KMEM__free(Data);
  } else {
    Result=0;
  }
  R->Data=Data;
  return(Result);
}

#if __b__TIGHT_MEMORY__
#define STAGE_SIZE 128
#else
#define STAGE_SIZE 1024
#endif

typedef struct {
  TGenericDirHandle H;
  TFileSpec DirSpec;
  void *Dir;
  uint32 FirstID;
} TOpenDir;

typedef struct {
  uint32 Next;
  char *Name;
  TOpenDir *Dir;
} TIdentificatorInfo;

static uint32 IDCount;
static uint32 FirstFreeID;
static TIdentificatorInfo *Idents[STAGE_SIZE];

static TIdentificatorInfo *PointID(uint32 Ident)
{
  TIdentificatorInfo *Stage;
  uint32 StageNum;

  /*
    Get and check the number of the first stage. If it is too large, return
    NULL.
  */
  StageNum=Ident / STAGE_SIZE;
  if (StageNum>=STAGE_SIZE) return(NULL);
  /*
    Get pointer to the first stage in which the identifier resides. These
    pointers are stored in the "Ident" table, which is in fact second stage
    identifier table.
  */
  Stage=Idents[StageNum];
  /*
    If there is no first stage, return failure.
  */
  if (Stage==NULL) return(NULL);
  /*
    The first stage contains the identificator information structures
    themselves. Select the apropiate one and return pointer to it.
  */
  return(Stage + (Ident % STAGE_SIZE));
}

static uint32 AllocateIdent(void)
{
  uint32 Ident;
  TIdentificatorInfo *ID;
  TIdentificatorInfo *Stage;
  uint16 StageNum;

  /*
    Fixup the pointer to the first free identifier. At program startup both
    variables will contain zero. Since zero is a valid identifier, we detect
    this situation by examining IDCount for zero and if so, we initialize
    that variable.
  */
  if (IDCount==0) FirstFreeID=-1;
  /*
    Check for free identifiers.
  */
  if (FirstFreeID!=-1) {
    /*
      There are free identifiers. Allocate one.
    */
    Ident=FirstFreeID;
    ID=PointID(Ident);
    assert(ID!=NULL);
    FirstFreeID=ID->Next;
  } else {
    /*
      No free identifiers found. Create one identifier.
    */
    Ident=IDCount;
    /*
      If the identifier is a first identifier in a stage, the stage does not
      exist yet, so create it. Return failure if the stage cannot be created.
    */
    if (Ident % STAGE_SIZE == 0) {
      StageNum=Ident / STAGE_SIZE;
      Stage=NULL;
      if (StageNum<STAGE_SIZE) Stage=KMEM__malloc(sizeof(*Stage)*STAGE_SIZE);
      if (Stage==NULL) return(beNOMEM);
      Idents[StageNum]=Stage;
    }
    IDCount++;
  }
  /*
    Return the allocated identifier.
  */
  return(Ident);
}

static void FreeIdent(uint16 Ident)
{
  TIdentificatorInfo *ID;

  ID=PointID(Ident);
  assert(ID!=NULL);
  assert(ID->Dir!=NULL);
  ID->Dir=NULL;
  ID->Next=FirstFreeID;
  FirstFreeID=Ident;
}

static int16 DirectReadItem(void *dd,TDirItem *I)
{
  TOpenDir *Dir;
  int16 NameLen;
  uint32 Ident;
  TIdentificatorInfo *ID;
  char *Name;
  bool Invalid;
  int32 Result;

  /*
    Convert the integer descriptor to pointer to the information structure.
    If the conversion fails, return with error.
  */
  Dir=dd;
  /*
    We need to read the items in a loop, because some of them need to be
    filtered out to make the call compatible with OSHS.
  */
  do {
    /*
      Try to read the item from the directory.
    */
    Result=LowLevelReadItem(Dir->Dir,I);
    if (Result<=0) return(Result);
    /*
      Check if the item is valid.
    */
    Invalid=false;
#if __b__CASE_SENSE__
    {
      char *Name;
      char Ch;

      Name=I->Name;
      do {
        Ch=*(Name++);
	if (Ch>='A' && Ch<='Z') break;
      } while (Ch);
      Invalid=Invalid || Ch;
    }
#endif
  } while (Invalid);
  /*
    Determine the length of the name of the file.
  */
  NameLen=StrLen(I->Name);
  /*
    Create a copy of the file name. If the copy could not be created, return
    a "not enough memory" failure status.
  */
  Name=KMEM__malloc(NameLen+1);
  if (Name==NULL) return(beNOMEM);
  MemCpy(Name,I->Name,NameLen+1);
  /*
    Allocate an identifier for the file. If no identificator could be
    allocated, return failure.
  */
  Ident=AllocateIdent();
  if (Ident==-1) {
    KMEM__free(Name);
    return(beNOMEM);
  }
  /*
    Fill the informational structure about the identifier.
  */
  ID=PointID(Ident);
  assert(ID!=NULL);
  ID->Name=Name;
  ID->Dir=Dir;
  /*
    Link the identifier into list of identifiers of the directory.
  */
  ID->Next=Dir->FirstID;
  Dir->FirstID=Ident;
  /*
    Set the identifier in the returned buffer.
  */
  I->ID=Ident+(dtDIRECT<<24);
  /*
    Return the success flag
  */
  return(sizeof(*I));
}

static int16 DirectCloseDir(void *dd)
{
  TOpenDir *Dir;
  int16 Result;

  /*
    Convert the integer descriptor to pointer to the information structure.
    If the conversion fails, return with error.
  */
  Dir=dd;
  /*
    Attempt to close the directory associated with the descriptor. Keep the
    result in a variable but don't return it yet.
  */
  Result=LowLevelCloseDir(Dir->Dir);
  /*
    Return the kept result flag.
  */
  return(Result);
}

static void DirectDestroyDir(void *dd)
{
  TOpenDir *Dir;
  uint32 Ident;
  TIdentificatorInfo *ID;

  Dir=dd;
  /*
    Free the identifiers associated with the directory.
  */
  for (;;) {
    Ident=Dir->FirstID;
    if (Ident==-1) break;
    ID=PointID(Ident);
    assert(ID!=NULL);
    KMEM__free(ID->Name);
    Dir->FirstID=ID->Next;
    FreeIdent(Ident);
  }
}

int16 DirectDirChMod(void *fd,uint16 NewAttr)
{
  TOpenDir *H;

  H=fd;
#if __e__CHOWN__ == __v__secure__
  return(DoDirectChMod(GetOpenDirID(H->Dir),NewAttr));
#else
  return(DoDirectChMod(H->DirSpec,NewAttr));
#endif
}

int16 DirectDirChOwn(void *fd,uint16 NewOwner)
{
  TOpenDir *H;

  H=fd;
#if __e__CHOWN__ == __v__secure__
  return(DoDirectChOwn(GetOpenDirID(H->Dir),NewOwner));
#else
  return(DoDirectChOwn(H->DirSpec,NewOwner));
#endif
}

int16 DirectDirChGrp(void *fd,uint16 NewGroup)
{
  TOpenDir *H;

  H=fd;
#if __e__CHOWN__ == __v__secure__
  return(DoDirectChGrp(GetOpenDirID(H->Dir),NewGroup));
#else
  return(DoDirectChGrp(H->DirSpec,NewGroup));
#endif
}

int16 DirectOpenDir(const char *DirSpec,TOpenReport *R)
{
  int16 Len;
  TOpenDir *Dir;
  void *DStream;
  int16 Result;

  /*
    Check the length of the directory specification.
  */
  Len=StrLen(DirSpec);
  if (Len>=MAX_FILESPEC_SIZE) return(fseNAMELEN);
  /*
    Fill in the file descriptor structure obtained from the caller
  */
  Dir=CreateFileHandleData(sizeof(*Dir));
  if (Dir==NULL) return(beNOMEM);
  Dir->H.ReadDir=DirectReadItem;
  R->Close=DirectCloseDir;
  R->Destroy=DirectDestroyDir;
  Dir->H.H.ChMod=DirectDirChMod;
  Dir->H.H.ChOwn=DirectDirChOwn;
  Dir->H.H.ChGrp=DirectDirChGrp;
  /*
    Create a copy of the directory specification for the open directory slot.
  */
  MemCpy(Dir->DirSpec,DirSpec,Len+1);
  /*
    Open the directory and keep the pointer around.
  */
  Result=LowLevelOpenDir(DirSpec,(void **)(&DStream));
  if (Result<0) {
    KMEM__free(Dir);
    return(Result);
  }
  /*
    Store the data in the open directory slot.
  */
  Dir->Dir=DStream;
  Dir->FirstID=-1;
  /*
    Return the number.
  */
  R->Data=Dir;
  return(0);
}

#if __b__TIGHT_MEMORY__
#define MAX_BUF_SIZE 1024
#else
#define MAX_BUF_SIZE 32768
#endif

static char FileNameBuf[MAX_BUF_SIZE];

static int16 PrepareFileSpec(uint32 Number)
{
  char *Ptr;
  TIdentificatorInfo *ID;
  bool Invalid;

  /*
    Get the pointer to the data about the identifier. If the identifier is
    invalid, activate the undefined situation trap and then return failure
    status.
  */
  ID=PointID(Number);
  Invalid=(ID==NULL);
  if (!Invalid) {
    if (ID->Dir==NULL) Invalid=true;
  }
  if (Invalid) {
    TrapUndefined("ReadFileInfo() with invalid file ID");
    return(beINVAL);
  }
  /*
    Combine the directory specification with the file specification. If the
    combination is too long, return error eNAMETOOLONG.
  */
  Ptr=StrNCpy(FileNameBuf,ID->Dir->DirSpec,MAX_BUF_SIZE);
  Ptr--;
  if ((*Ptr)!='/') {
    Ptr++;
    (*Ptr)='/';
  }
  Ptr=StrNCpy(Ptr+1,ID->Name,MAX_BUF_SIZE-(Ptr-FileNameBuf+1));
  if (Ptr-FileNameBuf>=MAX_BUF_SIZE) return(fseNAMELEN);
  /*
    The filename was successfully prepared, so return success
  */
  return(0);
}

int16 DirectReadFileInfo(uint32 Number,TFileInfo *Info)
{
  int16 Result;

  /*
    Prepare the file specification. Return failure status if failed.
  */
  Result=PrepareFileSpec(Number);
  if (Result!=0) return(Result);
  /*
    Stat the specified file and return the result.
  */
  Result=LowLevelStat(FileNameBuf,Info);
  return(Result);
}

int16 DirectMkDir(const char *DirSpec)
{
  int16 Result;

  PrepareSystemCall();
#if __e__DIR_MANIP__ == __v__posix__
  Result=mkdir(DirSpec,S_IRWXU+S_IRWXG+S_IRWXO);
#elif __t__USE_BORLAND_MKDIR__
  Result=mkdir(DirSpec);
#endif
  if (Result==-1) Result=TranslateCommonErrors(GetError(true));
  return(Result);
}

int16 DirectRmDir(const char *DirSpec)
{
  int16 Result;

  PrepareSystemCall();
  Result=rmdir(DirSpec);
  if (Result==-1) Result=TranslateCommonErrors(GetError(true));
  return(Result);
}

int16 DirectRemove(const char *FileSpec)
{
  int16 Result;

  PrepareSystemCall();
  Result=unlink(FileSpec);
  if (Result==-1) Result=TranslateCommonErrors(GetError(true));
  return(Result);
}

int16 DirectRename(const char *OldName,const char *NewName)
{
  int16 Result;

  PrepareSystemCall();
  Result=rename(OldName,NewName);
  if (Result==-1) Result=TranslateCommonErrors(GetError(true));
  return(Result);
}
