#include "defs.gh"

#if __e__GETCWD__ == __v__posix__
#include <unistd.h>
#elif __e__GETCWD__ == __v__bcmsdos__
#include <dos.h>
#include <dir.h>
#endif
#if __e__DEF_DRIVES__ == __v__bcmsdos__
#include <direct.h>
#endif
#if __e__DEF_DRIVES__ == __v__mingw__
#include <direct.h>
#include <windows.h>
#endif

#include "types.h"
#include "conio.h"
#include "process.h"
#include "errno.h"
#include "strings.h"
#include "memory.h"
#include "traps.h"
#include "env.h"
#include "kernel.h"
#include "pdfsys.h"
#include "kernmsg.h"
#include "pdfileio.h"
#include "diskdata.h"
#include "cfgfile.h"

#define __INTERFACE__
#include "disklett.h"
#undef __INTERFACE__

#include "typesnd.h"

#if __e__FSPEC_TYPE__ == __v__unix__
#define FSPEC_DELIMITER '/'
#elif __e__FSPEC_TYPE__ == __v__msdos__
#define FSPEC_DELIMITER '\\'
#endif

int16 Host2OSHS(char *Buf,const char *FileSpec,int16 Offset)
{
  char Ch;

  assert(Offset>=0);
  Buf+=Offset;
  do {
    if (Offset>=MAX_FILESPEC_SIZE) return(fseNAMELEN);
    Offset++;
    Ch=*(FileSpec++);
    if (Ch==FSPEC_DELIMITER) Ch='/';
    *(Buf++)=Ch;
  } while (Ch);
  return(Offset);
}

int16 OSHS2Host(char *Buf,const char *FileSpec,int16 Offset)
{
  char Ch;

  assert(Offset>=0);
  Buf+=Offset;
  do {
    if (Offset>=MAX_FILESPEC_SIZE) return(fseNAMELEN);
    Offset++;
    Ch=*(FileSpec++);
    if (Ch=='/') Ch=FSPEC_DELIMITER;
    *(Buf++)=Ch;
  } while (Ch);
  return(Offset);
}

static bool TranslateSpec(const char *DiskSpec,TEmulatedDisk *Disk)
{
  char FSType;
  char *TheEnd;
  uint16 Len;

  Disk->Type=dtDIRECT;
  FSType=DiskSpec[0];
  switch (FSType) {
    case 'M': Disk->FSType=fsMSDOS;break;
    case 'W': Disk->FSType=fsWINLFN;break;
    case 'U': Disk->FSType=fsUNIX;break;
    default: return(false);
  }
  if (DiskSpec[1]!=',') return(false);
  Disk->Cwd[0]='\0';
  TheEnd=StrNCpy(Disk->Dir,DiskSpec+2,MAX_FILESPEC_SIZE);
  Len=TheEnd-Disk->Dir;
  if (Len==0 || Len>=MAX_FILESPEC_SIZE) return(false);
#if __e__FSPEC_TYPE__ == __v__unix__
  if (Disk->Dir[0]!='/') return(false);
#elif __e__FSPEC_TYPE__ == __v__msdos__
  if (Disk->Dir[1]!=':' || Disk->Dir[2]!='\\' ) return(false);
#endif
  return(*(TheEnd-1)==FSPEC_DELIMITER);
}

#if __e__DEF_DRIVES__ == __v__mingw__
static uint16 CurDrive=0;

void _dos_getdrive(uint16 *DriveP)
{
  *DriveP=CurDrive;
}

void _dos_setdrive(uint16 Drive,uint16 *P)
{
  char DrivePath[4];
  UINT DriveType;
  if (Drive<1 || Drive>26) return;
  DrivePath[0]=Drive-1+'A';
  DrivePath[1]=':';
  DrivePath[2]='\\';
  DrivePath[3]='\0';
  DriveType=GetDriveType(DrivePath);
  if (DriveType!=1) CurDrive=Drive;
}
#elif __e__DEF_DRIVES__ == __v__tcmsdos__
void _dos_getdrive(uint16 *DriveP)
{
  union REGS inregs;
  union REGS outregs;

  inregs.h.ah=0x19;
  int86(0x21,&inregs,&outregs);
  *DriveP=outregs.h.al+1;
}

void _dos_setdrive(uint16 Drive,uint16 *P)
{
  union REGS inregs;
  union REGS outregs;

  inregs.h.ah=0x0e;
  inregs.h.dl=Drive-1;
  int86(0x21,&inregs,&outregs);
}
#endif

#if __e__DEF_DRIVES__ != __v__none__
static void PlaceDefaultDrives(TRootStatus *S)
{
#undef __t__DOS_DRIVES__
#undef __t__WINDOWS_DRIVES__
#define __t__WINDOWS_DRIVES__ 0
#if __e__DEF_DRIVES__ == __v__bcmsdos__
#define __t__DOS_DRIVES__ 1
#elif __e__DEF_DRIVES__ == __v__tcmsdos__
#define __t__DOS_DRIVES__ 1
#elif __e__DEF_DRIVES__ == __v__mingw__
#define __t__DOS_DRIVES__ 1
#undef __t__WINDOWS_DRIVES__
#define __t__WINDOWS_DRIVES__ 1
#else
#define __t__DOS_DRIVES__ 0
#endif
#if __t__DOS_DRIVES__
  uint16 CurDrive;
  uint16 Drive;
  int16 i;
  TEmulatedDisk *Disk;
  char Cwd[MAX_FILESPEC_SIZE];
  bool Valid;

  _dos_getdrive(&CurDrive);
  Disk=S->Drives;
  Disk--;
  for (i=1;i<27;i++) {
    Disk++;
    _dos_setdrive(i,&Drive);
    _dos_getdrive(&Drive);
    if (Drive!=i) continue;
    if (Disk->Type==dtEMPTY) {
      Cwd[0]=i+'A'-1;
      Cwd[1]=':';
      Cwd[2]='\\';
      Cwd[3]='\0';
      Valid=true;
#if __e__DEF_DRIVES__ != __v__tcmsdos__
      if (i>2) Valid=_getdcwd(i,Cwd,MAX_FILESPEC_SIZE)!=NULL;
#else
      if (i>2) Valid=getcwd(Cwd,MAX_FILESPEC_SIZE)!=NULL;
#endif
      if (Valid) {
        Disk->Type=dtDIRECT;
#if __t__WINDOWS_DRIVES__
        Disk->FSType=fsWINLFN;
#else
        Disk->FSType=fsMSDOS;
#endif
        if (Host2OSHS(Disk->Cwd,Cwd+3,0)<0) {
	  Fatal("Current working directory spec of platform drive too long");
	}
	Cwd[3]='\0';
        StrCpy(Disk->Dir,Cwd);
      }
    }
  }
  _dos_setdrive(CurDrive,&Drive);
#endif
#undef __t__DOS_DRIVES__
#if __e__DEF_DRIVES__ == __v__unix__
  TEmulatedDisk *Disk;
  Disk=S->Drives+20;
  Disk->Type=dtDIRECT;
  Disk->FSType=fsUNIX;
  StrCpy(Disk->Dir,"/");
  Disk->Cwd[0]='\0';
#endif
}
#endif

static void SetupCodeBaseDisk(TRootStatus *S)
{
  TEmulatedDisk *Disk;

  /*
    Get access to Z:
  */
  Disk=S->Drives+25;
  /*
    Setup the drive
  */
  Disk->Type=dtDIRECT;
  Disk->FSType=fsMSDOS;
  StrCpy(Disk->Dir,CodeBaseDir);
  Disk->Cwd[0]='\0';
}

void InitDisks(TRootStatus *S)
{
  int16 i;
  char MsgBuf[80];
  char *DiskName;
  TEmulatedDisk *Disk;
  const char *DiskSpec;
  bool Valid;
  char Cwd[MAX_FILESPEC_SIZE];
  char Tmp;
  int16 Result;
  uint16 Len;

  /*
    Get the current working directory before it gets corrupted by the drive
    letter detection code. On Windows changing drivers will ocassionally 
    chdir to the root of the current drive.
  */
  if (getcwd(Cwd,MAX_FILESPEC_SIZE)==NULL) {
    Fatal("Error getting current working directory");
  }
  /*
    Initialize the variable used to traverse through the disks. Formely here
    was the device filesystem, but I replaced it with the
  */
  Disk=S->Drives;
  /*
    Search for OSHSDISKx enviroment variables
  */
  DiskName=StrCpy(MsgBuf,"Invalid value of enviroment variable ");
  StrCpy(DiskName,"OSHSDISKA");
  for (i=1;i<27;i++) {
    Disk->Type=dtEMPTY;
    DiskSpec=GetEnv(DiskName);
    Valid=false;
    if (DiskSpec!=NULL) {
      Valid=TranslateSpec(DiskSpec,Disk);
      if (!Valid) Fatal(MsgBuf);
    }
    DiskName[8]++;
    Disk++;
  }
  /*
    Setup the codebase disk Z:
  */
  SetupCodeBaseDisk(S);
  /*
    Place the default image of drives. The drives that were already
    configured are skipped.
  */
#if __e__DEF_DRIVES__ != __v__none__
  PlaceDefaultDrives(S);
#endif
  /*
    Determine the current drive. To obtain this information the system needs
    to obtain the current (platform dependent) directory and to search for
    the drive which mapped directory is a prefix of the current working
    directory. If no such drive is found or the current working directory
    cannot be determined, the program is aborted using the Fatal() call.
  */
  S->CurrentDrive='\0';
  Disk=S->Drives;
  for (i=1;i<27;i++) {
    if (Disk->Type==dtDIRECT) {
      Len=StrLen(Disk->Dir);
      Tmp=Cwd[Len];
      Cwd[Len]='\0';
      Result=StrCmp(Disk->Dir,Cwd);
      Cwd[Len]=Tmp;
      if (Result==0) {
        S->CurrentDrive=i+'A'-1;
        if (Host2OSHS(Disk->Cwd,Cwd+Len,0)<0) {
          Fatal("Platform's current working directory spec too long");
        }
        break;
      }
    }
    Disk++;
  }
  if (S->CurrentDrive=='\0') {
    Fatal("Current working directory not accessible from OSHS enviroment");
  }
}

TEmulatedDisk *GetDrive(TRootStatus *S,char DriveLetter)
{
  DriveLetter=UpCase(DriveLetter);
  if (DriveLetter<'A' || DriveLetter>'Z') return(NULL);
  return(S->Drives+(DriveLetter-'A'));
}
