#include "defs.gh"

/*
  Low level file I/O module. This is used by the file I/O api.
*/

#if __b__UNISTD__
#include <unistd.h>
#endif
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#if __e__DIR_MANIP__ != __v__tcmsdos__
#include <dirent.h>
#endif
#if __b__NEED_IO__
#include <io.h>
#endif
#undef __t__INCLUDE_DIR_H__
#if __e__DIR_MANIP__ == __v__bcmsdos__
#define __t__INCLUDE_DIR_H__ 1
#elif __e__DIR_MANIP__ == __v__tcmsdos__
#include <dos.h>
#define __t__INCLUDE_DIR_H__ 1
#else
#define __t__INCLUDE_DIR_H__ 0
#endif
#if __t__INCLUDE_DIR_H__
#include <dir.h>
#endif
#include <errno.h>

#include "types.h"
#include "traps.h"
#include "errno.h"
#include "memory.h"
#include "strings.h"
#include "kernel.h"
#include "kernmsg.h"
#include "pdbase.h"
#include "pdfsys.h"
#include "pdfileio.h"
#include "diskdata.h"
#include "disklett.h"
#include "diskdumb.h"

#define __INTERFACE__
#include "disklow.h"
#undef __INTERFACE__

#include "typesnd.h"

#define oeACTION (3<<5)
#define onACTION (1<<4)
#define oMODE 3

int16 TranslateCommonErrors(int16 ErrorCode)
{
  switch (ErrorCode) {
    case 0: return(0);
    case EEXIST: return(fseEXIST);
    case EACCES: return(fseACCESS);
    case ENOENT: return(fseNOENT);
    case ENOMEM: return(beNOMEM);
#if __i__ERRNO__posix__
    case ETXTBSY: return(fsePRGUSE);
    case ELOOP: return(fseLOOP);
#endif
#if __i__ERRNO__unix__
    case EISDIR: return(fseISDIR);
    case EPERM: return(fseNOTSUP);
    case ENOSPC: return(fseNOSPC);
    case EIO: return(beIO);
    case EFBIG: return(fseFILELEN);
    case ESPIPE: return(fseNOSEEK);
    case ENAMETOOLONG: return(fseNAMELEN);
    case ENOTDIR: return(fsePLNDIR);
    case ENXIO: return(fseNOIO);
    case EROFS: return(fseROFS);
    case EFAULT: return(beFAULT);
#endif
    default: return(beUNKERR);
  }
}

int16 LowLevelSeek(int16 ID,int32 Where,int16 How)
{
  bool Result;

  switch (How) {
    case seekSET: How=SEEK_SET; break;
    case seekCUR: How=SEEK_CUR; break;
    case seekEND: How=SEEK_END; break;
  }
  PrepareSystemCall();
  Result=(lseek(ID,Where,How)!=-1);
  return(TranslateCommonErrors(GetError(!Result)));
}

int32 LowLevelTell(int16 ID)
{
  PrepareSystemCall();
  return(lseek(ID,0,SEEK_CUR));
}

int32 LowLevelRead(int16 ID,void *Buf,int32 Size)
{
  int32 Result;
#if !__b__IO32__
  upint8 *Ptr;
  int16 TSize;
  int16 Transferred;
#endif

#if __b__IO32__
  PrepareSystemCall();
  Result=read(ID,Buf,Size);
  if (Result==-1) Result=TranslateCommonErrors(GetError(true));
#else
  Ptr=Buf;
  Result=0;
  while (Size>0) {
    TSize=16384;
    if (TSize>Size) TSize=Size;
    Size-=TSize;
    PrepareSystemCall();
    Transferred=read(ID,Ptr,TSize);
    if (Transferred==-1) {
      Result=TranslateCommonErrors(GetError(true));
      Transferred=0;
    }
    Result+=Transferred;
    if (Transferred<TSize) break;
    Ptr+=TSize;
  }
#endif
  return(Result);
}

int32 LowLevelWrite(int16 ID,const void *Buf,int32 Size)
{
  int32 Result;
#if !__b__IO32__
  const upint8 *Ptr;
  int16 TSize;
  int16 Transferred;
#endif

#if __b__IO32__
  PrepareSystemCall();
  Result=write(ID,Buf,Size);
  if (Result==-1) Result=TranslateCommonErrors(GetError(true));
#else
  Ptr=Buf;
  Result=0;
  while (Size>0) {
    TSize=16384;
    if (TSize>Size) TSize=Size;
    Size-=TSize;
    PrepareSystemCall();
    Transferred=write(ID,Ptr,TSize);
    if (Transferred==-1) {
      Result=TranslateCommonErrors(GetError(true));
      Transferred=0;
    }
    Result+=Transferred;
    if (Transferred<TSize) break;
    Ptr+=TSize;
  }
#endif
  return(Result);
}

int16 ConvertStatToInfo(void *S,TFileInfo *Info)
{
  int16 SMode;
  int16 Mode;
  struct stat *Stat;

  Stat=S;
  /*
    Convert the file mode to our format
  */
  SMode=Stat->st_mode;
#if __e__STATINFO__ == __v__unix__
  if (S_ISREG(SMode)) {
    Mode=fmREGULAR;
  } else if (S_ISDIR(SMode)) {
    Mode=fmDIRECTORY;
  } else if (S_ISLNK(SMode)) {
    Mode=fmSYMLINK;
  } else if (S_ISCHR(SMode)) {
    Mode=fmCHARDEV;
  } else if (S_ISBLK(SMode)) {
    Mode=fmBLOCKDEV;
  } else {
    return(beINVDAT);
  }
  if (SMode & S_IRUSR) Mode+=fmUSRREAD;
  if (SMode & S_IWUSR) Mode+=fmUSRWRITE;
  if (SMode & S_IXUSR) Mode+=fmUSREXEC;
  if (SMode & S_IRGRP) Mode+=fmGRPREAD;
  if (SMode & S_IWGRP) Mode+=fmGRPWRITE;
  if (SMode & S_IXGRP) Mode+=fmGRPEXEC;
  if (SMode & S_IROTH) Mode+=fmOTHREAD;
  if (SMode & S_IWOTH) Mode+=fmOTHWRITE;
  if (SMode & S_IXOTH) Mode+=fmOTHEXEC;
  if (SMode & S_ISUID) Mode+=fmSETUID;
  if (SMode & S_ISGID) Mode+=fmSETGID;
  if (SMode & S_ISVTX) Mode+=fmSTICKY;
#elif __e__STATINFO__ == __v__bcmsdos__
  if (SMode & S_IFDIR) {
    Mode=fmDIRECTORY|fmUSREXEC;
  } else if (SMode & S_IFREG) {
    Mode=fmREGULAR;
  } else {
    return(beINVDAT);
  }
  if (SMode & S_IREAD) Mode+=fmUSRREAD;
  if (SMode & S_IWRITE) Mode+=fmUSRWRITE;
#endif

  Info->Attr=Mode;
  /*
    Copy the rest of the file.
  */
  Info->CreatedTime=Stat->st_ctime;
  Info->ModifiedTime=Stat->st_mtime;
  Info->Size=Stat->st_size;
#if __e__STATINFO__ == __v__unix__
  Info->Owner=Stat->st_uid;
  Info->Group=Stat->st_gid;
  Info->ID=Stat->st_ino;
#elif __e__STATINFO__ == __v__bcmsdos__
  Info->Owner=0;
  Info->Group=0;
  Info->ID=0;
#endif
  /*
    Return success;
  */
  return(sizeof(*Info));
}

int16 LowLevelStat(const char *FileSpec,TFileInfo *Info)
{
  struct stat buf;
  int16 Result;

  PrepareSystemCall();
  Result=stat(FileSpec,&buf);
  if (Result==-1) Result=TranslateCommonErrors(GetError(true));
  if (Result==0) Result=ConvertStatToInfo(&buf,Info);
  return(Result);
}

int16 LowLevelFStat(int16 ID,TFileInfo *Info)
{
  struct stat buf;
  int16 Result;

  PrepareSystemCall();
  Result=fstat(ID,&buf);
  if (Result==-1) Result=TranslateCommonErrors(GetError(true));
  if (Result==0) Result=ConvertStatToInfo(&buf,Info);
  return(Result);
}

int16 LowLevelClose(int16 ID)
{
  int16 Result;

  PrepareSystemCall();
  Result=close(ID);
  if (Result==-1) Result=TranslateCommonErrors(GetError(true));
  return(Result);
}

int16 LowLevelTruncate(int16 ID)
{
  int16 Result;
#if __e__TRUNC__ == __v__ftruncate__
  off_t FilePos;
#endif

  PrepareSystemCall();
#if __e__TRUNC__ == __v__ftruncate__
  FilePos=lseek(ID,0,SEEK_CUR);
  Result=-1;
  if (FilePos!=-1) Result=ftruncate(ID,FilePos);
  if (Result==-1) Result=TranslateCommonErrors(GetError(true));
#elif __e__TRUNC__ == __v__reopen__
  if (ID==0);
  Result=beNOTSUP;
#endif
  return(Result);
}

int16 LowLevelOpen(const char *FileSpec,int16 Mode)
{
  int16 M;
  int16 EAction;
  int16 NAction;
  int16 fd;
  int16 Result;
  int16 FileMode;

  /*
    Initialize the converted flags.
  */
#if __e__OPEN_MODE__ == __v__force_binary__
  M=O_BINARY;
#elif __e__OPEN_MODE__ == __v__zero__
  M=0;
#endif
  /*
    Convert the file mode specification to the POSIX.

    FIXME: Temporary files are not supported yet,
  */
  if ((Mode&oRDWR) == oRDWR) {
    M+=O_RDWR;
  } else {
    if (Mode & oRDONLY) M+=O_RDONLY;
    if (Mode & oWRONLY) M+=O_WRONLY;
    if ((Mode & oMODE)==0) return(beNOTSUP);
  }
  /*
    Convert the actions. If the non-existence action specifies
    "create the file", set the "O_CREAT" flag. If opening existing files is
    not permitted, set also "O_EXCL" flag.
  */
  EAction=Mode & oeACTION;
  NAction=Mode & onACTION;
  if (NAction==onCREAT) {
    M+=O_CREAT;
    if (EAction==0) M+=O_EXCL;
  }
  /*
    Convert the file synchronisation and append flags.
  */
  if (Mode & oAPPEND) M+=O_APPEND;
  if (Mode & oSYNC) {
#if __b__SYNC_SUPPORT__
    M+=O_SYNC;
#else
    return(beNOTSUP);
#endif
  }
  /*
    If OPEN_ON_DIR_BAD is set, we can't rely on open() returning EISDIR when
    called with a specification of a directory rather than a file. This isn't
    always true and OPEN_ON_DIR_BAD says when.
  */
#if __b__OPEN_ON_DIR_BAD__
  {
    TFileInfo I;
    if (LowLevelStat(FileSpec,&I)>=0) {
      if ((I.Attr&fmTYPE)==fmDIRECTORY) return(fseISDIR);
    }
  }
#endif
  /*
    Attempt to open the file.
  */
#if __e__MODE__ == __v__posix__
  FileMode=(S_IRWXU+S_IRWXG+S_IRWXO);
#elif __e__MODE__ == __v__msdos__
  FileMode=(S_IREAD|S_IWRITE);
#elif __e__MODE__ == __v__zero__
  FileMode=0;
#endif
  PrepareSystemCall();
  fd=open(FileSpec,M,FileMode);
  Result=TranslateCommonErrors(GetError(fd==-1));
  /*
    If no action is allowed, set the apropiate error message and exit.
  */
  if (EAction==0 && NAction==0) {
    Result=fseNOENT;
    if (fd!=-1) {
      Result=fseEXIST;
      close(fd);
      fd=-1;
    }
  }
  /*
    If the call succeeded, check if the file is to be truncated and if yes,
    do so. Also set the inheritance flags.
  */
  if (fd!=-1) {
    if (EAction==oeTRUNC) {
#if __e__TRUNC__ == __v__reopen__
      PrepareSystemCall();
      Result=close(fd);
      if (Result==-1) return(TranslateCommonErrors(GetError(true)));
      fd=open(FileSpec,M|O_TRUNC,FileMode);
      if (fd==-1) return(TranslateCommonErrors(GetError(true)));
#else
      Result=LowLevelTruncate(fd);
      if (Result!=0) {
        close(fd);
	fd=-1;
      }
#endif
    }
  }
  return(fd==-1?Result:fd);
}

#if __e__DIR_MANIP__ == __v__tcmsdos__
typedef struct {
  struct ffblk Item;
  bool ItemReady;
  bool EODHit;
} TDirHandle;
#endif

int16 LowLevelOpenDir(const char *DirSpec,void **Result)
{
#if __e__DIR_MANIP__ == __v__tcmsdos__
  char Buffer[129];
  char *Ptr;
  char Ch;
  TDirHandle *D;
  int16 Res;
  int16 Len;
  int16 Attr;

  Len=StrLen(DirSpec);
  if (DirSpec[Len-1]!='\\') Len+=1;
  if (Len>125) return(fseNAMELEN);
  Ptr=StrCpy(Buffer,DirSpec);
  if (*(Ptr-1)!='\\') *(Ptr++)='\\';
  StrCpy(Ptr,"*.*");
  D=KMEM__malloc(sizeof(*D));
  *Result=D;
  if (D!=NULL) {
    PrepareSystemCall();
    Attr=FA_RDONLY|FA_HIDDEN|FA_SYSTEM|FA_DIREC|FA_ARCH;
    Res=findfirst(Buffer,&D->Item,Attr);
    Res=GetError(Res==-1);
    if (Res==0 || Res==ENMFILE) {
      D->ItemReady=(Res==0);
      D->EODHit=!D->ItemReady;
      Res=0;
    } else {
      KMEM__free(D);
      D=NULL;
    }
  }
  *Result=D;
  return(TranslateCommonErrors(Res));
#else
  PrepareSystemCall();
  *Result=opendir((char *)DirSpec);
  return(TranslateCommonErrors(GetError(*Result==NULL)));
#endif
}

#if __e__CHOWN__ == __v__secure__
int16 GetOpenDirID(void *Dir)
{
  return(dirfd(Dir));
}
#endif

int16 LowLevelReadItem(void *Dir,TDirItem *I)
{
#if __e__DIR_MANIP__ == __v__tcmsdos__
  TDirHandle *D;
  int16 Result;
  bool Invalid;
  char *Name;

  D=Dir;
  if (D->EODHit) return(0);
  Invalid=false;
  do {
    if (D->ItemReady) {
      Result=0;
      D->ItemReady=false;
    } else {
      PrepareSystemCall();
      Result=findnext(&D->Item);
      Result=GetError(Result==-1);
    }
    if (Result==0) {
      Name=D->Item.ff_name;
      Invalid=(StrCmp(Name,".")==0 || StrCmp(Name,"..")==0);
    } else if (Result==ENMFILE) {
      Result=0;
      D->EODHit=1;
      return(0);
    } else {
      return(TranslateCommonErrors(Result));
    }
  } while (Invalid);
  StrCpy(I->Name,Name);
#else
  struct dirent *Item;
  int16 NameLen;
  bool Invalid;
  int32 Result;

  /*
    We need to read the items in a loop, because some of them need to be
    filtered out to make the call compatible with JojoDOS.
  */
  do {
    /*
      Try to read the item from the directory.
    */
    PrepareSystemCall();
    Item=readdir(Dir);
#if __b__READDIR_BAD__
    Result=0;
#else
    Result=TranslateCommonErrors(GetError(Item==NULL));
#endif
    /*
      If the attempt to read things from the directory failed, return failure.
      The ErrorCode is already set, so don't bother with it now.
    */
    if (Item==NULL) {
      return(Result);
    }
    /*
      Check if the item is valid.
    */
    Invalid=(StrCmp(Item->d_name,".")==0 || StrCmp(Item->d_name,"..")==0);
  } while (Invalid);
  /*
    Determine the length of the name of the file.
  */
  NameLen=StrLen(Item->d_name);
  /*
    The name must be short enough to fit into the directory item structure.
    If it isn't, set ErrorCode to eNAMETOOLONG and return failure.
  */
  if (NameLen>MAX_FILENAME_SIZE) return(fseNAMELEN);
  /*
    Copy the name into the provided buffer. Use MemCpy since the size of the
    copied string is known. Remember to copy the trailing zero (it is not
    counted in the NameLen variable containing the length of the string).
  */
  MemCpy(I->Name,Item->d_name,NameLen+1);
#endif
  return(sizeof(*I));
}

int16 LowLevelCloseDir(void *Dir)
{
#if __e__DIR_MANIP__ == __v__tcmsdos__
  KMEM__free(Dir);
  return(0);
#else
  int16 Result;

  PrepareSystemCall();
  Result=closedir(Dir);
  if (Result==-1) Result=TranslateCommonErrors(GetError(true));
  return(Result);
#endif
}
