#include "defs.gh"

#if __e__GETCWD__ == __v__posix__
#include <unistd.h>
#elif __e__GETCWD__ == __v__bcmsdos__
#include <dos.h>
#include <dir.h>
#endif
#if __e__DEF_DRIVES__ == __v__bcmsdos__
#include <direct.h>
#endif
#if __e__DEF_DRIVES__ == __v__mingw__
#include <direct.h>
#endif

#include "types.h"
#include "conio.h"
#include "process.h"
#include "errno.h"
#include "strings.h"
#include "memory.h"
#include "traps.h"
#include "env.h"
#include "kernel.h"
#include "pdfsys.h"
#include "kernmsg.h"
#include "pdfileio.h"
#include "diskdata.h"
#include "disklett.h"

#define __INTERFACE__
#include "diskspec.h"
#undef __INTERFACE__

#include "typesnd.h"

#if __e__FSPEC_TYPE__ == __v__unix__
#define FSPEC_DELIMITER '/'
#elif __e__FSPEC_TYPE__ == __v__msdos__
#define FSPEC_DELIMITER '\\'
#endif

/*** Convert a filespec using the MSDOS method ***/
static int16 ConvertMSDOS(char *Buffer)
{
  char Ch;
  char *Ptr;
  uint8 NameLen;
  uint8 ExtLen;
  bool Ext;

  assert(Buffer!=NULL);
  if (*Buffer=='\0') return(0);
  Ext=false;
  Ptr=Buffer;
  NameLen=0;
  ExtLen=0;
  do {
    Ch=DownCase(*Ptr);
    if (Ch=='/' || Ch=='\0') {
      if (NameLen==0 || NameLen>8) return(fseBADNAME);
      if (Ext && ExtLen>3) return(fseBADNAME);
      Ext=false;
      NameLen=0;
    } else if (Ch=='.') {
      if (Ext) return(fseBADNAME);
      Ext=true;
    } else if ((Ch<'0') || ((Ch>'9') && (Ch<'a')) || (Ch>'z')) {
      return(fseBADNAME);
    } else {
      if (Ext) {
        ExtLen++;
      } else {
        NameLen++;
      }
    }
    *(Ptr++)=Ch;
  } while (Ch);
  return(0);
}

/*
  The FollowPath() call uses some local calls and these local calls need
  access to some local variables of the FollowPath() call. One method is to
  make them global variables but this would lead to problems with thread
  safety. So I gathered these variables into a structure which FollowPath()
  will pass around it's local calls.
*/
typedef struct {
  uint16 PathLen;
  char *PathPtr;
  uint16 PathDotCnt;
} TPathParserStatus;

static int16 AddIntoPath(TPathParserStatus *P,char What)
{
  P->PathLen++;
  if (P->PathLen>=MAX_FILESPEC_SIZE) return(fseNAMELEN);
  *(P->PathPtr++)=What;
  return(0);
}

static int16 FlushDots(TPathParserStatus *P)
{
  int16 Res;

  while (P->PathDotCnt) {
    Res=AddIntoPath(P,'.');
    if (Res!=0) return(Res);
    P->PathDotCnt--;
  }
  return(0);
}

static int16 KillLastItem(TPathParserStatus *P)
{
  char Ch;

  if (P->PathLen==0) return(fseNOENT);
  do {
    P->PathLen--;
    P->PathPtr--;
    Ch=*P->PathPtr;
  } while ((Ch!='/') && P->PathLen);
  return(0);
}

/*** Follow a filesystem path from given place ***/
int16 FollowPath(
  /*
    The path to be followed.
  */
  const char *Path,
  /*
    The place from which start.
  */
  char *Result
  /*
    On sucess the call returns zero and Result specifies the place to which
    the Path leads. On failure returns negative error code and content of
    Result is undefined.
  */
)
/*
  This call is used to follow filesystem paths. The "Path" contains the
  argument
  of the "chdir" command and the "Result" contains working directory that was
  set before the "chdir" command took place. This call updates "Result" to
  show the directory that will be the current working directory after the
  "chdir" command took place.

  The path following is called "virtual", because no filesystem accesses are
  made and no checks if all the components of the path exist and are
  directories. It simply splits the "Path" to items and traverses them.
  If the item is ".", it is ignored, if it is "..", the last component of
  "Result" is removed. Other items are appended to the end of the "Result".

  The "Path" may be any valid OSHS path. The content "Result" is expected to
  be canonized OSHS path (all delimiters converted to slashes, no slash at the
  beginning nor the end of the path, and no "." or ".." items in it.
*/
{
  int16 Res;
  char Ch;
  /*
    This is set to true if the analyzer looks at start of path componene or
    at some dots at start of this component.
  */
  bool StartOfComponent;
  /*
    The bunch of "exported local variables".
  */
  TPathParserStatus P;

  assert(Path!=NULL);
  assert(Result!=NULL);
  /*
    Trivial case: if the path to follow is empty, we have nothing to do
  */
  if (*Path=='\0') return(0);
  /*
    Determine the end of the path in Result.
  */
  P.PathLen=StrLen(Result);
  P.PathPtr=Result+P.PathLen;
  /*
    Start traversing of the path
  */
  StartOfComponent=true;
  P.PathDotCnt=0;
  Res=0;
  do {
    /*
      Read character from the path
    */
    Ch=*(Path++);
    /*
      Convert delimiters to slashes
    */
    if (Ch=='\\') Ch='/';
    /*
      Now a large IF processing the starts of the components comes.
    */
    if (StartOfComponent) {
      /*
        Dots at the beginning of the component name are counted and gathered
        into 'PathDotCnt'.
      */
      if (Ch=='.') {
        P.PathDotCnt++;
        continue;
      }
      /*
        When this condition becomes true, we found a path component whose
        name contains only dots. The count of the dots is in the "exported
        local variable" called 'PathDotCnt'.
      */
      if (Ch=='/' || Ch=='\0') {
        /*
          No dots here means two consecutive slashes in the path. This is not
          allowed so reject such path.
        */
        if (P.PathDotCnt==0) return(fseBADNAME);
        /*
          Two dots refer to the ".." item. This means we need to remove last
          component in the resulting path. After the component was removed
          skip the rest of the loop.
        */
        if (P.PathDotCnt==2) {
          Res=KillLastItem(&P);
          if (Res!=0) return(Res);
          P.PathDotCnt=0;
          continue;
        }
        /*
          One dot means "." item which is ignored (refers to the current
          directory). Also here will be the rest of the loop skipped.
        */
        if (P.PathDotCnt==1) {
          P.PathDotCnt=0;
          continue;
        }
      }
      /*
        Control goes here only if the component is nonempty and is not "."
        neither "..". Since there is no trailing slash in the path, we need
        to add one. However this trailing slash is not needed when we are at
	the root.
      */
      if (P.PathLen) {
        Res=AddIntoPath(&P,'/');
        if (Res!=0) return(Res);
      }
      /*
        Flush the dots in the component name.
      */
      Res=FlushDots(&P);
      if (Res!=0) return(Res);
      /*
        Reset the start of component flag. If 'Ch' is "/" (so we in fact
        are at the start of a component), the flag will be set by the next
        command so we don't bother
      */
      StartOfComponent=false;
    }
    /*
      If we found a slash, set the component start flag (because we are at
      the start of the next component). Otherwise write the character into
      the path.
    */
    if (Ch=='/') {
      StartOfComponent=true;
    } else {
      if (Ch) {
        Res=AddIntoPath(&P,Ch);
        if (Res!=0) return(Res);
      }
    }
  } while (Ch && P.PathPtr);
  /*
    Strip garbage from the end of the new path
  */
  *P.PathPtr='\0';
  /*
    Success
  */
  return(0);
}

TEmulatedDisk *ConvertFileSpecEx(
  TRootStatus *R,
  const char *FileSpec,
  char *Buffer
)
{
  uint16 Len;
  char DriveLetter;
  TEmulatedDisk *Drive;
  TFileSpec Buf;
  char *Ptr;

  /*
    Determine the disk that is accessed. If the disk is invalid, return
    failure.
  */
  DriveLetter=R->CurrentDrive;
  if (FileSpec[1]==':') {
    DriveLetter=UpCase(FileSpec[0]);
    FileSpec+=2;
  }
  Drive=GetDrive(R,DriveLetter);
  if (Drive==NULL) {
    R->Result=fseINVDLET;
    return(NULL);
  }
  /*
    Find the disk and check if it is a valid one
  */
  if (Drive->Type==dtEMPTY) {
    R->Result=fseINVDRV;
    return(NULL);
  }
  /*
    Combine the current working directory with the file specification (the
    drive letter was stripped by the code above)
  */
  Buf[0]='\0';
  if (FileSpec[0]!='/' && FileSpec[0]!='\\') {
    StrCpy(Buf,Drive->Cwd);
  } else {
    FileSpec++;
  }
  R->Result=FollowPath(FileSpec,Buf);
  if (R->Result!=0) return(NULL);
  /*
    Check validity and convert the file specification according to the
    namespace type.
  */
  Ptr=Buf;
  switch (Drive->FSType) {
    case fsMSDOS: {
      R->Result=ConvertMSDOS(Buf);
      if (R->Result!=0) return(NULL);
      Ptr=Buf;
      break;
    }
  }
  if (Ptr==NULL) return(NULL);
  /*
    Copy the beginning of the directory in the drive specification into the
    buffer.
  */
  Ptr=StrNCpy(Buffer,Drive->Dir,MAX_FILESPEC_SIZE);
  Len=Ptr-Buffer;
  if (Len>=MAX_FILESPEC_SIZE) {
    R->Result=fseNAMELEN;
    return(NULL);
  }
  /*
    Concatenate with the specification of the file. Also convert the
    specification from OSHS format to the platform specific one.
  */
  R->Result=OSHS2Host(Buffer,Buf,Len);
  if (R->Result<0) return(NULL);
  /*
    WINLFN filesystems need some special processing on UNIX filesystems.
    Since this special processing is currently unavailable, we report no
    support for directory scanning on them.
  */
#if __b__CASE_SENSE__
  if (Drive->FSType==fsWINLFN) {
    R->Result=fseNOTSUP;
    return(NULL);
  }
#endif
  /*
    Everything went OK, so return the buffer
  */
  return(Drive);
}

const char *ConvertFileSpec(
  TRootStatus *R,
  const char *FileSpec,
  char *Buffer
)
{
  void *Result;

  Result=ConvertFileSpecEx(R,FileSpec,Buffer);
  if (Result==NULL) return(NULL);
  return(Buffer);
}
