#include "defs.gh"

#include <stdlib.h>

#undef __t__EXEC_IN_THREAD__
#define __t__EXEC_IN_THREAD__ 0
#include "types.h"
#include "kernel.h"
#include "init.h"
#include "pdgfxcon.h"
#if __e__SCREEN_DRV__ == __v__windows__
#undef __t__EXEC_IN_THREAD__
#define __t__EXEC_IN_THREAD__ 1
#include "gcwdata.h"
#include "gcww32.h"
#endif
#include "memcheck.h"

#define __INTERFACE__
#include "env.h"
#undef __INTERFACE__

#include "typesnd.h"

static uint16 ArgCnt;
static char **Args;
static char **EnvPtr;
static char **CurEnv;
static char *ProgSpec;

int16 Main(void);

int16 main(int16 argc, char **argv, char **envp)
{
  int16 Result;

  /*
    If requested, activate the memory heap checking facility. Most memory
    heap checking facilities must be activated before any other action is
    taken by the program.
  */
  InitializeMemoryChecking();
  /*
    Fetch informations about the arguments
  */
  ArgCnt=argc-1;
  Args=argv+1;
  EnvPtr=envp;
  ProgSpec=argv[0];
  /*
    Initialize the subsystems of the library
  */
  InitializeSYSLIB();
#if __t__EXEC_IN_THREAD__
  /*
    Execute the program in another thead. The main thread must handle the
    window or Windows will crash.
  */
  Result=RunTheProgram(&Main);
#else
  /*
    Run the program
  */
  Result=Main();
#endif
  /*
    Perform the cleanup.
  */
  CleanupSYSLIB();
  /*
    Return the result from the program's main procedure
  */
  return(Result);
}

uint16 GetArgCnt(void)
{
  return(ArgCnt);
}

char *GetArg(uint16 Num)
{
  if (Num>=ArgCnt) return(NULL);
  return(Args[Num]);
}

char * const *GetArgs(void)
{
  return(Args);
}

char *GetSelf(void)
{
  return(ProgSpec);
}

const char *GetEnv(const char *Name)
{
  return(getenv(Name));
}

const char *GetFirstEnv(void)
{
  CurEnv=EnvPtr;
  return(GetNextEnv());
}

const char *GetNextEnv(void)
{
  if (*CurEnv==NULL) return(NULL);
  return(*(CurEnv++));
}
