#include "defs.gh"

#include <errno.h>
#include <string.h>

#include "types.h"
#include "kernel.h"
#include "pdbase.h"

#define __INTERFACE__
#include "errno.h"
#undef __INTERFACE__

#include "typesnd.h"

int16 ReadERRNO(void)
{
  return(errno);
}

/*
  Error code translator.
*/
int16 ReadAndTranslateERRNO(void)
{
  switch (errno) {
    default: return(beUNKERR);
    case 0:return(0);
    case E2BIG:return(be2BIG);
    case EACCES:return(beACCESS);
    case EBADF:return(beBADF);
    case EDOM:return(beDOM);
    case EEXIST:return(beEXIST);
    case EFAULT:return(beFAULT);
    case EINVAL:return(beINVAL);
    case EMFILE:return(beMFILE);
    case ENODEV:return(beNODEV);
    case ENOENT:return(beNOENT);
    case ENOEXEC:return(beNOEXEC);
    case ENOMEM:return(beNOMEM);
    case ERANGE:return(beRANGE);
    case EXDEV:return(beXDEV);
#if __i__ERRNO__unix__
    case EAGAIN:return(beAGAIN);
    case EBUSY:return(beBUSY);
    case ECHILD:return(beCHILD);
    case EFBIG:return(beFBIG);
    case EINTR:return(beINTR);
    case EIO:return(beIO);
    case EISDIR:return(beISDIR);
    case EMLINK:return(beMLINK);
    case ENFILE:return(beNFILE);
    case ENOSPC:return(beNOSPC);
    case ENOTDIR:return(beNOTDIR);
    case ENOTTY:return(beNOTTY);
    case ENXIO:return(beNXIO);
    case EPERM:return(bePERM);
    case EPIPE:return(bePIPE);
    case EROFS:return(beROFS);
    case ESPIPE:return(beSPIPE);
    case ESRCH:return(beSRCH);
#endif
#if __i__ERRNO__posix__
    case EBADMSG:return(beBADMSG);
    case ECANCELED:return(beCANCELLED);
    case EDEADLK:return(beDEADLK);
    case EINPROGRESS:return(beINPROGRESS);
    case EMSGSIZE:return(beMSGSIZE);
    case ENAMETOOLONG:return(beNAMETOOLONG);
    case ENOLCK:return(beNOLOCK);
    case ENOSYS:return(beNOSYS);
    case ENOTEMPTY:return(beNOTEMPTY);
    case ENOTSUP:return(beNOTSUP);
    case ETIMEDOUT:return(beTIMEDOUT);
#endif
  }
}

/*
  Prepare the error reporting variables for an incoming system call.
*/
void PrepareSystemCall(void)
{
  errno=0;
}

/*
  Complain the last error occured
*/
const char *ComplainLastSystemError(void)
{
  return(strerror(errno));
}
