#include "defs.gh"

#include <errno.h>
#if __e__EXEC_DRV__ == __v__bcmsdos__
#include <process.h>
#elif  __e__EXEC_DRV__ == __v__unix__
#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>
#include <fcntl.h>
#elif  __e__EXEC_DRV__ == __v__mingw__
#include <windows.h>
#endif

#include "types.h"
#include "traps.h"
#include "memory.h"
#include "strings.h"
#include "kernel.h"
#include "pdbase.h"
#include "pdfsys.h"
#include "kernmsg.h"
#include "pdfileio.h"
#include "pdexec.h"
#include "diskdata.h"
#include "disklett.h"
#include "diskspec.h"
#if __e__COMPILER_DRV__ != __v__none__
#include "ltmain.h"
#endif
#include "typesnd.h"

#define MAX_PROGRAM_ARGUMENTS 1024

#undef __t__NEEDS_ERROR_TRANSLATOR__
#define __t__NEEDS_ERROR_TRANSLATOR__ 0
#if __e__EXEC_DRV__ != __v__none__
#if __e__EXEC_DRV__ != __v__mingw__
#undef __t__NEEDS_ERROR_TRANSLATOR__
#define __t__NEEDS_ERROR_TRANSLATOR__ 1
#endif
#endif
#if __t__NEEDS_ERROR_TRANSLATOR__
static int16 ReadExecError(void)
{
  switch (errno) {
    case EACCES: return(fseACCESS);
    case E2BIG: return(execeMANYARGS);
#if __e__EXEC_DRV__ == __v__unix__
    case EINTR: return(1);
    case EISDIR:
    case EINVAL:
    case ELIBBAD:
#endif
    case ENOEXEC: return(execeFORMAT);
    case ENOENT: return(fseNOENT);
    case ENOMEM: return(beNOMEM);
#if __e__EXEC_DRV__ == __v__unix__
    case EPERM: return(execeSUIDACC);
    case EFAULT: return(beFAULT);
    case ENAMETOOLONG: return(fseNAMELEN);
    case ENOTDIR: return(fsePLNDIR);
    case ELOOP: return(fseLOOP);
    case ETXTBSY: return(execeTXTBSY);
    case EIO: return(beIO);
    case ENFILE:
#endif
    case EMFILE: return(keNOHND);
    default: return(beUNKERR);
  }
}
#endif

int16 ExecutePlatformSpecificProgram(
  /*
    The binary to be executed. Note that this may differ from argv[0],
    especially when executing .bpf files on non-OSHS platforms.
  */
  const char *ProgSpec,
  /*
    The argument vector to be passed to the program. The first item of this
    vector is the specification of the program being executed.
  */
  const char *const* Args,
  /*
    The enviroment to be passed to the program. If NULL, the executed program
    will inherit the enviroment of the parent.
  */
  const char *const* Env
)
{
#if __e__EXEC_DRV__ == __v__bcmsdos__
  char Buffer[128];
  char *Ptr;
  char *Ext;
  char Ch;
  bool NoExt;
#endif
#if __e__EXEC_DRV__ == __v__unix__
  pid_t PID;
  pid_t Wait;
  int16 Status;
  int16 FileDes[2];
  pint16 Error;
  int16 Count;
#endif
#if  __e__EXEC_DRV__ == __v__mingw__
#define MAX_CMDLINE_LEN 32767
  char CmdLine[MAX_CMDLINE_LEN+1];
  char *CmdPtr;
  int16 ArgNum;
  const char *Arg;
  STARTUPINFO si;
  PROCESS_INFORMATION pi;
  BOOL ExecResult;
#endif
#if __e__EXEC_DRV__ == __v__none__
  return(beNOTSUP);
#else
#if  __e__EXEC_DRV__ != __v__mingw__
  int16 Result;

#endif
#endif
#if __e__EXEC_DRV__ == __v__bcmsdos__
  /*
    Setup a trap for long filenames. Filenames may be no longer than 128
    characters under DOS so the longer filenames need to be rejected with
    eNAMETOOLONG. This is done so that the last byte of the buffer is set to
    zero byte and then StrNCpy() is called using the buffer. If the user
    passed filename that is too long for us, the StrNCpy() overwrites the
    last byte with a nonzero byte so if so happened, we know the filename
    does not fit, and complain. This is the first command is for.

    The second command is used due to the "standard DOS search algorithm"
    Borland's spawn() wants to use when no extension of the file is given.
    Since I want to rip off this platform dependent feature, I need to add
    a dot at the end of the file specification if this is detected to prevent
    this behavior.
  */
  Buffer[127]='\0';
  Buffer[126]='\0';
  /*
    Prepare the filename into the buffer.
  */
  Ptr=StrNCpy(Buffer,ProgSpec,128);
  if (Buffer[127]!='\0') return(fseNAMELEN);
  /*
    Check if an explicit extension in the filename is present. Extensions may
    be no longer than 3 characters under MSDOS.
  */
  Ext=Ptr-4;
  if (Ptr-Buffer<4) Ext=Buffer;
  do {
    Ch=*(Ext++);
    NoExt=(Ch!='.');
  } while (NoExt || Ch=='\0');
  /*
    If there is no extension, the library adds a dot at the end of the
    filespec to prevent spawn() from trying to execute "filename.com" and
    "filename.exe". Remember to check if there is enough space in the buffer
    (if not, report eNAMETOOLONG).
  */
  if (NoExt) {
    if (Buffer[126]!='\0') return(fseNAMELEN);
    StrCpy(Ptr,".");
  }
  /*
    Everything was set up, attempt to execute the program.
  */
  if (Env==NULL) {
    Result=spawnv(P_WAIT,Buffer,(void *)Args);
  } else {
    Result=spawnve(P_WAIT,Buffer,(void *)Args,(void *)Env);
  }
  if (Result==-1) Result=ReadExecError();
  /*
    Return the result of the call.
  */
  return(Result);
#endif
#if __e__EXEC_DRV__ == __v__unix__
  /*
    I need to create a pipe, because this is the only way how to signal the
    errors from the child to the parent.
  */
  Status=pipe((void *)&FileDes);
  if (Status==-1) return(beUNKERR);
  /*
    Fork the process first.
  */
  PID=fork();
  if (PID==-1) return(beUNKERR);
  /*
    In the parent wait for the child and return any errors.
  */
  if (PID) {
    close(FileDes[1]);
    for (;;) {
      Wait=waitpid(PID,&Status,0);
      Result=0;
      if (Wait==-1) {
        Result=ReadExecError();
      } else {
        Error=0;
        Count=read(FileDes[0],&Error,2);
        if (Count==-1) {
          Result=beUNKERR;
        } else if (Count==0) {
          Result=Status & 32767;
        } else {
          Result=Error;
        }
        return(Result);
      }
    }
  }
  /*
    Here the child handling goes. Set the close-on-exec flag of the writing
    end of the pipe and close the reading end.
  */
  close(FileDes[0]);
  fcntl(FileDes[1],F_SETFD,FD_CLOEXEC);
  /*
    Attempt to execute the program
  */
  if (Env==NULL) {
    execv(ProgSpec,(void *)Args);
  } else {
    execve(ProgSpec,(void *)Args,(void *)Env);
  }
  /*
    If control comes here, the exec failed. Read the error code and send it
    to the parent.
  */
  Error=ReadExecError();
  write(FileDes[1],&Error,2);
  /*
    Terminate the process (I don't want the child process to run around and
    bother the parent when the exec failed.
  */
  _exit(0);
#elif  __e__EXEC_DRV__ == __v__mingw__
  CmdPtr=CmdLine;
  ArgNum=0;
  while (true) {
    Arg=Args[ArgNum++];
    if (Arg==NULL) break;
    if (CmdPtr!=CmdLine) *(CmdPtr++)=' ';
    CmdPtr=StrNCpy(CmdPtr,Arg,MAX_CMDLINE_LEN+1-(CmdPtr-CmdLine));
    if (CmdPtr==CmdLine+MAX_CMDLINE_LEN+1) return(execeMANYARGS);
  }
  si.cb=sizeof(si);
  si.lpReserved=NULL;
  si.lpDesktop=NULL;
  si.lpTitle=NULL;
  si.dwFlags=STARTF_FORCEONFEEDBACK;
  si.cbReserved2=0;
  si.lpReserved2=NULL;
  ExecResult=CreateProcess(
    NULL,                             /* lpApplicationName */
    CmdLine,
    NULL, /* */
    NULL,
    TRUE,
    NORMAL_PRIORITY_CLASS,
    (void *)Env,
    NULL,
    &si,
    &pi
  );
  if (!ExecResult) return(beUNKERR);
  WaitForSingleObject(pi.hProcess,-1);
  return(0);
#endif
}

#if __e__COMPILER_DRV__ != __v__none__
/*** Execute the file as specified ***/
/*
  This call executes the one file specified by the user. Only the file
  specified by the user will be attempted to be executed.
*/
static int32 Call(
  /*
    The status structure of the root filesystem.
  */
  TRootStatus *Root,
  /*
    The argument vector to be passed to the program. The first item of this
    vector is the specification of the program being executed.
  */
  const char *const* ArgV,
  /*
    The enviroment to be passed to the program. If NULL, the executed program
    will inherit the enviroment of the parent.
  */
  const char *const* Env
)
{
  /*
    Check for the 'libtool' program. The 'libtool' is currently located in
    the device filesystem. If the 'libtool' program is requested, call the
    libtool procedure and return whatever the procedure returns.
  */
  if (StrCmp(ArgV[0],"@:/compile")==0) {
#if __e__COMPILER_DRV__ == __v__none__
    PrintSTDERR("Compilation not supported on this platform.\n");
    return(1);
#else
    return(RunCompile(ArgV+1));
#endif
  }
  if (StrCmp(ArgV[0],"@:/link")==0) {
#if __e__COMPILER_DRV__ == __v__none__
    PrintSTDERR("Linking not supported on this platform.\n");
    return(1);
#else
    return(RunLink(Root,ArgV+1));
#endif
  }
  /*
    No 'libtool' program -> return the apropiate return value
  */
  return(0x11223344);
}

static int16 LoadLibraryIntoMemory(TRootStatus *Root,void *Msg)
{
  TLoadCmd *C;
  char Buffer[MAX_FILESPEC_SIZE+1];
  const char *Ptr;

  C=Msg;
  Ptr=ConvertFileSpec(Root,C->LibSpec,Buffer);
  if (Ptr==NULL) return(Root->Result);
  return(LoadDynamicLibrary(Buffer));
}
#endif

typedef struct {
  TRootStatus *Root;
} TExecStatus;

int16 HandleExec(void *Msg, int16 Size, void *Out, void *PrivateData)
{
  TExecCmd *C;
  int8 Cmd;
#if __e__COMPILER_DRV__ != __v__none__
  TExecStatus *S;
  TExecReply *R;
  int16 Result;
  int32 CallResult;
#endif

  C=Msg;
  Cmd=C->Command;
  if (Size<1) return(beREQLEN);
#if __e__COMPILER_DRV__ == __v__none__
  if (Cmd!=cmdLOAD && Cmd!=cmdCALL) return(beINVCMD);
  return(beNOTSUP);
#else
  S=PrivateData;
  R=Out;
  if (Cmd==cmdLOAD) return(LoadLibraryIntoMemory(S->Root,Msg));
  if (Cmd!=cmdCALL) return(beINVCMD);
  /*
    Check whether the program is one of the tools we are emulating. If yes,
    this call returns an int16 so we need to arrange proper return of this
    information. If not, the call returns a "magic cookie" we are checking
    here for.
  */
  CallResult=Call(S->Root,C->ArgList,C->EnvList);
  if (CallResult!=0x11223344) {
    assert(CallResult>=-32768 && CallResult<=32767);
    if (CallResult>=0) {
      R->ExitCode=CallResult;
      CallResult=2;
    }
    return(CallResult);
  }
  /*
    Execute the program using the libtool call 'RunProgram'. This api returns
    directly the value to be returned and it also fills in the output buffer.
  */
  Result=RunProgram(S->Root,C->ArgList,C->EnvList,R);
  return(Result);
#endif
}

void CreateExecHandle(TRootStatus *Root)
{
  void *Channel;
  TExecStatus *S;
  int16 Result;

  S=KMEM__malloc(sizeof(*S));
  if (S==NULL) Fatal("No memory for execution driver");
  S->Root=Root;
  Result=CreateChannel(&Channel);
  assert(Result==hndEXECDRV);
  InitializeChannel(Channel,HandleExec,S,DumbClose,DumbDestroy);
}
