#include <stdio.h>
#include <stdlib.h>
#include <string.h>

static FILE *CurFile;
static char FName[80];
static int WasEOF;

void ReportFileReadingError(const char *Msg)
{
  fprintf(stderr,"%s\n",Msg);
  exit(255);
}

/*
  Open the specified text file for reading.
*/
int OpenTheFile(const char *FileName)
{
  strncpy(FName,FileName,79);
  FName[79]='\0';
  CurFile=fopen(FileName,"rt");
  WasEOF=0;
  return(CurFile!=NULL);
}

/*
  Read the line from the current file into the buffer with the specified
  size. Returns nonzero if the read was successfull.
*/
int ReadLine(char *Buffer,int Size)
{
  int Ch;

  if (Size==0) ReportFileReadingError("FATAL ERROR IN ReadLine: Size=0");
  Size--;
  Ch='A';
  while (Size>0) {
    Ch=fgetc(CurFile);
    WasEOF=(Ch==-1);
    if (WasEOF || (Ch=='\n')) break;
    *(Buffer++)=Ch;
    Size--;
  }
  *Buffer='\0';
  return(Size>0);
}

/*
  Check the EOF flag of the file.
*/
int IsEOF(void)
{
  return(WasEOF);
}

/*
  Close the text file previously opened with OpenTheFile().
*/
void CloseTheFile(void)
{
  if (fclose(CurFile)==-1) {
    printf("Error closing file %s",FName);
  }
  CurFile=NULL;
}

/*
  Simply preprocess a C source before compilation. This simple preprocessing
  currently only expands #include "header.h" directives to avoid Turbo C++
  2.01 from misinterpreting them as #include <header.h>. On any errors error
  messages are generated and printed on the screen.

  This simple preprocessor includes each #include "file.h" only once. This
  is because the included files are always headers protected by #ifdef
  symbol against multiple inclusions.

  This implementation has a number of fixed limits, when it hits them,
  the action performed by it is aborted. Later a more advanced
  implementation using dynamic allocation etc will replace this.
*/
#define NAME_BUFFER_SIZE 4096
#define MAX_FILE_STACK_SIZE 4096
#define LINE_BUFFER_SIZE 4096
#define MAX_ACTIVE_FILE_LIST_SIZE 16384
typedef struct tagTFileInStack {
  struct tagTFileInStack *Prev;
  char *NamePtr;
  long Pos;
  long LineNum;
  char File[1];
} TFileInStack;
int PreprocessFile(const char *SrcFile, const char *DstFile)
{
  FILE *Stored;
  int Valid;
  char NameBuf[NAME_BUFFER_SIZE+1];
  char *Ptr;
  char *NPtr;
  const char *IPtr;
  char *NamePtr;
  const char *OrigName;
  char Ch;
  int MaxNameLen;
  FILE *OutF;
  char FileStackBuf[MAX_FILE_STACK_SIZE];
  TFileInStack *SP;
  TFileInStack *Next;
  int RestSize;
  int OpenFileOnStack;
  char Line[LINE_BUFFER_SIZE+1];
  char AFList[MAX_ACTIVE_FILE_LIST_SIZE];
  int AFSize;
  int AFPtr;
  int AFCurLen;
  int AFScanLen;
  /*
    Save the current file. We don't want this procedure to disturb it.
  */
  Stored=CurFile;
  /*
    Extract the directory name from the source file specification. Put the
    beginning of the file name into NamePtr.
  */
  if (strlen(SrcFile)>NAME_BUFFER_SIZE) {
    fprintf(stderr,"Filename '%s' is too long\n",SrcFile);
    return(0);
  }
  Ptr=NameBuf;
  IPtr=SrcFile;
  Ch='/';
  while (Ch!='\0') {
    if (Ch=='/') {
      NamePtr=Ptr;
      OrigName=IPtr;
    }
    Ch=*(IPtr++);
    *(Ptr++)=Ch;
    if (Ch=='\\') Ch='/';
  }
  /*
    Initialize the stack of files. Initially the specified input file will be
    in the stack but without the directory.
  */
  SP=(void *)FileStackBuf;
  SP->Prev=NULL;
  SP->Pos=0;
  SP->LineNum=1;
  SP->NamePtr=NamePtr;
  strcpy(SP->File,OrigName);
  /*
    Create the output file. We need to traverse through the files, search for
    #include "file.h" directives and replace them with the content of the
    referenced file.
  */
  AFSize=0;
  Valid=1;
  OpenFileOnStack=1;
  OutF=fopen(DstFile,"wt");
  if (OutF==NULL) {
    fprintf(stderr,"Error creating '%s'\n",DstFile);
    return(0);
  }
  do {
    if (OpenFileOnStack) {
      NameBuf[NAME_BUFFER_SIZE]='\0';
      NamePtr=SP->NamePtr;
      MaxNameLen=NAME_BUFFER_SIZE-(NamePtr-NameBuf)+1;
      strncpy(NamePtr,SP->File,MaxNameLen);
      if (NameBuf[NAME_BUFFER_SIZE]!='\0') {
        fprintf(stderr,"Name '%s' too long\n",SP->File);
        return(0);
      }
      if (!OpenTheFile(NameBuf)) {
	fprintf(stderr,"Error opening '%s'\n",NameBuf);
	return(0);
      }
      if (fseek(CurFile,SP->Pos,SEEK_SET)==-1) {
        fprintf(stderr,"Error seeking in '%s'\n",NameBuf);
        CloseTheFile();
        return(0);
      }
      fprintf(OutF,"#line %ld \"%s\"\n",SP->LineNum,NameBuf);
      Ptr=NamePtr;
      do {
        Ch=*(Ptr++);
	if (Ch=='\\' || Ch=='/') NamePtr=Ptr;
      } while (Ch!='\0');
      OpenFileOnStack=0;
    }
    if (!ReadLine(Line,LINE_BUFFER_SIZE)) {
      fprintf(stderr,"Error reading '%s'\n",NameBuf);
      return(0);
    }
    SP->LineNum++;
    if (!memcmp(Line,"#include \"",10)) {
      Ptr=Line+10;
      NPtr=Ptr;
      do {
	Ch=*(Ptr++);
      } while (Ch!='"' && Ch!='\0');
      if (Ch=='\0') {
	fprintf(
	  stderr,
	  "Malformed #include directive in '%s', line %ld\n",
	  NameBuf,SP->LineNum-1
	);
	Valid=0;
      }
      *(Ptr-1)='\0';
      OpenFileOnStack=1;
      AFCurLen=Ptr-NPtr-1;
      AFPtr=0;
      while (AFPtr<AFSize) {
	AFScanLen=AFList[AFPtr];
	if (AFScanLen==AFCurLen) {
	  if (!memcmp(AFList+AFPtr+1,NPtr,AFScanLen)) {
	    OpenFileOnStack=0;
	    break;
	  }
	}
	AFPtr+=1+AFScanLen;
      }
      if (OpenFileOnStack) {
	if (MAX_ACTIVE_FILE_LIST_SIZE-AFSize<AFCurLen+1) {
	  fprintf(
	    stderr,
	    "Included files list overflow in '%s', line %ld\n",
	    NameBuf,SP->LineNum-1
	  );
	  return(0);
	}
	AFList[AFSize]=AFCurLen;
	memcpy(AFList+AFSize+1,NPtr,AFCurLen);
	AFSize+=1+AFCurLen;
	Next=(void *)((char *)SP+sizeof(TFileInStack)+strlen(SP->File));
	RestSize=MAX_FILE_STACK_SIZE-(((char *)Next)-FileStackBuf);
	if (RestSize<sizeof(TFileInStack)+strlen(NPtr)) {
	  fprintf(
	    stderr,
	    "File stack overflow in '%s', line %ld\n",
	    NameBuf,SP->LineNum-1
	  );
	  return(0);
	}
	Next->Prev=SP;
	SP->Pos=ftell(CurFile);
	if (SP->Pos==-1) {
	  fprintf(
	    stderr,
	    "Error getting position in '%s', line %ld\n",
	    NameBuf,SP->LineNum-1
	  );
	  return(0);
	}
	CloseTheFile();
	SP=Next;
	SP->Pos=0;
	SP->LineNum=1;
	SP->NamePtr=NamePtr;
	strcpy(SP->File,NPtr);
      } else {
	fprintf(OutF,"\n");
      }
    } else if (Line[0]=='\0' && WasEOF) {
      CloseTheFile();
      SP=SP->Prev;
      OpenFileOnStack=1;
    } else {
      fprintf(OutF,"%s\n",Line);
    }
  } while (SP!=NULL);
  /*
    Restore enviroment to what it was before we had been called.
  */
  fclose(OutF);
  CurFile=Stored;
  /*
    Return success to indicate to the caller that everything went OK.
  */
  return(Valid);
}
