#ifndef __h__SYSLIB_GCWDATA__
#define __h__SYSLIB_GCWDATA__

typedef struct {
  uint8 R;
  uint8 RShift;
  uint8 RDrop;
  uint32 RMask;
  uint8 G;
  uint8 GShift;
  uint8 GDrop;
  uint32 GMask;
  uint8 B;
  uint8 BShift;
  uint8 BDrop;
  uint32 BMask;
  uint8 BitsPerPixel;
} TColorFormat;

void BuildColorFormat(TColorFormat *F);
void UnpackColor(TColorFormat *Format,uint32 Pixel);
uint32 PackColor(TColorFormat *F);
uint32 ConvertPixel(TColorFormat *In,TColorFormat *Out,uint32 Pixel);

typedef enum {
  mtLeftmostIsLSB,
  mtLeftmostIsMSB,
  mtPixmap,
  mtInvalidType
} TMaskType;

typedef struct {
  TColorFormat In;
  TColorFormat Out;
  const upint8 *InPtr;
  uint16 LineNumber;
  uint16 PixelsOnLine;
  uint16 PixelsToFill;
  uint16 LinePaddingBytes;
  upint8 *OutPtr;
  void *OutAuxInfo;
  uint32 FreePixels;
  uint32 PixelCount;
  bool UnprocessedPixels;
  upint8 *MaskPtr;
  uint32 TransparentPixel;
  uint32 InputSize;
  uint8 MaskBuffer;
  uint8 MaskBits;
  uint16 MaskPaddingBytes;
  uint8 MaskOutBuf;
  uint8 MaskOutBit;
  TMaskType MaskType;
  uint8 PixelSetR;
  uint8 PixelSetG;
  uint8 PixelSetB;
  uint8 PixelResetR;
  uint8 PixelResetG;
  uint8 PixelResetB;
  uint32 PixmapMaskDelta;
  uint16 SLine;
  uint16 SPixelsToFill;
  upint8 *SOutPtr;
  void *DestPixmap;
} TColorConversionInfo;

void ConvertPixels(TColorConversionInfo *I);
void InitMaskProcessing(TColorConversionInfo *I);
int16 ConvertPixelsOfMaskedBitmap(TColorConversionInfo *I);

typedef struct TPixmapInfo {
  void *Pixmap;
  uint16 Height;
  uint16 Width;
  uint16 ScreenX;
  uint16 ScreenY;
  struct TPixmapInfo *PrevMounted;
  struct TPixmapInfo *NextMounted;
} TPixmapInfo;

typedef struct {
  uint16 CountInBlock;
  TPixmapInfo Info[256];
} TPixmapInfoBlock;

typedef struct {
  TPixmapInfoBlock *Block[256];
  TPixmapInfo *FirstMounted;
} TPixmapInfoList;

void PixIDInitialize(TPixmapInfoList *List);
TPixmapInfoList *PixIDCreateList(void);
TPixmapInfo *PixIDGetInfo(TPixmapInfoList *List,uint16 ID);
TPixmapInfo *PixIDAllocate(TPixmapInfoList *List,uint16 ID);
int16 PixIDFree(TPixmapInfoList *List,uint16 ID);
void RegisterMountedPixmap(TPixmapInfoList *List,TPixmapInfo *I);
void UnregisterMountedPixmap(TPixmapInfoList *List,TPixmapInfo *I);

#define NO_PIXMAP PixIDAllocate

#endif
