#include "defs.gh"
#include "types.h"
#include "errno.h"
#include "memory.h"
#include "strings.h"
#include "kernel.h"
#include "kernmsg.h"
#include "pdbase.h"
#include "pdfsys.h"
#include "pdfileio.h"
#include "diskdata.h"
#include "diskdumb.h"
#include "diskmain.h"
#include "pdinet.h"
#include "inettype.h"
#include "pddev.h"
#include "devmain.h"

#if __e__INET_DRV__ == __v__linux__
#include "inetlin.h"
#elif __e__INET_DRV__ == __v__windows__
#include "inetwin.h"
#endif

#define __INTERFACE__
#include "inet.h"
#undef __INTERFACE__

#define MAX_SPARE_CHANNELS 1024

#if __b__WINSOCK_SUPPORT__
typedef struct {
  upint8 Buffer[256];
  upint8 UncheckedBytes;
  upint8 SpareBytes;
  upint8 Position;
} TWinsockHackCheckInfo;
#endif

typedef struct {
  TGenericFileHandle H;
  void *Socket;
  void *Other;
#if __b__WINSOCK_SUPPORT__
  TWinsockHackCheckInfo *CI;
  upint8 FirstIgnoredChar;
  upint8 SecondIgnoredChar;
  bool WasFirstIgnoredChar;
#endif
} TClientSocketInfo;

typedef struct {
  pint16 Handle;
  upint16 Next;
} TSpareChannel;

typedef struct {
  TSpareChannel Channels[MAX_SPARE_CHANNELS];
  int16 ListSize;
  int16 FirstFree;
} TSpareChannelList;

typedef struct {
  void *Socket;
  TSpareChannelList List;
} TServerSocketInfo;

typedef struct {
  upint8 Result;
} TGenericReply;

typedef struct {
  void *Channel;
  TResolverData *D;
  TSpareChannelList List;
} TMasterInfo;

static int32 SocketReadCore(TClientSocketInfo *I,void *Buf,int32 Size)
{
  int32 Result;
#if __b__WINSOCK_SUPPORT__
  uint16 Len;
  upint8 *OrigBuf;
  int32 SpareBytes;
  upint8 *RPtr;
  upint8 *WPtr;
  upint8 *BPtr;
  char *ChPtr;
  uint8 ReadSize;
  uint8 FreeSize;
  bool FreeCI;

  uint8 Byte;
  uint32 i;
  uint8 SizeToCheck;
#endif

#if __b__WINSOCK_SUPPORT__
  OrigBuf=Buf;
  SpareBytes=0;
  FreeCI=false;
  Len=0;
  ChPtr=NULL;
  BPtr=NULL;
  if (I->CI!=NULL) {
    Len=StrLen(WINSOCK_HACK_PROLOG_STR)+1;
    ChPtr=WINSOCK_HACK_PROLOG_STR;
  }
  if (I->FirstIgnoredChar!=I->SecondIgnoredChar) {
    if (I->WasFirstIgnoredChar) {
      *OrigBuf=I->FirstIgnoredChar;
      Size--;
      SpareBytes++;
      Buf=OrigBuf+1;
      I->WasFirstIgnoredChar=false;
    }
  } else if (I->CI!=NULL) {
    if (I->CI->UncheckedBytes==0) {
      ReadSize=I->CI->SpareBytes;
      if (ReadSize>Size) ReadSize=Size;
      if (ReadSize>0) {
        Buf=MemCpy(Buf,I->CI->Buffer+I->CI->Position,ReadSize);
        Size-=ReadSize;
        I->CI->Position+=ReadSize;
        I->CI->SpareBytes-=ReadSize;
        SpareBytes+=ReadSize;
        return(ReadSize);
      }
      FreeCI=(I->CI->SpareBytes==0);
    } else {
      Buf=I->CI->Buffer+I->CI->SpareBytes;
      FreeSize=sizeof(I->CI->Buffer) - I->CI->SpareBytes;
      assert(Size<=FreeSize);
    }
  }
#endif
  Result=InetSocketRead(I->Socket,Buf,Size);
#if __b__WINSOCK_SUPPORT__
  if (Result>0) {
    if (I->CI!=NULL) {
      if (I->CI->UncheckedBytes>0) {
        BPtr=I->CI->Buffer + I->CI->SpareBytes;
        I->CI->SpareBytes+=Result;
      }
      if (I->CI->UncheckedBytes==255) {
        if (*(BPtr++)==WINSOCK_HACK_TAG) {
          Result--;
          I->CI->UncheckedBytes=Len+2;
        } else {
          I->CI->UncheckedBytes=0;
          return(-65536);
        }
      }
      if (I->CI->UncheckedBytes>2 && Result>0) {
        SizeToCheck=I->CI->UncheckedBytes-2;
        if (SizeToCheck>Result) SizeToCheck=Result;
        ChPtr+=(BPtr - I->CI->Buffer) - 1;
        if (MemCmp(ChPtr,BPtr,SizeToCheck)==0) {
          I->CI->UncheckedBytes-=SizeToCheck;
          BPtr+=SizeToCheck;
          Result-=SizeToCheck;
        } else {
          I->CI->UncheckedBytes=0;
          return(-65536);
        }
      }
      if (I->CI->UncheckedBytes==2 && Result>0) {
        I->FirstIgnoredChar=*(BPtr++);
        I->SecondIgnoredChar=I->FirstIgnoredChar;
        I->CI->UncheckedBytes--;
        Result--;
      }
      if (I->CI->UncheckedBytes==1 && Result>0) {
        I->SecondIgnoredChar=*(BPtr++);
        I->CI->UncheckedBytes--;
        Result--;
        if (I->FirstIgnoredChar==I->SecondIgnoredChar) return(-65536);
        FreeCI=true;
      }
      if (I->CI->UncheckedBytes>0) {
        assert(BPtr - I->CI->Buffer == I->CI->SpareBytes);
      }
    }
    Result+=SpareBytes;
    if (I->FirstIgnoredChar!=I->SecondIgnoredChar) {
      RPtr=OrigBuf;
      WPtr=OrigBuf;
      for (i=0;i<Result;i++) {
        Byte=*(RPtr++);
        *(WPtr++)=Byte;
        if (I->WasFirstIgnoredChar) {
          if (Byte==I->SecondIgnoredChar) WPtr-=2;
        }
        I->WasFirstIgnoredChar=(Byte==I->FirstIgnoredChar);
      }
      if (I->WasFirstIgnoredChar) WPtr--;
      Result=WPtr-OrigBuf;
    }
    if (Result==0) Result=-65536;
  }
  if (FreeCI) {
    KMEM__free(I->CI);
    I->CI=NULL;
  }
#endif
  return(Result);
}

static int32 SocketRead(void *fd,void *Buf,int32 Size)
{
  TClientSocketInfo *I;
  int32 Result;
#if __b__WINSOCK_SUPPORT__
  bool Checking;
  uint8 ReadSize;
  upint8 *ReadPtr;
  int32 PreviousBytes;
#endif

  I=fd;
  if (I->Other==NULL) return(ineteSDOWN);
  UnlockChannels();
#if __b__WINSOCK_SUPPORT__
  ReadPtr=Buf;
  PreviousBytes=0;
  do {
    Checking=(I->CI!=NULL);
    if (Checking) Checking=(I->CI->UncheckedBytes>0);
    if (Checking) {
      ReadSize=I->CI->UncheckedBytes;
      if (ReadSize==255) ReadSize=1;
      if (ReadSize>Size) ReadSize=Size;
      Result=SocketReadCore(I,ReadPtr,ReadSize);
      if (Result>0) {
        ReadPtr+=Result;
        PreviousBytes+=Result;
        Size-=Result;
        if (Size==0) {
          Result=0;
        } else {
          Result=-65536;
        }
      }
    } else {
      Result=SocketReadCore(I,ReadPtr,Size);
    }
  } while (Result==-65536);
  Result+=PreviousBytes;
#else
  Result=SocketReadCore(I,Buf,Size);
#endif
  LockChannels();
  return(Result);
}

static int32 SocketWrite(void *fd,const void *Buf,int32 Size)
{
  int32 Result;
  TClientSocketInfo *I;

  I=fd;
  if (I->Other==NULL) return(ineteSDOWN);
  UnlockChannels();
  Result=InetSocketWrite(I->Socket,Buf,Size);
  LockChannels();
  return(Result);
}

static int16 SocketClose(void *Socket)
{
  int16 Result;

  UnlockChannels();
  Result=InetSocketClose(Socket);
  LockChannels();
  return(Result);
}

static int16 ClientSocketClose(void *fd)
{
  TClientSocketInfo *I;
  TClientSocketInfo *OI;

  I=fd;
  OI=I->Other;
  if (OI==NULL) return(0);
  OI->Other=NULL;
  return(SocketClose(I->Socket));
}

static void ClientSocketDestroy(void *fd)
{
  TClientSocketInfo *I;

  I=fd;
  if (I->Other!=NULL) {
    DestroySocketInfo(I->Socket);
    KMEM__free(I->Socket);
  }
}

static void InitSpareChannelList(TSpareChannelList *S)
{
  S->ListSize=0;
  S->FirstFree=-1;
}

static TSpareChannel *AllocSpareChannelSlot(TSpareChannelList *S)
{
  int16 Num;
  TSpareChannel *Slot;

  if (S->FirstFree>=0) {
    Num=S->FirstFree;
    S->FirstFree=S->Channels[Num].Next;
  } else {
    if (S->ListSize==MAX_SPARE_CHANNELS) return(NULL);
    Num=S->ListSize++;
  }
  Slot=S->Channels+Num;
  Slot->Handle=-1;
  return(Slot);
}

static void FreeSpareChannelSlot(TSpareChannelList *S,TSpareChannel *Slot)
{
  assert(Slot>=S->Channels && Slot<S->Channels+MAX_SPARE_CHANNELS);
  Slot->Handle=-1;
  Slot->Next=S->FirstFree;
  S->FirstFree=(uint16)(Slot-S->Channels);
}

static void CloseSpareChannels(TSpareChannelList *S)
{
  int16 i;
  TSpareChannel *Slot;

  Slot=S->Channels;
  for (i=0;i<S->ListSize;i++,Slot++) {
    if (Slot->Handle>=0) CloseCoreNoLock(Slot->Handle);
  }
}

typedef struct {
  TSpareChannelList *L;
  TSpareChannel *Slot;
  int16 Handle;
  TClientInfo Client;
  void *Channel;
  void *ServerChannel;
} TInetTaskInfo;

static int16 CreateStructures(TInetTaskInfo *I)
{
  I->Handle=-1;
  I->Slot=AllocSpareChannelSlot(I->L);
  I->Client.Socket=AllocSocketInfo();
  if (I->Slot==NULL || I->Client.Socket==NULL) {
    return(beNOMEM);
  }
  I->Handle=CreateChannelNoLock(&I->Channel);
  if (I->Handle<0) return(I->Handle);
  I->Slot->Handle=I->Handle;
  return(0);
}

static void FillClientSocket(TClientSocketInfo *FD,TInetTaskInfo *I)
{
  FD->Socket=I->Client.Socket;
  InitializeOpenFileChannel(
    I->Channel,
    FD,
    ClientSocketClose,
    ClientSocketDestroy
  );
#if __b__WINSOCK_SUPPORT__
  FD->FirstIgnoredChar=0;
  FD->SecondIgnoredChar=0;
  FD->WasFirstIgnoredChar=false;
#endif
}

static void FreeStructures(TInetTaskInfo *I)
{
  if (I->Handle>=0) CloseCoreNoLock(I->Handle);
  if (I->Slot!=NULL) FreeSpareChannelSlot(I->L,I->Slot);
  if (I->Client.Socket) {
    DestroySocketInfo(I->Client.Socket);
    KMEM__free(I->Client.Socket);
  }
}

typedef int16 TInetOpener(TClientInfo *Client);

static int16 GenericOpen(void *Buf,TInetTaskInfo *I,TInetOpener *Open)
{
  TOpenTCPServerReply *Hnd;
  int16 Result;

  Hnd=Buf;
  Result=CreateStructures(I);
  if (Result==0) {
    UnlockChannels();
    Result=Open(&I->Client);
    LockChannels();
  } while (false);
  if (Result<0) {
    FreeStructures(I);
    return(Result);
  } else {
    FreeSpareChannelSlot(I->L,I->Slot);
  }
  Hnd->Handle=I->Handle;
  return(sizeof(*Hnd));
}

static int16 OpenClient(void *Buf,TInetTaskInfo *I,TInetOpener *Open)
{
  int16 Result;
  int16 Handle;
  TClientSocketInfo *FDR,*FDW;
  void *WChannel;
  TOpenTCPClientReply *R;
#if __b__WINSOCK_SUPPORT__
  TWinsockHackCheckInfo *CI;
#endif

  Result=CreateChannelNoLock(&WChannel);
  Handle=Result;
  FDR=NULL;
  FDW=NULL;
#if __b__WINSOCK_SUPPORT__
  CI=NULL;
#endif
  if (Result>=0) {
    FDR=CreateFileHandleData(sizeof *FDR);
    FDW=CreateFileHandleData(sizeof *FDW);
    if (FDR==NULL || FDW==NULL) Result=beNOMEM;
#if __b__WINSOCK_SUPPORT__
    CI=KMEM__malloc(sizeof *CI);
    if (CI==NULL) Result=beNOMEM;
#endif
  }
  if (Result>=0) Result=GenericOpen(Buf,I,Open);
  if (Result>=0) {
    FillClientSocket(FDR,I);
    I->Channel=WChannel;
    FillClientSocket(FDW,I);
    FDR->H.Read=SocketRead;
    FDW->H.Write=SocketWrite;
    FDR->Other=FDW;
    FDW->Other=FDR;
    R=Buf;
    R->HWrite=Handle;
    Result=4;
#if __b__WINSOCK_SUPPORT__
    CI->UncheckedBytes=255;
    CI->SpareBytes=0;
    CI->Position=0;
    FDR->CI=CI;
    FDW->CI=NULL;
#endif
  } else {
    if (FDR!=NULL) KMEM__free(FDR);
    if (FDW!=NULL) KMEM__free(FDW);
#if __b__WINSOCK_SUPPORT__
    if (CI!=NULL) KMEM__free(CI);
#endif
  }
  return(Result);
}

static int16 ServerListen(void *Msg,int16 Size,void *Buf,void *fd)
{
  TServerSocketInfo *S;
  TConnectionRecord *Rec;
  int16 Result;
  TInetTaskInfo I;

  S=fd;
  Rec=Buf;
  I.L=&S->List;
  I.Client.Host=&Rec->Host;
  I.Client.Port=&Rec->Port;
  I.Client.ServerSocket=S->Socket;
  Result=OpenClient(Buf,&I,InetServerSocketRead);
  if (Result>0) Result=sizeof(*Rec);
  return(Result);
}

static int16 ServerClose(void *fd)
{
  TServerSocketInfo *I;

  I=fd;
  CloseSpareChannels(&I->List);
  return(SocketClose(I->Socket));
}

static int16 ExecuteOpenTCPClient(void *Buf,const void *Cmd,void *fd)
{
  const TOpenTCPClientCommand *Packet;
  TInetTaskInfo I;
  TMasterInfo *M;

  M=fd;
  Packet=Cmd;
  I.L=&M->List;
  I.Client.Host=(void *)&Packet->Host;
  I.Client.Port=(void *)&Packet->Port;
  return(OpenClient(Buf,&I,InetOpenTCPClient));
}

static int16 ExecuteOpenTCPServer(void *Buf,const void *Cmd,void *fd)
{
  const TOpenTCPServerCommand *Packet;
  int16 Result;
  TInetTaskInfo I;
  TServerSocketInfo *FD;
  TMasterInfo *M;

  M=fd;
  Packet=Cmd;
  I.L=&M->List;
  I.Client.QueueSize=Packet->QueueSize;
  I.Client.Port=(void *)&Packet->Port;
  FD=KMEM__malloc(sizeof *FD);
  if (FD==NULL) return(beNOMEM);
  Result=GenericOpen(Buf,&I,InetOpenTCPServer);
  if (Result>=0) {
    FD->Socket=I.Client.Socket;
    InitializeChannel(I.Channel,ServerListen,FD,ServerClose,DumbDestroy);
    InitSpareChannelList(&FD->List);
  } else {
    KMEM__free(FD);
  }
  return(Result);
}

static void FreeResolverData(TMasterInfo *Hnd)
{
  if (Hnd->D!=NULL) {
    KMEM__free(Hnd->D);
    Hnd->D=NULL;
  }
}

static int16 PrepareNextPendingAddr(void *Out,const void *Cmd,void *fd)
{
  uint16 Index;
  void *Ptr;
  TMasterInfo *Hnd;
  TOpenTCPClientCommand *R;

  Hnd=fd;
  R=Out;
  if (Hnd->D==NULL) return(0);
  Index=Hnd->D->CurrentPending++;
  Ptr=Hnd->D->Data+Index;
  R->Command=inetcmdOpenTCPClient;
  MemCpy(&R->Host,Ptr,sizeof(Hnd->D->Data[0]));
  if (Index+1==Hnd->D->PendingCnt) FreeResolverData(Hnd);
  return(sizeof(*R));
}

static int16 ExecuteResolveName(void *Buf,const void *Cmd,void *fd)
{
  TMasterInfo *Hnd;
  TResolverData *D;
  int16 Result;

  /*
    Create the resolver data structure. If there is already a resolver
    structure in the master handle, reuse it and don't bother with creating
    it.
  */
  Hnd=fd;
  D=Hnd->D;
  if (D==NULL) {
    D=KMEM__malloc(sizeof(*D));
    if (D==NULL) return(beNOMEM);
    Hnd->D=D;
  }
  D->PendingCnt=0;
  D->CurrentPending=0;
  /*
    Execute the resolver on the name from the user. The name is starting from
    the second byte of the command packet.
  */
  UnlockChannels();
  Result=InetResolveName(((const char *)Cmd)+1,D);
  LockChannels();
  if (Result==0) {
    Result=PrepareNextPendingAddr(Buf,NULL,fd);
  } else {
    FreeResolverData(Hnd);
  }
  return(Result);
}

typedef int16 (*TExecutor)(void *Buf,const void *Cmd,void *fd);

static int16 INETHandler(void *Buf, int16 Size, void *Out, void *fd)
{
  TExecutor Exec;
  uint16 ReqSize;
  uint8 Cmd;
  TMasterInfo *Hnd;
  bool DeleteResolverData;

  /*
    Typecast the handle private info
  */
  Hnd=fd;
  /*
    Get the command code.
  */
  if (Size<1) return(beREQLEN);
  Cmd=*((upint8 *)Buf);
  /*
    Determine how to execute the command. The "Exec" pointer will be set to
    the procedure that executes the command in question. The ReqSize will be
    set to the minimal size that a packet must have. Also here is determined
    if the resolver data is to be deleted.
  */
  DeleteResolverData=true;
  switch (Cmd) {
    default: {
      return(beINVCMD);
    }
    case inetcmdOpenTCPServer: {
      ReqSize=sizeof(TOpenTCPServerCommand);
      Exec=ExecuteOpenTCPServer;
      break;
    }
    case inetcmdOpenTCPClient: {
      ReqSize=sizeof(TOpenTCPClientCommand);
      Exec=ExecuteOpenTCPClient;
      break;
    }
    case inetcmdResolveName: {
      if (Size>1 && *(((char *)(Buf)+Size-1))!='\0') return(ineteMISNULL);
      ReqSize=sizeof(TResolveNameCommand);
      Exec=ExecuteResolveName;
      DeleteResolverData=false;
      break;
    }
    case inetcmdNextAddress: {
      if (Hnd->D==NULL) return(ineteNORESOLV);
      ReqSize=1;
      Exec=PrepareNextPendingAddr;
      break;
    }
  }
  /*
    If the resolver data is to be deleted, do so now.
  */
  if (DeleteResolverData) FreeResolverData(fd);
  /*
    Check if enough bytes were passed. If not, set the result to BADSIZE and
    ignore the command.
  */
  if (Size<ReqSize) return(beREQLEN);
  /*
    Execute the command.
  */
  return(Exec(Out,Buf,fd));
}

static int16 CloseINET(void *Data)
{
  TMasterInfo *I;

  I=Data;
  CloseSpareChannels(&I->List);
  return(0);
}

static void DestroyINET(void *Data)
{
  TMasterInfo *I;

  I=Data;
  FreeResolverData(I);
}

static int16 OpenINET(TOpenReport *R)
{
  TMasterInfo *Hnd;

  Hnd=KMEM__malloc(sizeof(*Hnd));
  if (Hnd==NULL) return(beNOMEM);
  Hnd->Channel=R->Channel;
  R->Data=Hnd;
  R->Handler=INETHandler;
  R->Close=CloseINET;
  R->Destroy=DestroyINET;
  Hnd->D=NULL;
  InitSpareChannelList(&Hnd->List);
  return(0);
}

void AddInternetDevices(void)
{
  AddDevice(dtInternet,"Internet access point",OpenINET);
}
