#include "defs.gh"
#include "types.h"
#include "traps.h"
#include "kernel.h"
#include "kernmsg.h"
#include "pdfsys.h"
#include "pdfileio.h"
#include "diskdata.h"
#include "disklett.h"
#include "thread.h"
#include "cfgfile.h"
#include "diskmain.h"
#include "pddev.h"
#include "devmain.h"

#define __INTERFACE__
#include "init.h"
#undef __INTERFACE__

#define MAX_SHUTDOWN_PROCS 64

static int16 ShutdownProcCnt;
TShutdownProc ProcList[MAX_SHUTDOWN_PROCS];

void InitializeSYSLIB(void)
{
  /*
    Initialize the list of procedures to be performed at program shutdown.
  */
  ShutdownProcCnt=0;
  /*
    Read the configuration file. This must be done before any other
    initialization takes place.
  */
  ReadConfig();
  /*
    Setup various variables that are affected by the content of the config
    file.
  */
  SetupConfigFileVariables();
  /*
    Initialize the thread subsystem
  */
  SYSInitThreads();
  /*
    Initialize the kernel interface
  */
  InitializeChannels();
  InitDevices();
  CreateStandardHandles();
}

void InstallShutdownProc(TShutdownProc Proc)
{
  if (ShutdownProcCnt==MAX_SHUTDOWN_PROCS) {
    ReportBug("Too many shutdown procedures");
  }
  ProcList[ShutdownProcCnt++]=Proc;
}

void CleanupSYSLIB(void)
{
  TShutdownProc Proc;

  while (ShutdownProcCnt>0) {
    Proc=ProcList[--ShutdownProcCnt];
    Proc();
  }
}
