#include "defs.gh"
#include "types.h"
#include "ltwd.h"
#define __INTERFACE__
#include "ltargs.h"
#undef __INTERFACE__
#include "memory.h"
#include "strings.h"
#include "kernel.h"
#include "pdbase.h"
#include "pdfileio.h"
#include "pdfsys.h"
#include "disklow.h"

/*
  Argument lists handling module.
*/

static TArgumentList *First;

TArgumentList *CreateList(void)
{
  TArgumentList *List;

  List=KMEM__malloc(sizeof(*List));
  if (List!=NULL) {
    List->ArgBlockPtr=List->ArgBlock;
    List->ArgListPtr=List->ArgList;
    *List->ArgListPtr=NULL;
    List->Next=First;
    First=List;
  }
  return(List);
}

char *AddArgument(TArgumentList *List,const char *Arg)
{
  uint32 Len;

  assert(List!=NULL);
  if (List->ArgListPtr-List->ArgList==MAX_ARGUMENT_COUNT) {
    return("Argument pointer table full");
  }
  Len=StrLen(Arg)+1;
  if (MAX_ARGUMENTS_SIZE-(List->ArgBlockPtr-List->ArgBlock)<Len) {
    return("No room for new argument");
  }
  *(List->ArgListPtr++)=List->ArgBlockPtr;
  *List->ArgListPtr=NULL;
  MemCpy(List->ArgBlockPtr,Arg,Len);
  List->ArgBlockPtr+=Len;
  return(NULL);
}

char *AddList(TArgumentList *List,const char *const*Args)
{
  const char *Ptr;

  assert(List!=NULL);
  do {
    Ptr=*(Args++);
    if (List->ArgListPtr-List->ArgList==MAX_ARGUMENT_COUNT) {
      return("Argument pointer table full");
    }
    *List->ArgListPtr=Ptr;
    if (Ptr!=NULL) List->ArgListPtr++;
  } while (Ptr!=NULL);
  return(NULL);
}

static char *AddSYSLIBLibrary(TArgumentList *List)
{
  char Buffer[8192];
  char *Ptr;
  const char *IPtr;

  Ptr=PutInternalDirSpec("lib",Buffer);
  Ptr=StrCpy(Ptr,"syslib.");
#if __i__LIB_SUPPORT__shared__
  IPtr="so";
#if __i__LIB_SUPPORT__static__
  if (LinkSystemLibsStatic) IPtr="a";
#endif
#elif __i__LIB_SUPPORT__static__
  IPtr="a";
#endif
  StrCpy(Ptr,IPtr);
  return(AddArgument(List,Buffer));
}

static char *AddPlatformLibraries(TArgumentList *List)
{
  char Buffer[8192];
  char *Ptr;
  char *Arg;
  char Ch;
  char *Result;
  int16 fd;
  uint32 Size;

  Ptr=PutInternalDirSpec(NULL,Buffer);
  Ptr=StrNCpy(Ptr,"lib/syslib.lib",sizeof(Buffer)-(Ptr-Buffer));
  if (Ptr<Buffer+sizeof(Buffer)) {
    fd=LowLevelOpen(Buffer,oRDONLY+oeOPEN);
    if (fd==-1) {
      Buffer[0]='\0';
    } else {
      Size=LowLevelRead(fd,Buffer,sizeof(Buffer));
      LowLevelClose(fd);
      if (Size==sizeof(Buffer)) Size=0;
      Buffer[Size]='\0';
    }
  }
  if (Buffer[0]=='\0') {
    assert(StrLen(__x__ADDON_LIBS__)<sizeof(Buffer));
    StrCpy(Buffer,__x__ADDON_LIBS__);
  }
  Arg=Buffer;
  Ptr=Buffer;
  Result=NULL;
  while (true) {
    Ch=*(Ptr++);
    if (Ch=='\0') break;
    if (Ch==' ') {
      *(Ptr-1)='\0';
      if (Arg!=Ptr) {
        Result=AddArgument(List,Arg);
        if (Result!=NULL) return(Result);
        Arg=Ptr;
      }
    }
  }
  if (Arg!=Ptr) Result=AddArgument(List,Arg);
  return(Result);
}

char *AddSystemLibraries(TArgumentList *List)
{
  char *Result;

  Result=AddSYSLIBLibrary(List);
  if (Result==NULL) Result=AddPlatformLibraries(List);
  return(Result);
}

void DestroyArgumentLists(void)
{
  TArgumentList *List;

  while (First!=NULL) {
    List=First;
    First=First->Next;
    KMEM__free(List);
  }
}
