#ifndef __h__SYSLIB_LTBIN__
#define __h__SYSLIB_LTBIN__
#include "hhead.h"

typedef struct {
  /*
    Full name of the output file.
  */
  const char *Output;
  /*
    Object files to be linked into the program section. Libraries usually
    have this set to NULL. If both sections are created in the binary, the
    resulting binary will be an executable library.
  */
  const char **ProgObj;
  /*
    Libraries to be linked statically into the program section.
  */
  const char **ProgLibStatic;
  /*
    Libraries to be linked dynamically into the program section. Must be NULL
    if there are no object modules nor statically linked libraries to be
    linked into the program section.
  */
  const char **ProgLibDynamic;
  /*
    Object files to be linked into the library section. If set to NULL, the
    resulting binary will have no library section.
  */
  const char **LibObj;
  /*
    Libraries to be linked statically into the library section.
  */
  const char **LibLibStatic;
  /*
    Libraries to be linked dynamically into the library section. Must be NULL
    if there are no object modules nor statically linked libraries to be
    linked into the library section.
  */
  const char **LibLibDynamic;
} TLinkingInfo;

extern uint16 OBSAreaSize;

bool ReadOBSAreaSize(const char *Arg);
int16 AssembleResultingBinaryFile(
  const char *FileName,
  const char *const* FileList,
  const char *Signature,
  const char *LibName
);
int16 CombineCompilationResults(
  const char *const* argv,
  const char *const* FileList,
  const char *Signature
);
int16 ExtractLibraryName(int16 FD,char *Buffer,bool Verbose);
int32 ExtractContent(int16 InFD,int16 OutFD,int16 Number,bool Verbose);
bool ExtractNumberedContents(
  const char **Files,
  int16 PartNum,
  const char *Prefix,
  const char *Ext,
  TArgumentList *List
);
int16 ConvertAndOpenFile(TRootStatus *Root,const char *FileSpec,int16 Mode);

#include "hfoot.h"
#endif
