#include "defs.gh"
#include "types.h"

#include "conio.h"

#define __INTERFACE__
#include "ltopts.h"
#undef __INTERFACE__

#define MAX_ARGUMENTS 128

static const char *NewArgV[MAX_ARGUMENTS];
static int16 CurrentArg;

static void ReportInvalidOption(const char *Arg)
{
  PrintSTDERR("Invalid option ");
  PrintSTDERR(Arg);
  PrintSTDERR("\n");
}

void InitNewArgV(void)
{
  CurrentArg=0;
}

void AddArg(const char *Arg)
{
  if (CurrentArg==MAX_ARGUMENTS) {
    PrintSTDERR("Too many arguments to copy\n");
    CurrentArg++;
  } else if (CurrentArg<MAX_ARGUMENTS) {
    NewArgV[CurrentArg++]=Arg;
  }
}

const char * const *FinalizeNewArgV(void)
{
  AddArg(NULL);
  return(NewArgV);
}

typedef enum {
  stPassOn,
  stProcessed,
  stInvalid
} TOptionProcessingResult;

typedef TOptionProcessingResult TOptionProcessingProc(const char *Arg);

const char * const *ProcessOptions(
  TOptionProcessingProc ProcessArg,
  const char * const *argv
)
{
  const char *Arg;
  TOptionProcessingResult Result;

  InitNewArgV();
  while (true) {
    Arg=*(argv++);
    if (Arg==NULL) break;
    Result=ProcessArg(Arg);
    switch (Result) {
      case stPassOn: {
        AddArg(Arg);
        break;
      }
      case stInvalid: {
        ReportInvalidOption(Arg);
        break;
      }
      default: {
        break;
      }
    }
  }
  return(FinalizeNewArgV());
}

TOptionProcessingResult ProcessCompilerOption(const char *Arg)
{
  TOptionProcessingResult Result;

  Result=stPassOn;
  if (Arg[0]=='-') {
    Result=stProcessed;
    switch (Arg[1]) {
      case 'w': {
        if (Arg[2]!='\0') return(stInvalid);
        TreatWarningsAsErrors=true;
        break;
      }
      default: {
        Result=stPassOn;
        break;
      }
    }
  }
  return(Result);
}

const char * const *ProcessCompilerOptions(const char * const *argv)
{
  TreatWarningsAsErrors=false;
  return(ProcessOptions(ProcessCompilerOption,argv));
}
