#ifndef __h__SYSLIB_PDINET__
#define __h__SYSLIB_PDINET__

/*
  The Internet Network is accessed through the packet device "INET".
*/

/*
  Error codes for the INET device
*/
#define ineteNOHOST	-0x0403	/* The host is unknown */
#define ineteNOADDR	-0x0404	/* The host has no address */
#define ineteUNSERR	-0x0405	/* Unrecoverable nameserver error */
#define ineteTMPERROR	-0x0406	/* Temporary nameserver error */
#define ineteNRERR	-0x0407	/* Unknown error during name resolution */
#define inetePROTOSUP	-0x0408	/* Protocol not supported */
#define ineteAFSUP	-0x0409	/* Adress family not supported */
#define ineteTOOMANY	-0x040A	/* Too many open sockets */
#define ineteSACCESS	-0x040B	/* Access denied to the socket */
#define ineteADDRACC	-0x040D	/* Access to adress denied */
#define ineteADDRUSE	-0x040E	/* Adress in use by another socket */
#define ineteSSETUP	-0x040F	/* Error performing socket setup */
#define ineteCMDSUP	-0x0410	/* Command not supported */
#define ineteMISNULL	-0x0411	/* Name is not null-terminated */
#define ineteMADDRS	-0x0412	/* Too many adresses returned */
#define ineteADDRFMT	-0x0413	/* Invalid host adresses format */
#define ineteREFUSED	-0x0414	/* Connection refused */
#define ineteTIMEOUT	-0x0415	/* Connection timed out */
#define ineteUNREACH	-0x0416	/* Network unreachable */
#define ineteCACCESS	-0x0417	/* Connection access denied */
#define ineteCERROR	-0x0418	/* Unknown connection error */
#define ineteNORESOLV	-0x0419	/* No address resolution in progress */
#define ineteSDOWN      -0x041A /* Socket is down */

/*
  It is IPv6 enabled. If the address we want to store here is an IPv4 one,
  for example A.B.C.D, the address is filled as follows:
  
    Host[0..4]=0;
    Host[5]=0xFFFF;
    Host[6]=A*16+B;
    Host[7]=C*16+D;

  An IPv6 address is stored as a tuple of 8 16-bit value into the array. The
  first number of the adress goes into Host[0] and the last one into Host[7].
*/
typedef upint16 THostAddr[8];

/*
  Open a TCP server socket

  This command opens a TCP server socket listening on the specified port and
  using the specified size of accept queue.

  The command if successfull returns a socket. Connections can be obtained
  from this socket using the Read() call. Each connection produces one
  TConnectionRecord.
*/

#define inetcmdOpenTCPServer	0

typedef struct {
  upint8 Command;
  upint16 Port;
  pint16 QueueSize;
} TOpenTCPServerCommand;

typedef struct {
  pint16 Handle;
} TOpenTCPServerReply;

typedef struct {
  pint16 HRead;
  pint16 HWrite;
} TOpenTCPClientReply;

typedef struct {
  pint16 HRead;
  pint16 HWrite;
  THostAddr Host;
  upint16 Port;
} TConnectionRecord;

/*
  Open a TCP client socket

  Opens a client socket connected to the specified server.

  The command, if successfull, returns TOpenTCPReply message with the handle
  of the socket connected to the specified machine. The handle is emulating
  a nonseekable file handle.
*/

#define inetcmdOpenTCPClient	1

typedef struct {
  upint8 Command;
  THostAddr Host;
  upint16 Port;
} TOpenTCPClientCommand;

/*
  Resolve a host name
  
  This command resolves a hostname using platform specific means. If
  successfull, the driver replies with sequence of TOpenTCPClient structures,
  one for each of the adresses. The sequence is terminated by an empty record
  (a record with 0 bytes). The command byte of each structure is set to zero,
  which is both, the success indicator and the command to open a TCP client
  connection to the specified host. To complete the TCP connection request
  the caller needs only to write the port number into the structure.

  On failure the driver returns an error message.
  
  The driver is required to return at least one reply structure. If the host
  has no address, the driver must respond with ineterrNOADDR error message.
*/

#define inetcmdResolveName	2
#define inetcmdNextAddress	3

typedef struct {
  upint8 Command;
  char Name[MAX_MESSAGE_SIZE-1];
} TResolveNameCommand;

#endif
