#include "defs.gh"

#include <stdio.h>
#include <time.h>
#if __e__SLEEP_DRV__ == __v__posix__
#include <errno.h>
#endif
#if __e__SLEEP_DRV__ == __v__bcmsdos__
#include <dos.h>
#endif
#if __e__SLEEP_DRV__ == __v__sleep__
#include <unistd.h>
#endif
#if __e__SLEEP_DRV__ == __v__usleep__
#include <unistd.h>
#endif
#if __e__SLEEP_DRV__ == __v__usleep2__
#include <unistd.h>
#endif
#if __e__SLEEP_DRV__ == __v__windows__
#include <windows.h>
#endif
#if __e__SLEEP_DRV__ == __v__select__
#include <sys/types.h>
#endif
#if __e__SLEEP_DRV__ == __v__apollo__
#include <apollo/base.h>
#include <apollo/time.h>
#endif
#undef __t__USE_POLL__
#define __t__USE_POLL__ 0
#if __e__SLEEP_DRV__ == __v__poll__
#include <poll.h>
#undef __t__USE_POLL__
#define __t__USE_POLL__ 1
#elif __e__SLEEP_DRV__ == __v__syspoll__
#include <sys/poll.h>
#undef __t__USE_POLL__
#define __t__USE_POLL__ 1
#endif

#include "types.h"
#include "traps.h"

#define __INTERFACE__
#include "time.h"
#undef __INTERFACE__

#include "typesnd.h"

int32 GetTimeShift(void)
/*
  TYPE: Platform Dependent Public Code

  PURPOSE: Determine the UTC<->Localtime conversion constant

  This code determines the shift which must be added to convert the UTC
  timestamp to local timestamp. Subtract the value returned if you want to
  convert the local timestamp back to UTC.
*/
{
  int32 Shift;
#if __e__TIMESHIFT__ == __v__always_zero__
  Shift=0;
#elif __e__TIMESHIFT__ == __v__localtime__
  struct tm *Ptr;
  time_t Time;

  Time=3600*12;
  Ptr=localtime(&Time);
  Shift=12-(Ptr->tm_hour*3600+Ptr->tm_min*60+Ptr->tm_sec);
#elif __e__TIMESHIFT__ == __v__thread_safe__
  /*
  struct {
    int Sec;
    int Min;
    int Hour;
    int Day;
    int Month;
    int Year;
    int WDay;
    int YDay;
    int IsDST;
  } Time;
  */
  struct tm Time;
  /*
  time_t UTime;

  Time.tm_sec=0;
  Time.tm_min=0;
  Time.tm_hour=0;
  Time.tm_mday=2;
  Time.tm_mon=1;
  Time.tm_year=1970;
  Time.tm_isdst=0;
  UTime=mktime(&Time);
  */
  Shift=0;
  localtime_r(&Shift,&Time);
  Shift=Time.tm_gmtoff;
#endif
  return(Shift);
}

uint64 GetTime(void)
{
  return(time(NULL));
}

/*** DelayThread the process execution to the specified number of microseconds */
void DelayThread(uint64 MicroSeconds)
{
#if __e__SLEEP_DRV__ == __v__posix__
  struct timespec Time1,Time2;
  struct timespec *Req,*Rem,*Tmp;
  int16 Result;

  Time1.tv_nsec=(MicroSeconds % 1000000)*1000;
  Time1.tv_sec=MicroSeconds/1000000;
  Req=&Time1;
  Rem=&Time2;
  do {
    Result=nanosleep(Req,Rem);
    if (Result==-1) {
      if (errno==EINTR) {
        Tmp=Req;
	Req=Rem;
	Rem=Tmp;
      } else {
        Result=0;
      }
    }
  } while (Result==-1);
#elif __e__SLEEP_DRV__ == __v__sleep__
  uint32 Seconds;

  Seconds=MicroSeconds/1000000;
  while (Seconds>0) {
    Seconds=sleep(Seconds);
  }
#elif __e__SLEEP_DRV__ == __v__windows__
  Sleep(MicroSeconds/1000);
#elif __e__SLEEP_DRV__ == __v__select__
  struct timeval tv;

  tv.tv_sec = MicroSeconds / 1000000;
  tv.tv_usec = MicroSeconds % 1000000;
  select(0, 0, 0, 0, &tv);
#elif __e__SLEEP_DRV__ == __v__bcmsdos__
  delay(MicroSeconds/1000);
#elif __e__SLEEP_DRV__ == __v__usleep__
  usleep(MicroSeconds);
#elif __e__SLEEP_DRV__ == __v__usleep2__
  if (MicroSeconds>1000000) {
    sleep(MicroSeconds/1000000);
    MicroSeconds%=1000000;
  }
  if (MicroSeconds>0) {
    if (usleep(MicroSeconds)!=0) {
      ReportBug("usleep() delay failed");
    }
  }
#elif __e__SLEEP_DRV__ == __v__apollo__
  /*
    FIXME: Do these vars need to be static on Domain/OS ???
  */
  static time_$clock_t	DomainDelay;
  static status_$t	DomainStatus;

  /*
    DomainDelay is a 48 bit value that defines how many 4uS periods to
    delay.  Since the input value range is 32 bits, the upper 16 bits of
    DomainDelay must be zero.  So we just divide the input value by 4 to
    determine how many "ticks" to wait
  */
  DomainDelay.c2.high16 = 0;
  DomainDelay.c2.low32 = usec / 4;
  time_$wait (time_$relative, DomainDelay, &DomainStatus);
#endif
#if __t__USE_POLL__
  if (poll(0, 0, MicroSeconds/1000) < 0) {
    ReportBug("poll() delay failed");
  }
#endif
}
