#include "defs.gh"

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>

#define __INTERFACE__
#include "types.h"
#undef __INTERFACE__

#include "memory.h"
#include "strings.h"
#include "traps.h"
#include "typesnd.h"

void KINT__ReportFileAndLine(const char *File,int16 Line)
{
  fprintf(stderr,"File:\t%s\nLine:\t%d\n\n",File,Line);
  abort();
}

void KINT__ReportUndefinedSituation(const char *Msg)
{
  fprintf(stderr,
    "--- UNDEFINED SITUATION TRAP ACTIVATED ---\n"
    "Reason:\t%s\n",
    Msg
  );
}

void KINT__ReportDetectedBug(const char *Msg)
{
  fprintf(stderr,
    "--- BUG DETECTION SYSTEM TRAP ACTIVATED ---\n"
    "Reason:\t%s\n",
    Msg
  );
}

void KINT__ReportAssertFail(const char *Expr)
{
  fprintf(stderr,
    "--- ASSERTION FAILED ---\n"
    "Cond:\t%s\n",
    Expr
  );
}

void KINT__ReportInternalError(const char *Msg)
{
  fprintf(stderr,
    "--- UNRECOVERABLE INTERNAL ERROR ---\n"
    "Error:\t%s\n",
    Msg
  );
}

void KINT__ReportFatalInitError(const char *Msg)
{
  fprintf(stderr,
    "--- FATAL INITIALIZATION ERROR ---\n"
    "Error:\t%s\n",
    Msg
  );
  if (errno) perror("System complains");
#ifndef __DEBUG__
  abort();
#endif
}

#define MAX_NESTED_DATABLOCKS 16

int16 KINT__DBCounter;
static bool Alloc;
static upint8 *WPtr;

typedef struct {
  int16 DBCounter;
  void *WPtr;
} TStackRecord;

static TStackRecord Stack[MAX_NESTED_DATABLOCKS];
static uint16 SP;

void KINT__LockDataBlockDriver(void)
{
}

void KINT__StartDataBlockGenerator(void)
{
  TStackRecord *R;

  if (Alloc) {
    assert(SP<MAX_NESTED_DATABLOCKS);
    R=Stack+SP;
    SP++;
    R->DBCounter=KINT__DBCounter;
    R->WPtr=WPtr;
  }
  WPtr=NULL;
  Alloc=true;
}

void KINT__UnlockDataBlockDriver(void)
{
}

void KINT__StopDataBlockGenerator(void)
{
  TStackRecord *R;

  if (SP>0) {
    SP--;
    R=Stack+SP;
    KINT__DBCounter=R->DBCounter;
    WPtr=R->WPtr;
    Alloc=true;
  }
}

static void WriteMem(const void *Data,uint16 Size)
{
  if (KINT__DBCounter==1) MemCpy(WPtr,Data,Size);
  WPtr+=Size;
}

void *KINT__AllocateDataBlock(void)
{
  uint32 Size;
  upint32 *Ptr;

  if (KINT__DBCounter==1 && Alloc) {
    Size=WPtr-(upint8 *)NULL;
    Ptr=KMEM__malloc((int16)(Size+4));
    if (Ptr==NULL) InternalError("No memory for generated datablock");
    *(Ptr++)=Size;
    WPtr=(void *)Ptr;
    Alloc=false;
    return(WPtr);
  } else {
    return(&WPtr);
  }
}

void KINT__AddIntoStream(int16 Type,void *Data)
{
  upint8 *R;
  int16 *Ptr;
  int16 ReaderAddSize;
  upint8 Vi8u;
  pint8 Vi8;
  upint16 Vi16u;
  pint16 Vi16;
  upint32 Vi32u;
  pint32 Vi32;
  void *Vptr;

  R=(void *)&Data;
  Ptr=(void *)R;
  ReaderAddSize=0;
  switch (Type) {
    case 0:
    case 1:
    case 2:
    case 3: ReaderAddSize=sizeof(int16); break;
    case 4:
    case 5: ReaderAddSize=sizeof(int32); break;
    case 6:
    case 7: ReaderAddSize=sizeof(void *); break;
  }

  while (
    *(Ptr+ 0)!= -1 || *(Ptr+ 1)!= -1 || *(Ptr+ 2)!= -1 || *(Ptr+ 3)!= -1 ||
    *(Ptr+ 4)!= 84 || *(Ptr+ 5)!= 83 || *(Ptr+ 6)!= 72 || *(Ptr+ 7)!= 32 ||
    *(Ptr+ 8)!=121 || *(Ptr+ 9)!= 98 || *(Ptr+10)!= 32 || *(Ptr+11)!= 83 ||
    *(Ptr+12)!= 72 || *(Ptr+13)!= 83 || *(Ptr+14)!= 79 || *(Ptr+15)!= 32 ||
    *(Ptr+16)!=115 || *(Ptr+17)!=101 || *(Ptr+18)!=115 || *(Ptr+19)!= 85 ||
    *(Ptr+20)!= -1 || *(Ptr+21)!= -1 || *(Ptr+22)!= -1 || *(Ptr+23)!= -1
  ) {
    switch (Type) {
      case 0: {
        Vi8=*((int16 *)R);
        WriteMem(&Vi8,sizeof(Vi8));
        break;
      }
      case 1: {
        Vi8u=*((uint16 *)R);
        WriteMem(&Vi8u,sizeof(Vi8u));
        break;
      }
      case 2: {
        Vi16=*((int16 *)R);
        WriteMem(&Vi16,sizeof(Vi16));
        break;
      }
      case 3: {
        Vi16u=*((uint16 *)R);
        WriteMem(&Vi16u,sizeof(Vi16u));
        break;
      }
      case 4: {
        Vi32=*((int32 *)R);
        WriteMem(&Vi32,sizeof(Vi32));
        break;
      }
      case 5: {
        Vi32u=*((uint32 *)R);
        WriteMem(&Vi32u,sizeof(Vi32u));
        break;
      }
      case 6:
      case 7: {
        Vptr=*((void **)R);
        if (Type==7) {
          WriteMem(Vptr,StrLen(Vptr)+1);
        } else {
          WriteMem(&Vptr,sizeof(Vptr));
        }
        break;
      }
    }
    R+=ReaderAddSize;
    Ptr=(void *)R;
  }
}
