#ifndef __h__SYSLIB_TYPES__
#define __h__SYSLIB_TYPES__

typedef unsigned char upint8;
typedef unsigned short int upint16;
typedef unsigned long int upint32;
typedef unsigned long long upint64;

typedef signed char pint8;
typedef signed short int pint16;
typedef signed long int pint32;
typedef signed long long pint64;

typedef unsigned int uint8;
typedef unsigned int uint16;
typedef unsigned long int uint32;
typedef unsigned long long uint64;

typedef signed int int8;
typedef signed int int16;
typedef signed long int int32;
typedef signed long long int64;

enum boolvals { false, true };
typedef enum boolvals bool;

#ifndef NULL
#define NULL (void *)0L
#endif

#define ptr2int(X) ((uint32)(X))

/*
  The dim() takes an array and returns number of components in the array.
  The itemsize() takes an array and returns the size of the components in
  the array.
*/
#define itemsize(v) sizeof((v)[0])
#define dim(v) (sizeof(v)/itemsize(v))

/*
  Avoid using "int", "signed" etc in programs using SYSLIB. This trap is
  kludgy: the type names are redefined to expand to syntactically incorrect
  string so the compiler is fooled to think that these reserved words are not
  allowed at all. Problem with this kludge is that nothing may be defined
  using "typedef" for these keywords but this is not really a problem.
*/
#define int #bad#
#define signed #bad#
#define unsigned #bad#
#define short #bad#
#define long #bad#

/*
  Disallow the user  to declare main() function. That would break the
  library. OSHS software has Main() as main function. Uses the same kludgy
  trick than the int,signed,unsigned,etc disabler above.
*/
#define main #bad#

/*
  Declare assert() here. The assert() directive which is implemented as a
  macro in standard C is a part of the core language in EPL.
*/

void KINT__ReportFileAndLine(const char *File,int16 Line);
void KINT__ReportInternalError(const char *Msg);

#ifdef __NO_DEBUG__
#undef __DEBUG__
#else
#define __DEBUG__ 1
#endif
#ifndef __DEBUG__
#define assert(x)
#else

void KINT__ReportAssertFail(const char *Expr);
void KINT__ReportUndefinedSituation(const char *Msg);
void KINT__ReportDetectedBug(const char *Msg);

#define assert(x) \
  if (!(x)) {						\
    KINT__ReportAssertFail(#x);				\
    KINT__ReportFileAndLine(__FILE__,__LINE__);		\
  }

#endif

#if __t__SYSLIB_internals__
void KINT__ReportFatalInitError(const char *Msg);
#endif

/*
  ESPL core language users can declare strings that need to be
  internationalized using the m"string" format (i.e. prepend the string with
  the "m" character which stands for "message"). Note that there must be no
  space between the "m" character and the string. Such a token will be parsed
  as two tokens by standard C compiler: an identifier with the value "m" and
  a string. So to allow this ESPL core language feature being compiled by
  standard C compiler force the "m" identifier to expand to nothing by the
  C preprocessor:
*/
#define m

/*
  P language uses "private" in structures to hide specific fields from the
  public. To enable C compilers to compile such structure force "private" to
  expand to nothing.
*/
#define private

/*
  The 'pubconst' and 'privconst' are allowed only in headers. They are
  defined to 'pubconst=<empty>' and 'privconst=const' in the header included
  into its module and vice versa in other headers.
*/
#define pubconst #bad#
#define privconst #bad#

/*
  These symbols are used as a stack where the 'extern' states are kept. This
  stack is required due to the fact that some headers are nested.
*/
#define __t__extern5__ 1
#define __t__extern4__ 1
#define __t__extern3__ 1
#define __t__extern2__ 1
#define __t__extern1__ 1

/*
  The following things were suggested and designed by The Holy Spirit. I did
  only the hard work of coding His ideas in C.
*/

/*
  The "datablock" keyword support. In C compatible mode of the P language
  datablocks must be defined by this syntactical construction:

  datablock(<name>)
    <definition>
  enddatablock

  where <definition> is a set of clauses from which each one has one of these
  forms:

    streami8 <data> endstream
    streami8u <data> endstream
    streami16 <data> endstream
    streami16u <data> endstream
    streami32 <data> endstream
    streami32u <data> endstream
    streamptr <data> endstream
    streamstr <data> endstream

  where the "streamXXX" start the stream and "endstream" ends it.

  The extdatablock(name) can be used to declare an external datablock in a
  header. This datablock must be supplied by the module to which the header
  belongs.

  The datasize() operator can be used to obtain a size of the datablock.
  Works even with the external datablocks.
*/

#define i__EnterDataBlock__i(Name,DataBlockClass) \
extern int16 KINT__DBCounter;                                              \
void KINT__LockDataBlockDriver(void);                                      \
void KINT__StartDataBlockGenerator(void);                                  \
void *KINT__AllocateDataBlock(void);                                       \
void KINT__StopDataBlockGenerator(void);                                   \
void KINT__UnlockDataBlockDriver(void);                                    \
DataBlockClass void *Name(void)                                            \
{                                                                          \
  static void *Ptr=NULL;                                                   \
                                                                           \
  if (Ptr==NULL) {                                                         \
    KINT__LockDataBlockDriver();                                           \
    if (Ptr!=NULL) {                                                       \
      KINT__UnlockDataBlockDriver();                                       \
      return(Ptr);                                                         \
    }                                                                      \
    KINT__StartDataBlockGenerator();                                       \
    for (KINT__DBCounter=0;KINT__DBCounter<2;KINT__DBCounter++) {          \
      Ptr=KINT__AllocateDataBlock();

#define datablock(Name) i__EnterDataBlock__i(Name,m)
#define staticdatablock(Name) i__EnterDataBlock__i(Name,static)

#define enddatablock \
    }                                                                      \
    KINT__StopDataBlockGenerator();                                        \
    KINT__UnlockDataBlockDriver();                                         \
  }                                                                        \
  return(Ptr);                                                             \
}

#define extdatablock(Name) void *Name(void)

#ifndef __INTERFACE__
void KINT__AddIntoStream(int16 Type,...);
#endif

#define PutIntoStream__macro__(Type) \
  KINT__AddIntoStream(Type,

#define endstream ,-1,-1,-1,-1,84,83,72,32,121,98,32,83,72,83,79,32,115,\
101,115,85,-1,-1,-1,-1);

#define streami8 PutIntoStream__macro__(0)
#define streami8u PutIntoStream__macro__(1)
#define streami16 PutIntoStream__macro__(2)
#define streami16u PutIntoStream__macro__(3)
#define streami32 PutIntoStream__macro__(4)
#define streami32u PutIntoStream__macro__(5)
#define streamptr PutIntoStream__macro__(6)
#define streamstr PutIntoStream__macro__(7)

#define datasize(x) (*(((upint32 *)(x()))-1))

/*
  The bitfields declaration support. The format of the declaration is as
  follows:
  
    bitsdef(Name,Type)
      <declarations>
    endbitsdef(Name)

  - The Name stated in "bitsdef" and "endbitsdef" must be the same.

  <declarations> is one or more bitfield declaration. A single bit bitfield
  declaration is:

    bitmask(Name,Value)
  
  - Expression composed of any previous bitmask may be used as Value.

  and a multibit bitfield declaration is:

    bitfield
      bitmask(Name1,Value1)
      bitmask(Name2,Value2)
        ...
      bitmask(NameN,ValueN)
    endbitfield(Name,Name1|Name2|...|NameN)

  - The second argument must contain all the names declared in the bitfield
    and all these names must be listed here in the same order as in the
    declaration itself. The names must be separated by the "|" character.

  Access to the content can be performed using GET or SET:

  - get(Value,Flag) extracts the value of the Flag from Value
  - set(LValue,Flag,NewValue) sets the value of the Flag in the LValue to
    NewValue.

  For single bit flags the get() returns false if the flag is clear and true
  if it is set.

  Each Flag must be a single flag taken from the declaration. For single bit
  bitfields the NewValue is treated as boolean value. If it evaulates to 0,
  it will unset the flag otherwise it will set the flag. For multiple bit
  bitfields the Value must be evaulated to 0 or any bitmasks defined for it,
  otherwise the effect of the set() command is unspecified.
*/

#define bitsdef(Name,Type) typedef enum {
#define bitfield
#define endbitfield(Name,Value) Name=(Value),
#define bitmask(Name,Value) Name=(Value),
#define endbitsdef(Name) } Name;

#define get(Value,Flag) \
(                                                                       \
  (                                                                     \
    Flag==1 || Flag==1<<1 || Flag==1<<2 || Flag==1<<3 || Flag==1<<4 ||  \
    Flag==1<<5 || Flag==1<<6 || Flag==1<<7 || Flag==1<<8 ||             \
    Flag==1<<9 || Flag==1<<10 || Flag==1<<11 || Flag==1<<12 ||          \
    Flag==1<<13 || Flag==1<<14 || Flag==1<<15 || Flag==1<<16 ||         \
    Flag==1<<17 || Flag==1<<18 || Flag==1<<19 || Flag==1<<20 ||         \
    Flag==1<<21 || Flag==1<<22 || Flag==1<<23 || Flag==1<<24 ||         \
    Flag==1<<25 || Flag==1<<26 || Flag==1<<27 || Flag==1<<28 ||         \
    Flag==1<<29 || Flag==1<<30 || Flag==1U<<31                          \
  ) ? (                                                                 \
    ((Value&Flag)!=0) ? true : false                                    \
  ) : (                                                                 \
    Value&Flag                                                          \
  )                                                                     \
)

#define set(LValue,Flag,NewValue) {                                     \
  LValue&=~(Flag);                                                      \
  if (                                                                  \
    Flag==1 || Flag==1<<1 || Flag==1<<2 || Flag==1<<3 || Flag==1<<4 ||  \
    Flag==1<<5 || Flag==1<<6 || Flag==1<<7 || Flag==1<<8 ||             \
    Flag==1<<9 || Flag==1<<10 || Flag==1<<11 || Flag==1<<12 ||          \
    Flag==1<<13 || Flag==1<<14 || Flag==1<<15 || Flag==1<<16 ||         \
    Flag==1<<17 || Flag==1<<18 || Flag==1<<19 || Flag==1<<20 ||         \
    Flag==1<<21 || Flag==1<<22 || Flag==1<<23 || Flag==1<<24 ||         \
    Flag==1<<25 || Flag==1<<26 || Flag==1<<27 || Flag==1<<28 ||         \
    Flag==1<<29 || Flag==1<<30 || Flag==1U<<31                          \
  ) {                                                                   \
    if ((NewValue)!=0) LValue|=Flag;                                    \
  } else {                                                              \
    LValue|=(NewValue);                                                 \
  }                                                                     \
}

#endif
