/*
  Test: Computations up to 32 bit

  Computations with integers of different length. The compiler is required to
  use the largest range of all types during all computations. Some buggy
  compilers will truncate some results when using different sizes for
  arguments and for the result.
*/

#include "types.h"
#include "conio.h"

void Complain(const char *Msg)
{
  PrintSTDOUT(Msg);
  PrintSTDOUT("\n");
}

int16 Main(void)
{
  pint8 Num08A;
  pint8 Num08B;
  pint16 Num16A;
  pint16 Num16B;
  pint32 Num32A;
  pint32 Num32B;
  pint8 Res08;
  pint16 Res16;
  pint32 Res32;

  /*
    Load common constants.
  */
  Num08A=0x12;
  Num08B=0x34;
  Num16A=0x1234;
  Num16B=0x2030;
  Num32A=0x12345678;
  Num32B=0x20304050;
  /*
    8 bit operands with 16 bit result:

    1. 0x0012 * 0x0034 = 0x03A8
  */
  Res16=Num08A*Num08B;
  if (Res16!=0x3A8) Complain("8 bit ops; 16 bit result");
  /*
    16 bit expression with 16 bit result:

    1. 0x0012 * 0x0034 = 0x03A8
  */
  Res08=(Num16A+Num16B) / 0x1000;
  if (Res08!=0x3) Complain("16 bit ops; 8 bit result");
  /*
    32 bit expression with 16 bit result:

    1. 0x12345678 + 0x20304050 = 0x326496C8
    2. 0x326496C8 / 0x00100000 = 0x00000326
  */
  Res16=(Num32A+Num32B) / 0x100000;
  if (Res16!=0x326) Complain("32 bit ops; 16 bit result");
  /*
    16 bit expression with 32 bit result.
    
    1. 0x00001234 * 0x00002030 = 0x0249E9C0
  */
  Res32=Num16A*Num16B;
  if (Res32!=0x249E9C0) Complain("16 bit ops; 32 bit result");
  /*
    The end.
  */
  return(0);
}
