#include "types.h"
#include "kernel.h"
#include "pdfsys.h"
#include "conio.h"
#include "memory.h"
#include "strings.h"
#include "time.h"

#include "complain.i"
#include "fileio.i"

uint32 BuildDir(const char *Dir)
{
  uint64 T;
  uint64 T1;
  bool Display;

  T=GetTime();
  Display=true;
  do {
    if (!NewMkDir(Dir)) break;
    T1=GetTime();
    if (T1!=T) {
      NewRmDir(Dir);
      if (!NewMkDir(Dir)) break;
      T=GetTime();
      if (T!=T1) {
        PrintSTDOUT("The directory create operation lasted too long\n");
	break;
      }
    }
    Display=false;
  } while (0);
  if (Display) {
    PrintSTDOUT("Directory: ");
    PrintSTDOUT(Dir);
    PrintSTDOUT("\n");
  }
  return(T);
}

bool CreateFile(const char *File,int16 Size)
{
  char Buffer[4096];
  int16 fd;
  bool Res;

  fd=NewOpen(File,onCREAT+oWRONLY);
  if (fd==-1) {
    Complain("Unable to build test file");
    return(false);
  }
  if (NewWrite(fd,Buffer,Size)!=Size) {
    Complain("Error writing test file");
    return(false);
  }
  Res=NewClose(fd);
  if (!Res) Complain("Error closing test file");
  return(Res);
}

uint32 BuildFile(const char *File,int16 Size)
{
  uint64 T;
  uint64 T1;
  bool Display;

  Display=true;
  do {
    T=GetTime();
    if (!CreateFile(File,Size)) break;
    T1=GetTime();
    if (T1!=T) {
      NewRemove(File);
      if (!CreateFile(File,Size)) break;
      T=GetTime();
      if (T!=T1) {
        PrintSTDOUT("The file create operation lasted too long\n");
	break;
      }
    }
    Display=false;
  } while (0);
  if (Display) {
    PrintSTDOUT("File: ");
    PrintSTDOUT(File);
    PrintSTDOUT("\nSize: ");
    PrintNum(Size);
    PrintSTDOUT("\n");
  }
  return(T);
}

#define MAX_ITEMS 1024

int16 Test(bool TimesToo)
{
  int16 Dir;
  TDirItem *Items[MAX_ITEMS];
  TDirItem *Item;
  uint16 ItemCnt;
  int16 i;
  TFileInfo I;
  int16 IsSize;
  uint64 Times[10];
  uint64 CurrentTime;

  /*
    Activate the temporary drive
  */
  if (!NewSetCurrentDrive('T')) Complain("Cannot activate the test drive");
  /*
    Create test directory structure.
  */
  if (!NewMkDir("TEST")) return(1);
  Times[0]=BuildDir("TEST\\001");
  Times[1]=BuildDir("TEST\\002");
  Times[2]=BuildFile("TEST\\003",1234);
  Times[3]=BuildDir("TEST\\004");
  Times[4]=BuildFile("TEST\\ABC.BIN",21);
  Times[5]=BuildFile("TEST\\DEF.OBJ",121);
  Times[6]=BuildDir("TEST\\FINALIZE");
  Times[7]=BuildFile("TEST\\OK.BIN",999);
  Times[8]=BuildFile("TEST\\TEST",333);
  Times[9]=BuildFile("TEST\\ZERO",0);
  /*
    Open the test directory. Report any errors.
  */
  Dir=NewOpenDir("TEST");
  if (Dir==-1) {
    Complain("Unable to open the testing directory");
    return(1);
  }
  /*
    Read the items from the current directory into the memory. Put their list
    in the sorted order into "Items".
  */
  ItemCnt=0;
  for (;;) {
    Item=KMEM__malloc(sizeof(*Item));
    if (Item==NULL) {
      Complain("Error allocating memory for directory item");
      break;
    }
    if (!NewReadItem(Dir,Item)) {
      Complain("Error reading directory item");
      break;
    }
    if (Item->ID==-1) break;
    if (
      StrCmp(Item->Name,"CVS")==0 ||
      StrCmp(Item->Name,"A.OUT")==0 ||
      StrCmp(Item->Name,"A.CE")==0 ||
      StrCmp(Item->Name,"A.CO")==0 ||
      StrCmp(Item->Name,"A.ES")==0 ||
      StrCmp(Item->Name,"SYSLIB")==0 ||
      StrCmp(Item->Name,"A.OS")==0
    ) continue;
    i=ItemCnt;
    ItemCnt++;
    while (i>0) {
      if (StrCmp(Item->Name,Items[i-1]->Name)>0) break;
      Items[i]=Items[i-1];
      i--;
    }
    Items[i]=Item;
  }
  /*
    Dump the items to stdout. Show some informations about each file.
  */
  for (i=0;i<ItemCnt;i++) {
    /*
      Fetch the current time from the time array.
    */
    CurrentTime=-1;
    if (i<10) CurrentTime=Times[i];
    /*
      Fetch the item being displayed.
    */
    Item=Items[i];
    if (!NewReadFileInfo(Item->ID,&I)) {
      Complain("Can't get informations about item");
      PrintSTDOUT("Item: ");
      PrintSTDOUT(Item->Name);
      NewLine();
      continue;
    }
    /*
      Display the file name.
    */
    PrintSTDOUT(Item->Name);
    PrintSTDOUT("\n");
    /*
      Display the type of the file.
    */
    IsSize=0;
    switch (I.Attr & fmTYPE) {
      case fmREGULAR:	PrintSTDOUT("Reg"); IsSize=1; break;
      case fmDIRECTORY:	PrintSTDOUT("Dir"); break;
      case fmSYMLINK:	PrintSTDOUT("Lnk"); break;
      case fmFIFO:	PrintSTDOUT("FIFO"); break;
      case fmCHARDEV:	PrintSTDOUT("ChDev"); break;
      case fmBLOCKDEV:	PrintSTDOUT("BlkDev"); break;
      default: PrintSTDOUT("???"); break;
    }
    /*
      After the attributes display their numerical value.
    */
    if (TimesToo) {
      PrintSTDOUT("\t");
      PrintSTDOUT(I.CreatedTime==CurrentTime?"OK":"Bad");
      PrintSTDOUT("\t");
      PrintSTDOUT(I.ModifiedTime==CurrentTime?"OK":"Bad");
    }
    PrintSTDOUT("\t");
    if (IsSize) {
      PrintNum(I.Size);
    } else {
      PrintSTDOUT("???");
    }
    NewLine();
  }
  /*
    Close the current directory.
  */
  if (!NewCloseDir(Dir)) {
    Complain("Error closing the testing directory");
    return(-1);
  }
  /*
    Destroy the test directory structure.
  */
  NewRmDir("TEST\\001");
  NewRmDir("TEST\\002");
  NewRemove("TEST\\003");
  NewRmDir("TEST\\004");
  NewRemove("TEST\\abc.bin");
  NewRemove("TEST\\def.obj");
  NewRmDir("TEST\\finalize");
  NewRemove("TEST\\ok.bin");
  NewRemove("TEST\\test");
  NewRemove("TEST\\zero");
  NewRmDir("TEST");
  /*
    Terminate.
  */
  return(0);
}
