/*
  Test: The directory path following
*/

#include "types.h"
#include "conio.h"
#include "process.h"
#include "strings.h"
#include "kernel.h"
#include "pdfsys.h"
#include "kernmsg.h"
#include "pdfileio.h"
#include "diskdata.h"
#include "disklett.h"
#include "diskspec.h"

#include "complain.i"

char Example[2];

void ExError(char *Msg)
{

  PrintSTDOUT("Error in example ");
  PrintSTDOUT(Example);
  PrintSTDOUT("\n");
  PrintSTDOUT(Msg);
  PrintSTDOUT("\n");
}

void NextExample(void)
{
  Example[0]++;
}

void Valid(char *Root,char *Path,char *Answer)
{
  char Spec[MAX_FILESPEC_SIZE+1];

  do {
    Spec[MAX_FILESPEC_SIZE]='\377';
    StrCpy(Spec,Root);
    ErrNo=FollowPath(Path,Spec);
    if (ErrNo<0) {
      ExError("Error following path");
      ComplainSystemSaid();
      break;
    }
    if (StrCmp(Spec,Answer)) {
      ExError("Bad answer");
      break;
    }
    if (Spec[MAX_FILESPEC_SIZE]!='\377') {
      ExError("Buffer overflow");
      break;
    }
  } while (false);
  NextExample();
}

void Invalid(char *Root,char *Path,int16 Answer)
{
  char Spec[MAX_FILESPEC_SIZE+1];

  do {
    Spec[MAX_FILESPEC_SIZE]='\377';
    StrCpy(Spec,Root);
    ErrNo=FollowPath(Path,Spec);
    if (ErrNo>=0) {
      ExError("Unexpected success following path");
      break;
    }
    if (ErrNo!=Answer) {
      ExError("Bad error code returned");
      ComplainSystemSaid();
      break;
    }
    if (Spec[MAX_FILESPEC_SIZE]!='\377') {
      ExError("Buffer overflow");
      break;
    }
  } while (false);
  NextExample();
}

int16 Main(void)
{
  TFileSpec Other;
  uint16 i;

  Example[0]='A';
  Example[1]='\0';
  /*
    Two empty file specifications
  */
  Valid(
    "",
    "",
    ""
  );
  /*
    Valid path
  */
  Valid(
    "test/foo/bar",
    "root/../../other/./another/../../text",
    "test/foo/text"
  );
  /*
    Path leading out of root
  */
  Invalid(
    "test/foo/bar",
    "root/../../../another/../../../text",
    fseNOENT
  );
  /*
    Invalid path containing two consecutive slashes
  */
  Invalid(
    "test/foo/bar",
    "root/..//other/text",
    fseBADNAME
  );
  /*
    Attempt to construct too long path
  */
  for (i=0;i<MAX_FILESPEC_SIZE-1;i++) Other[i]='X';
  Other[MAX_FILESPEC_SIZE-1]='\0';
  Invalid(
    "test/foo/bar",
    Other,
    fseNAMELEN
  );
  /*
    Valid path leading from root
  */
  Valid(
    "",
    "root/../other/./another/../else/../text",
    "other/text"
  );
  /*
    Valid path touching the root
  */
  Valid(
    "test/foo/bar",
    "../../other/../../another/./../text",
    "text"
  );
  /*
    All done OK.
  */
  return(0);
}
