/*
  Test: File seeking operations on small files
*/

#include "types.h"

#define TEST_BUF_SIZE 16384

upint16 Buffer[TEST_BUF_SIZE];
uint16 Position;
int16 FileH;

#include "complain.i"

void FillTestBuffer(void)
{
  int16 i;

  for (i=0;i<TEST_BUF_SIZE;i++) Buffer[i]=i+0x1234;
}

#include "fileio.i"

void ComplainClose(void)
{
  Complain("Error closing test file");
}

int16 OpenTestFile(uint16 Flags)
{
  return(NewOpen("T:\\TEST.BIN",Flags));
}

bool CreateTestFile(void)
{
  int16 fd;
  bool Result;

  Result=false;
  fd=OpenTestFile(onCREAT+oWRONLY+oeTRUNC);
  if (fd==-1) {
    Complain("Error creating test file");
  } else {
    if (NewWrite(fd,Buffer,sizeof(Buffer))!=sizeof(Buffer)) {
      Complain("Error writing test file");
    }
    Result=NewClose(fd);
    if (!Result) ComplainClose();
  }
  return(Result);
}

bool ReopenTestFile(void)
{
  bool Result;

  FileH=OpenTestFile(oRDONLY+oeOPEN);
  Result=FileH!=-1;
  if (!Result) Complain("Error reopening test file");
  return(Result);
}

void Test(uint16 Size)
{
  upint8 TstBuf[TEST_BUF_SIZE*2];
  int16 i;
  upint8 *Src;

  if (NewRead(FileH,TstBuf,Size)!=Size) {
    Complain("Error reading test file");
    return;
  }
  Src=(void *)Buffer;
  Src+=Position;
  for (i=0;i<Size;i++) {
    if (TstBuf[i]!=*(Src++)) {
      Complain("Data mismatch detected");
      return;
    }
  }
  Position+=Size;
}

void TestAbs(uint16 Where,uint16 Size)
{
  if (!NewSeek(FileH,Where)) Complain("Absolute seek error");
  Position=Where;
  Test(Size);
}

void TestRel(int16 Where,uint16 Size)
{
  if (!NewSeekRel(FileH,Where)) Complain("Relative seek error");
  Position+=Where;
  Test(Size);
}

int16 Main(void)
{
  FillTestBuffer();
  if (!CreateTestFile()) return(1);
  if (!ReopenTestFile()) return(1);
  TestAbs( 100,1000);
  TestAbs(1121,3456);
  TestRel(-236,7654);
  TestRel( 345,6789);
  if (!NewClose(FileH)) ComplainClose();
  return(0);
}
