/*
  Test: The classic I/O calls
*/

#include "types.h"
#include "process.h"
#include "kernel.h"
#include "pdfsys.h"
#include "pdfileio.h"
#include "memory.h"
#include "strings.h"

#include "complain.i"
#include "fileio.i"

void Fatal(char *Msg)
{
  PrintSTDOUT(Msg);
  NewLine();
  PerformProgramHalting(255);
}

void CFatal(char *Msg)
{
  Complain(Msg);
  NewLine();
  PerformProgramHalting(255);
}

/* #define DATA_SIZE (64*1024*16) */

#define DATA_SIZE 6000

int32 Seed;

int32 Rand(void)
{
  Seed=(Seed*1103515245+12345)&0x7FFFFFFF;
  return Seed;
}

int32 ClassicRead(int16 fd,void *Buf,int32 Size)
{
  TReadAndSendCmd C;
  TDataCmd R;
  int32 Read;
  int16 ErrNo;

  C.Command=cmdREADANDSEND;
  C.Size=Size;
  Read=0;
  ErrNo=KCALL__SEND(fd,&C,sizeof(C));
  if (ErrNo<0) {
    SetErrNo(ErrNo);
    return(0);
  }
  while (Size>0) {
    ErrNo=KCALL__RECV(fd,&R,sizeof(R),NULL);
    if (ErrNo<0) {
      SetErrNo(ErrNo);
      return(0);
    }
    if (ErrNo==0) return(Read);
    if (ErrNo==1 || R.Command!=cmdDATA) Fatal("Bad data packet");
    ErrNo--;
    if (ErrNo>Size) Fatal("Overread");
    Buf=MemCpy(Buf,R.Data,ErrNo);
    Read+=ErrNo;
    Size-=ErrNo;
  }
  ErrNo=KCALL__RECV(fd,&R,sizeof(R),NULL);
  if (ErrNo<0) {
    SetErrNo(ErrNo);
    return(0);
  } else if (ErrNo>0) {
    Fatal("Badly terminated data from the file");
  }
  return(Read);
}

int32 ClassicWrite(int16 fd,const void *Buf,int32 Size)
{
  TDataCmd C;
  TIOReply R;
  int32 Written;
  TServerInfo I;

  I.Server=fd;
  I.Cmd=&C;
  I.Reply=&R;
  I.RepSize=sizeof(R);
  C.Command=cmdDATA;
  Written=0;
  while (Size>0) {
    I.CmdSize=sizeof(C)-1;
    if (I.CmdSize>Size) I.CmdSize=Size;
    MemCpy(C.Data,Buf,I.CmdSize);
    Buf+=I.CmdSize;
    Size-=I.CmdSize;
    I.CmdSize++;
    if (!SendCmd(&I)) break;
    Written+=R.Size;
    if (R.Size!=I.CmdSize-1) break;
  }
  return Written;
}

int16 Main(void)
{
  void *Buffer;
  void *Buf2;
  pint32 *Ptr;
  pint32 *Ptr1;
  int16 fd;
  int32 SizeToIO;
  int32 Size;
  int32 i;
  int32 Count;

  Buffer=KMEM__hugemalloc(DATA_SIZE);
  Buf2=KMEM__hugemalloc(DATA_SIZE);
  if (Buffer==NULL || Buf2==NULL) Fatal("Out of memory");

  Count=DATA_SIZE/4;
  Ptr=Buffer;
  for (i=0;i<Count;i++) *(Ptr++)=Rand();
  Ptr=Buf2;
  for (i=0;i<Count;i++) *(Ptr++)=Rand();

  fd=NewOpen("T:/pokus.bin",oWRONLY+oeTRUNC+onCREAT);
  if (fd==-1) CFatal("Error creating test file");
  SizeToIO=DATA_SIZE/3;
  Size=ClassicWrite(fd,Buffer,SizeToIO);
  if (Size!=SizeToIO) CFatal("Error writing data into the test file");
  SizeToIO=DATA_SIZE-SizeToIO;
  Size=ClassicWrite(fd,Buffer+DATA_SIZE/3,SizeToIO);
  if (Size!=SizeToIO) CFatal("Error writing data into the test file");
  if (!NewClose(fd)) CFatal("Error closing file after writing");

  fd=NewOpen("T:/pokus.bin",oRDONLY+oeOPEN);
  if (fd==-1) CFatal("Error opening just created test file");
  SizeToIO=DATA_SIZE/2;
  Size=ClassicRead(fd,Buf2,SizeToIO);
  if (Size!=SizeToIO) CFatal("Error reading test data back");
  SizeToIO=DATA_SIZE-SizeToIO;
  Size=ClassicRead(fd,Buf2+DATA_SIZE/2,SizeToIO);
  if (Size!=SizeToIO) CFatal("Error reading test data back");
  if (!NewClose(fd)) CFatal("Error closing file after reading");

  Ptr=Buffer;
  Ptr1=Buf2;
  for (i=0;i<Count;i++) {
    if (*(Ptr++)!=*(Ptr1++)) Fatal("Data error");
  }

  return(0);
}
