/*
  Test: Opening directory as a file

  If a directory is attempted to be opened with file open command and the
  directory is visible to the user, the fseISDIR error must be returned even
  if the user does not have the rights to read the directory. Returning
  fseACCESS or something similar in such a situation is wrong. This also
  checks the ChMod command.
*/

#include "types.h"
#include "conio.h"
#include "kernel.h"
#include "pdfsys.h"
#include "strings.h"

#define TEST_DIR "test"

#include "complain.i"
#include "fileio.i"

void CheckTheDirectory(void)
{
  int16 fd;

  /*
    Attempt to open it as a file for reading.
  */
  fd=NewOpen(TEST_DIR,oeOPEN+oRDONLY);
  /*
    The returned error must be eISDIR. If not, complain.
  */
  if ((fd!=-1) || (ErrNo!=fseISDIR)) {
    Complain("The directory test failed (incorrect error returned)");
    if (fd!=-1) NewClose(fd);
  }
}

bool ChangeTestDirMod(int16 NewMod)
{
  int16 fd;
  bool Result;

  /*
    This operation is somewhat complex, because OSHS's kernel call
    don't accept file specification but insists on files/directories being
    opened before their attributes can be altered. This is to avoid race
    conditions that are present in the case of traditional attribute-changing
    UNIX calls which accept file specification (in the time after the file
    specification is verified but before the kernel call is issued).
  */
  fd=NewOpenDir(TEST_DIR);
  if (fd==-1) {
    Complain("Unable to open parent directory");
    return(-1);
  }
  Result=true;
  if (!NewChMod(fd,NewMod)) {
    Complain("Error changing mode of the directory");
    Result=false;
  }
  if (!NewCloseDir(fd)) {
    Complain("Error closing parent directory");
    Result=false;
  }
  return(Result);
}

int16 Main(void)
{
  /*
    Activate the temporary drive
  */
  if (!NewSetCurrentDrive('T')) Complain("Cannot activate the test drive");
  /*
    Create a directory.
  */
  if (!NewMkDir(TEST_DIR)) {
    Complain("Unable to create the test directory");
    return(1);
  }
  /*
    Perform the test.
  */
  CheckTheDirectory();
  /*
    Change permissions of the directory to such that the current user can't
    read it.
  */
  if (!ChangeTestDirMod(0)) {
    Complain("Unable to change mode of the test directory");
    return(1);
  }
  /*
    Perform the test again.
  */
  CheckTheDirectory();
  /*
    Remove the directory.
  */
  if (!NewRmDir(TEST_DIR)) {
    Complain("Unable to remove the test directory");
    return(1);
  }
  /*
    The test is now done.
  */
  return(0);
}
