/*
  Test: Unblocking a server socket

  When a thread blocked by waiting to a connection on a server and another
  thread closes the socket on which the waiter is waiting, the waiting call
  in the waiter is required to return the eASYNCLOSE error. This check checks
  if this condition is met.
*/

#include "types.h"
#include "time.h"
#include "kernel.h"
#include "pddev.h"
#include "thread.h"
#include "conio.h"
#include "process.h"
#include "pdinet.h"

#include "complain.i"
#include "fileio.i"

static int16 Socket;
static bool Done;
static bool Done2;

static int16 INETHandle;

static bool CommandDevice(void *Cmd,uint16 CmdSize,void *Rep,uint16 RepSize)
{
  /*
    Send the command packet into the device
  */
  ErrNo=KCALL__SEND(INETHandle,Cmd,CmdSize);
  if (ErrNo!=0) {
    SetErrNo(ErrNo);
    return(false);
  }
  /*
    Read the reply from the screen device
  */
  ErrNo=KCALL__RECV(INETHandle,Rep,RepSize,NULL);
  SetErrNo(ErrNo);
  /*
    Return success flag
  */
  return(ErrNo>=0);
}

static bool OpenINETDevice(void)
{
  TNoParamCmd C;
  TDevInfoReply R;
  TOpenDeviceCmd OC;
  TOpenDeviceReply OR;

  INETHandle=hndDEVICES;
  C.Command=cmdFindFirstDevice;
  while (R.Type!=dtInvalid) {
    R.Type=dtInvalid;
    if (!CommandDevice(&C,sizeof(C),&R,sizeof(R))) return(true);
    if (R.Type==dtInternet) break;
    C.Command=cmdFindNextDevice;
  }
  if (R.Type==dtInvalid) {
    SetErrNo(beUNKERR);
    return(true);
  }
  OC.Command=cmdOpenDevice;
  OC.Device=R.ID;
  OR.Handle=-1;
  if (!CommandDevice(&OC,sizeof(OC),&OR,sizeof(OR))) return(true);
  if (OR.Handle==-1) {
    SetErrNo(beUNKERR);
    return(true);
  }
  INETHandle=OR.Handle;
  return(false);
}

static int16 OpenServerSocket(uint16 Port,uint16 QueueSize)
{
  TOpenTCPServerCommand C;
  TOpenTCPServerReply R;

  C.Command=inetcmdOpenTCPServer;
  C.Port=Port;
  C.QueueSize=QueueSize;
  if (!CommandDevice(&C,sizeof(C),&R,sizeof(R))) return(-1);
  return(R.Handle);
}

static int16 Listener(void *Data)
{
  TConnectionRecord R;

  ErrNo=KCALL__SEND(Socket,NULL,0);
  if (ErrNo==0) ErrNo=KCALL__RECV(Socket,&R,sizeof(R),NULL);
  if (ErrNo==sizeof(R)) {
    Complain("An unexpected connection was accepted");
  } else {
    if (ErrNo!=keCLOSED) Complain("Invalid error code returned");
  }
  Done=true;
  /*
    To keep the compiler happy
  */
  return(0);
}

static int16 Closer(void *Data)
{
  NewClose(Socket);
  Done2=true;
  return(0);
}

int16 Main(void)
{
  int16 ThreadID;
  int16 CloserID;

  /*
    Ensure the device is open
  */
  if (OpenINETDevice()) Complain("Unable to open INET device");
  /*
    Create the server socket
  */
  Socket=OpenServerSocket(5678,1);
  if (Socket==-1) Complain("Unable to open server socket");
  /*
    Create a thread listening on the server socket
  */
  Done=false;
  ThreadID=SYSCreateNewThread(Listener,NULL);
  if (ThreadID==-1) Complain("Unable to create the listener thread");
  /*
    Wait for a small amount of time
  */
  DelayThread(500000);
  /*
    Close the server socket
  */
  Done2=true;
  CloserID=SYSCreateNewThread(Closer,NULL);
  if (CloserID<0) Complain("Unable to create the closer thread");
  /*
    Wait for a larger amount of time again
  */
  DelayThread(500000);
  /*
    Check if the thread terminated
  */
  if (!Done || !Done2) Message("The threads are still alive");
  /*
    To keep the compiler happy
  */
  return(0);
}
